package org.objectweb.speedo.api;

/**
 * Defines the Speedo options which can be passed as properties to the
 * PersistenceManagerFactory
 */
public interface SpeedoProperties {

    String LOGGER_NAME = "org.objectweb.speedo";

	String DEBUG = "org.objectweb.speedo.debug"; 

    // Constant usable in the Properties of a PersistenceManagerFactory

    /**
     * Is the property value to pass to the PersistenceManagerFactory to
     * indicate that the size is unlimited.
     */
    String NO_LIMIT = "nolimit";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * cache size.
     */
    String CACHE_SIZE = "org.objectweb.speedo.cache.size";

	String CACHE_AUTO_CLEAN_SIZE = "org.objectweb.speedo.cache.autoCleanSize";

	String CACHE_REPLCAEMENT = "org.objectweb.speedo.cache.replacement";
	String CACHE_REPLCAEMENT_FIFO = "FIFO";
	String CACHE_REPLCAEMENT_LRU = "LRU";
	String CACHE_REPLCAEMENT_MRU = "MRU";

    /**
     * Is the property to pass to the PersistenceManagerFactory to indicate
     * if the transactionnal environnement is managed.
     */
    String MANAGED = "org.objectweb.perseus.connector.ra.jdo.managed";

	String TM_NAME = "org.objectweb.speedo.jca.TMName";
	/**
	 * @deprecated use TM_NAME
	 */
	String TM_NAME_old = "org.objectweb.perseus.connector.ra.jdo.TMName";
    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * maximum size of PersistenceManager pool.
     */
    String PM_POOL_MIN = "org.objectweb.speedo.persistencemanager.pool.min";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * minimum size of PersistenceManager pool.
     */
    String PM_POOL_MAX = "org.objectweb.speedo.persistencemanager.pool.max";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * maximum size of Query pool.
     */
    String QUERY_POOL_MIN = "org.objectweb.speedo.query.pool.min";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * minimum size of Query pool.
     */
    String QUERY_POOL_MAX = "org.objectweb.speedo.query.pool.max";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * maximum size of compiled query pool.
     */
    String COMPILED_QUERY_POOL_MIN = "org.objectweb.speedo.compiledquery.pool.min";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * minimum size of compiled query pool.
     */
    String COMPILED_QUERY_POOL_MAX = "org.objectweb.speedo.compiledquery.pool.max";

    /**
     * Is the property to pass to the PersistenceManagerFactory to specify if
     * the query must prefetch the data. The default value is true.
     */
    String PREFETCH_ON_QUERY = "org.objectweb.speedo.query.prefetch";

    /**
     * Is the property to pass to the PersistenceManagerFactory to specify the
	 * mapping structure management:
	 * - DO_NOTHING
	 * - CREATE_IF_REQUIRED
	 * - FORCE_CREATE
     */
    String MAPPING_STRUCTURE = "org.objectweb.speedo.mappingStructure";
    String MAPPING_STRUCTURE_DN = "DO_NOTHING";
    String MAPPING_STRUCTURE_CIR = "CREATE_IF_REQUIRED";
    String MAPPING_STRUCTURE_FC = "FORCE_CREATE";
    String MAPPING_STRUCTURE_DD = "DELETE_DATA";

	/**
	 * Is the property to pass to the PersistenceManagerFactory to specify if
	 * the query must prefetch the data. The default value is true.
	 */
	String TX_LISTENER = "org.objectweb.speedo.txListener";
	String CONNECTION_POOL_MIN = "org.objectweb.speedo.connection.pool.min";
	String CONNECTION_POOL_MAX = "org.objectweb.speedo.connection.pool.max";


    // Constant usable in the .jdo files (keys of the extension tags)

    String VENDOR_NAME = "speedo";

    String ID = "id";
    String ID_SEQUENCE = "sequence";
    String ID_POLYMORPH_2L = "polymorph2l";

    /**
     * is a string used in an extension tag in a.jdo file to specify
     * the sql column for a primitive field (not a reference).
     */
    String SQL_NAME = "sql-name";

    /**
     * is a string used in an extension tag in a.jdo file to specify
     * the name of the sequence used for the datastore id.
     */
    String SQL_SEQ_NAME = "sql-seq-name";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the sql type for a primitive field (not a reference).
     */
    String SQL_TYPE = "sql-type";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the size of primitive field (not a reference).
     */
    String SIZE = "size";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the foreign keys referencing the current persistent object.
     */
    String SOURCE_FK = "source-foreign-keys";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the foreign keys referencing the referenced persistent object.
     */
    String TARGET_FK = "target-foreign-keys";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the name of the reverse field.
     */
    String REVERSE_FIELD = "reverse-field";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the if persistent instance referenced by a field, must be removed when
     * the current class is deleted.
     */
    String CASCADE_DELETE = "cascade-delete";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the name of the join table in a multivalued reference (Collection, Map,
     * Array, Set, List).
     */
    String JOIN_TABLE = "join-table";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the name of the class permiting the conversion of a field between the
     * memory and the data support.
     */
    String FIELD_CONVERTER = "field-converter";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the column name of the index of the list or of the key in a map.
     */
    String INDEX = "index-sql-name";

    String INDEX_TYPE = "index-sql-type";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the column name of the element of the list or of the value in a map,
	 * when the element is a prmitive.
     */
    String ELEMENT = "element-sql-name";

    String ELEMENT_TYPE = "element-sql-type";
}