/**
 * copyright area
 */
package org.objectweb.speedo.generation;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.api.GeneratorComponent;
import org.objectweb.speedo.generation.api.SpeedoCompilerParameter;
import org.objectweb.speedo.generation.compiler.Compiler;
import org.objectweb.speedo.generation.enhancer.AccessEnhancer;
import org.objectweb.speedo.generation.enhancer.PersistenceAwareEnhancer;
import org.objectweb.speedo.generation.enhancer.FieldModifier;
import org.objectweb.speedo.generation.enhancer.DummyAccessorAdder;
import org.objectweb.speedo.generation.generator.SpeedoGenerator;
import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;
import org.objectweb.speedo.generation.parser.JdoParser;
import org.objectweb.speedo.generation.jorm.JormGenerator;
import org.objectweb.speedo.generation.mivisitor.MetaInfoVisitors;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.Monolog;

import java.util.HashMap;

/**
 * TODO: put the file of the meta information into the jar file
 * @author S.Chassande-Barrioz
 */
public class SpeedoCompiler extends AbstractGeneratorComponent {

    public final static String LOGGER_NAME =
            SpeedoProperties.LOGGER_NAME + ".generation.SpeedoCompiler";

    public Object[][] actions = null;

    // IMPLEMENTATION OF THE GeneratorComponent INTERFACE //
    //----------------------------------------------------//
    public SpeedoCompilerParameter getSpeedoCompilerParameter() {
        if (scp == null)
            scp = new SpeedoCompilerParameter();
        return scp;
    }

    public void init() throws SpeedoException {
        // Initialize the logger factory if necessary
        if (scp.loggerFactory == null) {
            if (scp.logPropFile == null) {
                scp.loggerFactory = Monolog.init();
            } else {
                scp.loggerFactory = Monolog.init(scp.logPropFile);
            }
            logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        } else if (logger == null) {
            logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        }
        scp.nmf.setLogger(logger);

        if (actions == null) {
            actions = new Object[][]{
                {new JdoParser(), "Loading jdo files..."},
                {new MetaInfoVisitors(), "Completing the meta data..."},
                {new JormGenerator(), "Generating Jorm files..."},
                {new SpeedoGenerator(), "Generating Java files..."},
                {new FieldModifier(true), null},
                {new DummyAccessorAdder(true), null},
                {new Compiler(), "Compiling Java files..."},
                {new FieldModifier(false), null},
                {new DummyAccessorAdder(false), null},
                {new AccessEnhancer(), "Enhancing Persistent classes..."},
                {new PersistenceAwareEnhancer(), "Enhancing Persistent aware classes..."}
            };
        }
        scp.xmldescriptor = new HashMap();
    }

    public void process() throws SpeedoException {
        if (scp.jdo.isEmpty())
            return;

        // Launch the 5 steps of the speedo compiler
        GeneratorComponent gc = null;
        for (int i = 0; i < actions.length; i++) {
            if (actions[i][1] != null) {
                logger.log(BasicLevel.INFO, actions[i][1]);
            }
            gc = (GeneratorComponent) actions[i][0];
            gc.setSpeedoCompilerParameter(scp);
            gc.init();
            gc.process();
        }
        logger.log(BasicLevel.INFO, "Done.");
    }
}
