/**
 * copyright area
 */
package org.objectweb.speedo.generation.enhancer;

import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoEnhancerException;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.asm.Constants;
import org.objectweb.util.monolog.api.Logger;

import java.util.Collection;
import java.util.Iterator;
import java.util.HashSet;
import java.util.Set;

public class DummyAccessorAdder extends EnhancerComponent {

    public final static String LOGGER_NAME
        = SpeedoProperties.LOGGER_NAME + ".generation.enhancer.fieldMdodifier";

    boolean add;

    public DummyAccessorAdder(boolean add) {
        this.add = add;
    }

    public void init() throws SpeedoException {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
    }

    public void process() throws SpeedoException {
        if (scp.xmldescriptor == null || scp.xmldescriptor.isEmpty())
            throw new SpeedoEnhancerException("xmldescriptor not initialised");
        Collection xmls = scp.xmldescriptor.values();
        for (Iterator itDesc = xmls.iterator(); itDesc.hasNext();) {
            SpeedoXMLDescriptor desc = (SpeedoXMLDescriptor) itDesc.next();
            for (Iterator itclass = desc.getSpeedoClasses().iterator(); itclass.hasNext();) {
                Set fields = new HashSet();
                SpeedoClass jdoclass = (SpeedoClass) itclass.next();
                Iterator it = jdoclass.jdoField.values().iterator();
                while (it.hasNext()) {
                    SpeedoField sf = (SpeedoField) it.next();
                    if (sf.relationType != SpeedoField.NO_RELATION) {
                        fields.add(sf);
                    }
                }
                if (fields.isEmpty()) {
                    continue;
                }
                ClassWriter cw = new ClassWriter(false);
                DummyAccessorModifier fam = new DummyAccessorModifier(
                    cw, logger, fields);
                String name = jdoclass.getFQName();
                ClassReader cr = loadJavaClass(false, name, scp.output, false);
                cr.accept(fam, false);
                writeJavaClass(name, cw, scp.output);
            }
        }
    }

    class DummyAccessorModifier extends LoggedClassAdapter implements Constants {

        private Set methods;

        /**
         * @param fields is a set of field names
         */
        public DummyAccessorModifier(ClassVisitor classVisitor,
                                     Logger logger,
                                     Set fields) {
            super(classVisitor, logger);
            this.methods = new HashSet();
            Iterator i = fields.iterator();
            while (i.hasNext()) {
                SpeedoField f = (SpeedoField)i.next();
                String getter = NamingRules.getterName(f.jdoClass, f.name);
                String setter = NamingRules.setterName(f.jdoClass, f.name);
                methods.add(getter);
                methods.add(setter);
                if (add) {
                    // add dummy getter
                    CodeVisitor cv = this.cv.visitMethod(ACC_PUBLIC, getter, "()" + f.desc, null);
                    cv.visitInsn(ACONST_NULL);
                    cv.visitInsn(ARETURN);
                    cv.visitMaxs(1, 1);
                    // add dummy setter
                    cv = this.cv.visitMethod(ACC_PUBLIC, setter, "(" + f.desc + ")V", null);
                    cv.visitInsn(RETURN);
                    cv.visitMaxs(2, 2);
                }
            }
        }

        public CodeVisitor visitMethod(int i, String s, String s1, String[] strings) {
            if (!add && methods.contains(s)) {
                // remove dummy getter or setter
                return null;
            } else {
                return super.visitMethod(i, s, s1, strings);
            }
        }
    }
}
