/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.naming.lib;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.metainfo.api.HelperClassMapping;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.Reference;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.api.JormFactory;
import org.objectweb.speedo.generation.jorm.JormMIMappingBuilder;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.naming.api.MIBuilderHelper;
import org.objectweb.speedo.naming.api.NamingManager;
import org.objectweb.speedo.runtime.binder.RdbSequenceBinder;
import org.objectweb.speedo.runtime.binder.RdbSequencePName;
import org.objectweb.util.monolog.api.Logger;

import java.util.Collection;
import java.util.Map;
import java.util.Iterator;
import java.util.ArrayList;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class RdbSequenceNamingManager implements NamingManager {

    private final static String BINDER_CLASS_NAME
            = "org.objectweb.speedo.runtime.binder.RdbSequenceBinder";


    public boolean canManage(SpeedoClass sc) {
        if (sc.identityType == SpeedoIdentity.CONTAINER_ID) {
            SpeedoExtension se = sc.getExtension(
                    SpeedoProperties.VENDOR_NAME, SpeedoProperties.ID);
            if (se != null && SpeedoProperties.ID_SEQUENCE.equals(se.value)) {
                return true;
            }
            se = sc.getExtension(
                    SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_NAME);
            return se != null;
        } else {
            return false;
        }
    }


    // IMPLEMENTATION OF THE METHOD FROM THE NamingManager INTERFACE //
    //---------------------------------------------------------------//

    public boolean supportPNamingcontext() {
        return false;
    }

    public PName decode(PNameCoder pnc,
                        Object oid,
                        Class clazz,
                        JormFactory jf) throws PException {
        if (oid instanceof String) {
            String stroid = (String) oid;
            if (pnc != null) {
                if (pnc instanceof RdbSequenceBinder) {
					int idx = stroid.indexOf(SEP);
                    if (idx != -1) {
                        //The oid contains the class name
                        return pnc.decodeString(stroid.substring(idx + SEP.length()));
                    } else {
                        //The oid must decoded directly
                        return pnc.decodeString(stroid);
                    }

                } else {
                    //The pnc is not a RdbSequenceBinder, then the oid cannot be managed
                    return null;
                }
            } else {
                //No pnc specified
				int idx = stroid.indexOf(SEP);
                if (idx != -1) {
                    //The oid contains the class name
                    String fqcn = stroid.substring(0, idx);
                    ClassLoader cl = jf.getClass().getClassLoader();
                    if (cl == null) {
                        cl = ClassLoader.getSystemClassLoader();
                    }
                    try {
                        pnc = jf.getPBinder(fqcn, cl);
                    } catch (Exception e) {
                    }
                    return (pnc instanceof RdbSequenceBinder
                        ? pnc.decodeString(stroid.substring(idx + SEP.length()))
                        : null);
                } else {
                    //The oid cannot be managed
                    return null;
                }
            }
        } else if (clazz != null && pnc != null){
            if (oid instanceof Integer) {
                pnc.decodeLong(((Integer) oid).intValue());
            } else if (oid instanceof Long) {
                pnc.decodeLong(((Long) oid).intValue());
            }
        }
        return null;
    }

    public Object encode(PName pn) throws PException {
        if (pn instanceof RdbSequencePName) {
            return pn.getPNameManager().getPType().getJormName()
                    + SEP + pn.encodeString();
        }
        return null;
    }

    public void setPMapper(PMapper mapper) {
    }

    public void setLogger(Logger logger) {
    }

    public boolean canProvidePBinder(Object hints, ClassLoader classLoader) {
        return hints instanceof String
                && ((String) hints).startsWith(BINDER_CLASS_NAME + SEP);
    }

    public boolean canProvidePNamingContext(Object hints, ClassLoader classLoader) {
        return canProvidePBinder(hints, classLoader);
    }

    public PBinder getPBinder(String className,
							  String hints,
							  ClassLoader cl,
							  byte mappingStructureRule) {
        return new RdbSequenceBinder(className,
				hints.substring(
                BINDER_CLASS_NAME.length() + SEP.length()),
				mappingStructureRule);
    }

    public PNamingContext getPNamingContext(String className,
                                            String hints,
                                            ClassLoader classLoader,
                                            Map binders) throws PException {
        return null;
    }

    public String getJormConfig(NameDef nd,
                                SpeedoClass targetClass,
                                MetaObject sourceMO) {
        SpeedoExtension se = targetClass.getExtension(
                SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_SEQ_NAME);
        String seqName;
        if (se != null) {
            seqName = se.value;
        } else {
            se = targetClass.getExtension(
                SpeedoProperties.VENDOR_NAME, SpeedoProperties.SQL_NAME);
            if (se == null) {
                seqName = targetClass.name.toUpperCase() + "_SEQ";
            } else {
                seqName = se.value + "_SEQ";
            }
        }
        return BINDER_CLASS_NAME + SEP + seqName;
    }

    public String getPNameHints(SpeedoClass sc, NameDef nd) {
        return "null";
    }

    public String getGCPNameHints(SpeedoClass sc, NameDef nd) {
        return "pname";
    }

    /**
     * The auto incremented field used as identifier can be a visble or
     * an hidden field. In case the field is visible, the
     */
    public void fillNameDef(MIBuilderHelper mibh,
                            Manager manager,
                            NameDef nd,
                            SpeedoClass tsc,
                            SpeedoClass ssc,
                            MetaObject mo,
                            Reference ref,
                            HelperClassMapping hcm,
                            JormMIMappingBuilder mb,
                            boolean isIdentifier,
                            boolean isInGenClass,
                            boolean createField,
                            Collection createdMOs) throws SpeedoException, PException {
        Iterator it = tsc.jdoField.values().iterator();
        SpeedoField sf = null;
        ArrayList al = null;
        while(it.hasNext()) {
            SpeedoField _sf = (SpeedoField) it.next();
            if (_sf.primaryKey) {
                if (sf != null) {
                    if (al == null) {
                        al = new ArrayList();
                    }
                    al.add(_sf.name);
                } else {
                    sf = _sf;
                }
            }
        }
        if (al != null) {
            al.add(sf);
            throw new SpeedoException(
                    "Impossible to use an auto incremented identifier if " +
                    "several fields have been marked as primary-key " + al
                    + " in the class '" + tsc.getFQName() + "'.");
        }
        String fieldName = null;
        fieldName = mibh.getNameDefFieldPrefix(ref, isIdentifier, isInGenClass);
        if (sf == null) {
            fieldName += "id_";
        } else {
            if (!"long".equals(sf.type())) {
                throw new SpeedoException(
                        "Impossible to use an auto incremented identifier: " +
                        "the field type of '" + sf.name + "' is '" + sf.type()
                        + "' and 'long' is expected (class '"
                        + tsc.getFQName() + "'.");
            }
            if (!isInGenClass && fieldName.length() == 0) {
                //The field already exists and its mapping is defined when primitive
                // fields are mapped.
                createField = false;
            }
            fieldName += sf.name;
        }
        PrimitiveElement id = null;
        if (createField) {
            id = mibh.createNameDefField(mo, fieldName, PTypeSpace.LONG);
        } else {
            id = mibh.getPrimitiveField(mo, fieldName);
            if (id == null) {
                throw new SpeedoException(
                        mibh.getErrorMessage(tsc, mo, ref)
                        + " Impossible to get the field '" + fieldName + "'");
            }
        }
        nd.setFieldName(fieldName);
        if (createField) {
            mb.createNameDefMapping(hcm, nd, ssc, isIdentifier, isInGenClass);
        }
    }

    public NamingField[] getNamingfields(SpeedoClass sc) throws PException {
        Iterator it = sc.jdoField.values().iterator();
        while(it.hasNext()) {
            SpeedoField sf = (SpeedoField) it.next();
            if (sf.primaryKey) {
                return new NamingField[] {
                    new NamingField(sf.name, Long.TYPE, "", PTypeSpace.LONG)};
            }
        }
        return null;
    }
}
