/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.naming.lib;

import org.objectweb.speedo.naming.api.NamingManager;
import org.objectweb.speedo.naming.api.MIBuilderHelper;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.generation.jorm.JormMIMappingBuilder;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.JormFactory;
import org.objectweb.speedo.api.UserId;
import org.objectweb.speedo.api.UserIdFactory;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.Reference;
import org.objectweb.jorm.metainfo.api.HelperClassMapping;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.NameRef;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.asm.Type;
import org.objectweb.util.monolog.api.Logger;

import javax.jdo.JDOUserException;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class UserIdCompositeNamingManager implements NamingManager {


    // IMPLEMENTATION OF THE METHOD FROM THE NamingManager INTERFACE //
    //---------------------------------------------------------------//
    public boolean supportPNamingcontext() {
        return false;
    }

    public boolean canManage(SpeedoClass sc) {
        if (sc.identityType == SpeedoIdentity.USER_ID) {
            ArrayList idFields = new ArrayList();
            Iterator fieldsIt = sc.jdoField.values().iterator();
            while (fieldsIt.hasNext()) {
                SpeedoField sp = (SpeedoField) fieldsIt.next();
                if (sp.primaryKey) {
                    idFields.add(sp);
                }
            }
            String objectidClass = sc.objectidClass;
            if (sc.generateObjectId()) {
                objectidClass = NamingRules.generatedObjectIdName(sc.getFQName());
            }
            return objectidClass != null && objectidClass.length() > 0;
        }
        return false;
    }

    public Object encode(PName pn) throws PException {
        if (pn instanceof UserIdFactory) {
            return ((UserIdFactory) pn).getNewUserId();
        }
        return null;
    }

    public PName decode(PNameCoder pnc,
                        Object oid,
                        Class clazz,
                        JormFactory jf) throws PException {
        if (oid instanceof UserId) {
            //the oid is a UserId
            if (pnc != null) {
                //Use the pnc found by the clazz
                return pnc.decodeAbstract(oid, null);
            } else if (((UserId) oid).jdoGetPersistentClassName() != null) {
                //No pnc but the class name is inside the UserId
                ClassLoader cl = oid.getClass().getClassLoader();
                if (cl == null) {
                    cl = jf.getClass().getClassLoader();
                }
                if (cl == null) {
                    cl = ClassLoader.getSystemClassLoader();
                }
                return jf.getPBinder(
                        ((UserId) oid).jdoGetPersistentClassName(), cl)
                        .decodeAbstract(oid, null);
            }
        } else if (pnc != null
                && (oid instanceof String)
                && (pnc.getNull() instanceof UserIdFactory)) {
            //The oid is a String which must be the parameter of the constructor
            Class idclass = ((UserIdFactory) pnc.getNull())
                    .getNewUserId().getClass();
            Object o = null;
            try {
                o = idclass.getConstructor(new Class[]{String.class})
                        .newInstance(new Object[]{(String) oid});
            } catch (Exception e) {
                throw new JDOUserException("No constructor with a String " +
                        "parameter available on the identifier class: "
                        + idclass.getName());
            }
            //recall with the UserId
            return decode(pnc, o, clazz, jf);
        }

        return null;
    }

    public void setPMapper(PMapper mapper) {
    }

    public void setLogger(Logger logger) {
    }

    public boolean canProvidePBinder(Object hints, ClassLoader classLoader) {
        if (!(hints instanceof String)) {
            return false;
        }
        try {
            classLoader.loadClass((String) hints);
            return true;
        } catch (ClassNotFoundException e) {
            return false;
        }
    }

    public boolean canProvidePNamingContext(Object hints, ClassLoader classLoader) {
        return canProvidePBinder(hints, classLoader);
    }

    public PBinder getPBinder(String className, String hints, ClassLoader classLoader, byte mappingStructureRule) throws PException {
        try {
            return (PBinder) classLoader.loadClass(hints).newInstance();
        } catch (Exception e) {
            throw new PException(e,
                    "Impossible to instanciate the binder of the class "
                    + hints);
        }
    }

    public PNamingContext getPNamingContext(String className,
                                            String hints,
                                            ClassLoader classLoader,
                                            Map binders) throws PException {
        return null;
    }

    public void fillNameDef(MIBuilderHelper mibh,
                            Manager manager,
                            NameDef nd,
                            SpeedoClass tsc,
                            SpeedoClass ssc,
                            MetaObject mo,
                            Reference ref,
                            HelperClassMapping hcm,
                            JormMIMappingBuilder mb,
                            boolean isIdentifier,
                            boolean isInGenClass,
                            boolean createField,
                            Collection createdMOs) throws SpeedoException, PException {
        ArrayList idFields = new ArrayList();
        Iterator fieldsIt = tsc.jdoField.values().iterator();
        while (fieldsIt.hasNext()) {
            SpeedoField sf = (SpeedoField) fieldsIt.next();
            if (sf.primaryKey) {
                idFields.add(sf);
            }
        }
        String objectidClass = tsc.objectidClass;
        if (tsc.generateObjectId()) {
            objectidClass = NamingRules.generatedObjectIdName(tsc.getFQName());
        }
        //The user has specified an object id ==> use it for the namedef
        CompositeName cn = manager.getCompositeName(objectidClass);
        boolean cnnotdefined = (cn == null);
        if (cnnotdefined) {
            cn = manager.createCompositeName(objectidClass);
            createdMOs.add(cn);
        }
        NameRef nr = nd.createNameRef(cn);
        for (int i = 0; i < idFields.size(); i++) {
            SpeedoField sf = (SpeedoField) idFields.get(i);
            int size = PType.NOSIZE;
            SpeedoExtension se = sf.getExtension(SpeedoProperties.VENDOR_NAME,SpeedoProperties.SIZE);
            if (se != null) {
                try {
                    size = Integer.parseInt(se.value);
                } catch (NumberFormatException e) {
                    //The error will occur during the parsing of the primitive
                    // field in the referenced class
                }
            }
            if (cnnotdefined) {
                cn.createCompositeNameField(sf.name,
                        mibh.getPrimitivePType(Type.getType(sf.desc)),
                        size);
            }
            String fn = mibh.getNameDefFieldPrefix(
                    ref, isIdentifier, isInGenClass) + sf.name;
            if (createField) {
                fn = mibh.createNameDefField(mo, fn,
                        mibh.getPrimitivePType(Type.getType(sf.desc)), size).getName();
            }
            nr.addProjection(sf.name, fn);
        }
        if (createField) {
            mb.createNameDefMapping(hcm, nd, ssc, isIdentifier, isInGenClass);
        }
    }

    public String getPNameHints(SpeedoClass sc, NameDef nd) {
        return "this";
    }

	public String getGCPNameHints(SpeedoClass sc, NameDef nd) {
		return "pname";
	}

    public String getJormConfig(NameDef nd,
                                SpeedoClass targetClass,
                                MetaObject sourceMO) {
        CompositeName cn = nd.getNameRef().getCompositeName();
        return NamingRules.binderName(cn.getFQName());
    }

    public NamingManager.NamingField[] getNamingfields(SpeedoClass sc) {
        return null;
    }
}
