/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.speedo.runtime.dom;

import org.objectweb.perseus.persistence.lib.AbstractDirtyObjectManager;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.perseus.persistence.api.CacheEntryFilter;
import org.objectweb.perseus.persistence.api.WorkingSet;
import org.objectweb.perseus.cache.api.CacheEntry;
import org.objectweb.speedo.api.SpeedoAccessor;
import org.objectweb.speedo.api.LifeCycle;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.List;

public class SpeedoDirtyObjectManager extends AbstractDirtyObjectManager {

    public void makeDirty(CacheEntry cacheEntry) {
        SpeedoAccessor sa = (SpeedoAccessor) cacheEntry;
        sa.jdoChangeStatus(LifeCycle.ACTION_WRITEFIELD_ACTIVEDATASTORETRANSACTION);
		sa.jdoSetFlushed(false);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, "Mark dirty an instance, oid="
					+ cacheEntry.getCeIdentifier(),
					new Exception("Stack trace for information"));
		}
    }

    public void makeClean(CacheEntry cacheEntry) {
        SpeedoAccessor sa = (SpeedoAccessor) cacheEntry;
        if (LifeCycle.isDirty(sa.jdoGetStatus())) {
            sa.jdoSetStatus(LifeCycle.PERSISTENT_CLEAN);
			logger.log(BasicLevel.DEBUG, "Mark as clean an instance, oid=" + cacheEntry.getCeIdentifier());
        }
		sa.jdoSetFlushed(false);
    }

    public void flush(Object context, List cacheEntries) throws PersistenceException {
		for (int i = 0; i < cacheEntries.size(); i++) {
			SpeedoAccessor sa = (SpeedoAccessor) cacheEntries.get(i);
            logger.log(BasicLevel.DEBUG, "flush: status" + LifeCycle.statusToString(sa.jdoGetStatus()) + " / isDirty" + LifeCycle.isDirty(sa.jdoGetStatus()));
            if (LifeCycle.isDirty(sa.jdoGetStatus())) {
				if (!sa.jdoHasBeenFlush()) {
                	sm.write(context, sa.jdoGetSpeedoProxy().getPName(), sa);
				} else {
					sa.jdoSetFlushed(false);
				}
            }
        }
    }

    public void flush(Object context, CacheEntry cacheEntry) throws PersistenceException {
        throw new PersistenceException(
                "Not supported: use flush method with list of cache entries");
    }

	/**
	 * This method permits to flush entries associated to a context.
	 * @param context is an identifier of the execution context. It can be a
	 * transaction handle.
	 * @param cefilter is a filter of the cache entry which must be flushed
	 * @throws PersistenceException if the context is not valid.
	 */
	public void flush(Object dataAccess,
					  Object context,
					  CacheEntryFilter cefilter) throws PersistenceException {
		List l = ((WorkingSet) context).entries();
		for(int i=0; i<l.size(); i++) {
			SpeedoAccessor sa = (SpeedoAccessor) l.get(i);
			if (LifeCycle.isDirty(sa.jdoGetStatus()) && cefilter.accept(sa)) {
				sa.jdoSetFlushed(true);
				sm.write(dataAccess, sa.getCeIdentifier(), sa.getCeObject());
			}
		}
	}

}
