/**
 * copyright area
 */

package org.objectweb.speedo.runtime.genclass;

import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.perseus.persistence.api.TransactionalPersistenceManager;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.perseus.persistence.api.RolledBackPersistenceException;
import org.objectweb.speedo.runtime.SpeedoTransaction;
import org.objectweb.speedo.api.SpeedoProxy;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.api.SpeedoAccessor;
import org.objectweb.speedo.api.SpeedoGenClassProxy;

import javax.jdo.PersistenceManager;
import javax.jdo.JDOFatalException;
import javax.jdo.JDOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * This class is an implementation of the iterator interface. It iterates over
 * an inner iterator which the elements are GenClassElement. The main roles of
 * this iterator implementation are
 * - to skip the GenClassElement marked as deleted
 * - to resolve PName references if the element are persistent object
 *
 * @author Sebastien Chassande-Barrioz
 */
public class PIndexedElemIterator implements Iterator {

    /**
     * Is the inner iterator over PIndexexElem instances
     */
    protected Iterator iter;

    /**
     * is the next element which has not been deleted
     */
    protected GenClassElement next;

    /**
     * indicates if the next element has been computed
     */
    protected boolean nextComputed;

    /**
     * is the object to synchronize if an element must be removed
     */
    protected Object synchro = null;

    /**
     * is the persistence manager used to resolve the PName into reference.
     * If this field has a null value, that means the elements of the gen class
     * are not references but primitives.
     */
    protected PersistenceManager pm = null;

    protected Logger logger = null;


    public PIndexedElemIterator(Collection _elements, Object _synchro,
            PersistenceManager pm, Logger l) {
        iter = _elements.iterator();
        synchro = _synchro;
        this.pm = pm;
        logger = l;
    }


    // IMPLEMENTATION OF THE Iterator INTERFACE //
    //------------------------------------------//

    public boolean hasNext() {
        if (!nextComputed) {
            while (iter.hasNext()) {
                next = (GenClassElement) iter.next();
                if (next.getElemStatus() != PIndexedElem.ELEM_DELETED) {
                    nextComputed = true;
                    return true;
                }
            }
            next = null;
            nextComputed = true;
            return false;
        } else {
            return next != null;
        }
    }

    public Object next() {
        if (!nextComputed) {
            hasNext();
        }
        if (next == null) {
            throw new NoSuchElementException();
        } else {
            nextComputed = false;
            return (pm == null ? next.getElement() : next.getRefElement(pm));
        }
    }

    public void remove() {
        synchronized (synchro) {
            if (next != null) {
                SpeedoAccessor sa = next.getSpeedoAccessor();
                ((SpeedoGenClassProxy) sa.jdoGetSpeedoProxy()).jdoWriteIntention(sa);
                next.setStatus(PIndexedElem.ELEM_DELETED);
            }
        }
    }
}
