/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.genclass.collection;

import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.util.api.Loggable;

import org.objectweb.speedo.runtime.genclass.GenClassElement;
import org.objectweb.speedo.runtime.genclass.PIndexedElemIterator;
import org.objectweb.speedo.runtime.genclass.GenClassAccessor;
import org.objectweb.speedo.runtime.genclass.GenClass;
import org.objectweb.speedo.api.SpeedoProxy;
import org.objectweb.speedo.api.ProxyManager;
import org.objectweb.speedo.api.SpeedoGenClassProxy;
import org.objectweb.speedo.api.SpeedoAccessor;
import org.objectweb.speedo.api.Debug;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;

/**
 * @author	S.Chassande-Barrioz
 */
public class CollectionAccessor
        extends GenClassAccessor
        implements Collection {

	/**
	 * is the internal collection used
	 */
	protected Collection collection = null;

    /**
     * Instanciates and initializes a new collection with an initial size.
     */
    public CollectionAccessor(SpeedoGenClassProxy jdoProxy) {
        super(jdoProxy);
		collection = (Collection) jdoProxy.createGenClass();
    }

    public String toString() {
        return "CollectionAccessor: id=" + jdoProxy.jdoGetGenClassId();
    }

	public GenClassElement speedoRemove2(Object o) {
		tmpelem.setElement(o);   // tmp type = etype.
		Iterator it = elements.iterator();
		while (it.hasNext()) {
			CollectionElem e = (CollectionElem) it.next();
			if (e.equals(tmpelem) && e.getElemStatus() != PIndexedElem.ELEM_DELETED) {
				e.setStatus(PIndexedElem.ELEM_DELETED);
				if (Debug.ON && getLogger() != null) {
					logger.log(BasicLevel.DEBUG, "speedoRemove2(" + o + "): removed");
				}
				return e;
			}
		}
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "speedoRemove2(" + o + "): not found");
		}
		return null;
	}

	protected GenClassElement speedoAdd2(Object o, boolean allowDuplicates) {
		if (!allowDuplicates && contains(o)) {
			if (Debug.ON && getLogger() != null) {
				logger.log(BasicLevel.DEBUG, "speedoAdd2(" + o + "): already exist");
			}
			return null;
		}
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "speedoAdd2(" + o + "): adding");
		}
		// Put it in the "elements" list
		GenClassElement element = (GenClassElement) createPIndexedElem();
		if (o instanceof SpeedoProxy) {
			ProxyManager pm = (ProxyManager) jdoProxy.jdoGetPersistenceManager();
			jdoProxy.jdoMakePersistent(pm, (SpeedoProxy) o);
		}
		element.setElement(o);
		synchronized (this) {
			if (elements.add(element)) {
				return element;
			} else {
				return null;
			}
		}
	}

	public void jdoDeletePersistent(ProxyManager pm) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "jdoDeletePersistent");
		}
		clear();
	}

	public void setGCValue(Object o) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "setGCValue(" + o + ")");
		}
		clear();
		addAll(((Collection) o));
	}

	public void jdoLoadFieldsFromAccessor(SpeedoAccessor sa) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "jdoLoadFieldsFromAccessor(" + sa + ")");
		}
		CollectionAccessor ca = (CollectionAccessor) sa;
		collection.clear();
		collection.addAll(ca.collection);
		inner = ca.getInner();
		supportDelta = ca.supportDelta;
		elements.clear();
		elements.addAll(ca.elements);
		jdoSetStatus(ca.jdoGetStatus());
	}

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE Collection INTERFACE
    // ------------------------------------------------------------------------

    public boolean add(Object o) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "add(" + o + ")");
		}
        GenClassElement gce = speedoAdd2(o, true);
        if (gce != null) {
			Object _o = gce.getRefElement(jdoProxy.jdoGetPersistenceManager());
            if (_o != null) {
				jdoProxy.fireSpeedoElementAdded(_o);
			}
        }
        return gce != null;
    }

    public boolean addAll(Collection c) {
        Iterator it = c.iterator();
        boolean changed = false;
        while (it.hasNext()) {
            changed |= add(it.next());
        }
        return changed;
    }

    public void clear() {
        if (elements.size() > 0) {
			if (Debug.ON && getLogger() != null) {
				logger.log(BasicLevel.DEBUG, "clear()");
			}
            Collection c = new ArrayList(this);
            Iterator i = c.iterator();
            while (i.hasNext()) {
                remove(i.next());
            }
            elements.clear(); // efficient, but does not work with relations
            supportDelta = false;
        } else {
			if (Debug.ON && getLogger() != null) {
				logger.log(BasicLevel.DEBUG, "clear(): empty collection");
			}
		}
    }

    public boolean contains(Object o) {
        tmpelem.setElement(o);
        Iterator it = elements.iterator();
        while (it.hasNext()) {
            PIndexedElem e = (PIndexedElem) it.next();
            if (e.equals(tmpelem) && e.getElemStatus() != PIndexedElem.ELEM_DELETED) {
				if (Debug.ON && getLogger() != null) {
					logger.log(BasicLevel.DEBUG, "contains(" + o + "): yes");
				}
                return true;
            }
        }
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "contains(" + o + "): no");
		}
        return false;
    }

    public boolean containsAll(Collection c) {
        Iterator it = c.iterator();
        while (it.hasNext()) {
            if (!contains(it.next())) {
                return false;
            }
        }
        return true;
    }

    public boolean equals(Object o) {
        return (o instanceof Collection) &&
                ((Collection) o).size() == size() &&
                containsAll((Collection) o);
    }

    public boolean isEmpty() {
        if (elements.isEmpty()) {
            return true;
        }
        Iterator it = elements.iterator();
        // The set is empty if it contains only deleted indexed elements
        while (it.hasNext()) {
            if (((PIndexedElem) it.next()).getElemStatus() != PIndexedElem.ELEM_DELETED) {
                return false;
            }
        }
        return true;
    }

    public Iterator iterator() {
        return new PIndexedElemIterator(elements, this,
                jdoProxy.jdoGetPersistenceManager(), ((Loggable) jdoProxy).getLogger());
    }

    public boolean remove(Object o) {
        GenClassElement gce = speedoRemove2(o);
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "remove: gce=" + gce);
		}
        if (gce != null) {
			Object _o = gce.getRefElement(jdoProxy.jdoGetPersistenceManager());
            if (_o != null) {
            	jdoProxy.fireSpeedoElementRemoved(_o);
			}
        }
        return gce != null;
    }

    public boolean removeAll(Collection c) {
        Iterator it = c.iterator();
        boolean removed = true;
        while (it.hasNext()) {
            removed &= remove(it.next());
        }
        return removed;
    }

    public boolean retainAll(Collection c) {
        throw new UnsupportedOperationException();
    }

    public int size() {
        Iterator it = elements.iterator();
        int size = 0;
        while (it.hasNext()) {
            if (((PIndexedElem) it.next()).getElemStatus() != PIndexedElem.ELEM_DELETED) {
                size++;
            }
        }
        return size;
    }

    public Object[] toArray() {
        ArrayList al = new ArrayList(elements.size());
        Iterator it = elements.iterator();
        while (it.hasNext()) {
            GenClassElement gce = (GenClassElement) it.next();
            if (gce.getElemStatus() != PIndexedElem.ELEM_DELETED) {
                al.add(gce.getElement());
            }
        }
        return al.toArray();
    }

    public Object[] toArray(Object[] a) {
        ArrayList al = new ArrayList(elements.size());
        Iterator it = elements.iterator();
        while (it.hasNext()) {
            GenClassElement gce = (GenClassElement) it.next();
            if (gce.getElemStatus() != PIndexedElem.ELEM_DELETED) {
                al.add(gce.getElement());
            }
        }
        return al.toArray(a);
    }

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE PGenClassAccessor INTERFACE
    // ------------------------------------------------------------------------

    public PIndexedElem createPIndexedElem(GenClassAccessor gca) {
        return new CollectionElem(gca);
    }

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE SpeedoGenClassCoherence INTERFACE
    // ------------------------------------------------------------------------

    public boolean speedoAdd(Object o) {
        return speedoAdd(o, false);
    }

    protected boolean speedoAdd(Object o, boolean allowDuplicates) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "speedoAdd(" + o + ", " + allowDuplicates + ")");
		}
        if (!allowDuplicates && contains(o)) {
			if (Debug.ON && getLogger() != null) {
				logger.log(BasicLevel.DEBUG, "speedoAdd(" + o + ", " + allowDuplicates + "): already exist");
			}
            return false;
        }
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "speedoAdd(" + o + ", " + allowDuplicates + "): adding");
		}
        // Put it in the "elements" list
        GenClassElement element = (GenClassElement) createPIndexedElem();
        if (o instanceof SpeedoProxy) {
            ProxyManager pm = (ProxyManager) jdoProxy.jdoGetPersistenceManager();
            jdoProxy.jdoMakePersistent(pm, (SpeedoProxy) o);
        }
        element.setElement(o);
        synchronized (this) {
            return elements.add(element);
        }
    }

    public boolean speedoRemove(Object o) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "speedoRemove(" + o + ")");
		}
		return speedoRemove2(o) != null;
    }
}
