/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.genclass.map;

import org.objectweb.speedo.api.SpeedoGenClassProxy;
import org.objectweb.speedo.api.SpeedoAccessor;
import org.objectweb.speedo.api.ProxyManager;
import org.objectweb.speedo.runtime.genclass.GenClassAccessor;
import org.objectweb.speedo.runtime.genclass.collection.SetAccessor;
import org.objectweb.jorm.type.api.PType;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Collection;
import java.util.HashMap;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class HashMapImpl
        extends HashMap
        implements SpeedoGenClassProxy {

    public HashMapImpl() {
        super();
		accessor = (MapAccessor) createAccessor();
    }

    public String toString() {
        return "";
    }

    public abstract void jdoWriteIntention(SpeedoAccessor sa);
    public abstract void jdoReadIntention(SpeedoAccessor sa);
    public abstract Object getInner();
    public abstract void jdoMakePersistent(ProxyManager pm, Iterator it);

	MapAccessor accessor;

	public GenClassAccessor getMainAccessor() {
		return accessor;
	}

    public Object createGenClass() {
        return new HashMap();
    }

    protected SpeedoAccessor createAccessor() {
        return new MapAccessor(this);
    }

    public void jdoMakePersistent(ProxyManager pm) {
        if (size() > 0) {
            jdoMakePersistent(pm, values().iterator());
        }
        if (getInner() != null) {
            jdoMakePersistent(pm, ((Map) getInner()).values().iterator());
        }
    }

    // IMPLEMENTS THE HashSet CLASS //
    //------------------------------//

    public int size() {
        if (!jdoIsActive()) {
            return accessor.size();
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.size();
        }
    }

    public boolean isEmpty() {
        if (!jdoIsActive()) {
            return accessor.isEmpty();
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.isEmpty();
        }
    }

    public boolean containsValue(Object o) {
        if (!jdoIsActive()) {
            return accessor.containsValue(o);
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.containsValue(o);
        }
    }

    public synchronized boolean containsKey(Object o) {
        if (!jdoIsActive()) {
            return accessor.containsKey(o);
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.containsKey(o);
        }
    }

    public synchronized Object get(Object o) {
        if (!jdoIsActive()) {
            return accessor.get(o);
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.get(o);
        }
    }

    public synchronized Object put(Object o, Object o1) {
        if (!jdoIsActive()) {
            return accessor.put(o, o1);
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.put(o, o1);
        }
    }

    public synchronized Object remove(Object o) {
        if (!jdoIsActive()) {
            return accessor.remove(o);
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.remove(o);
        }
    }

    public synchronized void putAll(Map map) {
        if (!jdoIsActive()) {
            accessor.putAll(map);
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            ma.putAll(map);
        }
    }

    public synchronized void clear() {
        if (!jdoIsActive()) {
            accessor.clear();
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            ma.clear();
        }
    }

    public Set keySet() {
        if (!jdoIsActive()) {
            return accessor.keySet();
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.keySet();
        }
    }

    public Set entrySet() {
        if (!jdoIsActive()) {
            return accessor.entrySet();
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.entrySet();
        }
    }

    public Collection values() {
        if (!jdoIsActive()) {
            return accessor.values();
        } else {
            MapAccessor ma = (MapAccessor) getSpeedoAccessor();
            jdoReadIntention(ma);
            return ma.values();
        }
    }
}

