/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo;

import junit.framework.TestCase;
import junit.framework.Assert;

import javax.jdo.PersistenceManagerFactory;
import javax.jdo.JDOHelper;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.speedo.log.Log;
import org.objectweb.speedo.api.SpeedoProperties;

import java.util.Properties;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.io.InputStream;
import java.io.IOException;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class SpeedoTestHelper extends TestCase {

	public final static String LOG_NAME = "org.objectweb.speedo";

	protected PersistenceManagerFactory pmf = null;
	protected Logger logger = null;
	protected LoggerFactory loggerFactory = null;


	public SpeedoTestHelper(String s) {
		super(s);
        getPMF();
        loggerFactory = Log.getLoggerFactory();
		logger = loggerFactory.getLogger(getLoggerName());
		logger.log(BasicLevel.DEBUG, "PersistenceManagerFactory instanciated (" + pmf.getConnectionURL() + ")");
	}

	protected abstract String getLoggerName();

	public PersistenceManagerFactory getPMF() {
        if (pmf == null) {
            pmf = JDOHelper.getPersistenceManagerFactory(getPMFProperties());
        }
        return pmf;
    }
    
	public Properties getPMFPropertiesFromFile() {
        Properties p = new Properties();
        InputStream is = getClass().getClassLoader()
                .getResourceAsStream("speedo.properties");
        if (is == null) {
            return null;
        }
        try {
            p.load(is);
        } catch (IOException e) {
            return null;
        }
        return p;
    }
	public Properties getPMFProperties() {
        if (Boolean.getBoolean("org.objectweb.speedo.useFile")) {
            return getPMFPropertiesFromFile();
        }
		String debug = System.getProperty("org.objectweb.speedo.debug", "false");
		String mapperName = System.getProperty("org.objectweb.speedo.mapperName", "rdb");
		String dcn = System.getProperty("javax.jdo.option.DriverClassName", "");
		String user = System.getProperty("javax.jdo.option.ConnectionUserName", "");
		String pass = System.getProperty("javax.jdo.option.ConnectionPassword", "");
		String url = System.getProperty("javax.jdo.option.ConnectionURL");
		if (url == null)
			Assert.fail("No database url specified");
		Properties p = new Properties();
		p.setProperty(
			"javax.jdo.PersistenceManagerFactoryClass",
			System.getProperty("javax.jdo.PersistenceManagerFactoryClass",
				"org.objectweb.speedo.Speedo"));
		p.setProperty("org.objectweb.speedo.debug", debug);
		p.setProperty("org.objectweb.speedo.mapperName", mapperName);
		p.setProperty("javax.jdo.option.DriverClassName", dcn);
		p.setProperty("javax.jdo.option.ConnectionUserName", user);
		p.setProperty("javax.jdo.option.ConnectionPassword", pass);
		p.setProperty("javax.jdo.option.ConnectionURL", url);
		p.setProperty(SpeedoProperties.CACHE_SIZE, "nolimit");
		return p;
	}

    protected void assertSameCollection(String msg, Collection expected, Collection found) {
        if(expected == null) {
            assertNull(msg + " null collection expected", found);
            return;
        }
        Assert.assertNotNull(msg + " non null collection expected", found);
        Assert.assertEquals(msg + " not same size", expected.size(), found.size());
        Iterator it = expected.iterator();
        while(it.hasNext()) {
            Object element  = it.next();
            Assert.assertTrue(msg + " the found collection does not contains the element "
                    + element , found.contains(element));
        }
        it = found.iterator();
        while(it.hasNext()) {
            Object element  = it.next();
            assertTrue(msg + " the found collection contains an unexpected element "
                    + element , found.contains(element));
        }
    }

    protected void assertSameList(String msg, List expected, List found) {
        if(expected == null) {
            Assert.assertNull(msg + " null list expected", found);
            return;
        }
        Assert.assertNotNull(msg + " non null list expected", found);
        Assert.assertEquals(msg + " not same size", expected.size(), found.size());
        for(int i=0; i<expected.size(); i++) {
            Assert.assertEquals(msg + " bad element at the index" + i,
                    expected.get(i), found.get(i));
        }
    }
}
