/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.bank.client;

import junit.framework.TestCase;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.Monolog;

import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.util.Properties;
import java.util.Iterator;
import java.util.Map;
import java.io.FileInputStream;
import java.io.IOException;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class J2EETestCase extends TestCase {

	public final static String SERVER_CFG_NAME = "org.objectweb.speedo.bank.serverCfgfile";
	public static LoggerFactory loggerFactory = null;
	public static Properties serverProps = null;
	public static InitialContext initialContext = null;

	public Logger logger;

	public J2EETestCase(String s) {
		super(s);
		if (loggerFactory == null) {
			synchronized (J2EETestCase.class) {
				loggerFactory = Monolog.init();
			}
		}
		logger = loggerFactory.getLogger(getLoggerName());
		if (serverProps == null) {
			synchronized (J2EETestCase.class) {
				if (serverProps == null) {
					String serverCfgFile = System.getProperty(SERVER_CFG_NAME);
					serverProps = new Properties();
					try {
						serverProps.load(new FileInputStream(serverCfgFile));
					} catch (IOException e) {
						String msg = "Impossible to load the server configuration file: "
								+ serverCfgFile;
						logger.log(BasicLevel.ERROR, msg);
						fail(msg);
					}
					Iterator it = serverProps.entrySet().iterator();
					while(it.hasNext()) {
						Map.Entry me = (Map.Entry) it.next();
						System.setProperty((String) me.getKey(), (String) me.getValue());
					}
				}
			}
		}
		if (initialContext == null) {
			synchronized (J2EETestCase.class) {
				if (initialContext == null) {
					try {
						initialContext = new InitialContext(serverProps);
					} catch (NamingException ex) {
						String msg = "We were unable to get a connection to the WebLogic "
								+ "server at Please make sure that the server is running.";
						logger.log(BasicLevel.ERROR, msg);
						fail(msg);
					}
				}
			}
		}
	}

	protected abstract String getLoggerName();

}
