package org.objectweb.speedo.bank.client;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.jms.JMSException;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.naming.NamingException;

import org.objectweb.speedo.bank.common.BankMessage;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.Monolog;


/**
 * This class illustrates calling a Message-Driven bean and publishing
 * quotes on a topic.
 *
 * @author Copyright (c) 1998-2002 by BEA Systems, Inc. All Rights Reserved.
 */

public abstract class TestMdb extends J2EETestCase {

	static public String TOPIC_FACTORY_NAME = "topic.factory.name";
	static public String TOPIC_NAME = "topic.name";

	private static TopicConnectionFactory tcf = null;
	private static TopicConnection tc = null;
	private static Topic topic = null;
	private static TopicPublisher tp = null;
	private static TopicSession ts = null;

	public TestMdb(String s) {
		super(s);
		if (tc == null) {
			synchronized(TestMdb.class) {
				if (tc == null) {
					try {
						String m_topicFactory = serverProps.getProperty(TOPIC_FACTORY_NAME);
						if (m_topicFactory == null)
							throw new CreateException(TOPIC_FACTORY_NAME + " properties is not set");
						logger.log(BasicLevel.DEBUG, TOPIC_FACTORY_NAME + "=" + m_topicFactory);
						int l = logger.getCurrentIntLevel();
							tcf = (TopicConnectionFactory)
							initialContext.lookup(m_topicFactory);
						String topicName = getTopicName();
						try {
							topic = (Topic) initialContext.lookup(topicName);
							logger.log(BasicLevel.DEBUG, "the topic '"
									+  topicName + "' has been found");
						} catch (NamingException ex) {
							String msg = "The topic '"
									+  topicName + "' does not exist: ";
							logger.log(BasicLevel.ERROR, msg, ex);
							fail(msg + ex.getMessage());
						}
						tc = tcf.createTopicConnection();
						Monolog.init();
						logger.setIntLevel(l);
						logger.log(BasicLevel.DEBUG, "TopicConnection created: " + tc);
						ts = tc.createTopicSession(
								false, Session.AUTO_ACKNOWLEDGE);
						logger.log(BasicLevel.DEBUG, "TopicSession created: " + ts);
						tp = ts.createPublisher(topic);
						logger.log(BasicLevel.DEBUG, "TopicPublisher created: " + tp);
						logger.log(BasicLevel.INFO, "Mdb initialized");
					} catch (Exception e) {
						logger.log(BasicLevel.ERROR, "Mdb initialization failed: ", e);
						fail(e.getMessage());
					}
				}
			}
		}
	}

	public void sendMessage(BankMessage msg)
			throws JMSException {
		logger.log(BasicLevel.INFO, "Sending BankMessage: " + msg);
		ObjectMessage om = ts.createObjectMessage();
		om.setObject(msg);
		tp.publish(om);
	}

	protected abstract String getTopicName();
}
