/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.bank.common;

import java.io.Serializable;
import java.util.List;
import java.util.ArrayList;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class BankMessage implements Serializable {

	public final static byte OP_BEGIN_TX 		= 1;
	public final static byte OP_COMMIT_TX 		= 2;
	public final static byte OP_ROLLBACK_TX 	= 3;

	public final static byte OP_CREATE_BANK 	= 4;

	public final static byte OP_CREATE_CLIENT 	= 5;
	public final static byte OP_REMOVE_CLIENT 	= 6;
	public final static byte OP_GET_CLIENT 		= 7;

	public final static byte OP_CREATE_ACCOUNT 	= 8;
	public final static byte OP_REMOVE_ACCOUNT 	= 9;
	public final static byte OP_CREDIT_ACCOUNT 	= 10;
	public final static byte OP_DEBIT_ACCOUNT 	= 11;

	public final static byte OP_CLIENT_POP 		= 12;
	public final static byte OP_ACCOUNT_POP 	= 13;


	public static String op2Name(byte op) {
		switch(op) {
		case OP_BEGIN_TX: return "OP_BEGIN_TX";
		case OP_COMMIT_TX: return "OP_COMMIT_TX";
		case OP_ROLLBACK_TX: return "OP_ROLLBACK_TX";

		case OP_CREATE_BANK: return "OP_CREATE_BANK";

		case OP_CREATE_CLIENT: return "OP_CREATE_CLIENT";
		case OP_REMOVE_CLIENT: return "OP_REMOVE_CLIENT";
		case OP_GET_CLIENT: return "OP_GET_CLIENT";

		case OP_CREATE_ACCOUNT: return "OP_CREATE_ACCOUNT";
		case OP_REMOVE_ACCOUNT: return "OP_REMOVE_ACCOUNT";
		case OP_CREDIT_ACCOUNT: return "OP_CREDIT_ACCOUNT";
		case OP_DEBIT_ACCOUNT: return "OP_DEBIT_ACCOUNT";

		case OP_CLIENT_POP: return "OP_CLIENT_POP";
		case OP_ACCOUNT_POP: return "OP_ACCOUNT_POP";
		default: return "UNKNOWN";
		}
	}

	public List operations;

	public BankMessage() {
		operations = new ArrayList();
	}

	public BankMessage(List operations) {
		this.operations = operations;
	}
}

