/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.bank.server;

import org.objectweb.speedo.bank.common.AccountInfo;

import java.rmi.RemoteException;
import java.util.List;

/**
 *
 * @author S.Chassande-Barrioz
 */
public interface BankService {

	/**
	 * ask the bank creation
	 * @return true is the bank is effectivly created, false is it already exists
	 */
	boolean createBank() throws RemoteException;

	// Clients management
	//--------------------

	/**
	 * Create a Client in the bank
	 */
	long createClient(String lastName,
					  String firstName,
					  String address) throws RemoteException;

	/**
	 * Retrieves a list of all Client identifier.
	 * @return list of Long objects
	 */
	List getClients() throws RemoteException;

	/**
	 * Retrieves a client identifier list. A null parameter means that the
	 * criterias is not used to find a client.
	 * @return list of Long objects
	 * @throws java.rmi.RemoteException
	 */
	List getClient(String lastName,
				   String firstName,
				   String address) throws RemoteException;

	/**
	 * Removes a client from from the bank only if the client does not have any Account
	 * @param cid is a client identifier
	 * @return true if the client was removed
	 */
	boolean removeClient(long cid) throws RemoteException;


	// Accounts management
	//--------------------

	/**
	 * Create a new Account for a client
	 * @param cid is a client identifier (last name, first name and address),
	 * the owner of the new account.
	 * @return the account number
	 */
	long createAccount(long cid) throws RemoteException;

	/**
	 * Retrieves information about an account of a client
	 * @param number is the account number
	 */
	AccountInfo getAccountInfo(long number) throws RemoteException;

	/**
	 * Retrieves a list of account for client.
	 * @param cid is the client identifier
	 * @return list of AccountInfo instance
	 */
	List getAccounts(long cid) throws RemoteException;

	/**
	 * Closes a account of a client.
	 * @param accountNumber is the number of account to close
	 * @return a AccountInfo if the account was removed. The accountInfo
	 * instance contains the value of the account. If the account does not exist
	 * a null value is returned
	 */
	AccountInfo closeAccount(long accountNumber) throws RemoteException;

	void debit(long accountNumber, double value) throws RemoteException;

	void credit(long accountNumber, double value) throws RemoteException;

	void debitCredit(long todebit, long tocredit, double value) throws RemoteException;


}
