/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.bank.server.jdo;

import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class Bank {

	/**
	 * identifier
	 */
	protected String name;

	protected Collection clients;

	protected Collection accounts;

	protected long lastAccountNumber;

	public Bank() {
	}

	public Bank(String name) {
		lastAccountNumber = 0;
		this.name = name;
		clients = new ArrayList();
		accounts = new ArrayList();
		Log.logger.log(BasicLevel.DEBUG, "Bank(" + name + ")");
	}

	public Client getClient(String fn,
							String ln,
							String ad) {
		Iterator it = clients.iterator();
		if (it.hasNext()) {
			while(it.hasNext()) {
				Client c = (Client) it.next();
				if (c.firstName.equals(fn)
					&& c.lastName.equals(ln) && c.address.equals(ad) ) {
					Log.logger.log(BasicLevel.DEBUG, "Bank.getClient(" + ln + ", "
							+ fn + ", " + ad + ") found");
					return c;
				} else {
					Log.logger.log(BasicLevel.DEBUG, "current: (" + c.lastName + ", "
							+ c.firstName + ", " + c.address + ") ");
				}
			}
			Log.logger.log(BasicLevel.DEBUG, "Bank.getClient(" + ln + ", "
					+ fn + ", " + ad + ") NOT found");
		} else {
			Log.logger.log(BasicLevel.DEBUG, "Bank.getClient(" + ln + ", "
					+ fn + ", " + ad + ") no clients currently");
		}
		return null;
	}

	public synchronized Client createClient(String fn,
											String ln,
											String ad) {
		Client client = getClient(fn, ln, ad);
		if (client == null) {
			client = new Client(fn, ln, ad, this);
		}
		return client;
	}


	public long getNextAccountNumber() {
		lastAccountNumber++;
		Log.logger.log(BasicLevel.DEBUG, "Bank.getNextAccountNumber():" + lastAccountNumber);
		return lastAccountNumber;
	}

	public static String getStringAccountNumber(long an) {
		String str = String.valueOf(an);
		for(int i=str.length(); i<11; i++) {
			str = '0' + str;
		}
		return str;
	}

}
