/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.basic;

import javax.jdo.PersistenceManager;
import javax.jdo.JDOException;
import javax.jdo.Extent;
import javax.jdo.JDOUserException;

import org.objectweb.speedo.pobjects.basic.BasicA;
import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.util.monolog.api.BasicLevel;
import junit.framework.Assert;

import java.util.Iterator;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestBasicA extends SpeedoTestHelper {

	public TestBasicA(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.basic.TestBasicA";
	}

	public void testCreateInTxAndRemoveInTx2PM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateInTxAndRemoveInTx2PM");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(ba);
		Object id = pm.getObjectId(ba);
		Assert.assertNotNull("Null object identifier", id);
		pm.currentTransaction().commit();
		pm.close();

		ba = null;

		pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		ba = (BasicA) pm.getObjectById(id, true);
		Assert.assertNotNull("Object not found", ba);
		Assert.assertEquals("Bad f1 value", "testCreateInTxAndRemoveInTx2PM", ba.f1);
		Assert.assertEquals("Bad f1 value", 2, ba.f2);

		pm.deletePersistent(ba);
		pm.currentTransaction().commit();
		try {
			Object o = pm.getObjectById(id, true);
			if (o != null)
				fail("Removed object always availlable: " + o);
		} catch (JDOUserException e) {
			//OK
		} catch (Exception e) {
			fail("Bad exception thrown when an object does not exist: " + e);
		}

		pm.close();
	}

	public void testCreateAndRemoveInTx2PM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateAndRemoveInTx2PM");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.makePersistent(ba);
		Object id = pm.getObjectId(ba);
		Assert.assertNotNull("Null object identifier", id);
		pm.close();

		ba = null;

		pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		ba = (BasicA) pm.getObjectById(id, true);
		Assert.assertNotNull("Object not found", ba);
		Assert.assertEquals("Bad f1 value", "testCreateAndRemoveInTx2PM", ba.f1);
		Assert.assertEquals("Bad f1 value", 2, ba.f2);

		pm.deletePersistent(ba);
		pm.currentTransaction().commit();
		pm.close();
	}


	public void testCreateAndRemoveInTx1PM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateAndRemoveInTx1PM");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.makePersistent(ba);
		pm.currentTransaction().begin();
		pm.deletePersistent(ba);
		pm.currentTransaction().commit();
		pm.close();
	}

	public void testCreateAndRemoveInSameTx1PM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateAndRemoveInSameTx1PM");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(ba);
		pm.deletePersistent(ba);
		try {
			pm.currentTransaction().commit();
		} catch (JDOException e) {
			logger.log(BasicLevel.ERROR, "testCreateAndRemoveInSameTx1PM", ExceptionHelper.getNested(e));
			fail("Jorm does not support export following by an unexport");
		}
		pm.close();
	}

	public void testCreateInTxAndRemove1PM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateInTxAndRemove1PM");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(ba);
        pm.currentTransaction().commit();
		pm.deletePersistent(ba);
        try {
            pm.close();
        } catch (JDOException e) {
            logger.log(BasicLevel.ERROR, "testCreateInTxAndRemove1PM", ExceptionHelper.getNested(e));
            fail("Speedo does not support the use of a PM after a transaction commit");
        }
	}

    public void testTransientAfterTx() {
        logger.log(BasicLevel.DEBUG, "Create BasicA");
        BasicA ba = new BasicA();
        ba.writeF1("testTransient");
        ba.writeF2(2);
        PersistenceManager pm = pmf.getPersistenceManager();
        pm.currentTransaction().begin();
        pm.makePersistent(ba);
        pm.currentTransaction().commit();
        Object oid = pm.getObjectId(ba);
        try {
            pm.makeTransient(ba);
            pm.close();
            Assert.assertEquals("Bad f1 value", "testTransient", ba.readF1());
            ba  = null;
        } catch (JDOException e) {
            logger.log(BasicLevel.ERROR, "testTransient", ExceptionHelper.getNested(e));
            fail("Speedo does not support the makeTransient operation after a Tx");
        } finally {
            pm = pmf.getPersistenceManager();
            ba = (BasicA) pm.getObjectById(oid, false);
            pm.deletePersistent(ba);
            pm.close();
        }
    }

    public void testTransientNoTx() {
        logger.log(BasicLevel.DEBUG, "Create BasicA");
        BasicA ba = new BasicA();
        ba.writeF1("testTransient");
        ba.writeF2(2);
        PersistenceManager pm = pmf.getPersistenceManager();
        pm.currentTransaction().begin();
        pm.makePersistent(ba);
        pm.currentTransaction().commit();
        Object oid = pm.getObjectId(ba);
        pm.close();
        ba = null;
        pm = pmf.getPersistenceManager();
        try {
            ba = (BasicA) pm.getObjectById(oid, false);
            pm.makeTransient(ba);
            pm.close();
            Assert.assertEquals("Bad f1 value", "testTransient", ba.readF1());
            ba  = null;
        } catch (JDOException e) {
            logger.log(BasicLevel.ERROR, "testTransientNoTx", ExceptionHelper.getNested(e));
            fail("Speedo does not support the makeTransient operation (notx)");
        } finally {
            pm = pmf.getPersistenceManager();
            ba = (BasicA) pm.getObjectById(oid, false);
            pm.deletePersistent(ba);
            pm.close();
        }
    }

	public void testImbricatedPM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testImbricatedPM");
		ba.writeF2(2);
		PersistenceManager pm1 = pmf.getPersistenceManager();
		PersistenceManager pm2 = pmf.getPersistenceManager();
        Assert.assertTrue("not the same persistence manager", pm1==pm2);
		pm2.makePersistent(ba);
        pm2.close();
        ba.readF1();
        int i = ba.f2;
		pm1.deletePersistent(ba);
        try {
            pm1.close();
        } catch (JDOException e) {
            logger.log(BasicLevel.ERROR, "testImbricatedPM", ExceptionHelper.getNested(e));
            fail("Speedo does not support the use of a PM after a transaction commit");
        }
	}

	public void testRollBackTxSamePM() {
		logger.log(BasicLevel.DEBUG, "Create BasicA");
		BasicA ba = new BasicA();
		ba.writeF1("testRollBack1");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();

        pm.currentTransaction().begin();
		pm.makePersistent(ba);
        pm.currentTransaction().commit();

        pm.currentTransaction().begin();
		ba.writeF2(3);
        Assert.assertEquals("Bad f2 value before roolback", 3, ba.readF2());
        pm.currentTransaction().rollback();
        Assert.assertEquals("Bad f2 value after roolback", 2, ba.readF2());

		pm.deletePersistent(ba);
        pm.close();
	}

    public void testRollBack2() {
        logger.log(BasicLevel.DEBUG, "Create BasicA");
        BasicA ba = new BasicA();
        ba.writeF1("testRollBack1");
        ba.writeF2(2);
        PersistenceManager pm = pmf.getPersistenceManager();

        pm.currentTransaction().begin();
        pm.makePersistent(ba);
        pm.currentTransaction().commit();

        pm.currentTransaction().begin();
        ba.writeF2(3);
        Assert.assertEquals("Bad f2 value before roolback", 3, ba.readF2());
        pm.currentTransaction().rollback();
        Assert.assertEquals("Bad f2 value after roolback", 2, ba.readF2());
        ba.writeF2(4);
        Assert.assertEquals("Bad f2 value after roolback", 4, ba.readF2());
        pm.close();

        pm = pmf.getPersistenceManager();
        Assert.assertEquals("Bad f2 value after roolback", 4, ba.readF2());
        pm.deletePersistent(ba);
        pm.close();
    }
}
