package org.objectweb.speedo.runtime.basic;

import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.jorm.mapper.rdb.lib.ConnectionSpecJDBC;

import javax.jdo.PersistenceManagerFactory;
import javax.naming.InitialContext;
import javax.sql.DataSource;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

import junit.framework.Assert;

/**
 * @author S.Chassande-Barrioz
 */
public class TestJavaxSqlDatasource extends TestBasicA {

    static {
        System.setProperty("java.naming.factory.initial",
            "org.objectweb.speedo.runtime.basic.SpeedoInitialNamingFactory");
    }

    public final static String CF_NAME = "ds";
    public TestJavaxSqlDatasource(String s) {
        super(s);
    }


    public Properties getPMFProperties() {
		Properties p = super.getPMFProperties();
		p.setProperty("javax.jdo.option.ConnectionFactoryName", CF_NAME);
		if (pmf == null) {
			String dcn = p.getProperty("javax.jdo.option.DriverClassName", "");
			String user = p.getProperty("javax.jdo.option.ConnectionUserName", "");
			String pass = p.getProperty("javax.jdo.option.ConnectionPassword", "");
			String url = p.getProperty("javax.jdo.option.ConnectionURL");
			if (url == null)
				fail("No database url specified");
			try {
				DriverManager.registerDriver((Driver)
							Class.forName(dcn).newInstance());
				Object cf = new ConnectionSpecJDBC(url, dcn, user, pass);
				InitialContext ic = new InitialContext();
				ic.rebind(CF_NAME, cf);
			} catch (Exception e) {
				fail(e.getMessage());
			}
		}
        return p;
    }

    public void testGetSqlConnection() {
        logger.log(BasicLevel.DEBUG, "testGetSqlConnection");
        Object cf = pmf.getConnectionFactory();
        Assert.assertTrue("The connection factory is not a DataSource",
            cf instanceof DataSource);
        try {
            ((DataSource) cf).getConnection().close();
        } catch (SQLException e) {
            fail("Impossible to get a connection on the factory " + cf);
        }
    }
}