/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.query;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.util.monolog.api.BasicLevel;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestParallelQueries extends SpeedoTestHelper {

    private static TestQueries test;
    private static Method[] methods;

    static {
        test = new TestQueries("TestParallelQueries");
        Method[] allMethods = TestQueries.class.getMethods();
        ArrayList testMethods = new ArrayList(allMethods.length);
        for(int i=0; i<allMethods.length; i++) {
            Method m = allMethods[i];
            if (m.getReturnType().equals(Void.TYPE)
                && m.getParameterTypes().length == 0
                && Modifier.isPublic(m.getModifiers())
                && m.getName().startsWith("test")) {
                testMethods.add(m);
                //System.out.println(i + ": " + m.getName());
            }
        }
        methods = (Method[]) testMethods.toArray(new Method[testMethods.size()]);
    }

	public TestParallelQueries(String name) {
		super(name);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.query.TestParallelQueries";
	}

    private void launchTest(int idx, int threadId) {
        try {
            methods[idx].invoke(test, new Object[]{});
        } catch (Exception e) {
            Exception ie = ExceptionHelper.getNested(e);
            String msg = "Thread " + threadId + ", method " + idx + ": "
                    + methods[idx].getName() + " : ";
            logger.log(BasicLevel.ERROR, msg, ie);
            fail(msg + ie.getMessage());
        }
    }

    public void test1() {
        final int nbThread = 20;
        Thread[] ts = new Thread[nbThread];
        for(int i=0; i<nbThread; i++) {
            final int threadId = i;
            ts[i] = new Thread(
                    new Runnable () {
                        public void run () {
                            try {
                                Thread.sleep(1);
                            } catch (InterruptedException e) {
                            }
                            for(int j=0; j<10; j++) {
                                launchTest(j, threadId);
                            }
                        }
                    }
            );
        }
        for(int i=0; i<nbThread; i++) {
            ts[i].start();
        }
        try {
            for(int i=0; i<nbThread; i++) {
                ts[i].join();
            }
        } catch (InterruptedException e) {
            fail(e.getMessage());
        }
    }
}
