/**
 * copyright area
 */
package org.objectweb.speedo.runtime.relations;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pobjects.relations.A;
import org.objectweb.speedo.pobjects.relations.B;

import javax.jdo.PersistenceManager;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Arrays;

public class TestRelations extends SpeedoTestHelper {

	public TestRelations(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + "rt.relations.Relations";
	}

	public void testOneOne() {
        A a1 = new A("a1");
        A a2 = new A("a2");
        B b1 = new B("b1");
        B b2 = new B("b2");

        a1.b = b1;
        assertTrue(b1.a == a1);

        a2.b = b2;
        assertTrue(b2.a == a2);

        a1.b = b2;
        assertTrue(b1.a == null);
        assertTrue(b2.a == a1);
        assertTrue(a2.b == null);

        b1.a = a2;
        assertTrue(a2.b == b1);

        b1.a = a1;
        assertTrue(a1.b == b1);
        assertTrue(b2.a == null);
        assertTrue(a2.b == null);
	}

    public void testOneMany() { // tests on active persistent objects
        A a1 = new A("a1");
        A a2 = new A("a2");

        B b1 = new B("b1");
        B b2 = new B("b2");
        B b3 = new B("b3");
        B b4 = new B("b4");

        PersistenceManager pm = pmf.getPersistenceManager();
        pm.makePersistent(a1);
        pm.makePersistent(a2);
        pm.makePersistent(b1);
        pm.makePersistent(b2);
        pm.makePersistent(b3);
        pm.makePersistent(b4);
        pm.close();

        try {
            pm = pmf.getPersistenceManager();

            a1.nbs.add(b1);
            assertTrue(b1.na == a1);
            a1.nbs.add(b2);
            assertTrue(b2.na == a1);

            a2.nbs.add(b3);
            assertTrue(b3.na == a2);
            a2.nbs.add(b4);
            assertTrue(b4.na == a2);

            a1.nbs.add(b3);
            assertTrue(b3.na == a1);
            assertTrue(!a2.nbs.contains(b3));

            a1.nbs.remove(b1);
            assertTrue(b1.na == null);

            b3.na = a2;
            assertTrue(!a1.nbs.contains(b3));
            assertTrue(a2.nbs.contains(b3));

            a1.nbs = a2.nbs;
            // reload objects before testing effects of the above assignement
            pm.close();
            pm = pmf.getPersistenceManager();

            assertTrue(!a1.nbs.contains(b1));
            assertTrue(!a1.nbs.contains(b2));
            assertTrue(a1.nbs.contains(b3));
            assertTrue(a1.nbs.contains(b4));
            assertTrue(a1.nbs.size() == 2);

            assertTrue(!a2.nbs.contains(b1));
            assertTrue(!a2.nbs.contains(b2));
            assertTrue(!a2.nbs.contains(b3));
            assertTrue(!a2.nbs.contains(b4));

            assertTrue(b1.na == null);
            assertTrue(b2.na == null);
            assertTrue(b3.na == a1);
            assertTrue(b4.na == a1);

            pm.close();
        } finally {
            pm = pmf.getPersistenceManager();
			pm.deletePersistent(a1);
			pm.deletePersistent(a2);
			pm.deletePersistent(b1);
			pm.deletePersistent(b2);
			pm.deletePersistent(b3);
			pm.deletePersistent(b4);
            pm.close();
        }
    }

    public void testOneMany2() { // tests on non persistent objects
        A a1 = new A();
        A a2 = new A();

        B b1 = new B();
        B b2 = new B();
        B b3 = new B();
        B b4 = new B();

        PersistenceManager pm = pmf.getPersistenceManager();

        b1.na = a1;
        // tests that a1.nbs, initially null, is created and contains b1
        assertTrue(a1.nbs != null);
        assertTrue(a1.nbs.contains(b1));

        b2.na = a1;
        assertTrue(a1.nbs.contains(b2));

        b3.na = a2;
        assertTrue(a2.nbs != null);
        assertTrue(a2.nbs.contains(b3));

        b4.na = a2;
        assertTrue(a2.nbs.contains(b4));

        pm.makePersistent(a1);
        pm.makePersistent(a2);

        pm.close();

        try {
            pm = pmf.getPersistenceManager();
            Collection c = new ArrayList();
            c.addAll(a2.nbs);
            a1.nbs = c;

            pm.close();

            pm = pmf.getPersistenceManager();
            assertTrue(!a1.nbs.contains(b1));
            assertTrue(!a1.nbs.contains(b2));
            assertTrue(a1.nbs.contains(b3));
            assertTrue(a1.nbs.contains(b4));

            assertTrue(a2.nbs.size() == 0);

            assertTrue(b1.na == null);
            assertTrue(b2.na == null);
            assertTrue(b3.na == a1);
            assertTrue(b4.na == a1);

            pm.close();
        } finally {
            pm = pmf.getPersistenceManager();
			pm.deletePersistent(a1);
			pm.deletePersistent(a2);
			pm.deletePersistent(b1);
			pm.deletePersistent(b2);
			pm.deletePersistent(b3);
			pm.deletePersistent(b4);
            pm.close();
        }
    }

    public void testOneMany3() { // tests with manually managed consistency
        A a1 = new A();

        B b1 = new B();
        B b2 = new B();

        PersistenceManager pm = pmf.getPersistenceManager();

        ArrayList l = new ArrayList();
        l.add(b1);
        l.add(b2);
        a1.nbs = l;

        b1.na = a1;
        b2.na = a1;

        assertTrue(a1.nbs.size() == 2);

        pm.makePersistent(a1);

        pm.close();

        try {
            pm = pmf.getPersistenceManager();
            assertTrue(a1.nbs.size() == 2);
            pm.close();
        } finally {
            pm = pmf.getPersistenceManager();
            pm.deletePersistent(a1);
            pm.deletePersistent(b1);
            pm.deletePersistent(b1);
            pm.close();
        }
    }

    public void testOneManyDelete() {
        A a1 = new A("a1");
        A a2 = new A("a2");

        B b1 = new B("b1");
        B b2 = new B("b2");
        B b3 = new B("b3");
        B b4 = new B("b4");

        PersistenceManager pm = pmf.getPersistenceManager();
        pm.makePersistent(a1);
        pm.makePersistent(a2);
        pm.makePersistent(b1);
        pm.makePersistent(b2);
        pm.makePersistent(b3);
        pm.makePersistent(b4);
        pm.close();

        try {
            pm = pmf.getPersistenceManager();

            a1.nbs.add(b1);
            assertTrue(b1.na == a1);
            a1.nbs.add(b2);
            assertTrue(b2.na == a1);

            a2.nbs.add(b3);
            assertTrue(b3.na == a2);
            a2.nbs.add(b4);
            assertTrue(b4.na == a2);

            pm.close();

            pm = pmf.getPersistenceManager();
            pm.deletePersistent(a1);
			assertNull("reverse reference of a1.nbs (b1.na) must be null", b1.na);
			assertNull("reverse reference of a1.nbs (b2.na) must be null", b2.na);
            pm.deletePersistent(b3);
			assertTrue("reverse reference of b3.na (a2.nbs) must not contain b3",
					!a2.nbs.contains(b3));
            pm.close();

            pm = pmf.getPersistenceManager();
			assertTrue("reverse reference of b3.na (a2.nbs) must not contain b3",
					!a2.nbs.contains(b3));

            assertEquals("Bad a2.nbs size", 1, a2.nbs.size());
			assertTrue("reverse reference of b3.na (a2.nbs) must not contain b3",
					!a2.nbs.contains(b3));
            assertTrue(a2.nbs.contains(b4));

			assertNull("reverse reference of a1.nbs (b1.na) must be null", b1.na);
			assertNull("reverse reference of a1.nbs (b2.na) must be null", b2.na);
            assertTrue(b4.na == a2);

            pm.close();
        } finally {
            pm = pmf.getPersistenceManager();
            pm.deletePersistent(a2);
            pm.deletePersistent(b1);
            pm.deletePersistent(b2);
            pm.deletePersistent(b4);
            pm.close();
        }
    }

    public void testManyMany() { // tests on active persistent objects
        A a1 = new A("a1");
        A a2 = new A("a2");
        A a3 = new A("a3");

        B b1 = new B("b1");
        B b2 = new B("b2");
        B b3 = new B("b3");

        PersistenceManager pm = pmf.getPersistenceManager();
        pm.makePersistent(a1);
        pm.makePersistent(a2);
        pm.makePersistent(a3);
        pm.makePersistent(b1);
        pm.makePersistent(b2);
        pm.makePersistent(b3);
        pm.close();

        try {
            pm = pmf.getPersistenceManager();

            a1.bs.add(b1);
            assertTrue(b1.as.contains(a1));
            a1.bs.add(b2);
            assertTrue(b2.as.contains(a1));

            a2.bs.add(b1);
            assertTrue(b1.as.contains(a2));
            a2.bs.add(b2);
            assertTrue(b2.as.contains(a2));
            a2.bs.add(b3);
            assertTrue(b3.as.contains(a2));

            a3.bs.add(b2);
            assertTrue(b2.as.contains(a3));
            a3.bs.add(b3);
            assertTrue(b3.as.contains(a3));

            a1.bs.add(b3);
            assertTrue(b3.as.contains(a1));

            a1.bs.remove(b1);
            assertTrue(!b1.as.contains(a1));

            b2.as.remove(a2);
            assertTrue(!a2.bs.contains(b2));

            a1.bs = a2.bs;
            // reload objects before testing effects of the above assignement
            pm.close();
            pm = pmf.getPersistenceManager();

            assertTrue(a1.bs.contains(b1));
            assertTrue(!a1.bs.contains(b2));
            assertTrue(a1.bs.contains(b3));

            assertTrue(a2.bs.contains(b1));
            assertTrue(!a2.bs.contains(b2));
            assertTrue(a2.bs.contains(b3));

            assertTrue(!a3.bs.contains(b1));
            assertTrue(a3.bs.contains(b2));
            assertTrue(a3.bs.contains(b3));

            assertTrue(b1.as.contains(a1));
            assertTrue(b1.as.contains(a2));
            assertTrue(!b1.as.contains(a3));

            assertTrue(!b2.as.contains(a1));
            assertTrue(!b2.as.contains(a2));
            assertTrue(b2.as.contains(a3));

            assertTrue(b3.as.contains(a1));
            assertTrue(b3.as.contains(a2));
            assertTrue(b3.as.contains(a3));

            pm.close();
        } finally {
            pm = pmf.getPersistenceManager();
            pm.deletePersistent(a1);
            pm.deletePersistent(a2);
            pm.deletePersistent(a3);
            pm.deletePersistent(b1);
            pm.deletePersistent(b2);
            pm.deletePersistent(b3);
            pm.close();
        }
    }

    public void testManyMany2() { // tests on non persistent objects
        A a1 = new A("a1");
        A a2 = new A("a2");
        A a3 = new A("a3");

        B b1 = new B();
        B b2 = new B();
        B b3 = new B();

        PersistenceManager pm = pmf.getPersistenceManager();

        a1.bs.add(b1);
        a1.bs.add(b2);

        a2.bs.add(b1);
        a2.bs.add(b2);
        a2.bs.add(b3);

        a3.bs.add(b2);
        a3.bs.add(b3);

        pm.makePersistent(a1);
        pm.makePersistent(a2);
        pm.makePersistent(a3);

        pm.close();

        try {
            pm = pmf.getPersistenceManager();

            assertTrue(b1.as.contains(a1));
            assertTrue(b2.as.contains(a1));

            assertTrue(b1.as.contains(a2));
            assertTrue(b2.as.contains(a2));
            assertTrue(b3.as.contains(a2));

            assertTrue(b2.as.contains(a3));
            assertTrue(b3.as.contains(a3));

            Collection c = new ArrayList();
            c.addAll(a2.bs);
            a1.bs = c;

            pm.close();

            pm = pmf.getPersistenceManager();

            assertTrue(a1.bs.contains(b1));
            assertTrue(a1.bs.contains(b2));
            assertTrue(a1.bs.contains(b3));

            assertTrue(a2.bs.contains(b1));
            assertTrue(a2.bs.contains(b2));
            assertTrue(a2.bs.contains(b3));

            assertTrue(!a3.bs.contains(b1));
            assertTrue(a3.bs.contains(b2));
            assertTrue(a3.bs.contains(b3));

            assertTrue(b1.as.contains(a1));
            assertTrue(b1.as.contains(a2));
            assertTrue(!b1.as.contains(a3));

            assertTrue(b2.as.contains(a1));
            assertTrue(b2.as.contains(a2));
            assertTrue(b2.as.contains(a3));

            assertTrue(b3.as.contains(a1));
            assertTrue(b3.as.contains(a2));
            assertTrue(b3.as.contains(a3));

            pm.close();
        } finally {
            pm = pmf.getPersistenceManager();
            pm.deletePersistent(a1);
            pm.deletePersistent(a2);
            pm.deletePersistent(a3);
            pm.deletePersistent(b1);
            pm.deletePersistent(b2);
            pm.deletePersistent(b3);
            pm.close();
        }
    }

    public void testManyMany3() { // tests with manually managed consistency
        A a1 = new A();
        A a2 = new A();
        A a3 = new A();

        B b1 = new B();
        B b2 = new B();
        B b3 = new B();

        PersistenceManager pm = pmf.getPersistenceManager();

        a1.bs = Arrays.asList(new Object[] { b1, b2 });
        a2.bs = Arrays.asList(new Object[] { b1, b2, b3 });
        a3.bs = Arrays.asList(new Object[] { b2, b3 });

        b1.as = Arrays.asList(new Object[] { a1, a2 });
        b2.as = Arrays.asList(new Object[] { a1, a2, a3 });
        b3.as = Arrays.asList(new Object[] { a2, a3 });

        pm.makePersistent(a1);
        pm.makePersistent(a2);
        pm.makePersistent(a3);

        pm.close();

        try {
            pm = pmf.getPersistenceManager();

            assertTrue(a1.bs.size() == 2);
            assertTrue(a1.bs.contains(b1));
            assertTrue(a1.bs.contains(b2));

            assertTrue(a2.bs.size() == 3);
            assertTrue(a2.bs.contains(b1));
            assertTrue(a2.bs.contains(b2));
            assertTrue(a2.bs.contains(b3));

            assertTrue(a3.bs.size() == 2);
            assertTrue(a3.bs.contains(b2));
            assertTrue(a3.bs.contains(b3));

            assertTrue(b1.as.size() == 2);
            assertTrue(b1.as.contains(a1));
            assertTrue(b1.as.contains(a2));

            assertTrue(b2.as.size() == 3);
            assertTrue(b2.as.contains(a1));
            assertTrue(b2.as.contains(a2));
            assertTrue(b2.as.contains(a3));

            assertTrue(b3.as.size() == 2);
            assertTrue(b3.as.contains(a2));
            assertTrue(b3.as.contains(a3));

            pm.close();
        } finally {
            pm = pmf.getPersistenceManager();
            pm.deletePersistent(a1);
            pm.deletePersistent(a2);
            pm.deletePersistent(a3);
            pm.deletePersistent(b1);
            pm.deletePersistent(b2);
            pm.deletePersistent(b3);
            pm.close();
        }
    }

    public void testManyManyDelete() {
        A a1 = new A("a1");
        A a2 = new A("a2");
        A a3 = new A("a3");

        B b1 = new B("b1");
        B b2 = new B("b2");
        B b3 = new B("b3");

        PersistenceManager pm = pmf.getPersistenceManager();
        pm.makePersistent(a1);
        pm.makePersistent(a2);
        pm.makePersistent(a3);
        pm.makePersistent(b1);
        pm.makePersistent(b2);
        pm.makePersistent(b3);
        pm.close();

        try {
            pm = pmf.getPersistenceManager();

            a1.bs.add(b1);
            assertTrue(b1.as.contains(a1));
            a1.bs.add(b2);
            assertTrue(b2.as.contains(a1));

            a2.bs.add(b1);
            assertTrue(b1.as.contains(a2));
            a2.bs.add(b2);
            assertTrue(b2.as.contains(a2));
            a2.bs.add(b3);
            assertTrue(b3.as.contains(a2));

            a3.bs.add(b2);
            assertTrue(b2.as.contains(a3));
            a3.bs.add(b3);
            assertTrue(b3.as.contains(a3));

            pm.close();

            pm = pmf.getPersistenceManager();
            pm.deletePersistent(b2);
            pm.close();

            pm = pmf.getPersistenceManager();

            assertTrue(a1.bs.size() == 1);
            assertTrue(a1.bs.contains(b1));

            assertTrue(a2.bs.size() == 2);
            assertTrue(a2.bs.contains(b1));
            assertTrue(a2.bs.contains(b3));

            assertTrue(a3.bs.size() == 1);
            assertTrue(a3.bs.contains(b3));

            assertTrue(b1.as.contains(a1));
            assertTrue(b1.as.contains(a2));
            assertTrue(!b1.as.contains(a3));

            assertTrue(!b3.as.contains(a1));
            assertTrue(b3.as.contains(a2));
            assertTrue(b3.as.contains(a3));

            pm.close();
        } finally {
            pm = pmf.getPersistenceManager();
            pm.deletePersistent(a1);
            pm.deletePersistent(a2);
            pm.deletePersistent(a3);
            pm.deletePersistent(b1);
            pm.deletePersistent(b3);
            pm.close();
        }
    }
}
