/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.userid;

import junit.framework.Assert;
import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pobjects.userid.AutoIncFieldId;
import org.objectweb.speedo.pobjects.userid.AutoIncId;
import org.objectweb.speedo.pobjects.userid.BasicB;
import org.objectweb.speedo.pobjects.userid.IntUserId;
import org.objectweb.speedo.pobjects.userid.Ref2AutoIncFieldId;
import org.objectweb.speedo.pobjects.userid.StringLongId;
import org.objectweb.speedo.pobjects.userid.StringUserId;
import org.objectweb.speedo.pobjects.userid.InnerUserId;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.jdo.PersistenceManager;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestUserId extends SpeedoTestHelper {

    private final static String OID_FILE_NAME = "TestUserId_test1.id";

	public TestUserId(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + "rt.userid";
	}

    public void test1MakePersistent() {
        try {
            PersistenceManager pm = pmf.getPersistenceManager();
            BasicB bb = new BasicB();
            bb.id1 = "azerty";
            bb.id2 = 217;
            bb.f1 = "blabla";
            bb.f2 = 456;
            pm.makePersistent(bb);
            StringLongId id = (StringLongId) pm.getObjectId(bb);
            logger.log(BasicLevel.DEBUG, "UserId=" + id);
            pm.close();
            Assert.assertNotNull("Null Object id", id);
            ObjectOutputStream os = new ObjectOutputStream(
                    new FileOutputStream(OID_FILE_NAME));
            os.writeObject(id);
            os.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void test2LoadNDelete() {
        try {
            PersistenceManager pm = pmf.getPersistenceManager();
            ObjectInputStream is = new ObjectInputStream(
                    new FileInputStream(OID_FILE_NAME));
            StringLongId id = (StringLongId) is.readObject();
            is.close();
            new File(OID_FILE_NAME).delete();
            BasicB bb = (BasicB) pm.getObjectById(id, true);
            Assert.assertNotNull("Null Object", bb);
            pm.deletePersistent(bb);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testUserId1() {
        try {
            String id1 = "wxcvb";
            int id2 = 3;
            String f1 = "sdgh";
            int f2 = 68746;

            PersistenceManager pm = pmf.getPersistenceManager();
            BasicB bb = new BasicB();
            bb.id1 = id1;
            bb.id2 = id2;
            bb.f1 = f1;
            bb.f2 = f2;
            pm.makePersistent(bb);
            StringLongId id = (StringLongId) pm.getObjectId(bb);
            logger.log(BasicLevel.DEBUG, "UserId=" + id);
            Assert.assertNotNull("identifier is null", id);
            String strid = id.toString();
            bb = null;
            pm.close();

            pm = pmf.getPersistenceManager();
            Object o = pm.newObjectIdInstance(BasicB.class,  strid);
            bb = (BasicB) pm.getObjectById(o, false);
            Assert.assertNotNull("returned object is null", bb);
            Assert.assertEquals("Bad field 'id1' value", id1, bb.id1);
            Assert.assertEquals("Bad field 'di2' value", id2, bb.id2);
            Assert.assertEquals("Bad field 'f1' value", f1, bb.f1);
            Assert.assertEquals("Bad field 'f1' value", f2, bb.f2);
            pm.deletePersistent(bb);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testUserId2() {
        try {
            String id1 = "querty";
            int id2 = 12000;
            String f1 = "blabla";
            int f2 = 456;

            PersistenceManager pm = pmf.getPersistenceManager();
            BasicB bb = new BasicB();
            bb.id1 = id1;
            bb.id2 = id2;
            bb.f1 = f1;
            bb.f2 = f2;
            pm.makePersistent(bb);
            StringLongId id = (StringLongId) pm.getObjectId(bb);
            logger.log(BasicLevel.DEBUG, "UserId=" + id);
            Assert.assertNotNull("identifier is null", id);
            bb = null;
            pm.close();

            pm = pmf.getPersistenceManager();
            id = new StringLongId(id1, id2);
            bb = (BasicB) pm.getObjectById(id, false);
            Assert.assertNotNull("returned object is null", bb);
            Assert.assertEquals("Bad field 'id1' value", id1, bb.id1);
            Assert.assertEquals("Bad field 'di2' value", id2, bb.id2);
            Assert.assertEquals("Bad field 'f1' value", f1, bb.f1);
            Assert.assertEquals("Bad field 'f1' value", f2, bb.f2);
            pm.deletePersistent(bb);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testInnerUserId() {
        try {
			String f1 = "testInnerUserId";
			int f2 = 12;
			InnerUserId.Oid iuio = new InnerUserId.Oid(0,0);
            PersistenceManager pm = pmf.getPersistenceManager();
            InnerUserId iui = new InnerUserId(f1, f2, iuio);
            pm.makePersistent(iui);
            InnerUserId.Oid iuio2 = (InnerUserId.Oid) pm.getObjectId(iui);
            logger.log(BasicLevel.DEBUG, "UserId=" + iuio2);
            Assert.assertNotNull("identifier is null", iuio2);
            iui = null;
            pm.close();

            pm = pmf.getPersistenceManager();
            iuio2 = new InnerUserId.Oid(0, 0);
            iui = (InnerUserId) pm.getObjectById(iuio2, false);
            Assert.assertNotNull("returned object is null", iui);
            Assert.assertEquals("Bad field 'oid1' value", iuio.oid1, iui.getOid1());
            Assert.assertEquals("Bad field 'oid2' value", iuio.oid2, iui.getOid2());
            Assert.assertEquals("Bad field 'f1' value", f1, iui.getF1());
            Assert.assertEquals("Bad field 'f2' value", f2, iui.getF2());
            pm.deletePersistent(iui);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testStringId() {
        try {
            String name = "testStringId1";
            int f1 = 12;
            Object id;
            String strid;
            PersistenceManager pm = pmf.getPersistenceManager();
            StringUserId sui = new StringUserId(name, f1);
            pm.makePersistent(sui);
            id = pm.getObjectId(sui);
            Assert.assertNotNull("identifier is null", id);
            strid = id.toString();
            id = null;
            pm.close();
            sui = null; // garbage object
            pm = pmf.getPersistenceManager();
            id = pm.newObjectIdInstance(StringUserId.class, strid);
            sui = (StringUserId) pm.getObjectById(id, false);
            Assert.assertNotNull("returned object is null", sui);
            Assert.assertEquals("Bad field 'name' value", name, sui.getName());
            Assert.assertEquals("Bad field 'f1' value", f1, sui.getF1());
            pm.deletePersistent(sui);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testStringId2() {
        try {
            String name = "testStringId2";
            int f1 = 12;
            Object id;
            PersistenceManager pm = pmf.getPersistenceManager();
            StringUserId sui = new StringUserId(name, f1);
            pm.makePersistent(sui);
            id = pm.getObjectId(sui);
            Assert.assertNotNull("identifier is null", id);
            id = null;
            pm.close();
            sui = null; // garbage object
            pm = pmf.getPersistenceManager();
            id = pm.newObjectIdInstance(StringUserId.class, name);
            sui = (StringUserId) pm.getObjectById(id, false);
            Assert.assertNotNull("returned object is null", sui);
            Assert.assertEquals("Bad field 'name' value", name, sui.getName());
            Assert.assertEquals("Bad field 'f1' value", f1, sui.getF1());
            pm.deletePersistent(sui);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testIntId() {
        try {
            String f1 = "testInt";
            int name = 13;
            Object id;
            PersistenceManager pm = pmf.getPersistenceManager();
            IntUserId iui = new IntUserId(name, f1);
            pm.makePersistent(iui);
            id = pm.getObjectId(iui);
            Assert.assertNotNull("identifier is null", id);
            pm.close();
            iui = null; // garbage object
            pm = pmf.getPersistenceManager();
            id = pm.newObjectIdInstance(IntUserId.class, id.toString());
            Assert.assertNotNull("built identifier is null", id);
            iui = (IntUserId) pm.getObjectById(id, false);
            Assert.assertNotNull("returned object is null", iui);
            Assert.assertEquals("Bad field 'name' value", name, iui.getName());
            Assert.assertEquals("Bad field 'f1' value", f1, iui.getF1());
            pm.deletePersistent(iui);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testRdbSequence1() {
        try {
            PersistenceManager pm = pmf.getPersistenceManager();
            AutoIncId aii = new AutoIncId();
            pm.makePersistent(aii);
            Object id = pm.getObjectId(aii);
            logger.log(BasicLevel.DEBUG, "id(AutoIncId): " +id);
            Assert.assertNotNull("identifier is null", id);
            pm.close();
            pm = pmf.getPersistenceManager();
            id = pm.newObjectIdInstance(AutoIncId.class, id.toString());
            Assert.assertNotNull("built identifier is null", id);
            aii = (AutoIncId) pm.getObjectById(id, false);
            Assert.assertNotNull("returned object is null", aii);
            pm.deletePersistent(aii);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testRdbSequenceUserField() {
        try {
            PersistenceManager pm = pmf.getPersistenceManager();
            AutoIncFieldId aifi = new AutoIncFieldId();
            Assert.assertEquals("Bad id value before 'makePersistent'", -1, aifi.getId());
            pm.makePersistent(aifi);
            long lid = aifi.getId();
            Assert.assertTrue("Bad id value before 'makePersistent'", 0 < lid);
            logger.log(BasicLevel.DEBUG, "long id: " +lid);
            Object id = pm.getObjectId(aifi);
            logger.log(BasicLevel.DEBUG, "id(AutoIncId): " +id);
            Assert.assertNotNull("identifier is null", id);
            pm.close();

            pm = pmf.getPersistenceManager();
            id = pm.newObjectIdInstance(AutoIncFieldId.class, id.toString());
            Assert.assertNotNull("built identifier is null (toString)", id);
            aifi = (AutoIncFieldId) pm.getObjectById(id, false);
            Assert.assertNotNull("returned object is null (toString)", aifi);
            pm.close();

            pm = pmf.getPersistenceManager();
            id = pm.newObjectIdInstance(AutoIncFieldId.class, "" + lid);
            Assert.assertNotNull("built identifier is null (long value)", id);
            aifi = (AutoIncFieldId) pm.getObjectById(id, false);
            Assert.assertNotNull("returned object is null (long value)", aifi);
            pm.deletePersistent(aifi);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }

    public void testRdbSequenceRefToUserField() {
        try {
            PersistenceManager pm = pmf.getPersistenceManager();
            pm.currentTransaction().begin();
            ArrayList al = new ArrayList();
            al.add(new AutoIncFieldId());
            al.add(new AutoIncFieldId());
            Ref2AutoIncFieldId ref = new Ref2AutoIncFieldId(new AutoIncFieldId(), al);
            pm.makePersistent(ref);
            pm.currentTransaction().commit();
            pm.deletePersistentAll(al);
			pm.deletePersistent(ref.getSimpleRef());
            pm.deletePersistent(ref);
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }
}
