/*
*  Copyright 2001 Sun Microsystems, Inc. All rights reserved.
*  Use is subject to license terms.
*/
package com.sun.jdotck;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.util.Calendar;
import java.util.Random;
import java.io.PrintWriter;
import java.math.*;

import javax.jdo.*;

public class InitCompany extends JDO_Test {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;

	public static void main(String args[]) {
		InitCompany company = new InitCompany();
		company.load(args, new PrintWriter(System.err), new PrintWriter(System.out));
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		return Status.passed("dummy method");
	}

	public void load(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			out.println("URL is " + pmf.getConnectionURL());
			out.println("URL is " + pmf.getConnectionUserName());
			pm = pmf.getPersistenceManager();

			loadPrimitiveTypes(pm);

			pm.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void loadPrimitiveTypes(PersistenceManager pm) {

		deleteObjects(pm, PrimitiveTypes.class);
		insertPrimitiveTypes(pm);
		verifyObjects(pm, PrimitiveTypes.class);
	}

	protected void insertPrimitiveTypes(PersistenceManager pm) {

		boolean bFlag = false;
		String strValue = "";
		char charValue = '\u0000';

		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();

			for (int i = 1; i <= 10; i++) {

				if (i % 2 == 1) {
					bFlag = true;
					strValue = "Odd" + i;
					charValue = 'O';
				} else {
					bFlag = false;
					strValue = "Even" + i;
					charValue = 'E';
				}

				PrimitiveTypes primitiveObject = new PrimitiveTypes((long) i,
						bFlag, new Boolean(bFlag), (byte) i, new Byte((byte) i),
						(short) i, new Short((short) i), (int) i, new Integer(i),
						(long) i, new Long(i), (float) i, new Float(i),
						(double) i, new Double(i), charValue, new Character(charValue),
						Calendar.getInstance().getTime(), strValue,
						new BigDecimal((double) i), new BigInteger(64, new Random()),
						new Long(i));
				pm.makePersistent(primitiveObject);
			}
			tx.commit();
		} catch (Exception e) {
			e.printStackTrace();
		}
		//System.out.println("Objects inserted: 10");
	}

	protected void deleteObjects(PersistenceManager pm, Class cls) {

		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();

			Query query = pm.newQuery();
			query.setClass(cls);
			query.setCandidates(pm.getExtent(cls, false));

			Collection result = (Collection) query.execute();

			if (result.size() != 0) {
				for (Iterator iter = result.iterator(); iter.hasNext();) {
					Object pc = iter.next();
					pm.deletePersistent(pc);
				}
			}

			//System.out.println("Deleted the existing " + cls.getName() + " object...");
			tx.commit();

		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	protected void verifyObjects(PersistenceManager pm, Class cls) {

		//System.out.println("Following objects inserted of type " + cls.getName() + ":");
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();

			Query query = pm.newQuery();
			query.setClass(cls);
			query.setCandidates(pm.getExtent(cls, false));

			Collection result = (Collection) query.execute();

			if (result.size() != 0) {
				for (Iterator iter = result.iterator(); iter.hasNext();) {
					Object pc = iter.next();
					//System.out.println("Object = " + pc );
				}
			}

			tx.commit();

		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}


}
