package com.sun.jdotck.api.instancecallbacks;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import javax.jdo.*;
import java.io.*;
import java.util.*;

/**
 *<B>JavaTest Title:</B> Access Other Instances In Prestore Method
 *<BR>
 *<B>JavaTest Keywords:</B> instancecallbacks
 *<BR>
 *<B>Assertion ID:</B> A10.2-3.
 *<BR>
 *<B>Assertion Description: </B>
 The context in which a call is made to <code>jdoPreStore()</code> allows access
 to the <code>PersistenceManager</code> and other persistent JDO instances.

 *@test
 *@title Access Other Instances In Prestore Method
 *@executeClass com.sun.jdotck.api.instancecallbacks.AccessOtherInstancesInPrestore
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source AccessOtherInstancesInPrestore.java
 *@keywords instancecallbacks
 */


/*
 *   Within jdoPreStore(), locate the PersistenceManager (JDOHelper.getPersistenceManager()) and use it
 *   to call currentTransaction().  Use the Transaction to call isActive() and verify active.  These operations
 *   are done to help verify that we had access to a usable PersistenceManager.
 *
 *   Create objects.
 *   Within jdoPreStore(), capture information about objects referenced.
 *   This includes a String, Date and Collection attribute.
 *   After commit() verify jdoPreStore() called and found what expected.
 */

public class AccessOtherInstancesInPrestore extends TestParts {

	public AccessOtherInstancesInPrestore() {
	}

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */

	public static void main(String[] args) {
		AccessOtherInstancesInPrestore testInstance = new AccessOtherInstancesInPrestore();
		Status s = testInstance.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		try {
			return run2(args, errPrt, outPrt);
		} catch (Exception e) {
			outPrt.println("Assertion A10.2-3 (AccessOtherInstancesInPrestore) unexpected exception:");
			outPrt.println(e);
			incrementErrorCount();
			e.printStackTrace(outPrt);
			return Status.failed("Assertion A10.2-3 (AccessOtherInstancesInPrestore) unexpected exception");
		}
	}

	Status run2(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		initializeArgs(args, errPrt, outPrt);

		PersistenceManagerFactory fact = getPMF();
		PersistenceManager pm = fact.getPersistenceManager();
		Transaction t = pm.currentTransaction();

		InstanceCallbackClass.initializeStaticsForTest();
		t.begin();
		InstanceCallbackClass.removeAllInstances(pm); // always start fresh with no instances
		t.commit();

		InstanceCallbackClass.performPreStoreTests = true;
		t.begin();
		//  Create primaryObj which has a collection (children) of two objects and references another (nextObj).
		//  None of the other objects have a non-empty collection.  The also all have nextObj set to null.
		Date createTime = new Date();
		InstanceCallbackClass secondaryObj = new InstanceCallbackClass("secondaryObj", createTime, 2, 2.0, (short) -1, '2', null);
		InstanceCallbackClass primaryObj = new InstanceCallbackClass("primaryObj", createTime, 1, 1.0, (short) 3, '1', secondaryObj);
		InstanceCallbackClass childA = new InstanceCallbackClass("childA", createTime, 3, 3.0, (short) -2, '3', null);
		InstanceCallbackClass childB = new InstanceCallbackClass("childB", createTime, 4, 4.0, (short) -3, '4', null);
		pm.makePersistent(primaryObj);
		primaryObj.addChild(childA);
		primaryObj.addChild(childB);
		t.commit();

		// check that jdoPreStore provides access to instances for primaryObj
		checkInstances("jdoStore instance access:  ", 1, "secondaryObj", 2, 7);
		// check that jdoPreStore provides access to PersistenceManager for primaryObj
		checkPMAccess("jdoPreDelete PersistenceManager access:  ", 1, true);

		// check that jdoPreStore provides access to instances for secondaryObj
		checkInstances("jdoStore instance access:  ", 2, null, 0, 0);
		// check that jdoPreStore provides access to PersistenceManager for secondaryObj
		checkPMAccess("jdoPreDelete PersistenceManager access:  ", 2, true);

		// There is no need to check childA or childB.  They are no different than secondaryObj.

		pm.close();

		if (getErrorCount() > 0)
			return Status.failed("Assertion A10.2-3 (AccessOtherInstancesInPrestore) failed");
		else
			return Status.passed("Assertion A10.2-3 (AccessOtherInstancesInPrestore) passed");
	}
}
