/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.instancecallbacks;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.*;
import java.io.*;
import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Accessing Fields In Predelete
 *<BR>
 *<B>JavaTest Keywords:</B> instancecallbacks
 *<BR>
 *<B>Assertion ID:</B> A10.4-2.
 *<BR>
 *<B>Assertion Description: </B>
 Access to field values within a call to <code>jdoPreDelete()</code>
 of a class implementing <code>InstanceCallbacks</code> are valid.

 *@test
 *@title Accessing Fields In Predelete
 *@executeClass com.sun.jdotck.api.instancecallbacks.AccessingFieldsInPredelete
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source AccessingFieldsInPredelete.java
 *@keywords instancecallbacks
 */



/*
 *   Define a PersistenceCapable class containing some primative types,
 *      a Date attribute, a String attribute,
 *      one reference to an object of the same class and one
 *      Set collection attribute containing two instances of objects of this class.
 *
 *   Create an object of this class (along with the referenced object and two members in the Set) and commit.
 *   Delete the object.
 *   Within jdoPreDelete(), capture the values of attribues and delete the referenced object
 *       and one of the two Set members.
 *
 *  In the next transaction, verify the data captured from the attriubtes was what was expected and
 *  that all the objects that should have been deleted have been deleted and the
 *  Set member that was not deleted is still present.
 *  Also verify that jdoPreDelete() is called on each of these objects deleted within the first jdoPreDelete().
 */

public class AccessingFieldsInPredelete extends TestParts {

	public AccessingFieldsInPredelete() {
	}

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		AccessingFieldsInPredelete testInstance = new AccessingFieldsInPredelete();
		Status s = testInstance.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		try {
			return run2(args, errPrt, outPrt);
		} catch (Exception e) {
			outPrt.println("Assertion A10.4-2 (AccessingFieldsInPredelete) unexpected exception");
			outPrt.println(e);
			incrementErrorCount();
			e.printStackTrace(outPrt);
			outPrt.flush();
			return Status.failed("Assertion A10.4-2 (AccessingFieldsInPredelete) unexpected exception");
		}
	}

	Status run2(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		initializeArgs(args, errPrt, outPrt);
		PersistenceManagerFactory fact = getPMF();
		PersistenceManager pm = fact.getPersistenceManager();
		Transaction t = pm.currentTransaction();

		InstanceCallbackClass.initializeStaticsForTest();
		t.begin();
		InstanceCallbackClass.removeAllInstances(pm); // always start fresh with no instances
		t.commit();

		InstanceCallbackClass.performPreDeleteTests = true;
		t.begin();
		Date createTime = new Date();
		InstanceCallbackClass secondaryObj = new InstanceCallbackClass("secondaryObj", createTime, 2, 2.0, (short) -1, '2', null);
		InstanceCallbackClass primaryObj = new InstanceCallbackClass("primaryObj", createTime, 1, 1.0, (short) 3, '1', secondaryObj);
		InstanceCallbackClass childA = new InstanceCallbackClass("childA", createTime, 3, 3.0, (short) -2, '3', null);
		InstanceCallbackClass childB = new InstanceCallbackClass("childB", createTime, 4, 4.0, (short) -3, '4', null);
		pm.makePersistent(primaryObj);
		pm.makePersistent(secondaryObj);
		pm.makePersistent(childA);
		pm.makePersistent(childB);
		primaryObj.addChild(childA);
		primaryObj.addChild(childB);
		Object secondaryObjId = pm.getObjectId(secondaryObj);
		Object primaryObjId = pm.getObjectId(primaryObj);
		Object childAId = pm.getObjectId(childA);
		Object childBId = pm.getObjectId(childB);
		t.commit();

		t.begin();
		primaryObj = (InstanceCallbackClass) pm.getObjectById(primaryObjId, true);
		pm.deletePersistent(primaryObj);
		t.commit();

		t.begin();
		try {
			primaryObj = (InstanceCallbackClass) pm.getObjectById(primaryObjId, true);
			outPrt.println("primaryObj deleted but getObjectById() on its Id succeeded.");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected one of these exceptions
		} catch (JDODataStoreException e) {
			// expected one of these exceptions
		}

		// check that jdoPreDelete() provided proper access to the attributes in primaryObj
		checkFieldValues("jdoPreDelete attribute access:  ", 1, "primaryObj", createTime, 1.0, (short) 3, '1');
		checkInstances("jdoPreDelete instance access:  ", 1, "secondaryObj", 2, 7);
		checkPMAccess("jdoPreDelete PersistenceManager access:  ", 1, true);

		// check that secondaryObj had jdoPreDelete() called on it and provided proper access to its attributes.
		checkFieldValues("jdoPreDelete attribute access:  ", 2, "secondaryObj", createTime, 2.0, (short) -1, '2');
		checkInstances("jdoPreDelete instance access:  ", 2, null, 0, 0);
		checkPMAccess("jdoPreDelete PersistenceManager access:  ", 2, true);

		// check that secondaryObj was deleted.
		try {
			secondaryObj = (InstanceCallbackClass) pm.getObjectById(secondaryObjId, true);
			outPrt.println("secondaryObj should have been deleted but getObjectById() on its Id succeeded.");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected one of these exceptions
		} catch (JDODataStoreException e) {
			// expected one of these exceptions
		}

		// check that first added member of Set had jdoPreDelete() called on it and provided proper access to its attributes.
		checkFieldValues("jdoPreDelete attribute access:  ", 3, "childA", createTime, 3.0, (short) -2, '3');
		checkInstances("jdoPreDelete instance access:  ", 3, null, 0, 0);
		checkPMAccess("jdoPreDelete PersistenceManager access:  ", 3, true);

		// verify first added member of Set was actaully deleted.
		try {
			childA = (InstanceCallbackClass) pm.getObjectById(childAId, true);
			outPrt.println("First added member of Set primaryObj.children should have been deleted but getObjectById() on its Id succeeded.");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected one of these exceptions
		} catch (JDODataStoreException e) {
			// expected one of these exceptions
		}

		// check that the second added member of Set did not have jdoPreDelete() called on it and it was not deleted.
		if (InstanceCallbackClass.processedIndex[4] == true) {
			outPrt.println("jdoPreDelete() called on childB--it was not deleted.");
			incrementErrorCount();
		}

		try {
			childB = (InstanceCallbackClass) pm.getObjectById(childBId, true);
			if (childB == null) {
				outPrt.println("childB object is null");
			} else {
				if (childB.name == null)
					outPrt.println("childB.name is null");
			}
			if (!childB.name.equals("childB")) {
				outPrt.println("childB.name should be \"childB\".  Instead its value is \"" + childB.name + "\".");
				incrementErrorCount();
			}
		} catch (JDOUserException e) {
			outPrt.println("Second added member of Set primaryObj.children should exist but getObjectById() got JDOUserException " + e);
			incrementErrorCount();

		} catch (JDODataStoreException e) {
			outPrt.println("Second added member of Set primaryObj.children should exist but getObjectById() got JDODataStoreException " + e);
			incrementErrorCount();
		}
		t.rollback();
		pm.close();

		if (getErrorCount() > 0)
			return Status.failed("Assertion A10.4-2 (AccessingFieldsInPredelete) failed");
		else
			return Status.passed("Assertion A10.4-2 (AccessingFieldsInPredelete) passed");
	}
}

