/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.instancecallbacks;


import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.*;
import java.io.*;
import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Calling Jdo Preclear
 *<BR>
 *<B>JavaTest Keywords:</B> instancecallbacks
 *<BR>
 *<B>Assertion ID:</B> A10.3-1.
 *<BR>
 *<B>Assertion Description: </B>
 The <code>jdoPreClear()</code> method of a class implementing
 <code>InstanceCallbacks</code> is called before the values in the instance
 are cleared. (This happens during the state transition to hollow.)

 *@test
 *@title Calling Jdo Preclear
 *@executeClass com.sun.jdotck.api.instancecallbacks.CallingJdoPreclear
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source CallingJdoPreclear.java
 *@keywords instancecallbacks
 */

/*
 *   Create a new object, reference an existing object and modify an existing object and then
 *   perform a commit.
 *
 *   Check that each instance contains the values set into the persistent attributes prior to commit.
 */

public class CallingJdoPreclear extends TestParts {

	public CallingJdoPreclear() {
	}

	public static void main(String[] args) {
		CallingJdoPreclear testInstance = new CallingJdoPreclear();
		Status s = testInstance.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status run(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		Status status = null;

		try {
			status = run2(args, errPrt, outPrt);
		} catch (Exception e) {
			outPrt.println("Assertion A10.3-1 (CallingJdoPreclear) unexpected exception");
			outPrt.println(e);
			e.printStackTrace(outPrt);
			outPrt.flush();
			incrementErrorCount();
			status = Status.failed("Assertion A10.3-1 (CallingJdoPreclear) unexpected exception");
		}
		errPrt.flush();
		outPrt.flush();
		return status;
	}

	Status run2(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		initializeArgs(args, errPrt, outPrt);

		PersistenceManagerFactory fact = getPMF();
		PersistenceManager pm = fact.getPersistenceManager();
		Transaction t = pm.currentTransaction();
		t.setRetainValues(false);  // instances transition to hollow after commit

		InstanceCallbackClass.initializeStaticsForTest();
		t.begin();
		InstanceCallbackClass.removeAllInstances(pm); // always start fresh with no instances
		t.commit();

		t.begin();
		Calendar cal = Calendar.getInstance();
		cal.set(1999, 1, 15, 12, 0);
		Date createTime = cal.getTime();
		cal.set(2002, 1, 15, 12, 0);
		Date laterDate = cal.getTime();
		InstanceCallbackClass secondaryObj = new InstanceCallbackClass("secondaryObj", createTime, 2, 2.2, (short) -20, '2', null);
		InstanceCallbackClass primaryObj = new InstanceCallbackClass("primaryObj", laterDate, 1, 1.1, (short) -10, '1', secondaryObj);
		pm.makePersistent(primaryObj);
		pm.makePersistent(secondaryObj);
		Object secondaryObjId = pm.getObjectId(secondaryObj);
		Object primaryObjId = pm.getObjectId(primaryObj);
		t.commit();

		InstanceCallbackClass.performPreClearTests = true;
		t.begin();
		try {
			primaryObj = (InstanceCallbackClass) pm.getObjectById(primaryObjId, true);
			touchFields(primaryObj); // load fields of primaryObj (make it persistent-clean)

		} catch (JDOUserException e) {
			errPrt.println("Failed to find primaryObj created in previous transaction.  Got JDOUserException " + e);
			incrementErrorCount();
			return Status.failed("CallingJdoPreclear: Failed to find primaryObj created in previous transaction.");
		} catch (JDODataStoreException e) {
			errPrt.println("Failed to find primaryObj created in previous transaction.  Got JDODataStoreException " + e);
			incrementErrorCount();
			return Status.failed("CallingJdoPreclear: Failed to find primaryObj created in previous transaction.");
		}

		secondaryObj = primaryObj.nextObj;
		if (secondaryObj == null) {
			errPrt.println("Failed to find secondaryObj created in previous transaction using reference from primaryObj.");
			incrementErrorCount();
			return Status.failed("CallingJdoPreclear: Failed to find secondaryObj created in previous transaction.");
		}
		touchFields(secondaryObj);

		primaryObj.addChild(secondaryObj);  // primaryObj contains one child;  secondaryObj contains none. primaryObj is now dirty

		cal.set(2005, 6, 28, 0, 0);
		Date stillLaterDate = cal.getTime();
		InstanceCallbackClass ternaryObj = new InstanceCallbackClass("ternaryObj", stillLaterDate, 3, 3.3, (short) -30, '3', null);
		pm.makePersistent(ternaryObj);
		ternaryObj.addChild(secondaryObj);
		ternaryObj.addChild(primaryObj);
		t.commit();

		// verify attributes in what was persistent-clean object--secondaryObj
		checkFieldValues("jdoPreClear attribute access:  ", 2, "secondaryObj", createTime, 2.2, (short) -20, '2');

		// verify attributes in what was persistent-dirty object--primaryObj
		checkFieldValues("jdoPreClear attribute access:  ", 1, "primaryObj", laterDate, 1.1, (short) -10, '1');


		// verify attributes in what was persistent-new object--ternaryObj
		checkFieldValues("jdoPreClear attribute access:  ", 3, "ternaryObj", stillLaterDate, 3.3, (short) -30, '3');
		pm.close();

		if (getErrorCount() > 0)
			return Status.failed("Assertion A10.3-1 (CallingJdoPreclear) failed");
		else
			return Status.passed("Assertion A10.3-1 (CallingJdoPreclear) passed");
	}
}
