/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.instancecallbacks;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.io.*;
import javax.jdo.*;
import java.util.Iterator;

/**
 *<B>JavaTest Title:</B> Calling JDO Predelete
 *<BR>
 *<B>JavaTest Keywords:</B> instancecallbacks
 *<BR>
 *<B>Assertion ID:</B> A10.4-1.
 *<BR>
 *<B>Assertion Description: </B>
 <code>jdoPreDelete()</code> is called during the execution of
 <code>deletePersistent</code> before the state transition to persistent-deleted
 or persistent-new-deleted.

 *@test
 *@title Calling JDO Predelete
 *@executeClass com.sun.jdotck.api.instancecallbacks.CallingJdoPredelete
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source CallingJdoPredelete.java
 *@keywords instancecallbacks
 */

/*
 *    Use the same PersistenceCapable class and objects as in the test AccessingFieldsInPredelete (but the
 *      referenced object and Set need not be created).
 *
 *    Delete one such new persistent, one persistent clean and one persistent dirty object.
 *
 *    For each of these objects:
 *       determine jdoPredelete is called after deletePersistent is called
 *       determine jdoPredelete is called before statements after deletePersistent is called.
 *       determine that in jdoPredelete, the state of the object is persistent-new, persistent-clean, or persistent-dirty
 *         (appropriately).
 *       determine that the state of the object after return from deletePersistent is persistent-deleted or
 *         persitent-new-deleted (appropriately).
 *
 */

public class CallingJdoPredelete extends JDO_Test {
	public CallingJdoPredelete() {
	}

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		CallingJdoPredelete testInstance = new CallingJdoPredelete();
		Status s = testInstance.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(java.lang.String[] args, java.io.PrintWriter errPrt, java.io.PrintWriter outPrt) {
		try {
			return run2(args, errPrt, outPrt);
		} catch (Exception e) {
			outPrt.println("Assertion A10.4-1 (CallingJdoPredelete) unexpected exception");
			outPrt.println(e);
			incrementErrorCount();
			e.printStackTrace(outPrt);
			return Status.failed("Assertion A10.4-1 (CallingJdoPredelete) unexpected exception");
		}
	}

	Status run2(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		initializeArgs(args, errPrt, outPrt);

		PersistenceManagerFactory fact = getPMF();
		PersistenceManager pm = fact.getPersistenceManager();
		Transaction t = pm.currentTransaction();

		InstanceCallbackClass.initializeStaticsForTest();
		t.begin();
		InstanceCallbackClass.removeAllInstances(pm); // always start fresh with no instances
		t.commit();

		InstanceCallbackClass.performPreDeleteTests = true; // enabling PreDelete tests
		t.begin();
		InstanceCallbackClass a = new InstanceCallbackClass(null, null, 1, 1.0, (short) -1, '1', null);
		InstanceCallbackClass b = new InstanceCallbackClass(null, null, 2, 2.0, (short) -1, '2', null);
		pm.makePersistent(a);
		pm.makePersistent(b);
		Object aId = pm.getObjectId(a);
		Object bId = pm.getObjectId(b);
		t.commit();

		t.begin();
		try {
			a = (InstanceCallbackClass) pm.getObjectById(aId, true);
		} catch (JDOUserException e) {
			errPrt.println("Could not relocate first object created, got JDOUserException " + e);
			incrementErrorCount();
			return Status.failed("CallingJdoPredelete: Failed to find object a created in previous transaction.");
		}
		try {
			b = (InstanceCallbackClass) pm.getObjectById(bId, true);
		} catch (JDOException e) {
			errPrt.println("Could not relocate second object created, got JDOUserException " + e);
			incrementErrorCount();
			return Status.failed("CallingJdoPredelete: Failed to find object b created in previous transaction.");
		}

		int getfld = a.intValue;    // cause instance to be persistent-clean
		int getfld2 = a.childToDelete;
		b.childToDelete++;          // make second object dirty
		InstanceCallbackClass c = new InstanceCallbackClass(null, null, 3, 3.0, (short) -1, '3', null);
		pm.makePersistent(c);       // create persistent-new object

		InstanceCallbackClass.preDeleteCalled = false;
		pm.deletePersistent(a);
		if (!InstanceCallbackClass.preDeleteCalled) {
			outPrt.println("jdoPreDelete not called when PERSISTENT_CLEAN object deleted.");
			incrementErrorCount();
		} else {
			if (InstanceCallbackClass.objectState == PERSISTENT_DELETED) {
				outPrt.println("Error, state was persistent-deleted in jdoPredelete callback");
				incrementErrorCount();
			}
		}

		InstanceCallbackClass.preDeleteCalled = false;
		pm.deletePersistent(b);
		if (!InstanceCallbackClass.preDeleteCalled) {
			outPrt.println("jdoPreDelete not called when PERSISTENT_DIRTY object deleted.");
			incrementErrorCount();
		} else {
			if (InstanceCallbackClass.objectState == PERSISTENT_DELETED) {
				outPrt.println("Error, state was persistent-deleted in jdoPredelete callback");
				incrementErrorCount();
			}
		}

		InstanceCallbackClass.preDeleteCalled = false;
		pm.deletePersistent(c);
		if (!InstanceCallbackClass.preDeleteCalled) {
			outPrt.println("jdoPreDelete not called when PERSISTENT_NEW object deleted.");
			incrementErrorCount();
		} else {
			if (InstanceCallbackClass.objectState == PERSISTENT_NEW_DELETED) {
				outPrt.println("Error, state was persistent-new-deleted in jdoPredelete callback");
				incrementErrorCount();
			}
		}
		t.rollback();
		pm.close();
		if (getErrorCount() > 0)
			return Status.failed("Assertion A10.4-1 (CallingJdoPredelete) failed");
		else
			return Status.passed("Assertion A10.4-1 (CallingJdoPredelete) passed");
	}

}
