/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.instancecallbacks;

import java.util.*;
import javax.jdo.*;

import com.sun.jdotck.*;

import java.io.*;
import java.io.Serializable;

public class InstanceCallbackClass implements InstanceCallbacks {
	static boolean performPreClearTests;
	static boolean performPreDeleteTests;
	static boolean performPreStoreTests;

	// The following two variables used in CallingJdoPreDelete tests
	static boolean preDeleteCalled;
	static int objectState;

	// The rest of these variables used in FieldsInPredelete tests, except to set a variable to make object dirty.
	static int arraySize = 5;
	static String[] capturedName = new String[arraySize];
	static Date[] capturedTimeStamp = new Date[arraySize];
	static double[] capturedDoubleValue = new double[arraySize];
	static short[] capturedChildToDelete = new short[arraySize];
	static char[] capturedCharValue = new char[arraySize];
	static String[] capturedNextObjName = new String[arraySize];
	static int[] numberOfChildren = new int[arraySize];
	static int[] sumOfChildrenIntValue = new int[arraySize];
	static boolean[] processedIndex = new boolean[arraySize];

	static boolean[] transactionActive = new boolean[arraySize];

	static private int nextKeyValue = 1;
	private int keyValue;  // persistent--used as key field in application identity
	String name;
	Date timeStamp;
	InstanceCallbackClass nextObj;
	HashSet children;
	int intValue;
	double doubleValue;
	short childToDelete;
	char charValue;

	static void initializeStaticsForTest() {
		performPreClearTests = false;
		performPreDeleteTests = false;
		performPreStoreTests = false;
		preDeleteCalled = false;
		for (int i = 0; i < arraySize; ++i) {
			processedIndex[i] = false;
			transactionActive[i] = false;
		}
	}

	static void removeAllInstances(PersistenceManager pm) {
		Extent e = pm.getExtent(com.sun.jdotck.api.instancecallbacks.InstanceCallbackClass.class, true);
		Iterator i = e.iterator();
		while (i.hasNext()) {
			pm.deletePersistent(i.next());
		}
	}

	InstanceCallbackClass() { // not used by application
	}

	InstanceCallbackClass(String label, Date createTime, int intValue, double doubleValue, short childToDelete, char charValue, InstanceCallbackClass obj) {
		keyValue = nextKeyValue++;
		name = label;
		timeStamp = createTime;
		nextObj = obj;
		children = new HashSet();
		this.intValue = intValue;
		this.doubleValue = doubleValue;
		this.childToDelete = childToDelete;
		this.charValue = charValue;
	}

	void addChild(InstanceCallbackClass child) {
		children.add(child);
	}

	public void jdoPreStore() {
		if (!performPreStoreTests) {
			return;
		}
		captureValues();
		PersistenceManager pm = JDOHelper.getPersistenceManager(this);
		usePM(pm);
	}

	public void jdoPreDelete() {
		if (!performPreDeleteTests) {
			return;
		}
		// The following two variables set for CallingJdoPreDelete tests
		preDeleteCalled = true;
		objectState = JDO_Test.currentState(this);

		captureValues();

		// The rest of this routine used for Accessing FieldsInPredelete tests
		// check that intValue is a valid index
		if (intValue >= 0 & intValue < arraySize) {
			PersistenceManager pm = JDOHelper.getPersistenceManager(this);
			usePM(pm);
			if (nextObj != null) {
				pm.deletePersistent(nextObj);  // delete referenced object

				// delete designated child
				for (Iterator i = children.iterator(); i.hasNext();) {
					InstanceCallbackClass obj = (InstanceCallbackClass) i.next();
					if (obj.intValue == childToDelete) {
						pm.deletePersistent(obj);
						break;
					}
				}
			}
		}
	}

	public void jdoPostLoad() {
	}

	public void jdoPreClear() {
		if (!performPreClearTests) {
			return;
		}
		// the following code is copied from captureValues, because it must
		// not be enhanced for execution during jdoPreClear.
		// check that intValue is a valid index
		if (intValue >= 0 & intValue < arraySize) {
			processedIndex[intValue] = true;

			// capture values of the attributes
			capturedName[intValue] = name;
			capturedTimeStamp[intValue] = timeStamp;
			capturedDoubleValue[intValue] = doubleValue;
			numberOfChildren[intValue] = children.size();
			sumOfChildrenIntValue[intValue] = 0;
			for (Iterator i = children.iterator(); i.hasNext();) {
				InstanceCallbackClass o = (InstanceCallbackClass) i.next();
				sumOfChildrenIntValue[intValue] += o.intValue;
			}
			capturedChildToDelete[intValue] = childToDelete;
			capturedCharValue[intValue] = charValue;
			if (nextObj != null) {
				capturedNextObjName[intValue] = nextObj.name;
			} else {
				capturedNextObjName[intValue] = null;
			}
		}
	}

	void captureValues() {
		// check that intValue is a valid index
		if (intValue >= 0 & intValue < arraySize) {
			processedIndex[intValue] = true;

			// capture values of the attributes
			capturedName[intValue] = name;
			capturedTimeStamp[intValue] = timeStamp;
			capturedDoubleValue[intValue] = doubleValue;
			numberOfChildren[intValue] = children.size();
			sumOfChildrenIntValue[intValue] = 0;
			for (Iterator i = children.iterator(); i.hasNext();) {
				InstanceCallbackClass o = (InstanceCallbackClass) i.next();
				sumOfChildrenIntValue[intValue] += o.intValue;
			}
			capturedChildToDelete[intValue] = childToDelete;
			capturedCharValue[intValue] = charValue;
			if (nextObj != null) {
				capturedNextObjName[intValue] = nextObj.name;
			} else {
				capturedNextObjName[intValue] = null;
			}
		}
	}

	void usePM(PersistenceManager pm) {
		if (intValue >= 0 & intValue < arraySize) {
			Transaction t = pm.currentTransaction();
			if (t.isActive()) {
				transactionActive[intValue] = true;
			}
		}
	}

	public static class KeyClass implements Serializable {
		public int keyValue;

		public KeyClass() {
		}

		public KeyClass(String s) {
			try {
				keyValue = Integer.parseInt(s);
			} catch (NumberFormatException e) {
				keyValue = 0;
			}
		}

		public boolean equals(Object obj) {
			if (obj == null || !this.getClass().equals(obj.getClass()))
				return false;
			else
				return keyValue == ((KeyClass) obj).keyValue;
		}

		public int hashCode() {
			return keyValue;
		}

		public String toString() {
			return Integer.toString(keyValue);
		}
	}
}
