/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.instancecallbacks;


import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import javax.jdo.*;
import java.io.*;
import java.util.*;


/**
 *<B>JavaTest Title:</B> Modification of Nontransactional Nonpersistent Fields
 *<BR>
 *<B>JavaTest Keywords:</B> instancecallbacks
 *<BR>
 *<B>Assertion ID:</B> A6.4.1-3.
 *<BR>
 *<B>Assertion Description: </B>
 Nontransactional non-persistent fields may be modified during execution of
 user-written callbacks defined in interface <code>InstanceCallbacks</code>
 at specific points in the life cycle.

 *@test
 *@title Modification of Nontransactional Nonpersistent Fields
 *@executeClass com.sun.jdotck.api.instancecallbacks.ModificationOfNontransactionalNonpersistentFields
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source ModificationOfNontransactionalNonpersistentFields.java
 *@keywords instancecallbacks
 */

/*
 *  Test jdoPostLoad() by setting values into the non-managed and transactional attriubes.
 *    This includes creating and populating a collection and initializing a Date.
 *  Verify those values are set once the object is referenced.
 *
 *  Test jdoPreClear(), jdoPreDelete() & jdoPreStore() by setting values into the nonmanaged and transactional attributes.
 *   This includes setting a collection and Date to null.
 *  Verify that no exceptions are thrown.
 */

public class ModificationOfNontransactionalNonpersistentFields extends JDO_Test {

	public ModificationOfNontransactionalNonpersistentFields() {
	}

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		ModificationOfNontransactionalNonpersistentFields testInstance = new ModificationOfNontransactionalNonpersistentFields();
		Status s = testInstance.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		try {
			return run2(args, errPrt, outPrt);
		} catch (Exception e) {
			outPrt.println("Assertion A6.4.1-3 (ModificationOfNontransactionalNonpersistentFields) unexpected exception");
			outPrt.println(e);
			incrementErrorCount();
			e.printStackTrace(outPrt);
			return Status.failed("Assertion A6.4.1-3 (ModificationOfNontransactionalNonpersistentFields) unexpected exception");
		}
	}

	Status run2(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		initializeArgs(args, errPrt, outPrt);

		PersistenceManagerFactory fact = getPMF();
		PersistenceManager pm = fact.getPersistenceManager();
		Transaction t = pm.currentTransaction();

		InstanceCallbackNonPersistFdsClass.initializeStaticsForTest();
		t.begin();
		InstanceCallbackNonPersistFdsClass.removeAllInstances(pm); // always start fresh with no instances
		t.commit();

		t.begin();
		// create instance
		InstanceCallbackNonPersistFdsClass obj1 = new InstanceCallbackNonPersistFdsClass(1.1f, 1);
		pm.makePersistent(obj1);
		Object objPtr1 = pm.getObjectId(obj1);
		obj1.setNonPersist(1, (char) 1, 0, (short) 0);
		t.commit();  // jdoPreStore() and jdoPreClear() called

		t.begin();
		try {
			obj1 = (InstanceCallbackNonPersistFdsClass) pm.getObjectById(objPtr1, true);  // jdoPostLoad() called
		} catch (JDOUserException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDOUserException " + e + " could not locate obj1.");
			incrementErrorCount();
			return Status.failed("ModificationOfNontransactionalNonpersistentFields:  Could not locate persistent object created in previous transaction");
		} catch (JDODataStoreException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDODataStoreException " + e + " could not locate obj1.");
			incrementErrorCount();
			return Status.failed("ModificationOfNontransactionalNonpersistentFields:  Could not locate persistent object created in previous transaction");
		}

		// check fields set in jdoPostLoad()
		if (obj1.i != -10) {
			outPrt.println("jdoPostLoad:  Value incorrect, obj1.i != -10; it is " + obj1.i);
			incrementErrorCount();
		}
		if (obj1.c != '2') {
			outPrt.println("jdoPostLoad:  Value incorrect, obj1.c != '2'; it is " + obj1.c);
			incrementErrorCount();
		}
		if (obj1.d != 30.0) {
			outPrt.println("jdoPostLoad:  Value incorrect, obj1.d != 30.0; it is " + obj1.d);
			incrementErrorCount();
		}
		if (obj1.s != 40) {
			outPrt.println("jdoPostLoad:  Value incorrect, obj1.s != 40; it is " + obj1.s);
			incrementErrorCount();
		}
		if (obj1.loadTime != InstanceCallbackNonPersistFdsClass.savedLoadTime) {
			outPrt.println("jdoPostLoad:  Value incorrect, obj1.loadTime != " +
					InstanceCallbackNonPersistFdsClass.savedLoadTime + "; it is " + obj1.loadTime);
			incrementErrorCount();
		}
		boolean childrenValueIncorrect = false;
		if (obj1.children.size() != 3) {
			childrenValueIncorrect = true;
		} else {
			if (!obj1.children.contains(InstanceCallbackNonPersistFdsClass.member1)) {
				childrenValueIncorrect = true;
			}
			if (!obj1.children.contains(InstanceCallbackNonPersistFdsClass.member2)) {
				childrenValueIncorrect = true;
			}
			if (!obj1.children.contains(InstanceCallbackNonPersistFdsClass.member3)) {
				childrenValueIncorrect = true;
			}
		}
		if (childrenValueIncorrect) {
			incrementErrorCount();
			outPrt.println("jdoPostLoad:  Value incorrect, obj1.children does not contain the correct String members");
			outPrt.println("The members should be: " + InstanceCallbackNonPersistFdsClass.member1 + ", " +
					InstanceCallbackNonPersistFdsClass.member2 + ", " + InstanceCallbackNonPersistFdsClass.member3);
			outPrt.println("obj1.children contains " + obj1.children.size() + " members");
			if (obj1.children.size() != 0) {
				outPrt.println("Those members are:");
				for (Iterator i = obj1.children.iterator(); i.hasNext();) {
					outPrt.println((String) i.next());
				}
			}
		}

		pm.deletePersistent(obj1);  // jdoPreDelete() called
		t.commit();
		// check result of jdoPreStore(), jdoPreClear() and jdoPreDelete()
		if (!InstanceCallbackNonPersistFdsClass.preClearCalled) {
			incrementErrorCount();
			outPrt.println("jdoPreClear() never called on obj1");
		}

		if (!InstanceCallbackNonPersistFdsClass.preStoreCalled) {
			incrementErrorCount();
			outPrt.println("jdoPreStore() never called on obj1");
		}

		if (!InstanceCallbackNonPersistFdsClass.preDeleteCalled) {
			incrementErrorCount();
			outPrt.println("jdoPreDelete() never called on obj1");
		}

		if (InstanceCallbackNonPersistFdsClass.exceptions.size() != 0) {
			for (int index = 0; index < InstanceCallbackNonPersistFdsClass.exceptions.size(); index++) {
				incrementErrorCount();
				outPrt.println(InstanceCallbackNonPersistFdsClass.callbackCalled.get(index) + "operation " +
						InstanceCallbackNonPersistFdsClass.attributeOpCausingExceptions.get(index) + " resulted in exception " +
						InstanceCallbackNonPersistFdsClass.exceptions.get(index));

			}
		}
		pm.close();
		if (getErrorCount() > 0)
			return Status.failed("Assertion A6.4.1-3 (ModificationOfNontransactionalNonpersistentFields) failed");
		else
			return Status.passed("Assertion A6.4.1-3 (ModificationOfNontransactionalNonpersistentFields) passed");
	}
}
