/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.instancecallbacks;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.*;
import java.io.*;
import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> No Access To Fields After Predelete
 *<BR>
 *<B>JavaTest Keywords:</B> instancecallbacks
 *<BR>
 *<B>Assertion ID:</B> A10.4-3.
 *<BR>
 *<B>Assertion Description: </B>
 Access to field values after a call to <code>jdoPreDelete()</code>
 of a class implementing <code>InstanceCallbacks</code> are disallowed.

 *@test
 *@title No Access To Fields After Predelete
 *@executeClass com.sun.jdotck.api.instancecallbacks.NoAccessToFieldsAfterPredelete
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source NoAccessToFieldsAfterPredelete.java
 *@keywords instancecallbacks
 */

/*
 *  Delete a persistent object and new object.  While the objects are in state persitent-deleted (or persistent-new-deleted)
 *     access persistent fields (except do not reference primary key fields).
 *
 *  Verify the field access results in a JDOUserException.
 */

public class NoAccessToFieldsAfterPredelete extends JDO_Test {

	public NoAccessToFieldsAfterPredelete() {
	}

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		NoAccessToFieldsAfterPredelete testInstance = new NoAccessToFieldsAfterPredelete();
		Status s = testInstance.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(java.lang.String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		try {
			return run2(args, errPrt, outPrt);
		} catch (Exception e) {
			outPrt.println("Assertion A10.4-3 (NoAccessToFieldsAfterPredelete) unexpected exception");
			outPrt.println(e);
			incrementErrorCount();
			e.printStackTrace(outPrt);
			return Status.failed("Assertion A10.4-3 (NoAccessToFieldsAfterPredelete) unexpected exception");
		}
	}

	Status run2(String[] args, PrintWriter errPrt, PrintWriter outPrt) {
		initializeArgs(args, errPrt, outPrt);

		PersistenceManagerFactory fact = getPMF();
		PersistenceManager pm = fact.getPersistenceManager();
		Transaction t = pm.currentTransaction();

		InstanceCallbackClass.initializeStaticsForTest();
		t.begin();
		InstanceCallbackClass.removeAllInstances(pm); // always start fresh with no instances
		t.commit();

		t.begin();
		// make intValue outside the range of zero to arraySize-1 to skip most jdoPreDelete() code.
		InstanceCallbackClass a = new InstanceCallbackClass("object a", null, InstanceCallbackClass.arraySize, 6.0, (short) -1, '6', null);
		pm.makePersistent(a);
		Object aId = pm.getObjectId(a);
		t.commit();

		t.begin();
		// relocte object.
		try {
			a = (InstanceCallbackClass) pm.getObjectById(aId, true);
		} catch (JDOUserException e) {
			errPrt.println("Could not locate persistent object created in previous transaction, got JDOUserException " + e);
			incrementErrorCount();
			return Status.failed("NoAccessToFieldsAfterPredelete:  Could not locate persistent object created in previous transaction");
		} catch (JDODataStoreException e) {
			errPrt.println("Attempt to locate object created in previous transaction, got JDODataStoreException " + e);
			incrementErrorCount();
			return Status.failed("NoAccessToFieldsAfterPredelete:  Could not locate persistent object created in previous transaction");
		}

		pm.deletePersistent(a);
		performAccessFieldTests("Object in state persistent-deleted:  ", a);

		InstanceCallbackClass b = new InstanceCallbackClass("object b", null, InstanceCallbackClass.arraySize + 1, 7.0, (short) -1, '7', null);
		pm.makePersistent(b);
		pm.deletePersistent(b);
		performAccessFieldTests("Object in state persistent-new-deleted:  ", b);

		t.rollback();
		pm.close();
		if (getErrorCount() > 0)
			return Status.failed("Assertion A10.4-3 (NoAccessToFieldsAfterPredelete) failed");
		else
			return Status.passed("Assertion A10.4-3 (NoAccessToFieldsAfterPredelete) passed");
	}

	void performAccessFieldTests(String title, InstanceCallbackClass o) {
		try {
			short x1 = o.childToDelete;
			out.println(title + "Accessed persistent short field childToDelete--should have gotten JDOUserException");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected
		}

		try {
			double x2 = o.doubleValue;
			out.println(title + "Accessed persistent double field doubleValue--should have gotten JDOUserException");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected
		}

		try {
			char x3 = o.charValue;
			out.println(title + "Accessed persistent char field charValue--should have gotten JDOUserException");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected
		}

		try {
			String x4 = o.name;
			out.println(title + "Accessed persistent String field name--should have gotten JDOUserException");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected
		}

		try {
			Date x5 = o.timeStamp;
			out.println(title + "Accessed persistent Date field timeStamp--should have gotten JDOUserException");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected
		}

		try {
			HashSet x6 = o.children;
			out.println(title + "Accessed persistent HashSet field Children--should have gotten JDOUserException");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected
		}

		try {
			InstanceCallbackClass x7 = o.nextObj;
			out.println(title + "Accessed persistent InstanceCallbackClass reference field nextObj--should have gotten JDOUserException");
			incrementErrorCount();
		} catch (JDOUserException e) {
			// expected
		}
	}
}
