/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.instancecallbacks;

import java.io.*;
import java.util.*;

import com.sun.jdotck.*;

public abstract class TestParts extends JDO_Test {

	public TestParts() {
	}

	void checkInstances(String label, int intValue, String capturedNextObjName, int numberOfChildren, int sumOfChildrenIntValue) {
		if (InstanceCallbackClass.processedIndex[intValue] != true) {
			out.println(label + "Callback never made on object with intValue = " + intValue);
			incrementErrorCount();
			return;
		}
		if (capturedNextObjName != null & InstanceCallbackClass.capturedNextObjName[intValue] == null) {
			out.println(label + "nextObj attribute for object with intValue = " + intValue + " should not have been null.");
			incrementErrorCount();
		} else if (capturedNextObjName == null & InstanceCallbackClass.capturedNextObjName[intValue] != null) {
			out.println(label + "nextObj attribute for object with intValue = " + intValue + " should  have been null.");
			incrementErrorCount();
		} else if (capturedNextObjName != null && !InstanceCallbackClass.capturedNextObjName[intValue].equals(capturedNextObjName)) {
			out.println(label + "nextObj.name attribute for object with intValue = " + intValue + " should  have been \"" +
					capturedNextObjName + "\".  It was \"" + InstanceCallbackClass.capturedNextObjName[intValue] + "\" instead.");
			incrementErrorCount();
		}

		if (InstanceCallbackClass.numberOfChildren[intValue] != numberOfChildren) {
			out.println(label + "Number of instances in attribute children for object with intValue = " + intValue + " should  have been " +
					numberOfChildren + ".  It was " + InstanceCallbackClass.numberOfChildren[intValue] + " instead.");
			incrementErrorCount();
		}

		if (InstanceCallbackClass.sumOfChildrenIntValue[intValue] != sumOfChildrenIntValue) {
			out.println(label + "Sum of intValue of instances in attribute children for object with intValue = " + intValue + " should have been " +
					sumOfChildrenIntValue + ".  It was " + InstanceCallbackClass.sumOfChildrenIntValue[intValue] + " instead.");
			incrementErrorCount();
		}
	}

	void checkPMAccess(String label, int intValue, boolean transactionActive) {
		if (InstanceCallbackClass.processedIndex[intValue] != true) {
			out.println(label + "Callback never made on object with intValue = " + intValue);
			incrementErrorCount();
			return;
		}
		// Only  verify isActive() returned true for the object if transactionActive is true
		if (transactionActive & InstanceCallbackClass.transactionActive[intValue] != true) {
			out.println(label + "PersistenceManager.currentTransaction.isAcive() returned false");
			incrementErrorCount();
		}
	}

	//  The attributes are:  label, name, date, intValue, doubleValue, childToDelete, charValue
	void checkFieldValues(String label, int intValue, String name, Date timeStamp, double doubleValue, short childToDelete, char charValue) {
		if (InstanceCallbackClass.processedIndex[intValue] != true) {
			out.println(label + "Callback never made on object with intValue = " + intValue);
			incrementErrorCount();
			return;
		}

		if (!InstanceCallbackClass.capturedName[intValue].equals(name)) {
			out.println(label + "name attribute for object with intValue = " + intValue + " should be \"" + name + "\".  It was \"" +
					InstanceCallbackClass.capturedName[intValue] + "\" instead.");
			incrementErrorCount();
		}

		if (!InstanceCallbackClass.capturedTimeStamp[intValue].equals(timeStamp)) {
			out.println(label + "timeStamp attribute for object with intValue = " + intValue + " should be " + timeStamp + ".  It was " +
					InstanceCallbackClass.capturedTimeStamp[intValue] + " instead.");
			incrementErrorCount();
		}

		if (InstanceCallbackClass.capturedDoubleValue[intValue] != doubleValue) {
			out.println(label + "doubleValue attribute for object with intValue = " + intValue + " should be " + doubleValue + ".  It was " +
					InstanceCallbackClass.capturedDoubleValue[intValue] + " instead.");
			incrementErrorCount();
		}

		if (InstanceCallbackClass.capturedCharValue[intValue] != charValue) {
			out.println(label + "charValue attribute for object with intValue = " + intValue + " should be " + charValue + ".  It was " +
					InstanceCallbackClass.capturedCharValue[intValue] + " instead.");
			incrementErrorCount();
		}

		if (InstanceCallbackClass.capturedChildToDelete[intValue] != childToDelete) {
			out.println(label + "childToDelete attribute for object with intValue = " + intValue + " should be " + childToDelete + ".  It was " +
					InstanceCallbackClass.capturedChildToDelete[intValue] + " instead.");
			incrementErrorCount();
		}
	}

	/** Touch fields to guarantee that they are loaded into the instance
	 */
	double touchFields(InstanceCallbackClass o) {
		// make a checksum from the fields and return it; this cannot be optimized out...
		double rc = o.doubleValue;
		rc += o.intValue;
		rc += o.charValue;
		rc += o.childToDelete;
		rc += o.name.length();
		rc += o.timeStamp.getTime();
		return rc;
	}

}
