/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */
package com.sun.jdotck.api.jdohelper;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get Objectid
 *<BR>
 *<B>JavaTest Keywords:</B> jdohelper
 *<BR>
 *<B>Assertion ID:</B> A8.3-1
 *<BR>
 *<B>Assertion Description: </B>
 The jdohelper.getObjectId method returns an ObjectId instance that re presents the object identity of the specified JDO Instance. Test: The method g etObjectById returns the exact same object, evaluating to true when == is used .

 *@test
 *@title get objectid
 *@executeClass com.sun.jdotck.api.jdohelper.GetObjectId
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetObjectId.java
 *@keywords jdohelper
 */



/*
* Revision History
* ================
* Author         :   	Date   : 	Version
* Azita Kamangar  	9/26/01		 1.0
*/

public class GetObjectId extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction transaction;
	private JDOHelper jhelp;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		GetObjectId t = new GetObjectId();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test JDOHelper.getObjectId(Object pc)
	 *
	*/
	public Status testGetObjectId() {
		try {
			Transaction tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);

			Object p2 = jhelp.getObjectId(p1);
			tx.commit();

			if (p2 == null)
				return Status.failed("FAILED in testGetObjectId()");
			else
				return Status.passed("PASSED in testGetObjectId()");
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in testGetObjectId()");
		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */


	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			testGetObjectId();

			Status s1 = this.testGetObjectId();
			pm.close();
			if (s1.isPassed())
				return Status.passed("PASSED in GetObjectId");
			else
				return Status.failed("FAILED in GetObjectId");

		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in GetObjectId()");
		}

	}

}
