/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.jdohelper;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Is Deleted
 *<BR>
 *<B>JavaTest Keywords:</B> jdohelper
 *<BR>
 *<B>Assertion ID:</B> A8.4.5-1
 *<BR>
 *<B>Assertion Description: </B>
 The jdohelper.isDeleted method returns true if the instance of persistence capable
 have been deleted in the current transaction
 evaluating to true when == is used.
 *@test
 *@title Is Deleted
 *@executeClass com.sun.jdotck.api.jdohelper.IsDeleted
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source IsDeleted.java
 *@keywords jdohelper
 */
/*
* Revision History
* ================
* Author         :   	Date   : 	Version
* Azita Kamangar  	9/26/01		 1.0
*/

public class IsDeleted extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction transaction;
	private JDOHelper jhlpr;
	private boolean flag;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		IsDeleted t = new IsDeleted();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test jdohelper.isDeleted(Object pc)
	 *
	*/
	Status testIsDeleted() {
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			pm.deletePersistent(p1);
			flag = jhlpr.isDeleted(p1);
			tx.commit();
			if (flag == true)

				return Status.passed("PASSED in testIsDeleted()");
			else
				return Status.failed("FAILED in testIsDeleted()");
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in testIsDeleted()");
		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */


	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			testIsDeleted();
			Status s1 = this.testIsDeleted();
			pm.close();
			if (s1.isPassed())
				return Status.passed("PASSED in IsDeleted");
			else
				return Status.failed("FAILED in IsDeleted");

		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in IsDeleted()");
		}

	}

}
