/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.jdohelper;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Is New False
 *<BR>
 *<B>JavaTest Keywords:</B> jdohelper
 *<BR>
 *<B>Assertion ID:</B> A8.4.4-2
 *<BR>
 *<B>Assertion Description: </B>
 The jdohelper.isnew method returns false for the instances that have not
 been made persistent in the current transaction
 evaluating to true when == is used.
 *@test
 *@title Is New False
 *@executeClass com.sun.jdotck.api.jdohelper.IsNewFalse
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source IsNewFalse.java
 *@keywords jdohelper
 */

/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/2/01		 1.0
 */
public class IsNewFalse extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction transaction;
	private JDOHelper jhlpr;
	private boolean flag;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		IsNewFalse t = new IsNewFalse();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test JDOHelper.isNew(Object pc)
	 *
	*/
	Status testIsNewFalse() {
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);

			flag = jhlpr.isNew(p1);
			tx.commit();
			if (flag == false)

				return Status.passed("PASSED in testIsNewFalse()");
			else
				return Status.failed("FAILED in testIsNewFalse()");
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in testIsNewFalse()");
		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			testIsNewFalse();
			Status s1 = this.testIsNewFalse();
			pm.close();
			if (s1.isPassed())
				return Status.passed("PASSED in IsNewFalse");
			else
				return Status.failed("FAILED in IsNewFalse");

		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in IsNewFalse()");
		}

	}

}
