/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.jdohelper;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Is Persistent Not Persistence Capable
 *<BR>
 *<B>JavaTest Keywords:</B> jdohelper
 *<BR>
 *<B>Assertion ID:</B> A8.4.3-4
 *<BR>
 *<B>Assertion Description: </B>
 The jdohelper.isPersistent method returns false If the instance does not implement PersistenceCapable
 evaluating to true when == is used.
 *@test
 *@title Is Persistent Not Persistence Capable
 *@executeClass com.sun.jdotck.api.jdohelper.IsPersistentNotPersistenceCapable
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source IsPersistentNotPersistenceCapable.java
 *@keywords jdohelper
 */
/*
* Revision History
* ================
* Author         :   	Date   : 	Version
* Azita Kamangar  	10/2/01		 1.0
*/

public class IsPersistentNotPersistenceCapable extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction transaction;
	private JDOHelper jhlpr;
	private boolean flag;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		IsPersistentNotPersistenceCapable t = new IsPersistentNotPersistenceCapable();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test JDOHelper.isPersistent(Object pc)
	 *
	*/
	Status testIsPersistentNotPersistenceCapable() {
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();
			Point p1 = new Point(1, 3);

			flag = jhlpr.isPersistent(p1);
			tx.commit();
			if (flag == false)

				return Status.passed("PASSED in testIsPersistentNotPersistenceCapable()");
			else
				return Status.failed("FAILED in testIsPersistentNotPersistenceCapable()");
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in testIsPersistentNotPersistenceCapable()");
		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */


	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			testIsPersistentNotPersistenceCapable();
			Status s1 = this.testIsPersistentNotPersistenceCapable();
			pm.close();
			if (s1.isPassed())
				return Status.passed("PASSED in IsPersistentNotPersistenceCapable");
			else
				return Status.failed("FAILED in IsPersistentNotPersistenceCapable");

		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in IsPersistentNotPersistenceCapable()");
		}

	}

}
