/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.jdohelper;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Is Transactional
 *<BR>
 *<B>JavaTest Keywords:</B> jdohelper
 *<BR>
 *<B>Assertion ID:</B> A8.4.2-1
 *<BR>
 *<B>Assertion Description: </B>
 The jdohelper.isTransactional method returns true and delegates to the parameter instance
 and instances whose state is associated with the current transaction .  evaluating to true when == is used.
 *@test
 *@title Is Transactional
 *@executeClass com.sun.jdotck.api.jdohelper.IsTransactional
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source IsTransactional.java
 *@keywords jdohelper
 */

/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/4/01		 1.0
 */
public class IsTransactional extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction transaction;
	private JDOHelper jhlpr;
	private boolean flag;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		IsTransactional t = new IsTransactional();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test JDOHelper.IsTransactional(Object pc)
	 *
	*/

	public void testIsTransactional() {
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			flag = jhlpr.isTransactional(p1);
			tx.commit();
			if (flag == true) {
				;
			} else {
				incrementErrorCount();
			}
			Transaction tx1 = pm.currentTransaction();
			tx1.begin();
			flag = jhlpr.isTransactional(p1);
			tx1.commit();
			if (flag == false) {
				;
			} else {
				incrementErrorCount();
			}
		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexpected exception caught in IsTransactional");
			incrementErrorCount();
		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();
		testIsTransactional();
		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in IsTransactional");
		else
			return Status.failed("FAILED in IsTransactional: number of errors detected is " + errCnt);

	}


}


