/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> AfterCloseAllMethodsThrowException
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion IDs:</B> A12.5-6
 *<BR>
 *<B>Assertion Description: </B>
 After the PersistenceManager.close method completes,
 all methods on PersistenceManager except isClosed throw a
 JDOFatalUserException.

 *@test
 *@title AfterCloseAllMethodsThrowException
 *@executeClass com.sun.jdotck.api.persistencemanager.AfterCloseAllMethodsThrowException
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source AfterCloseAllMethodsThrowException.java
 *@keywords exception
 */

public class AfterCloseAllMethodsThrowException extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		AfterCloseAllMethodsThrowException t = new AfterCloseAllMethodsThrowException();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testAfterCloseAllMethodsThrowException();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("AfterCloseAllMethodsThrowException tests passed");
		} else {
			return Status.failed("AfterCloseAllMethodsThrowException: number of errors detected is " + errCnt);
		}

	}

	public void testAfterCloseAllMethodsThrowException() {

		pm.close();
		testEvict();
		testRefresh();
		testIsClosed();

	}


	public void testEvict() {
		try {
			pm.evictAll();
			out.println(" \nFAILED in testEvict() - expect exception");
			incrementErrorCount();
		} catch (Exception ex) {
			assertJDOFatalUserException(ex, "testEvict()");
		}

	}

	public void testRefresh() {
		try {
			pm.refreshAll();
			out.println(" \nFAILED in testRefresh() - expect exception");
			incrementErrorCount();
		} catch (Exception ex) {
			assertJDOFatalUserException(ex, "testRefresh()");
		}

	}


	public void testIsClosed() {
		try {
			if (!pm.isClosed())
				incrementErrorCount();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testIsClosed()");
		}

	}


}

