/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Calling EvictAll With Collection Containing Nulls
 *<BR>
 *<B>JavaTest Keywords:</B> cache
 *<BR>
 *<B>Assertion IDs:</B> A12.5-11
 *<BR>
 *<B>Assertion Description: </B>
 Passing a non-null Object[] or Collection arguments to evictAll that
 contain null elements will have the documented behavior for non-null elements, and the null elements
 will be ignored.

 *@test
 *@title Calling EvictAll With Collection Containing Null
 *@executeClass com.sun.jdotck.api.persistencemanager.CallingEvictAllWithCollectionContainingNulls
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source CallingEvictAllWithCollectionContainingNulls.java
 *@keywords cache
 */

public class CallingEvictAllWithCollectionContainingNulls extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1;
	PCPoint p2;
	PCPoint p3;
	PCPoint p4;
	PCPoint p5;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		CallingEvictAllWithCollectionContainingNulls t = new CallingEvictAllWithCollectionContainingNulls();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testCallingEvictAllWithCollectionContainingNulls();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("CallingEvictAllWithCollectionContainingNulls tests passed");
		} else {
			return Status.failed("CallingEvictAllWithCollectionContainingNulls: number of errors detected is " + errCnt);
		}

	}

	public void testCallingEvictAllWithCollectionContainingNulls() {
		createObjects();
		testEvictAll1();
		testEvictAll2();

	}

	public void createObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(4, 3);
			p3 = new PCPoint(4, 2);
			p4 = new PCPoint(3, 3);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);

			tx.commit();

		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	/* test evictAll (Collection col1) */
	public void testEvictAll1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p5 = null;
			Collection col1 = new java.util.HashSet();
			col1.add(p1);
			col1.add(p5);
			col1.add(p2);

			pm.evictAll(col1);
			out.println(" \nPASSED in testEvictAll1()");
			tx.rollback();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testEvictAll1()");
			tx.rollback();
		}

	}


	/* test evictAll (Object[] objArray) */
	public void testEvictAll2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p5 = null;
			Collection col1 = new java.util.HashSet();
			col1.add(p3);
			col1.add(p4);
			col1.add(p5);

			pm.evictAll(col1.toArray());
			out.println(" \nPASSED in testEvictAll2()");
			tx.rollback();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testEvictAll2()");
			tx.rollback();
		}

	}


}
