/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.lang.reflect.*;

/**
 *<B>JavaTest Title:</B> Changing ObjectId Has No Effect On Instance
 *<BR>
 *<B>JavaTest Keywords:</B> identity
 *<BR>
 *<B>Assertion ID:</B> A12.5.6-13.
 *<BR>
 *<B>Assertion Description: </B>
 If the application makes a change to the <code>ObjectId</code> instance
 returned by <code>PersistenceManager.getObjectId</code>, there is no effect
 on the instance from which the <code>ObjectId</code> was obtained.

 *@test
 *@title Changing ObjectId Has No Effect On Instance
 *@executeClass com.sun.jdotck.api.persistencemanager.ChangingObjectIdHasNoEffectOnInstance
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source ChangingObjectIdHasNoEffectOnInstance.java
 *@keywords identity
 */

public class ChangingObjectIdHasNoEffectOnInstance extends PersistenceManagerTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		ChangingObjectIdHasNoEffectOnInstance t = new ChangingObjectIdHasNoEffectOnInstance();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		testChangingObjectIdHasNoEffectOnInstance();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in ChangingObjectIdHasNoEffectOnInstance tests ");
		} else {
			return Status.failed("FAILED in ChangingObjectIdHasNoEffectOnInstance: number of errors detected is " + errCnt);
		}
	}

	public void testChangingObjectIdHasNoEffectOnInstance() {
		Object oid = createPCPointInstance();
		Object p1 = getPM().getObjectById(oid, false);
		Object oid2 = getPM().getObjectId(p1);
		try {
			mangleObject(oid);
			// now oid3 should equal oid2 even though we mangled oid
			Object oid3 = getPM().getObjectId(p1);
			if (!oid2.equals(oid3)) {
				incrementErrorCount();
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "ChangingObjectIdHasNoEffectOnInstance()");
		}

	}
}
