/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import java.util.Properties;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.PCPoint;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Concurrent Persistence Managers
 *<BR>
 *<B>JavaTest Keywords:</B> concurrency multipleJDOimpls
 *<BR>
 *<B>Assertion ID:</B> A5.2-1.
 *<BR>
 *<B>Assertion Description: </B>
 An implementation should support its own <code>PersistenceManager</code>
 concurrently with another <code>PersistenceManager</code>.
 A non-reference implementation will be tested concurrently
 with the Reference Implementation.

 *@test
 *@title Concurrent Persistence Managers
 *@executeClass com.sun.jdotck.api.persistencemanager.ConcurrentPersistenceManagers
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source ConcurrentPersistenceManagers.java
 *@keywords concurrency multipleJDOimpls
 */

public class ConcurrentPersistenceManagers extends JDO_Test {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		ConcurrentPersistenceManagers t = new ConcurrentPersistenceManagers();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Properties pmfProperties = loadPMF2Properties();
		PersistenceManagerFactory pmf2 = JDOHelper.getPersistenceManagerFactory(pmfProperties);
		PersistenceManager pm2 = pmf2.getPersistenceManager();
		Transaction tx2 = pm2.currentTransaction();

		Transaction tx = getPM().currentTransaction();
		tx.begin();
		tx2.begin();
		tx.commit();
		tx2.commit();
		pm.close();
		pm2.close();

		out.flush();

		return Status.passed("Assertion A5.2-2 passed");
	}

	Properties loadPMF2Properties() {
		String PMF2 = System.getProperty("PMF2Properties", "jdori2.properties");
		out.println("Got PMF2Properties file name:" + PMF2);
		Properties ret = loadProperties(PMF2);
		out.println("Got PMF2Properties: " + ret);
		return ret;
	}

}
