/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import java.util.Properties;
import java.util.Collection;
import java.util.Iterator;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.PCPoint;
import com.sun.jdotck.models.mylib.PCRect;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Same Classes with Concurrent Persistence Managers
 *<BR>
 *<B>JavaTest Keywords:</B> concurrency multipleJDOimpls
 *<BR>
 *<B>Assertion ID:</B> A5.2-2.
 *<BR>
 *<B>Assertion Description: </B>
 The same classes must be supported concurrently by multiple
 <code>PersistenceManager</code>s from different implementations.

 *@test
 *@title Same Classes with Concurrent Persistence Managers
 *@executeClass com.sun.jdotck.api.persistencemanager.ConcurrentPersistenceManagersSameClasses
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source ConcurrentPersistenceManagersSameClasses.java
 *@keywords concurrency multipleJDOimpls
 */

public class ConcurrentPersistenceManagersSameClasses extends JDO_Test {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		ConcurrentPersistenceManagersSameClasses t = new ConcurrentPersistenceManagersSameClasses();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Properties pmfProperties = loadPMF2Properties();
		PersistenceManagerFactory pmf2 = JDOHelper.getPersistenceManagerFactory(pmfProperties);
		PersistenceManager pm2 = pmf2.getPersistenceManager();
		Transaction tx2 = pm2.currentTransaction();

		Transaction tx = getPM().currentTransaction();
		tx.begin();
		tx2.begin();

		PCPoint p11 = new PCPoint(110, 120);
		PCPoint p12 = new PCPoint(120, 140);
		PCRect rect1 = new PCRect(0, p11, p12);
		pm.makePersistent(rect1);

		PCPoint p21 = new PCPoint(210, 220);
		PCPoint p22 = new PCPoint(220, 240);
		PCRect rect2 = new PCRect(0, p21, p22);
		pm2.makePersistent(rect2);

		tx.commit();
		tx2.commit();

		tx.begin();
		tx2.begin();

		PCPoint p11a = findPoint(pm, 110, 120);
		if (p11a != p11) {
			out.println("Found PCPoint: " + p11a.getX() + ", " + p11a.getY());
			out.println("Expected: 110, 120");
			incrementErrorCount();
		}
		PCPoint p21a = findPoint(pm2, 210, 220);
		if (p21a != p21) {
			out.println("Found PCPoint: " + p21a.getX() + ", " + p21a.getY());
			out.println("Expected: 210, 220");
			incrementErrorCount();
		}

		tx.commit();
		tx2.commit();

		pm.close();
		pm2.close();

		out.flush();

		if (getErrorCount() != 0)
			return Status.failed("Assertion A5.2-2 failed");
		else
			return Status.passed("Assertion A5.2-2 passed");
	}

	Properties loadPMF2Properties() {
		String PMF2 = System.getProperty("PMF2Properties", "jdori2.properties");
		out.println("Got PMF2Properties file name:" + PMF2);
		Properties ret = loadProperties(PMF2);
		out.println("Got PMF2Properties: " + ret);
		return ret;
	}

	PCPoint findPoint(PersistenceManager pm, int x, int y) {
		Query q = pm.newQuery(PCPoint.class);
		q.declareParameters("int px, int py");
		q.setFilter("x == px & y == py");
		Collection results = (Collection) q.execute(new Integer(x), new Integer(y));
		Iterator it = results.iterator();
		PCPoint ret = (PCPoint) it.next();
		return ret;
	}


}
