/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Delete Persistent
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-9
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.deletePersistent and deletePersistentAll delete a persistent instance(s) from the data store. It must be called in the context of an active transaction, or a JDOUserExceptionis thrown. The representation in the data store will be deleted when this instance is flushed to the data store (via commit, or evict).

 *@test
 *@title Delete Persistent
 *@executeClass com.sun.jdotck.api.persistencemanager.DeletePersistent
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source DeletePersistent.java
 *@keywords
 */

public class DeletePersistent extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		DeletePersistent t = new DeletePersistent();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testDeletePersistent();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("DeletePersistent tests passed");
		} else {
			return Status.failed("DeletePersistent: number of errors detected is " + errCnt);
		}

	}

	public void testDeletePersistent() {
		createObjects();

		/* positive tests */
		/* method called in the context of an active transaction */
		testDeletePersistent1();
		testDeletePersistent2();
		testDeletePersistent3();

		/* negative tests */
		/* method called in the context of an non-active transaction */
		testDeletePersistent4();
		testDeletePersistent5();
		testDeletePersistent6();

	}


	private void createObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);

			tx.commit();

		} catch (Exception ex) {
			out.println(" - createObjects() FAILED: ");
			ex.printStackTrace(out);
		}
	}


	/* test deletePersistent (Object pc) */
	public void testDeletePersistent1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.deletePersistent(p1);
			tx.commit();
			out.println(" \nPASSED in testDeletePersistent1()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testDeletePersistent1()");
			if (tx.isActive())
				tx.rollback();
		}

	}


	/* test deletePersistentAll (Collection pcs) */
	public void testDeletePersistent2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			pm.deletePersistentAll(col1);
			tx.commit();
			out.println(" \nPASSED in testDeletePersistent2()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testDeletePersistent2()");
			if (tx.isActive())
				tx.rollback();
		}

	}

	/* test deletePersistentAll (Object[] o) */
	public void testDeletePersistent3() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			Object[] obj1 = col1.toArray();

			pm.deletePersistentAll(obj1);
			tx.commit();
			out.println(" \nPASSED in testDeletePersistent3()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testDeletePersistent3()");
			if (tx.isActive())
				tx.rollback();
		}

	}


	/* test deletePersistent (Object pc) */
	public void testDeletePersistent4() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			PCPoint np1 = new PCPoint(1, 3);
			pm.makePersistent(np1);
			tx.commit();

			pm.deletePersistent(np1);
			out.println(" FAILED in testDeletePersistent4() - should not go that further ");
			incrementErrorCount();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistent4()");
			if (tx.isActive())
				tx.rollback();
		}
	}


	/* test deletePersistentAll (Collection pcs) */
	public void testDeletePersistent5() {
		try {

			tx = pm.currentTransaction();
			tx.begin();

			PCPoint np1 = new PCPoint(3, 3);
			PCPoint np2 = new PCPoint(4, 4);
			pm.makePersistent(np1);
			pm.makePersistent(np2);
			tx.commit();

			Collection col1 = new java.util.HashSet();
			col1.add(np1);
			col1.add(np2);

			pm.deletePersistentAll(col1);
			out.println(" FAILED in testDeletePersistent5() - should not go that further ");
			incrementErrorCount();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistent5()");
			if (tx.isActive())
				tx.rollback();
		}
	}

	/* test deletePersistentAll (Object[] o) */
	public void testDeletePersistent6() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint np1 = new PCPoint(3, 3);
			PCPoint np2 = new PCPoint(4, 4);

			pm.makePersistent(np1);
			pm.makePersistent(np2);
			tx.commit();

			Collection col1 = new java.util.HashSet();
			col1.add(np1);
			col1.add(np2);

			Object[] obj1 = col1.toArray();

			pm.deletePersistentAll(obj1);
			out.println(" FAILED in testDeletePersistent6() - should not go that further ");
			incrementErrorCount();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistent6()");
			if (tx.isActive())
				tx.rollback();
		}

	}

}
