/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> DeletePersistentAll Fails
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-2.
 *<BR>
 *<B>Assertion Description: </B>
 If a collection or array of instances is passed to PersistenceManager.deletePersistentAll, and one or more of the instances fail to complete the required operation, then all instances will be attempted, and a JDOUserException will be thrown which contains nested exceptions, each of which contains one of the failing instances. The succeeding instances will transition to the specified life cycle state, and the failing instances will remain in their current state.
 *@test
 *@title DeletePersistentAll Fails
 *@executeClass com.sun.jdotck.api.persistencemanager.DeletePersistentAllFails
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source DeletePersistentAllFails.java
 *@keywords exception
 */

public class DeletePersistentAllFails extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		DeletePersistentAllFails t = new DeletePersistentAllFails();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testDeletePersistentAllFails();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("DeletePersistentAllFails tests passed");
		} else {
			return Status.failed("DeletePersistentAllFails: number of errors detected is " + errCnt);
		}

	}

	public void testDeletePersistentAllFails() {
		createObjects();
		testDeletePersistentAllFails1();
		testDeletePersistentAllFails2();

	}

	private void createObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);

			tx.commit();

		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}


	/* test deletePersistentAll (Collection pcs) */
	public void testDeletePersistentAllFails1() {
		try {

			tx = pm.currentTransaction();
			tx.begin();

			Point np3 = new Point(5, 5);

			Collection col1 = new java.util.HashSet();
			col1.add(p1);
			col1.add(p2);
			col1.add(np3);

			pm.deletePersistentAll(col1);
			out.println(" FAILED in testDeletePersistentAllFails1() - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistentAllFails1()");
			tx.rollback();
		}
	}

	/* test deletePersistentAll (Object[] o) */
	public void testDeletePersistentAllFails2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Point np3 = new Point(5, 5);

			Collection col1 = new java.util.HashSet();
			col1.add(p3);
			col1.add(p4);
			col1.add(np3);

			Object[] obj1 = col1.toArray();

			pm.deletePersistentAll(obj1);
			out.println(" FAILED in testDeletePersistentAllFails2() - should not go that further ");
			incrementErrorCount();
			tx.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistentAllFails2()");
			tx.rollback();
		}

	}

}
