/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> DeletePersistent Fails If Instance Is Transient
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-12.
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.deletePersistent and deletePersistentAll will throw a JDOUserException if the parameter instance is transient.
 *@test
 *@title DeletePersistent Fails If Instance Is Transient
 *@executeClass com.sun.jdotck.api.persistencemanager.DeletePersistentFailsIfInstanceIsTransient
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source DeletePersistentFailsIfInstanceIsTransient.java
 *@keywords exception
 */

public class DeletePersistentFailsIfInstanceIsTransient extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		DeletePersistentFailsIfInstanceIsTransient t = new DeletePersistentFailsIfInstanceIsTransient();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testDeletePersistentFailsIfInstanceIsTransient();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("DeletePersistentFailsIfInstanceIsTransient tests passed");
		} else {
			return Status.failed("DeletePersistentFailsIfInstanceIsTransient: number of errors detected is " + errCnt);
		}

	}

	public void testDeletePersistentFailsIfInstanceIsTransient() {

		testDeletePersistent();
		testDeletePersistentAll1();
		testDeletePersistentAll2();

	}

	/* test deletePersistent (Object pc) */
	public void testDeletePersistent() {
//	out.println (" ** in testDeletePersistent() ");
		try {

			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p1 = new PCPoint(3, 3);

			pm.deletePersistent(p1);
			out.println(" FAILED in testDeletePersistent() - should no t go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistent()");
			tx.rollback();
		}
	}


	/* test deletePersistentAll (Collection pcs) */
	public void testDeletePersistentAll1() {
//	out.println (" ** in testDeletePersistentAll1()");
		try {

			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p1 = new PCPoint(3, 3);
			PCPoint p2 = new PCPoint(4, 4);

			Collection col1 = new java.util.HashSet();
			col1.add(p1);
			col1.add(p2);

			pm.deletePersistentAll(col1);
			out.println(" FAILED in testDeletePersistentAll1() - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistentAll1()");
			tx.rollback();
		}
	}

	/* test deletePersistentAll (Object[] o) */
	public void testDeletePersistentAll2() {
		out.println(" ** in testDeletePersistentAll2() ");
		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p1 = new PCPoint(3, 3);
			PCPoint p2 = new PCPoint(4, 4);

			Collection col1 = new java.util.HashSet();
			col1.add(p1);
			col1.add(p2);

			pm.deletePersistentAll(col1.toArray());
			out.println(" FAILED in testDeletePersistentAll2() - should not go that further ");
			incrementErrorCount();
			tx.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistentAll2()");
			tx.rollback();
		}

	}

}
