/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> DeletePersistent Fails If Instance Managed By Another PersistenceManager
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-11.
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.deletePersistent and deletePersistentAll will throw a JDOUserException if the parameter instance is managed by a different PersistenceManager.

 *@test
 *@title DeletePersistent Fails If Instance Managed By Another PersistenceManager
 *@executeClass com.sun.jdotck.api.persistencemanager.DeletePersistentFailsIfInstanceManagedByAnotherPersistenceManager
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source DeletePersistentFailsIfInstanceManagedByAnotherPersistenceManager.java
 *@keywords exception
 */

public class DeletePersistentFailsIfInstanceManagedByAnotherPersistenceManager extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm1;
	private PersistenceManager pm2;
	private Transaction tx1;
	private Transaction tx2;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		DeletePersistentFailsIfInstanceManagedByAnotherPersistenceManager t = new DeletePersistentFailsIfInstanceManagedByAnotherPersistenceManager();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm1 = pmf.getPersistenceManager();
		pm2 = pmf.getPersistenceManager();

		testDeletePersistentFailsIfInstanceManagedByAnotherPersistenceManager();

		pm1.close();
		pm2.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("DeletePersistentFailsIfInstanceManagedByAnotherPersistenceManager tests passed");
		} else {
			return Status.failed("DeletePersistentFailsIfInstanceManagedByAnotherPersistenceManager: number of errors detected is " + errCnt);
		}

	}

	public void testDeletePersistentFailsIfInstanceManagedByAnotherPersistenceManager() {
		createObjects();

		/* positive tests */
		testDeletePersistent();
		testDeletePersistentAll1();
		testDeletePersistentAll2();

	}


	private void createObjects() {
		try {
			tx1 = pm1.currentTransaction();

			tx1.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			pm1.makePersistent(p1);
			pm1.makePersistent(p2);
			pm1.makePersistent(p3);
			pm1.makePersistent(p4);
			pm1.makePersistent(p5);
			tx1.commit();

		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}


	/* test deletePersistent (Object pc) */
	public void testDeletePersistent() {
		try {

			tx2 = pm2.currentTransaction();
			tx2.begin();
			pm2.deletePersistent(p1);

			out.println(" \nFAILED in testDeletePersistent - " +
					" should not go that further ");
			incrementErrorCount();
			tx2.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistent()");
			tx2.rollback();
		}

	}


	/* test deletePersistentAll (Collection pcs) */
	public void testDeletePersistentAll1() {
		try {
			tx2.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			pm2.deletePersistentAll(col1);

			out.println(" \nFAILED in testDeletePersistentAll1 - " +
					" should not go that further ");
			incrementErrorCount();
			tx2.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistentAll1()");
			tx2.rollback();

		}


	}

	/* test deletePersistentAll (Object[] o) */
	public void testDeletePersistentAll2() {
		try {

			tx2.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			Object[] obj1 = col1.toArray();

			pm2.deletePersistentAll(obj1);

			out.println(" \nFAILED in testDeletePersistentAll2 - " +
					" should not go that further ");
			incrementErrorCount();
			tx2.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testDeletePersistentAll2()");
			tx2.rollback();

		}

	}


}
