/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> DeletePersistent Has No Effect On Deleted Instances
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-10.
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.deletePersistent and deletePersistentAll have no effect on parameter instances already deleted in the transaction.

 *@test
 *@title DeletePersistent Has No Effect On Deleted Instances
 *@executeClass com.sun.jdotck.api.persistencemanager.DeletePersistentHasNoEffectOnDeletedInstances
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source DeletePersistentHasNoEffectOnDeletedInstances.java
 *@keywords
 */

public class DeletePersistentHasNoEffectOnDeletedInstances extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		DeletePersistentHasNoEffectOnDeletedInstances t = new DeletePersistentHasNoEffectOnDeletedInstances();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testDeletePersistentHasNoEffectOnDeletedInstances();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("DeletePersistentHasNoEffectOnDeletedInstances tests passed");
		} else {
			return Status.failed("DeletePersistentHasNoEffectOnDeletedInstances: number of errors detected is " + errCnt);
		}

	}

	public void testDeletePersistentHasNoEffectOnDeletedInstances() {
		createObjects();
		testDeletePersistent();
		testDeletePersistentAll1();
		testDeletePersistentAll2();

	}


	private void createObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);

			tx.commit();

		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}


	/* test makePersistent (Object pc) */
	public void testDeletePersistent() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.deletePersistent(p1);
			pm.deletePersistent(p1);
			pm.deletePersistent(p1);
			pm.deletePersistent(p1);
			tx.commit();
			out.println(" \nPASSED in testDeletePersistent()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testDeletePersistent()");
			if (tx.isActive())
				tx.rollback();
		}

	}


	/* test makePersistentAll (Collection pcs) */
	public void testDeletePersistentAll1() {
		try {
			tx = pm.currentTransaction();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			tx.begin();
			pm.deletePersistentAll(col1);
			pm.deletePersistentAll(col1);
			pm.deletePersistentAll(col1);
			pm.deletePersistentAll(col1);
			tx.commit();
			out.println(" \nPASSED in testDeletePersistentAll1()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testDeletePersistentAll1()");
			if (tx.isActive())
				tx.rollback();
		}

	}

	/* test makePersistentAll (Object[] o) */
	public void testDeletePersistentAll2() {
		try {
			Transaction tx = pm.currentTransaction();
			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			Object[] obj1 = col1.toArray();

			tx.begin();
			pm.deletePersistentAll(obj1);
			pm.deletePersistentAll(obj1);
			pm.deletePersistentAll(obj1);
			pm.deletePersistentAll(obj1);
			tx.commit();
			out.println(" \nPASSED in testDeletePersistentAll2()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testDeletePersistentAll2()");
			if (tx.isActive())
				tx.rollback();
		}

	}

}
