/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> EvictAllWithNoParameters
 *<BR>
 *<B>JavaTest Keywords:</B> cache
 *<BR>
 *<B>Assertion IDs:</B> A12.5.1-2
 *<BR>
 *<B>Assertion Description: </B>
 If PersistenceManager.evict is called with no parameter, then all referenced instances are evicted. For each instance evicted, it:
 *<UL>
 *<LI> calls the jdoPreClearmethod on each instance, if the class of the instance implements InstanceCallbacks </LI>
 *<LI> clears persistent fields on each instance after the call to jdoPreClear()</LI>
 *<LI> changes the state of instances to hollow or persistent-nontransactional (cannot distinguish between these two states) this is not directly testable. </LI>
 *</UL>

 *@test
 *@title Delete Persistent
 *@executeClass com.sun.jdotck.api.persistencemanager.EvictAllWithNoParameters
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source EvictAllWithNoParameters.java
 *@keywords
 */

public class EvictAllWithNoParameters extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint2 pnt1 = null;
	PCPoint2 p1 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		EvictAllWithNoParameters t = new EvictAllWithNoParameters();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testEvictAllWithNoParameters();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("EvictAllWithNoParameters tests passed");
		} else {
			return Status.failed("EvictAllWithNoParameters: number of errors detected is " + errCnt);
		}

	}

	public void testEvictAllWithNoParameters() {
		createObjects();
		testEvictAll();

	}


	private void createObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pnt1 = new PCPoint2(1, 3);
			p1 = new PCPoint2(3, 5);
			pm.makePersistent(pnt1);
			pm.makePersistent(p1);
			tx.commit();

			/* P-nontransactional instance*/
			tx.setOptimistic(true);
			tx.begin();
			pnt1.getX();
			tx.commit();

			/* P-clean instance */
			tx.setOptimistic(false);
			tx.begin();
			p1.getX();
			tx.commit();

		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}


	public void testEvictAll() {
		out.println("**  in testEvictAll() ");
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.evictAll();

			if (!p1.wasClearCalled()) {
				out.println(" p1 jdoPreClear not called ");
				incrementErrorCount();
			}
			if (!pnt1.wasClearCalled()) {
				out.println(" pnt1 jdoPreClear not called ");
				incrementErrorCount();
			}

			if (testState(p1, HOLLOW, "hollow") ||
					testState(p1, PERSISTENT_NONTRANSACTIONAL, "persistent_nontransaction")) {
				;  // expected result

			} else {
				incrementErrorCount();
			}

			if (testState(pnt1, HOLLOW, "hollow") ||
					testState(pnt1, PERSISTENT_NONTRANSACTIONAL, "persistent_nontransaction")) {
				;  // expected result

			} else {
				incrementErrorCount();
			}

			tx.commit();
			out.println(" \nPASSED in testEvictAll()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testEvictAll()");
			tx.rollback();
		}

	}

}
