/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> GetExtentWithNoSubclasses
 *<BR>
 *<B>JavaTest Keywords:</B> inheritance extent
 *<BR>
 *<B>Assertion ID:</B> A12.5.4-2.
 *<BR>
 *<B>Assertion Description: </B>
 The getExtent method returns an Extent that contains all of the instances in the parameter class. With the subclasses parameter false, only instances of the specified class are returned.

 *@test
 *@title GetExtentWithNoSubclasses
 *@executeClass com.sun.jdotck.api.persistencemanager.GetExtentWithNoSubclasses
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetExtentWithNoSubclasses.java
 *@keywords inheritance extent
 */

public class GetExtentWithNoSubclasses extends PersistenceManagerTest {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;
	PCPoint p1 = null;
	PCPoint p2 = null;


	public static void main(String[] args) {

		GetExtentWithNoSubclasses t = new GetExtentWithNoSubclasses();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testGetExtentWithNoSubclasses();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetExtentWithNoSubclasses tests ");
		} else {
			return Status.failed("FAILED in GetExtentWithNoSubclasses: number of errors detected is " + errCnt);
		}

	}


	public void testGetExtentWithNoSubclasses() {
		createObjects();
		testGetExtent();
	}


	public void createObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			PCPoint p2 = new PCPoint(3, 5);
			pm.makePersistent(p1);
			pm.makePersistent(p2);
			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "createObjects()");
		}
	}


	public void testGetExtent() {

		try {

			tx = pm.currentTransaction();
			tx.begin();

			Extent e = pm.getExtent(PCPoint.class, false);

			int c = 0;
			for (Iterator i = e.iterator(); i.hasNext();) {
				PCPoint p = (PCPoint) i.next();
				out.println("p.getX() = " + p.getX());
				if (p.getX() == 1)
					;
				else {
					if (p.getX() == 3)
						;
					else
						incrementErrorCount();
				}

			}
			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetExtent()");
		}

	}

}
