/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import com.sun.jdotck.models.mylib.PCPoint;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get Object By Id
 *<BR>
 *<B>JavaTest Keywords:</B> identity cache
 *<BR>
 *<B>Assertion ID:</B> A12.5.6-9.
 *<BR>
 *<B>Assertion Description: </B>
 If <code>PersistenceManager.getObjectById</code> is called with a value of
 <code>true</code> for the second parameter named <code>validate</code>,
 and there is not an instance already in the cache with the same JDO identity
 as the oid parameter, then this method creates an instance with the specified
 JDO identity, verifies that it exists in the data store, and returns it.
 <OL TYPE=A>
 <LI>If there is no transaction in progress, the returned instance will be
 hollow or persistent-nontransactional, at the choice of the implementation.</LI>
 <LI>If there is a data store transaction in progress,
 the returned instance will be persistent-clean.</LI>
 <LI>If there is an optimistic transaction in progress,
 the returned instance will be persistent-nontransactional.</LI>
 </OL>

 *@test
 *@title Get Object By Id
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectById
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source GetObjectById.java
 *@keywords identity cache
 */

public class GetObjectById extends PersistenceManagerTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		GetObjectById t = new GetObjectById();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>runTest</code> method is invoked by JDO_Test to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		testGetObjectById();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectById tests ");
		} else {
			return Status.failed("FAILED in GetObjectById: number of errors detected is " + errCnt);
		}

	}


	public void testGetObjectById() {

		Transaction tx;
		try {

			Object oid = createPCPointInstance();

			// nontransactional access
			out.println("nontransactional");
			pm = null; // make sure to get a different pm so obj is not in cache
			tx = getPM().currentTransaction();
			tx.setNontransactionalRead(true);
			Object obj = pm.getObjectById(oid, true);
			int state = currentState(obj);
			if (state != PERSISTENT_NONTRANSACTIONAL & state != HOLLOW) {
				incrementErrorCount();
				out.println("Expected persistent-nontransactional or hollow; got " + getStateOfInstance(obj));
			}

			// pessimistic transactional access
			out.println("pessimistic");
			pm = null; // make sure to get a different pm so obj is not in cache
			tx = getPM().currentTransaction();
			tx.setOptimistic(false);
			tx.begin();
			obj = pm.getObjectById(oid, true);
			state = currentState(obj);
			tx.commit();
			if (state != PERSISTENT_CLEAN) {
				incrementErrorCount();
				out.println("Expected persistent-clean; got " + getStateOfInstance(obj));
			}

			// optimistic transactional access
			if (isOptimisticSupported()) {
				out.println("optimistic");
				pm = null; // make sure to get a different pm so obj is not in cache
				tx = getPM().currentTransaction();
				tx.setOptimistic(true);
				tx.begin();
				obj = pm.getObjectById(oid, true);
				state = currentState(obj);
				tx.commit();
				if (state != PERSISTENT_NONTRANSACTIONAL & state != HOLLOW) {
					incrementErrorCount();
					out.println("Expected persistent-nontransactional; got " + state + getStateOfInstance(obj));
				}
			} else {
				out.println("optimistic test skipped");
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetObjectById()");
		}
	}
}