/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get Object By Id No Validation, Instance In Cache, No State Change
 *<BR>
 *<B>JavaTest Keywords:</B> identity cache lifecycle
 *<BR>
 *<B>Assertion IDs:</B> A12.5.6-3
 *<BR>
 *<B>Assertion Description: </B>
 If PersistenceManager.getObjectById is called with a value of false for the second parameter named validate, and there is already an instance in the cache with the same JDO identity as the oid parameter, there is no change made to the state of the returned instance.
 *@test
 *@title get properties
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectByIdNoValidationInstanceInCacheNoStateChange
 *@executeArgs -PMFProperties $PMFProperties

 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source GetObjectByIdNoValidationInstanceInCacheNoStateChange.java
 *@keywords identity cache lifecycle
 */


public class GetObjectByIdNoValidationInstanceInCacheNoStateChange extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		GetObjectByIdNoValidationInstanceInCacheNoStateChange t = new GetObjectByIdNoValidationInstanceInCacheNoStateChange();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testGetObjectByIdNoValidationInstanceInCacheNoStateChange();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectByIdNoValidationInstanceInCacheNoStateChange tests ");
		} else {
			return Status.failed("FAILED in GetObjectByIdNoValidationInstanceInCacheNoStateChange: number of errors detected is " + errCnt);
		}

	}

	public void testGetObjectByIdNoValidationInstanceInCacheNoStateChange() {

		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			Object oid = pm.getObjectId(p1);
			tx.commit();
			int curr = currentState(p1);

			tx.begin();
			Object obj = pm.getObjectById(oid, false);

			int actualState = currentState(obj);

			if (actualState != curr) {
				out.println("State mismatched - Expected state: " +
						curr + " Actual state: " + actualState);
				incrementErrorCount();
			}
			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetObjectByIdNoValidationInstanceInCacheNoStateChange()");
		}

	}

}
