/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get Object By Id No Validation Instance Not In Cache, No Tx
 *<BR>
 *<B>JavaTest Keywords:</B> identity cache
 *<BR>
 *<B>Assertion IDs:</B> A12.5.6-4
 *<BR>
 *<B>Assertion Description: </B>
 If PersistenceManager.getObjectById is called with a value of false for the
 second parameter named validate, and there is not an instance already in the cache with the same JDO identity as the oid parameter, and there is no transaction in progress, then this method creates an instance with the specified JDO identity and returns it with a state of hollow or persistent-nontransactional, at the choice of the implementation.
 *@test
 *@title Get Object By Id No Validation Instance Not In Cache, No Tx
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectByIdNoValidationInstanceNotInCacheNoTx
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetObjectByIdNoValidationInstanceNotInCacheNoTx.java
 *@keywords identity cache
 */


public class GetObjectByIdNoValidationInstanceNotInCacheNoTx extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		GetObjectByIdNoValidationInstanceNotInCacheNoTx t = new GetObjectByIdNoValidationInstanceNotInCacheNoTx();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testGetObjectByIdNoValidationInstanceNotInCacheNoTx();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectByIdNoValidationInstanceNotInCacheNoTx tests ");
		} else {
			return Status.failed("FAILED in GetObjectByIdNoValidationInstanceNotInCacheNoTx: number of errors detected is " + errCnt);
		}

	}

	public void testGetObjectByIdNoValidationInstanceNotInCacheNoTx() {

		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			Object oid = pm.getObjectId(p1);
			pm.evict(p1);
			tx.commit();

			/* get oid for p1, no transaction in progress */
			Object obj = pm.getObjectById(oid, false);

			int curr = currentState(p1);

			if (curr == HOLLOW ||
					curr == PERSISTENT_NONTRANSACTIONAL ||
					curr == PERSISTENT_CLEAN) {
				; // expected result
			} else {
				out.println("returned state mismatched - " +
						" Actual state " + curr);
				incrementErrorCount();
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetObjectByIdNoValidationInstanceNotInCacheNoTx()");
		}

	}

}
