/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

/**
 *<B>JavaTest Title:</B> Get Object By Id No Validation Instance Not In Datastore
 *<BR>
 *<B>JavaTest Keywords:</B> identity exception
 *<BR>
 *<B>Assertion ID:</B> A12.5.6-6.
 *<BR>
 *<B>Assertion Description: </B>
 If <code>PersistenceManager.getObjectById</code> is called with a value of
 <code>false</code> for the second parameter named <code>validate</code>,
 and the instance does not exist in the data store, a subsequent access
 of the fields of the instance will throw a <code>JDODatastoreException</code>
 if the instance does not exist at that time.

 *@test
 *@title Get Object By Id No Validation Instance Not In Datastore
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectByIdNoValidationInstanceNotInDatastore
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source GetObjectByIdNoValidationInstanceNotInDatastore.java
 *@keywords identity exception
 */

public class GetObjectByIdNoValidationInstanceNotInDatastore extends PersistenceManagerTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		GetObjectByIdNoValidationInstanceNotInDatastore t = new GetObjectByIdNoValidationInstanceNotInDatastore();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>runTest</code> method is invoked by JDO_Test to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		testGetObjectByIdNoValidationInstanceNotInDatastore();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectByIdNoValidationInstanceNotInDatastore tests ");
		} else {
			return Status.failed("FAILED in GetObjectByIdNoValidationInstanceNotInDatastore: number of errors detected is " + errCnt);
		}
	}

	public void testGetObjectByIdNoValidationInstanceNotInDatastore() {
		Object oid = null;
		try {
			oid = createPCPointInstance();
			deletePCPointInstance(oid);
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetObjectByIdNoValidationInstanceNotInDatastore()");
		}

		try {
			PCPoint p1 = (PCPoint) pm.getObjectById(oid, false); // might throw exception here
			out.println("Got object in cache, even though not in datastore.");
			p1.getX(); // if not thrown above, throws exception here
		} catch (Exception ex) {
			assertJDODataStoreException(ex, "testGetObjectByIdNoValidationInstanceNotInDatastore()");
		}
	}
}
