/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get ObjectId For Null Or Not Persistent
 *<BR>
 *<B>JavaTest Keywords:</B> identity
 *<BR>
 *<B>Assertion IDs:</B> A12.5.6-1
 *<BR>
 *<B>Assertion Description: </B>
 In a call to PersistenceManager.getObjectById, if the PersistenceManager is
 unable to resolve the oid parameter to an ObjectId instance, then it throws a JDOUserException.
 *@test
 *@title Get ObjectId For Null Or Not Persistent
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectByIdNotResolved
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source GetObjectByIdNotResolved
 *@keywords identity
 */


public class GetObjectByIdNotResolved extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm2;
	private Transaction tx;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		GetObjectByIdNotResolved t = new GetObjectByIdNotResolved();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();
		pm2 = pmf.getPersistenceManager();

		testGetObjectByIdNotResolved();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectByIdNotResolved tests ");
		} else {
			return Status.failed("FAILED in GetObjectByIdNotResolved: number of errors detected is " + errCnt);
		}

	}

	/* passing null paramameter to getObjectId */
	public void testGetObjectByIdNotResolved() {

		out.println(" *** testObjectByIdNotResolved *** ");
		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			Object oid = pm.getObjectId(p1);
			pm.deletePersistent(p1);
			tx.commit();

			tx.begin();

			try {
				PCPoint p2 = (PCPoint) pm.getObjectById(oid, true);
			} catch (Exception e) {
				assertJDODataStoreException(e, "testGetObjectByIdNotResolved()");
				tx.rollback();
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetObjectByIdNotResolved()");
		}

	}

}
