/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get Object By Id With Validation Instance in Cache
 *<BR>
 *<B>JavaTest Keywords:</B> identity cache
 *<BR>
 *<B>Assertion IDs:</B> A12.5.6-7
 *<BR>
 *<B>Assertion Description: </B>
 If PersistenceManager.getObjectById is called with a value of true for the second parameter named validate, and there is already a transactional instance in the cache with the same JDO identity as the oid parameter, then this method returns it. There is no change made to the state of the returned instance.
 *@test
 *@title Get Object By Id With Validation Instance in Cache
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectByIdWithValidationInstanceInCache
 *@executeArgs -PMFProperties $PMFProperties

 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetObjectByIdWithValidationInstanceInCache.java
 *@keywords identity cache
 */


public class GetObjectByIdWithValidationInstanceInCache extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		GetObjectByIdWithValidationInstanceInCache t = new GetObjectByIdWithValidationInstanceInCache();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testGetObjectByIdWithValidationInstanceInCache();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectByIdWithValidationInstanceInCache tests ");
		} else {
			return Status.failed("FAILED in GetObjectByIdWithValidationInstanceInCache: number of errors detected is " + errCnt);
		}

	}

	public void testGetObjectByIdWithValidationInstanceInCache() {

		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			Object oid = pm.getObjectId(p1);
			tx.commit();
			int curr = currentState(p1);

			tx.begin();
			Object obj = pm.getObjectById(oid, false);
			int curr2 = currentState(obj);

			if (obj != p1)
				incrementErrorCount();

			if (curr2 != curr) {
				out.println("State changed in the returned instance ");
				out.println("Expected state: " + curr + " Actual state: " +
						curr2);
				incrementErrorCount();
			}

			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetObjectByIdWithValidationInstanceInCache()");
		}

	}

}
