/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get Object By Id With Validation, Instance in Cache, Not in Datastore
 *<BR>
 *<B>JavaTest Keywords:</B> identity cache
 *<BR>
 *<B>Assertion IDs:</B> A12.5.6-8.
 *<BR>
 *<B>Assertion Description: </B>
 If PersistenceManager.getObjectById is called with a value of true
 for the second parameter named validate, and there is an instance already in the cache with the
 same JDO identity as the oid parameter, and the instance is not transactional, and the instance
 does not exist in the datastore, then a JDODatastoreException is thrown.

 *@test
 *@title get properties
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectByIdWithValidationInstanceInCacheNotInDatastore
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetObjectByIdWithValidationInstanceInCacheNotInDatastore.java
 *@keywords identity
 */


public class GetObjectByIdWithValidationInstanceInCacheNotInDatastore extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm2;
	private Transaction tx;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		GetObjectByIdWithValidationInstanceInCacheNotInDatastore t = new GetObjectByIdWithValidationInstanceInCacheNotInDatastore();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();
		pm2 = pmf.getPersistenceManager();

		testGetObjectByIdWithValidationInstanceInCacheNotInDatastore();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectByIdWithValidationInstanceInCacheNotInDatastore tests ");
		} else {
			return Status.failed("FAILED in GetObjectByIdWithValidationInstanceInCacheNotInDatastore: number of errors detected is " + errCnt);
		}

	}

	/* passing null paramameter to getObjectId */
	public void testGetObjectByIdWithValidationInstanceInCacheNotInDatastore() {

		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			Object oid = pm.getObjectId(p1);
			pm.deletePersistent(p1);
			tx.commit();

			tx.begin();

			try {
				PCPoint p2 = (PCPoint) pm.getObjectById(oid, true);
			} catch (Exception e) {
				assertJDODataStoreException(e, "testGetObjectByIdWithValidationInstanceInCacheNotInDatastore()");
				tx.rollback();
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetObjectByIdWithValidationInstanceInCacheNotInDatastore()");
		}

	}

}
