/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Get Objectid
 *<BR>
 *<B>JavaTest Keywords:</B> identity
 *<BR>
 *<B>Assertion ID:</B> A12.5.6-10
 *<BR>
 *<B>Assertion Description: </B>
 The PersistenceManager.getObjectId method returns an ObjectId instance that re presents the object identity of the specified JDO Instance. Test: The method g etObjectById returns the exact same object, evaluating to true when == is used .

 *@test
 *@title get objectid
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectId
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetObjectId.java
 *@keywords identity
 */

public class GetObjectId extends PersistenceManagerTest {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;


	public static void main(String[] args) {

		GetObjectId t = new GetObjectId();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testGetObjectId();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectId tests ");
		} else {
			return Status.failed("FAILED in GetObjectId: number of errors detected is " + errCnt);
		}

	}


	public void testGetObjectId() {

		try {

			PCPoint p1 = new PCPoint(8, 8);

			Transaction tx = pm.currentTransaction();
			tx.begin();
			pm.makePersistent(p1);
			Object oid = pm.getObjectId(p1);
			tx.commit();

			Object obj = pm.getObjectById(oid, false);
			if (obj != p1) {
				incrementErrorCount();
				out.println(" Object mismatched");
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetObjectId()");
		}

	}


}
