/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get ObjectId Class For Abstract or Non-PersistenceCapable Class
 *<BR>
 *<B>JavaTest Keywords:</B> identity
 *<BR>
 *<B>Assertion IDs:</B> A12.10-2
 *<BR>
 *<B>Assertion Description: </B>
 A call to PersistenceManager.getObjectIdClass returns null if the class is abstract or not persistence-capable, or the parameter is null.
 *@test
 *@title get properties
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectIdClassForAbstractOrNonPersistenceCapableClass
 *@executeArgs -PMFProperties $PMFProperties

 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetObjectIdClassForAbstractOrNonPersistenceCapableClass.java
 *@keywords identity
 */


public class GetObjectIdClassForAbstractOrNonPersistenceCapableClass extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		GetObjectIdClassForAbstractOrNonPersistenceCapableClass t = new GetObjectIdClassForAbstractOrNonPersistenceCapableClass();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testGetObjectIdClassForAbstractOrNonPersistenceCapableClass();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectIdClassForAbstractOrNonPersistenceCapableClass tests ");
		} else {
			return Status.failed("FAILED in GetObjectIdClassForAbstractOrNonPersistenceCapableClass: number of errors detected is " + errCnt);
		}

	}

	public void testGetObjectIdClassForAbstractOrNonPersistenceCapableClass() {

		testNonPcClass();
		testNullParam();


	}


	public void testNonPcClass() {

		try {

			Class c1 = pm.getObjectIdClass(Point.class);

			if (c1 != null) {
				out.println(" class not returning null in testNonpcClass()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testNonPcClass()");
		}

	}


	public void testNullParam() {

		try {

			Class c1 = pm.getObjectIdClass(null);

			if (c1 != null) {
				out.println(" class not returning null in testNullParam()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testNullParam()");
		}

	}


}
