/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> GetObjectIdForNullOrNotPersistent
 *<BR>
 *<B>JavaTest Keywords:</B> identity
 *<BR>
 *<B>Assertion IDs:</B> A12.5.6-14.
 *<BR>
 *<B>Assertion Description: </B>
 If a call is made to PersistenceManager.getObjectId and the
 parameter pc is not persistent, or is null, then null is returned.
 *@test
 *@title GetObjectIdForNullOrNotPersistent
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectIdForNullOrNotPersistent
 *@executeArgs -PMFProperties $PMFProperties

 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetObjectIdForNullOrNotPersistent.java
 *@keywords identity
 */


public class GetObjectIdForNullOrNotPersistent extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		GetObjectIdForNullOrNotPersistent t = new GetObjectIdForNullOrNotPersistent();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testGetObjectIdForNullOrNotPersistent();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectIdForNullOrNotPersistent tests ");
		} else {
			return Status.failed("FAILED in GetObjectIdForNullOrNotPersistent: number of errors detected is " + errCnt);
		}

	}

	public void testGetObjectIdForNullOrNotPersistent() {

		testNonPcClass();
		testNullParam();


	}


	public void testNonPcClass() {

		try {

			Point p = new Point(1, 3);
			Object oid = pm.getObjectId(p);

			if (oid != null) {
				out.println(" oid not returning null in testNonpcClass()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testNonPcClass()");
		}

	}


	public void testNullParam() {

		try {

			Object oid = pm.getObjectId(null);

			if (oid != null) {
				out.println(" oid not returning null in testNullParam()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testNullParam()");
		}

	}


}
