/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import javax.jdo.*;

import com.sun.jdotck.models.mylib.PCPoint;

/**
 *<B>JavaTest Title:</B> Get ObjectId With Application Modifying Identity
 *<BR>
 *<B>JavaTest Keywords:</B> identity applicationidentity
 *<BR>
 *<B>Assertion ID:</B> A12.5.6-11.
 *<BR>
 *<B>Assertion Description: </B>
 If the object identity is being changed in the transaction,
 by the application modifying one or more of the application key fields,
 then the method <code>PersistenceManager.getObjectId</code> returns the
 identity as of the beginning of the transaction. The value returned by
 <code>getObjectId</code> will be different following <code>afterCompletion</code>
 processing for successful transactions.

 *@test
 *@title Get ObjectId With Application Modifying Identity
 *@executeClass com.sun.jdotck.api.persistencemanager.GetObjectIdWithApplicationModifyingIdentity
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source GetObjectIdWithApplicationModifyingIdentity.java
 *@keywords identity applicationidentity
 */

public class GetObjectIdWithApplicationModifyingIdentity extends PersistenceManagerTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		GetObjectIdWithApplicationModifyingIdentity t = new GetObjectIdWithApplicationModifyingIdentity();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>runTest</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		testGetObjectByIdWithApplicationModifyingIdentity();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectByIdWithApplicationModifyingIdentity tests ");
		} else {
			return Status.failed("FAILED in GetObjectByIdWithApplicationModifyingIdentity: number of errors detected is " + errCnt);
		}

	}

	public void testGetObjectByIdWithApplicationModifyingIdentity() {
		if (isChangeApplicationIdentitySupported()) {
			Object oid = createPCPointInstance();
			PCPoint p1 = (PCPoint) getPM().getObjectById(oid, false);
			Transaction tx = pm.currentTransaction();
			tx.begin();
			p1.setX(10000); // this is the primary key
			Object oid2 = pm.getObjectId(p1); // should be the same
			tx.commit();
			Object oid3 = pm.getObjectId(p1); // should be same as oid3
			if (!oid.equals(oid2)) {
				out.println("Oid before is different from oid after modifying pk");
				incrementErrorCount();
			}
			if (oid.equals(oid3)) {
				out.println("Oid before is the same as oid after modifying pk");
				incrementErrorCount();
			}
		}
	}
}
