/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Get/Set User Object
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion ID:</B> A12.8-1.
 *<BR>
 *<B>Assertion Description: </B>
 The PersistenceManager.setUserObject method is used to store an object associated with the PersistenceManager. One uses the method getUserObject to later
 retrieve the object.


 *@test
 *@title Get/Set User Object
 *@executeClass com.sun.jdotck.api.persistencemanager.GetSetUserObject
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetSetUserObject.java
 *@keywords
 */

public class GetSetUserObject extends PersistenceManagerTest {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;


	public static void main(String[] args) {

		GetSetUserObject t = new GetSetUserObject();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testGetSetUserObject();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetSetUserObject tests ");
		} else {
			return Status.failed("FAILED in GetSetUserObject: number of errors detected is " + errCnt);
		}

	}


	public void testGetSetUserObject() {

		try {

			tx = pm.currentTransaction();
			tx.begin();
			Point p = new Point(10, 10);
			pm.setUserObject(p);
			tx.commit();

			tx.begin();
			Object obj = pm.getUserObject();

			if (obj != p)
				incrementErrorCount();
			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetSetUserObject()");
		}

	}


}
