/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import com.sun.jdotck.models.mylib.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get Transactional ObjectId When ObjectId Being Changed
 *<BR>
 *<B>JavaTest Keywords:</B> identity applicationidentity
 *<BR>
 *<B>Assertion ID:</B> A12.5.6-15.
 *<BR>
 *<B>Assertion Description: </B>
 If the object identity is being changed in the transaction, by the application
 modifying one or more of the application key fields, then
 <code>PersistenceManager.getTransactionalObjectId</code> returns the current
 identity in the transaction.

 *@test
 *@title Get Transactional ObjectId When ObjectId Being Changed
 *@executeClass com.sun.jdotck.api.persistencemanager.GetTransactionalObjectIdWhenObjectIdBeingChanged
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source GetTransactionalObjectIdWhenObjectIdBeingChanged.java
 *@keywords identity applicationidentity
 */

public class GetTransactionalObjectIdWhenObjectIdBeingChanged extends PersistenceManagerTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		GetTransactionalObjectIdWhenObjectIdBeingChanged t = new GetTransactionalObjectIdWhenObjectIdBeingChanged();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>runTest</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		testGetObjectByIdWhenObjectIdBeingChanged();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetObjectByIdWhenObjectIdBeingChanged tests ");
		} else {
			return Status.failed("FAILED in GetObjectByIdWhenObjectIdBeingChanged: number of errors detected is " + errCnt);
		}

	}

	public void testGetObjectByIdWhenObjectIdBeingChanged() {
		if (isChangeApplicationIdentitySupported()) {
			Object oid = createPCPointInstance();
			PCPoint p1 = (PCPoint) getPM().getObjectById(oid, false);
			Transaction tx = pm.currentTransaction();
			tx.begin();
			p1.setX(10000); // this is the primary key
			Object oid2 = pm.getTransactionalObjectId(p1); // should be different
			tx.commit();
			Object oid3 = pm.getObjectId(p1); // should be same as oid2
			if (oid.equals(oid2)) {
				out.println("Oid before is the same as transactional oid after modifying pk");
				incrementErrorCount();
			}
			if (!oid2.equals(oid3)) {
				out.println("Oid after is different from transactional oid after modifying pk");
				incrementErrorCount();
			}
		}
	}
}
