/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Get Transactional ObjectId With No Transaction
 *<BR>
 *<B>JavaTest Keywords:</B> identity
 *<BR>
 *<B>Assertion ID:</B> A12.5.6-16
 *<BR>
 *<B>Assertion Description: </B>
 If there is no transaction in progress, or if none of the key fields is being
 modified, then PersistenceManager.getTransactionalObjectId has the same behavior as
 getObjectId.


 *@test
 *@title Get Transactional ObjectId With No Transaction
 *@executeClass com.sun.jdotck.api.persistencemanager.GetTransactionalObjectIdWithNoTransaction
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetTransactionalObjectIdWithNoTransaction.java
 *@keywords identity
 */

public class GetTransactionalObjectIdWithNoTransaction extends PersistenceManagerTest {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;


	public static void main(String[] args) {

		GetTransactionalObjectIdWithNoTransaction t = new GetTransactionalObjectIdWithNoTransaction();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testGetTransactionalObjectIdWithNoTransaction();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in GetTransactionalObjectIdWithNoTransaction tests ");
		} else {
			return Status.failed("FAILED in GetTransactionalObjectIdWithNoTransaction: number of errors detected is " + errCnt);
		}

	}


	public void testGetTransactionalObjectIdWithNoTransaction() {

		try {

			PCPoint p1 = new PCPoint();

			Transaction tx = pm.currentTransaction();
			tx.begin();
			pm.makePersistent(p1);
			Object oid = pm.getObjectId(p1);
			tx.commit();

			Object toid = pm.getTransactionalObjectId(p1);
			if (!toid.equals(oid)) {
				incrementErrorCount();
				out.println(" Object mismatched");
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testGetTransactionalObjectIdWithNoTransaction()");
		}

	}


}
