/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> MakeNontransactionalAllFails
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion ID:</B> A12.5.7-5
 *<BR>
 *<B>Assertion Description: </B>
 If a collection or array of instances is passed to PersistenceManager.makeNontransactionalAll, and one or more of the instances fail to complete the required operation, then all instances will be attempted, and a JDOUserException will be thrown which contains a nested exception array, each exception of which contains one of the failing instances. The succeeding instances will transition to the specified life cycle state, and the failing instances will remain in their current state.

 *@test
 *@title MakeNontransactionalAllFails
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeNontransactionalAllFails
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source MakeNontransactionalAllFails.java
 *@keywords transienttransactional lifecycle
 */

public class MakeNontransactionalAllFails extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	Point p5 = null;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeNontransactionalAllFails t = new MakeNontransactionalAllFails();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testTransactionalInst();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeNontransactionalAllFails tests ");
		} else {
			return Status.failed("FAILED in MakeNontransactionalAllFails: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInst() {
		createObjects();
		testMakeNontransactional1();
		testMakeNontransactionalAll1();
	}

	private void createObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	/* test makeNontransactional (Object pc) */
	public void testMakeNontransactional1() {

		try {
			tx = pm.currentTransaction();
			tx.begin();

			pm.makePersistent(p1);

			pm.makeNontransactional(p1);
			out.println(" FAILED in testMakeNontransactional1 - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeNontransactional1()");
			tx.rollback();

		}

	}


	/* test makeNontransactionalAll (Collection pcs) */
	public void testMakeNontransactionalAll1() {

		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.makePersistent(p2);
			tx.commit();

			tx.begin();
			pm.makePersistent(p3);
			pm.makePersistent(p4);

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);
			col1.add(p4);

			pm.makeNontransactionalAll(col1);
			out.println(" FAILED in testMakeNontransactionalAll1 - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "tMakeNontransactionalAll1()");
			tx.rollback();

		}

	}

	/* test makeNontransactionalAll (Object[] pcs) */
	public void testMakeNontransactionalAll2() {

		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.makePersistent(p2);
			tx.commit();

			tx.begin();
			pm.makePersistent(p3);
			pm.makePersistent(p4);

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);
			col1.add(p4);

			pm.makeNontransactionalAll(col1.toArray());
			out.println(" FAILED in testMakeNontransactionalAll2 - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeNontransactionalAll2()");
			tx.rollback();

		}

	}


}
