/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Make Nontransactional a Dirty Instance
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-28
 *<BR>
 *<B>Assertion Description: </B>
 If PersistenceManager.makeNontransactional or makeNontransactionalAll is called with an explicit dirty parameter instance, a JDOUserException is thrown.

 *@test
 *@title Make Nontransactional a Dirty Instance
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeNontransactionalDirtyInstance
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeNontransactionalDirtyInstance.java
 *@keywords exception
 */


public class MakeNontransactionalDirtyInstance extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;
	PCPoint p6 = null;
	PCPoint p7 = null;

	PCPoint c1 = null;
	PCPoint c2 = null;
	PCPoint c3 = null;
	PCPoint c4 = null;
	PCPoint c5 = null;
	PCPoint c6 = null;
	PCPoint c7 = null;

	Collection pcol1 = new java.util.HashSet();
	Collection pcol2 = new java.util.HashSet();
	Collection ccol1 = new java.util.HashSet();
	Collection ccol2 = new java.util.HashSet();

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeNontransactionalDirtyInstance t = new MakeNontransactionalDirtyInstance();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakeNontransactional();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeNontransactionalDirtyInstance tests ");
		} else {
			return Status.failed("FAILED in MakeNontransactionalDirtyInstance: number of errors detected is " + errCnt);
		}

	}

	public void testMakeNontransactional() {


		/*  call with an explicit persistent dirty parameter instance */
		String pstate = "persistent";
		createPersistentObjects();
		out.println(" -- testing with persistent dirty instance -- ");
		testMakeNontransactionalDirtyInstance(p1, pstate);
		testMakeNontransactionalDirtyInstanceAll1(pcol1, pstate);
		testMakeNontransactionalDirtyInstanceAll2(pcol2, pstate);

		/*  call with an explicit transient dirty parameter instance */
		pstate = "transient";
		createTcleanObjects();
		out.println(" -- testing with transient dirty instance -- ");
		testMakeNontransactionalDirtyInstance(c1, pstate);
		testMakeNontransactionalDirtyInstanceAll1(ccol1, pstate);
		testMakeNontransactionalDirtyInstanceAll2(ccol2, pstate);

	}


	private void createTcleanObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			c1 = new PCPoint(1, 3);
			c2 = new PCPoint(2, 4);
			c3 = new PCPoint(3, 5);
			c4 = new PCPoint(4, 6);
			c5 = new PCPoint(5, 7);
			c6 = new PCPoint(6, 8);
			c7 = new PCPoint(7, 9);

			ccol1.add(c2);
			ccol1.add(c3);
			ccol1.add(c4);

			ccol2.add(c5);
			ccol2.add(c6);
			ccol2.add(c7);

			pm.makeTransactional(c1);
			pm.makeTransactional(c2);
			pm.makeTransactional(c3);
			pm.makeTransactional(c4);
			pm.makeTransactional(c5);
			pm.makeTransactional(c6);
			pm.makeTransactional(c7);
			tx.commit();
		} catch (Exception ex) {
			out.println(" - createTcleanObjects() FAILED");
		}
	}


	private void createPersistentObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);
			p6 = new PCPoint(6, 8);
			p7 = new PCPoint(7, 9);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);
			pm.makePersistent(p6);
			pm.makePersistent(p7);

			pcol1.add(p2);
			pcol1.add(p3);
			pcol1.add(p4);

			pcol2.add(p5);
			pcol2.add(p6);
			pcol2.add(p7);
			tx.commit();

		} catch (Exception ex) {
			out.println(" - createPersistentObjects() FAILED");
		}
	}

	public void testMakeNontransactionalDirtyInstance(PCPoint obj, String state) {

		try {

			out.println(" ** in testMakeNontransactionalDirtyInstance()");
			int curr;

			tx = pm.currentTransaction();
			tx.begin();

			if (makeAndTestDirtyInstance(obj, state)) {
				pm.makeNontransactional(obj);

				incrementErrorCount();
			}
			tx.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeNontransactionalDirtyInstance()");
			tx.rollback();
		}

	}


	public void testMakeNontransactionalDirtyInstanceAll1(Collection tcol, String state) {

		boolean stopFlag = false;

		try {
			out.println("");
			out.println(" ** in testMakeNontransactionalDirtyInstanceAll1()");
			int curr;
			tx = pm.currentTransaction();
			tx.begin();

			Iterator iter = tcol.iterator();
			while (iter.hasNext()) {
				PCPoint p = (PCPoint) iter.next();
				if (makeAndTestDirtyInstance(p, state))
					stopFlag = true;
			}

			if (!stopFlag) {
				pm.makeNontransactionalAll(tcol);
				incrementErrorCount();
			}
			tx.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeNontransactionalDirtyInstanceAll1()");
			tx.rollback();
		}

	}

	public void testMakeNontransactionalDirtyInstanceAll2(Collection tcol, String state) {

		boolean stopFlag = false;
		try {
			out.println("");
			out.println(" ** in testMakeNontransactionalDirtyInstanceAll2()");
			tx = pm.currentTransaction();
			tx.begin();

			Iterator iter = tcol.iterator();
			while (iter.hasNext()) {
				PCPoint p = (PCPoint) iter.next();
				if (makeAndTestDirtyInstance(p, state))
					stopFlag = true;
			}

			Object[] objArray = tcol.toArray();

			if (!stopFlag) {
				pm.makeNontransactionalAll(objArray);
				incrementErrorCount();

			}
			tx.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeNontransactionalDirtyInstanceAll2()");

			tx.rollback();
		}

	}

	public boolean makeAndTestDirtyInstance(PCPoint obj, String state) {

		int i = obj.getX();
		i = i * 10;

		obj.setX(i);
		int curr = currentState(obj);

		if (state.equals("persistent")) {
			if (curr != PERSISTENT_DIRTY) {
				out.println(" Object not in persistent dirty state " +
						" for " + obj.getX() + " current state: " + curr);
				incrementErrorCount();
				return false;
			}
		} else {	// state = transient
			if (state.equals("transient")) {
				if (curr != TRANSIENT_DIRTY) {
					out.println(" Object not in transient dirty state " +
							" for " + obj.getX() + " current state: " + curr);
					incrementErrorCount();
					return false;
				}
			}
		}
		return true;
	}


}
