/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Make Nontransactional is immediate
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-29
 *<BR>
 *<B>Assertion Description: </B>
 The effect of PersistenceManager.makeNontransactional or
 makeNontransactionalAll is immediate and not subject to rollback.
 *@test
 *@title Make Nontransactional is immediate
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeNontransactionalIsImmediate
 *@executeArgs -PMFProperties $PMFProperties

 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeNontransactionalIsImmediate.java
 *@keywords
 */


public class MakeNontransactionalIsImmediate extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;
	PCPoint p6 = null;
	PCPoint p7 = null;

	Collection col1 = new java.util.HashSet();
	Collection col2 = new java.util.HashSet();

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeNontransactionalIsImmediate t = new MakeNontransactionalIsImmediate();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testTransactionalInst();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeNontransactionalIsImmediate tests ");
		} else {
			return Status.failed("FAILED in MakeNontransactionalIsImmediate: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInst() {
		createTcleanObjects();
		testMakeNontransactionalIsImmediate();
		testMakeNontransactionalIsImmediateAll1();
		testMakeNontransactionalIsImmediateAll2();
	}

	private void createTcleanObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);
			p6 = new PCPoint(6, 8);
			p7 = new PCPoint(7, 9);

			col1.add(p2);
			col1.add(p3);
			col1.add(p4);

			col2.add(p5);
			col2.add(p6);
			col2.add(p7);

			pm.makeTransactional(p1);
			pm.makeTransactional(p2);
			pm.makeTransactional(p3);
			pm.makeTransactional(p4);
			pm.makeTransactional(p5);
			pm.makeTransactional(p6);
			pm.makeTransactional(p7);
			tx.commit();
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	public void testMakeNontransactionalIsImmediate() {

		try {

			out.println(" ** in testMakeNontransactionalIsImmediate()");
			int curr;

			tx = pm.currentTransaction();
			tx.begin();


			if (testState(p1, TRANSIENT_CLEAN, "transient clean")) {
				pm.makeNontransactional(p1);
				if (!testState(p1, TRANSIENT, "transient"))
					incrementErrorCount();

			}
			tx.rollback();
			if (!testState(p1, TRANSIENT, "transient")) {
				out.println(" FAILED after rollback");
				incrementErrorCount();
			}


		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeNontransactionalIsImmediate()");
		}

	}


	public void testMakeNontransactionalIsImmediateAll1() {

		try {
			out.println(" ** in testMakeNontransactionalIsImmediateAll1()");
			int curr;
			tx = pm.currentTransaction();
			tx.begin();

			if (testState(p2, TRANSIENT_CLEAN, "transient clean") &&
					testState(p3, TRANSIENT_CLEAN, "transient clean") &&
					testState(p4, TRANSIENT_CLEAN, "transient clean")) {

				pm.makeNontransactionalAll(col1);
				if (!navigateAndTestTransientCol(col1))
					incrementErrorCount();

			}
			tx.rollback();

			tx.begin();
			if (!navigateAndTestTransientCol(col1)) {
				out.println(" FAILED after rollback");
				incrementErrorCount();
			}
			tx.rollback();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeNontransactionalIsImmediateAll1()");
		}

	}

	public void testMakeNontransactionalIsImmediateAll2() {

		try {
			out.println(" ** in testMakeNontransactionalIsImmediateAll2()");
			int curr;
			tx = pm.currentTransaction();
			tx.begin();

			Object[] objArray = col2.toArray();

			if (testState(p5, TRANSIENT_CLEAN, "transient clean") &&
					testState(p6, TRANSIENT_CLEAN, "transient clean") &&
					testState(p7, TRANSIENT_CLEAN, "transient clean")) {

				pm.makeNontransactionalAll(objArray);

				if (!navigateAndTestTransientArray(objArray)) {
					out.println(" FAILED before rollback");
					incrementErrorCount();
				}

			}
			tx.rollback();

			tx.begin();
			if (!navigateAndTestTransientArray(objArray)) {
				out.println(" FAILED after rollback");
				incrementErrorCount();
			}

			tx.rollback();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeNontransactionalIsImmediateAll2()");
		}

	}

	public boolean navigateAndTestTransientCol(Collection col) {

		boolean noError = true;
		Iterator iter = col1.iterator();
		while (iter.hasNext()) {
			PCPoint p = (PCPoint) iter.next();
			if (!testState(p, TRANSIENT, "transient")) {
				incrementErrorCount();
				noError = false;
			}
		}
		return noError;

	}

	public boolean navigateAndTestTransientArray(Object[] objArray) {

		boolean noError = true;
		for (int i = 0; i < objArray.length; i++) {
			PCPoint p = (PCPoint) objArray[i];
			if (!testState(p, TRANSIENT, "transient")) {
				incrementErrorCount();
				noError = false;
			}
		}
		return noError;


	}


}
