/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Make Nontransactional a Persistent-Clean Instance
 *<BR>
 *<B>JavaTest Keywords:</B> lifecycle
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-27
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.makeNontransactional and makeNontransactionalAll makes a persistent-clean instance nontransactional and causes a state transition to persistent-nontransactional.
 *@test
 *@title get properties
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeNontransactionalPersistentCleanInstance
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source MakeNontransactionalPersistentCleanInstance.java
 *@keywords lifecycle
 */


public class MakeNontransactionalPersistentCleanInstance extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;
	PCPoint p6 = null;
	PCPoint p7 = null;

	Collection col1 = new java.util.HashSet();
	Collection col2 = new java.util.HashSet();

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeNontransactionalPersistentCleanInstance t = new MakeNontransactionalPersistentCleanInstance();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testTransactionalInstance();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeNontransactionalPersistentCleanInstance tests ");
		} else {
			return Status.failed("FAILED in MakeNontransactionalPersistentCleanInstance: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInstance() {
		createObjects();
		testMakeNontransactionalPersistentCleanInstance();
		testMakeNontransactionalPersistentCleanInstanceAll1();
		testMakeNontransactionalPersistentCleanInstanceAll2();
	}

	private void createObjects() {
		try {
			tx = pm.currentTransaction();
			tx.setOptimistic(false);
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);
			p6 = new PCPoint(6, 8);
			p7 = new PCPoint(7, 9);

			col1.add(p2);
			col1.add(p3);
			col1.add(p4);

			col2.add(p5);
			col2.add(p6);
			col2.add(p7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);
			pm.makePersistent(p6);
			pm.makePersistent(p7);
			tx.commit();
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	public void testMakeNontransactionalPersistentCleanInstance() {

		try {

			//out.println (" ** in testMakeNontransactionalPersistentCleanInstance()");
			int curr;

			tx = pm.currentTransaction();
			tx.setOptimistic(false);
			tx.begin();

			if (makePersistentCleanInstance(p1)) {
				pm.makeNontransactional(p1);

				curr = currentState(p1);

				if (curr == HOLLOW ||
						curr == PERSISTENT_NONTRANSACTIONAL)
					;
				else
					incrementErrorCount();
			}
			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeNontransactionalPersistentCleanInstance()");
		}

	}


	public void testMakeNontransactionalPersistentCleanInstanceAll1() {

		try {
			out.println(" ** in testMakeNontransactionalPersistentCleanInstanceAll1()");
			int curr;
			tx = pm.currentTransaction();

			tx.setOptimistic(false);
			tx.begin();

			if (makePersistentCleanInstance(p2) &&
					makePersistentCleanInstance(p3) &&
					makePersistentCleanInstance(p4)) {

				pm.makeNontransactionalAll(col1);
				Iterator iter = col1.iterator();
				while (iter.hasNext()) {
					PCPoint p = (PCPoint) iter.next();
					curr = currentState(p);
					if (curr == HOLLOW ||
							curr == PERSISTENT_NONTRANSACTIONAL)
						;
					else
						incrementErrorCount();
				}
			}

			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeNontransactionalPersistentCleanInstanceAll1()");
		}

	}

	public void testMakeNontransactionalPersistentCleanInstanceAll2() {

		try {
			out.println(" ** in testMakeNontransactionalPersistentCleanInstanceAll2()");
			int curr;
			tx = pm.currentTransaction();
			if (tx.getOptimistic())
				out.println(" opt flag set to true");
			tx.begin();

			Object[] objArray = col2.toArray();

			if (makePersistentCleanInstance(p5) &&
					makePersistentCleanInstance(p6) &&
					makePersistentCleanInstance(p7)) {

				pm.makeNontransactionalAll(objArray);

				for (int i = 0; i < objArray.length; i++) {
					PCPoint p = (PCPoint) objArray[i];
					curr = currentState(p);
					if (curr == HOLLOW ||
							curr == PERSISTENT_NONTRANSACTIONAL)
						;
					else
						incrementErrorCount();

				}
			}
			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeNontransactionalPersistentCleanInstanceAll2()");
		}

	}

	public boolean makePersistentCleanInstance(PCPoint obj) {

		int val = obj.getX();

		if (testState(obj, PERSISTENT_CLEAN, "persistent clean"))
			return true;
		else
			return false;

	}


}
