/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> MakeNontransactionalTransientCleanInstance
 *<BR>
 *<B>JavaTest Keywords:</B> lifecycle
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-26
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.makeNontransactional and makeNontransactionalAll makes a transient-clean instance nontransactional and causes a state transition to transient. After the method completes, the instance does not observe transaction boundaries.
 *@test
 *@title get properties
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeNontransactionalTransientCleanInstance
 *@executeArgs -PMFProperties $PMFProperties

 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeNontransactionalTransientCleanInstance.java
 *@keywords lifecycle
 */


public class MakeNontransactionalTransientCleanInstance extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;
	PCPoint p6 = null;
	PCPoint p7 = null;

	Collection col1 = new java.util.HashSet();
	Collection col2 = new java.util.HashSet();

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeNontransactionalTransientCleanInstance t = new MakeNontransactionalTransientCleanInstance();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testTransactionalInst();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeNontransactionalTransientCleanInstance tests ");
		} else {
			return Status.failed("FAILED in MakeNontransactionalTransientCleanInstance: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInst() {
		createTcleanObjects();
		testMakeNontransactionalTransientCleanInstance();
		testMakeNontransactionalTransientCleanInstanceAll1();
		testMakeNontransactionalTransientCleanInstanceAll2();
	}

	private void createTcleanObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);
			p6 = new PCPoint(6, 8);
			p7 = new PCPoint(7, 9);

			col1.add(p2);
			col1.add(p3);
			col1.add(p4);

			col2.add(p5);
			col2.add(p6);
			col2.add(p7);

			pm.makeTransactional(p1);
			pm.makeTransactional(p2);
			pm.makeTransactional(p3);
			pm.makeTransactional(p4);
			pm.makeTransactional(p5);
			pm.makeTransactional(p6);
			pm.makeTransactional(p7);
			tx.commit();
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	public void testMakeNontransactionalTransientCleanInstance() {

		try {

			out.println(" ** in testMakeNontransactionalTransientCleanInstance()");
			int curr;

			tx = pm.currentTransaction();
			tx.begin();

			if (testState(p1, TRANSIENT_CLEAN, "transient clean")) {

				pm.makeNontransactional(p1);

				if (testState(p1, TRANSIENT, "transient"))
					; // expected result
				else
					incrementErrorCount();

			}
			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeNontransactionalTransientCleanInstance()");
		}

	}


	public void testMakeNontransactionalTransientCleanInstanceAll1() {

		try {
			out.println(" ** in testMakeNontransactionalTransientCleanInstanceAll1()");
			int curr;
			tx = pm.currentTransaction();
			tx.begin();

			if (testState(p2, TRANSIENT_CLEAN, "transient clean") &&
					testState(p3, TRANSIENT_CLEAN, "transient clean") &&
					testState(p4, TRANSIENT_CLEAN, "transient clean")) {

				pm.makeNontransactionalAll(col1);
				Iterator iter = col1.iterator();
				while (iter.hasNext()) {
					PCPoint p = (PCPoint) iter.next();
					if (testState(p, TRANSIENT, "transient"))
						; // expected result
					else
						incrementErrorCount();
				}
			}
			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeNontransactionalTransientCleanInstanceAll1()");
		}

	}

	public void testMakeNontransactionalTransientCleanInstanceAll2() {

		try {
			out.println(" ** in testMakeNontransactionalTransientCleanInstanceAll2()");
			int curr;
			tx = pm.currentTransaction();
			tx.begin();

			Object[] objArray = col2.toArray();

			if (testState(p5, TRANSIENT_CLEAN, "transient clean") &&
					testState(p6, TRANSIENT_CLEAN, "transient clean") &&
					testState(p7, TRANSIENT_CLEAN, "transient clean")) {

				pm.makeNontransactionalAll(objArray);

				for (int i = 0; i < objArray.length; i++) {
					PCPoint p = (PCPoint) objArray[i];
					if (testState(p, TRANSIENT, "transient"))
						; // expected result
					else
						incrementErrorCount();
				}
			}
			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeNontransactionalTransientCleanInstanceAll2()");
		}

	}

}
