/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> make instance persistent
 *<BR>
 *<B>JavaTest Keywords:</B> persistencemanager
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-6A
 *<BR>
 *<B>Assertion Description: </B>
 The method <code>PersistenceManager.makePersistent</code> and
 <code>makePersistentAll</code> makes a transient instance(s) persistent directly.
 It must be called in the context of an active transaction,
 or a <code>JDOUserException</code> is thrown.

 *@test
 *@title make instance persistent
 *@executeClass com.sun.jdotck.api.persistencemanager.MakePersistent
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakePersistent.java
 *@keywords persistencemanager
 */

public class MakePersistent extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakePersistent t = new MakePersistent();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakePersistent();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakePersistent tests passed");
		} else {
			return Status.failed("MakePersistent: number of errors detected is " + errCnt);
		}

	}

	public void testMakePersistent() {
		createObjects();

		/* positive tests */
		testMakePersistent1();
		testMakePersistent2();
		testMakePersistent3();

		/* negative tests */
		testMakePersistent4();
		testMakePersistent5();
		testMakePersistent6();

	}


	private void createObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}


	/* test makePersistent (Object pc) */
	public void testMakePersistent1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			pm.makePersistent(p1);
			tx.commit();
			out.println(" \nPASSED in testMakePersistent1()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakePersistent1()");
		}

	}


	/* test makePersistentAll (Collection pcs) */
	public void testMakePersistent2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			pm.makePersistentAll(col1);
			tx.commit();
			out.println(" \nPASSED in testMakePersistent2()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakePersistent2()");
		}

	}

	/* test makePersistentAll (Object[] o) */
	public void testMakePersistent3() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p1);
			col1.add(p2);

			Object[] obj1 = col1.toArray();

			pm.makePersistentAll(obj1);
			tx.commit();
			out.println(" \nPASSED in testMakePersistent3()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakePersistent3()");
		}

	}


	/* test makePersistent (Object pc) */
	public void testMakePersistent4() {
		try {

			PCPoint np1 = new PCPoint(1, 3);
			pm.makePersistent(np1);
			out.println(" FAILED in testMakePersistent4() - should not go that further ");
			incrementErrorCount();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakePersistent4()");
		}
	}


	/* test makePersistentAll (Collection pcs) */
	public void testMakePersistent5() {
		try {

			tx = pm.currentTransaction();
			if (tx.isActive())
				out.println(" tx is active ");
			else
				out.println(" tx is not active ");


			PCPoint np1 = new PCPoint(3, 3);
			PCPoint np2 = new PCPoint(4, 4);

			Collection col1 = new java.util.HashSet();
			col1.add(np1);
			col1.add(np2);

			pm.makePersistentAll(col1);
			out.println(" FAILED in testMakePersistent5() - should not go that further ");
			incrementErrorCount();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakePersistent5()");
		}
	}

	/* test makePersistentAll (Object[] o) */
	public void testMakePersistent6() {
		try {
			tx = pm.currentTransaction();
			if (tx.isActive())
				out.println(" tx is active ");
			else
				out.println(" tx is not active ");

			PCPoint np1 = new PCPoint(3, 3);
			PCPoint np2 = new PCPoint(4, 4);

			Collection col1 = new java.util.HashSet();
			col1.add(np1);
			col1.add(np2);

			Object[] obj1 = col1.toArray();

			pm.makePersistentAll(obj1);
			out.println(" FAILED in testMakePersistent6() - should not go that further ");
			incrementErrorCount();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakePersistent6()");
		}

	}

}
