/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> MakePersistentAll Fails
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-1.
 *<BR>
 *<B>Assertion Description: </B>
 If a collection or array of instances is passed to PersistenceManager.makePersistentAll, and one or more of the instances fail to complete the operation, then all instances will be attempted, and a JDOUserException will be thrown which contains a nested exception array, each exception of which contains one of the failing instances. The succeeding instances will transition to the specified life cycle state, and the failing instances will remain in their current state.
 *@test
 *@title MakePersistentAll Fails
 *@executeClass com.sun.jdotck.api.persistencemanager.MakePersistentAllFails
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakePersistentAllFails.java
 *@keywords exception
 */

public class MakePersistentAllFails extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakePersistentAllFails t = new MakePersistentAllFails();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakePersistentAllFails();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakePersistentAllFails tests passed");
		} else {
			return Status.failed("MakePersistentAllFails: number of errors detected is " + errCnt);
		}

	}

	public void testMakePersistentAllFails() {

		testMakePersistentAllFails1();
		testMakePersistentAllFails2();

	}


	/* test makePersistentAll (Collection pcs) */
	public void testMakePersistentAllFails1() {
		try {

			tx = pm.currentTransaction();
			tx.begin();

			PCPoint np1 = new PCPoint(3, 3);
			PCPoint np2 = new PCPoint(4, 4);
			Point np3 = new Point(5, 5);

			Collection col1 = new java.util.HashSet();
			col1.add(np1);
			col1.add(np2);
			col1.add(np3);

			pm.makePersistentAll(col1);
			out.println(" FAILED in testMakePersistentAllFails1() - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakePersistentAllFails1()");
			tx.rollback();
		}
	}

	/* test makePersistentAll (Object[] o) */
	public void testMakePersistentAllFails2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint np1 = new PCPoint(3, 3);
			PCPoint np2 = new PCPoint(4, 4);
			Point np3 = new Point(5, 5);

			Collection col1 = new java.util.HashSet();
			col1.add(np1);
			col1.add(np2);
			col1.add(np3);

			Object[] obj1 = col1.toArray();

			pm.makePersistentAll(obj1);
			out.println(" FAILED in testMakePersistentAllFails2() - should not go that further ");
			incrementErrorCount();
			tx.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakePersistentAllFails2()");
			tx.rollback();
		}

	}

}
