/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> make instance persistent
 *<BR>
 *<B>JavaTest Keywords:</B> identity
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-6B
 *<BR>
 *<B>Assertion Description: </B>
 The method PersistenceManager.makePersistent and
 makePersistentAll will assign an object identity to the instance
 and transitions it to persistent-new.

 *@test
 *@title make instance persistent
 *@executeClass com.sun.jdotck.api.persistencemanager.MakePersistentAssignsObjectId
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakePersistentAssignsObjectId.java
 *@keywords identity
 */

public class MakePersistentAssignsObjectId extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakePersistentAssignsObjectId t = new MakePersistentAssignsObjectId();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakePersistentAssignsObjectId();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakePersistentAssignsObjectId tests passed");
		} else {
			return Status.failed("MakePersistentAssignsObjectId: number of errors detected is " + errCnt);
		}

	}

	public void testMakePersistentAssignsObjectId() {
		createObjects();

		/* positive tests */
		testMakePersistentAssignsObjectId1();
		testMakePersistentAssignsObjectId2();
		testMakePersistentAssignsObjectId3();

	}


	private void createObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}


	/* test makePersistent (Object pc) */
	public void testMakePersistentAssignsObjectId1() {
		try {
			Transaction tx = pm.currentTransaction();
			tx.begin();

			pm.makePersistent(p1);

			Object oid = pm.getObjectId(p1);

			if (testObjectId(p1) &&
					testState(p1, PERSISTENT_NEW, "persistent_new")) {
				out.println(" \nPASSED in testMakePersistentAssignsObjectId1()");
			} else {
				out.println(" \nFAILED in testMakePersistentAssignsObjectId1()");
				incrementErrorCount();
			}

			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakePersistentAssignsObjectId1()");
		}

	}


	/* test makePersistentAll (Collection pcs) */
	public void testMakePersistentAssignsObjectId2() {
		try {
			Transaction tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			pm.makePersistentAll(col1);
			if (testCollection(col1)) {
				out.println(" \nPASSED in testMakePersistentAssignsObjectId2()");
			} else {
				out.println(" \nFAILED in testMakePersistentAssignsObjectId2()");
				incrementErrorCount();
			}
			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakePersistentAssignsObjectId2()");
		}

	}

	/* test makePersistentAll (Object[] o) */
	public void testMakePersistentAssignsObjectId3() {
		try {

			int NUM_OBJS = 2;
			Transaction tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			Object[] obj1 = col1.toArray();

			pm.makePersistentAll(obj1);
			if (testArray(obj1, NUM_OBJS)) {
				out.println(" \nPASSED in testMakePersistentAssignsObjectId3()");
			} else {
				out.println(" \nFAILED in testMakePersistentAssignsObjectId3()");
				incrementErrorCount();
			}
			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakePersistentAssignsObjectId3()");
		}

	}


	public boolean testObjectId(PCPoint obj) {
		try {

			Object oid = pm.getObjectId(obj);
			if (oid != null) {
				return true;
			} else {
				return false;
			}
		} catch (Exception ex) {
			out.println("Unexpected exception at testObjectid()");
			return false;
		}

	}

	public boolean testCollection(Collection mycol) {

		int flag = 0;
		Iterator iter = mycol.iterator();
		while (iter.hasNext()) {
			PCPoint p = (PCPoint) iter.next();
			if (testObjectId(p) &&
					testState(p, PERSISTENT_NEW, "persistent_new")) {
				;  // expected result
			} else {
				flag++;
			}
		}

		if (flag == 0)
			return true;
		else
			return false;
	}


	public boolean testArray(Object[] arrayObj, int num) {

		int flag = 0;
		int i = 0;
		for (i = 0; i < num; ++i) {
			PCPoint p = (PCPoint) arrayObj[i];
			if (testObjectId(p) &&
					testState(p, PERSISTENT_NEW, "persistent_new")) {
				;  // expected result
			} else {
				flag++;
			}
		}

		if (flag == 0) {
			return true;
		} else {
			return false;
		}
	}

}
