/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> MakePersistent Has No Effect On Persistent Instances
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-7.
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.makePersistent and makePersistentAll have no effect on parameter persistent instances already managed by the same PersistenceManager.

 *@test
 *@title MakePersistent Has No Effect On Persistent Instances
 *@executeClass com.sun.jdotck.api.persistencemanager.MakePersistentHasNoEffectOnPersistentInstances
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakePersistentHasNoEffectOnPersistentInstances.java
 *@keywords
 */

public class MakePersistentHasNoEffectOnPersistentInstances extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakePersistentHasNoEffectOnPersistentInstances t = new MakePersistentHasNoEffectOnPersistentInstances();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakePersistentHasNoEffectOnPersistentInstances();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakePersistentHasNoEffectOnPersistentInstances tests passed");
		} else {
			return Status.failed("MakePersistentHasNoEffectOnPersistentInstances: number of errors detected is " + errCnt);
		}

	}

	public void testMakePersistentHasNoEffectOnPersistentInstances() {
		createObjects();

		testMakePersistent();
		testMakePersistentAll1();
		testMakePersistentAll2();

	}


	private void createObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			tx.commit();

		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}


	/* test makePersistent (Object pc) */
	public void testMakePersistent() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.makePersistent(p1);
			pm.makePersistent(p1);
			pm.makePersistent(p1);
			pm.makePersistent(p1);
			tx.commit();
			out.println(" \nPASSED in testMakePersistent()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakePersistent()");
		}

	}


	/* test makePersistentAll (Collection pcs) */
	public void testMakePersistentAll1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			pm.makePersistentAll(col1);
			pm.makePersistentAll(col1);
			pm.makePersistentAll(col1);
			pm.makePersistentAll(col1);
			tx.commit();
			out.println(" \nPASSED in testMakePersistentAll1()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakePersistentAll1()");
		}

	}

	/* test makePersistentAll (Object[] o) */
	public void testMakePersistentAll2() {
		try {
			Transaction tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p1);
			col1.add(p2);

			Object[] obj1 = col1.toArray();

			pm.makePersistentAll(obj1);
			pm.makePersistentAll(obj1);
			pm.makePersistentAll(obj1);
			pm.makePersistentAll(obj1);
			tx.commit();
			out.println(" \nPASSED in testMakePersistentAll2()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakePersistentAll2()");
		}

	}

}
