/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> MakeTransactional
 *<BR>
 *<B>JavaTest Keywords:</B> transienttransactional lifecycle
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-20
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.makeTransactional and makeTransactionalAll makes a transient instance transactional and causes a state transition to transient-clean. After the method completes, the instance observes transaction boundaries.

 *@test
 *@title MakeTransactional
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransactional
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransactional.java
 *@keywords transienttransactional lifecycle
 */


public class MakeTransactional extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p = null;
	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransactional t = new MakeTransactional();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testTransactionalInst();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeTransactional tests ");
		} else {
			return Status.failed("FAILED in MakeTransactional: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInst() {
		createObjects();
		testMakeTransactional();
		testMakeTransactionalAll1();
		testMakeTransactionalAll2();
	}

	private void createObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	public void testMakeTransactional() {

		try {
			tx = pm.currentTransaction();
			tx.begin();

			pm.makeTransactional(p1);
			if (currentState(p1) != TRANSIENT_CLEAN) {
				out.println(" State is not transient clean ");
				incrementErrorCount();
			}

			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactional()");
		}

	}


	public void testMakeTransactionalAll1() {

		int curr;

		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p1);
			col1.add(p2);
			col1.add(p3);
			col1.add(p4);

			pm.makeTransactionalAll(col1);
			Iterator iter = col1.iterator();
			while (iter.hasNext()) {
				PCPoint p = (PCPoint) iter.next();
				curr = currentState(p);
				if (curr != TRANSIENT_CLEAN) {
					out.println(" State is not transient clean " +
							" for x = " + p.getX());
					incrementErrorCount();
				}
			}

			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalAll1()");
		}

	}

	public void testMakeTransactionalAll2() {

		try {

			tx = pm.currentTransaction();
			tx.begin();

			Object[] objArray = {p1, p2, p3, p4};

			pm.makeTransactionalAll(objArray);

			for (int i = 0; i < objArray.length; i++) {
				p = (PCPoint) objArray[i];
				if (currentState(p) != TRANSIENT_CLEAN) {
					out.println(" State is not transient clean " +
							" for x = " + p.getX());
					incrementErrorCount();
				}
			}

			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalAll2()");
		}

	}


}
