/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Make Transactional A Persistent Nontransactional Instance
 *<BR>
 *<B>JavaTest Keywords:</B> persistentnontransactional
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-24
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.makeTransactional and makeTransactionalAll can be used to mark a nontransactional persistent instance as being part of the read-consistency set of the transaction. In this case, the call must be made in the context of an active transaction, or a JDOUserException is thrown. To test this, get an instance in the persistence-nontransactional state, make it transactional, then in a
 different transaction commit a change to the instance. An exception should then be thrown when the first transaction commits.

 *@test
 *@title Make Transactional A Persistent Nontransactional Instance
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransactionalANontransactionalPersistentInstance
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransactionalANontransactionalPersistentInstance.java
 *@keywords persistentnontransactional
 */

public class MakeTransactionalANontransactionalPersistentInstance extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;
	PCPoint p6 = null;
	PCPoint p7 = null;

	Collection col1 = new java.util.HashSet();
	Collection col2 = new java.util.HashSet();

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransactionalANontransactionalPersistentInstance t = new MakeTransactionalANontransactionalPersistentInstance();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testTransactionalInstance();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeTransactionalANontransactionalPersistentInstance tests ");
		} else {
			return Status.failed("FAILED in MakeTransactionalANontransactionalPersistentInstance: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInstance() {
		createPNonTranObjects();

		/* Positive cases */
		testMakeTransactional();
		testMakeTransactionalAll1();
		testMakeTransactionalAll2();

		/* Negative cases */
		testMakeTransactionalNeg();
		testMakeTransactionalAll1Neg();
		testMakeTransactionalAll2Neg();
	}

	private void createPNonTranObjects() {
		try {
			tx = pm.currentTransaction();
			tx.setOptimistic(false);
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);
			p6 = new PCPoint(6, 8);
			p7 = new PCPoint(7, 9);

			col1.add(p2);
			col1.add(p3);
			col1.add(p4);

			col2.add(p5);
			col2.add(p6);
			col2.add(p7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);
			pm.makePersistent(p6);
			pm.makePersistent(p7);
			tx.commit();
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	public void testMakeTransactional() {

		try {

			out.println(" ** in testMakeTransactional()");
			int curr;

			tx = pm.currentTransaction();
			tx.setOptimistic(true);
			tx.begin();

			if (makePersistentNonTranInstance(p1)) {
				pm.makeTransactional(p1);
			}
			tx.commit();
			out.println(" \nPASSED in testMakeTransactional()");

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactional()");
		}

	}


	public void testMakeTransactionalNeg() {

		try {

			out.println(" ** in testMakeTransactionalNeg()");
			int curr;

			tx = pm.currentTransaction();
			tx.setOptimistic(true);
			tx.begin();

			if (makePersistentNonTranInstance(p1)) {
				; // expected result
			}
			tx.commit();

			try {
				pm.makeTransactional(p1);
				out.println(" FAILED - Should not go that further ");
				incrementErrorCount();

			} catch (Exception tex) {
				assertJDOUserException(tex, "testMakeTransactionalNeg()");
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalNeg()");
		}

	}


	public void testMakeTransactionalAll1() {

		try {
			out.println(" ** in testMakeTransactionalAll1()");
			int curr;
			tx = pm.currentTransaction();
			tx.setOptimistic(true);
			tx.begin();

			if (makePersistentNonTranInstance(p2) &&
					makePersistentNonTranInstance(p3) &&
					makePersistentNonTranInstance(p4)) {

				pm.makeTransactionalAll(col1);
			}

			tx.commit();
			out.println(" \nPASSED in testMakeTransactionalAll1()");

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalAll1()");
		}

	}


	public void testMakeTransactionalAll1Neg() {

		try {
			out.println(" ** in testMakeTransactionalAll1Neg()");
			int curr;
			tx = pm.currentTransaction();
			tx.setOptimistic(true);
			tx.begin();

			if (makePersistentNonTranInstance(p2) &&
					makePersistentNonTranInstance(p3) &&
					makePersistentNonTranInstance(p4)) {
				;  // expected result
			}
			tx.commit();

			try {
				pm.makeTransactionalAll(col1);
				out.println(" FAILED - Should not go that further ");
				incrementErrorCount();

			} catch (Exception tex) {
				assertJDOUserException(tex, "testMakeTransactionalAll1Neg()");
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalAll1Neg()");
		}

	}


	public void testMakeTransactionalAll2() {

		try {
			out.println(" ** in testMakeTransactionalAll2()");
			int curr;
			tx = pm.currentTransaction();
			tx.setOptimistic(true);
			tx.begin();

			Object[] objArray = col2.toArray();

			if (makePersistentNonTranInstance(p5) &&
					makePersistentNonTranInstance(p6) &&
					makePersistentNonTranInstance(p7)) {

				pm.makeTransactionalAll(objArray);

			}
			tx.commit();
			out.println(" \nPASSED in testMakeTransactionalAll2()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalAll2()");
		}

	}

	public void testMakeTransactionalAll2Neg() {

		try {
			out.println(" ** in testMakeTransactionalAll2Neg()");
			int curr;
			tx = pm.currentTransaction();
			tx.setOptimistic(true);
			tx.begin();

			Object[] objArray = col2.toArray();
			if (makePersistentNonTranInstance(p5) &&
					makePersistentNonTranInstance(p6) &&
					makePersistentNonTranInstance(p7)) {
				;  // expected result
			}
			tx.commit();

			try {
				pm.makeTransactionalAll(objArray);
				out.println(" FAILED - Should not go that further ");
				incrementErrorCount();

			} catch (Exception tex) {
				assertJDOUserException(tex, "testMakeTransactionalAll2Neg()");
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalAll2Neg()");
		}

	}


	public boolean makePersistentNonTranInstance(PCPoint obj) {

		int val = obj.getX();

		int curr = currentState(obj);
		if (curr != PERSISTENT_NONTRANSACTIONAL &&
				curr != HOLLOW) {
			out.println(" Object not in P-nontransactional " +
					" state for " + obj.getX() + "current state: " + curr);
			incrementErrorCount();
			return false;
		}
		return true;
	}

}
