/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> MakeTransactionalAllFails
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion ID:</B> A12.5.7-4
 *<BR>
 *<B>Assertion Description: </B>
 If a collection or array of instances is passed to PersistenceManager.makeTransactionalAll, and one or more of the instances fail to complete the required o peration, then all instances will be attempted, and a JDOUserException will be thrown which contains a nested exception array, each exception of which conta ins one of the failing instances. The succeeding instances will transition to the specified life cycle state, and the failing instances will remain in their current state.

 *@test
 *@title MakeTransactionalAllFails
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransactionalAllFails
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source MakeTransactionalAllFails.java
 *@keywords transienttransactional lifecycle
 */

public class MakeTransactionalAllFails extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	Point p2 = null;
	Collection col1 = new java.util.HashSet();

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransactionalAllFails t = new MakeTransactionalAllFails();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testTransactionalInst();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeTransactionalAllFails tests ");
		} else {
			return Status.failed("FAILED in MakeTransactionalAllFails: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInst() {
		createObjects();
		testMakeTransactionalAll1();
		testMakeTransactionalAll2();
	}

	private void createObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new Point(2, 4);
			col1 = new java.util.HashSet();
			col1.add(p1);
			col1.add(p2);
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	/* test makeTransactional (Collection pcs) */
	public void testMakeTransactionalAll1() {

		out.println(" in testMakeTransactionalAll2() ");
		try {
			tx = pm.currentTransaction();
			tx.begin();

			col1.add(p1);
			col1.add(p2);

			pm.makeTransactionalAll(col1);
			out.println(" FAILED in testMakeTransactional1 - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeTransactionalAll1()");
			tx.rollback();

		}

	}


	/* test makeTransactionalAll (Object[] pcs) */
	public void testMakeTransactionalAll2() {
		out.println(" in testMakeTransactionalAll2() ");

		try {
			tx = pm.currentTransaction();
			tx.begin();

			pm.makeTransactionalAll(col1.toArray());
			out.println(" FAILED in testMakeTransactionalAll2 - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeTransactionalAll2()");
			tx.rollback();

		}

	}
}
