/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Make Nontransactional is immediate
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-25
 *<BR>
 *<B>Assertion Description: </B>
 The effect of PersistenceManager.makeTransactional or
 makeTransactionalAll is immediate and not subject to rollback.
 *@test
 *@title Make Nontransactional is immediate
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransactionalIsImmediate
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransactionalIsImmediate.java
 *@keywords
 */


public class MakeTransactionalIsImmediate extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;
	PCPoint p6 = null;
	PCPoint p7 = null;

	Collection col1 = new java.util.HashSet();
	Collection col2 = new java.util.HashSet();

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransactionalIsImmediate t = new MakeTransactionalIsImmediate();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testTransactionalInst();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeTransactionalIsImmediate tests ");
		} else {
			return Status.failed("FAILED in MakeTransactionalIsImmediate: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInst() {
		createTcleanObjects();
		testMakeTransactionalIsImmediate();
		testMakeTransactionalIsImmediateAll1();
		testMakeTransactionalIsImmediateAll2();
	}

	private void createTcleanObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);
			p6 = new PCPoint(6, 8);
			p7 = new PCPoint(7, 9);

			col1.add(p2);
			col1.add(p3);
			col1.add(p4);

			col2.add(p5);
			col2.add(p6);
			col2.add(p7);

			pm.makeTransactional(p1);
			pm.makeTransactional(p2);
			pm.makeTransactional(p3);
			pm.makeTransactional(p4);
			pm.makeTransactional(p5);
			pm.makeTransactional(p6);
			pm.makeTransactional(p7);

		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	public void testMakeTransactionalIsImmediate() {

		try {

			//out.println (" ** in testMakeTransactionalIsImmediate()");
			int curr;

			tx = pm.currentTransaction();
			tx.begin();
			pm.makeTransactional(p1);
			tx.rollback();

			if (testState(p1, TRANSIENT_CLEAN, "transient_clean")) {
				out.println(" PASSED in testMakeTransactionalIsImmediate()");
			} else {
				incrementErrorCount();
				out.println(" FAILED in testMakeTransactionalIsImmediate()");
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalIsImmediate()");
		}

	}


	public void testMakeTransactionalIsImmediateAll1() {

		try {
			//out.println (" ** in testMakeTransactionalIsImmediateAll1()");
			int curr;
			int flag = 0;

			tx = pm.currentTransaction();
			tx.begin();
			pm.makeTransactionalAll(col1);
			tx.rollback();

			Iterator iter = col1.iterator();
			while (iter.hasNext()) {
				PCPoint p = (PCPoint) iter.next();
				if (testState(p, TRANSIENT_CLEAN, "transient_clean")) {
					; // expected result
				} else {
					flag++;
				}
			}

			if (flag == 0) {
				out.println(" PASSED in testMakeTransactionalIsImmediateAll1()");
			} else {
				out.println(" FAILED in testMakeTransactionalIsImmediateAll1()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalIsImmediateAll1()");
		}

	}

	public void testMakeTransactionalIsImmediateAll2() {

		try {
			//out.println (" ** in testMakeTransactionalIsImmediateAll2()");
			int curr;
			int flag = 0;
			tx = pm.currentTransaction();

			tx.begin();
			Object[] objArray = col2.toArray();
			pm.makeTransactionalAll(objArray);
			tx.rollback();

			for (int i = 0; i < objArray.length; i++) {
				PCPoint p = (PCPoint) objArray[i];
				if (testState(p, TRANSIENT_CLEAN, "transient_clean")) {
					; // expected result
				} else {
					flag++;
				}
			}

			if (flag == 0) {
				out.println(" PASSED in testMakeTransactionalIsImmediateAll2()");
			} else {
				out.println(" FAILED in testMakeTransactionalIsImmediateAll2()");
				incrementErrorCount();
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalIsImmediateAll2()");
		}

	}

}
