/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> MakeTransactional Prior To a Transaction Rolledback
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-23
 *<BR>
 *<B>Assertion Description: </B>
 If the transaction in which an instance is made transactional (by
 calling PersistenceManager.makeTransactional or makeTransactionalAll) is
 rolled back, then the transient instance takes its values as of the beginning of the
 transaction, if the call to makeTransactional was made prior to the beginning of the
 current transaction.

 *@test
 *@title  MakeTransactional Prior To a Transaction Rolledback
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransactionalPriorToTransactionRolledback
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransactionalPriorToTransactionRolledback.java
 *@keywords
 */


public class MakeTransactionalPriorToTransactionRolledback extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;
	PCPoint p6 = null;
	PCPoint p7 = null;

	Collection col1 = new java.util.HashSet();
	Collection col2 = new java.util.HashSet();

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransactionalPriorToTransactionRolledback t = new MakeTransactionalPriorToTransactionRolledback();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();
		pm.currentTransaction().setRestoreValues(true);

		testTransactionalInst();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeTransactionalPriorToTransactionRolledback tests ");
		} else {
			return Status.failed("FAILED in MakeTransactionalPriorToTransactionRolledback: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInst() {
		createObjects();
		testMakeTransactionalPriorToTransactionRolledback();
		testMakeTransactionalPriorToTransactionRolledbackAll1();
		testMakeTransactionalPriorToTransactionRolledbackAll2();
	}

	private void createObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			col1.add(p2);
			col1.add(p3);

			col2.add(p4);
			col2.add(p5);
		} catch (Exception ex) {
			out.println(" - createObjects() FAILED");
		}
	}

	public void testMakeTransactionalPriorToTransactionRolledback() {

		try {

			out.println(" ** in testMakeTransactionalPriorToTransactionRolledback()");
			tx = pm.currentTransaction();
			p1.setY(new Integer(50));
			tx.begin();
			p1.setY(new Integer(100));
			pm.makeTransactional(p1);
			p1.setY(new Integer(200));
			tx.rollback();

			p1.setY(new Integer(300));
			tx.begin();
			p1.setY(new Integer(400));
			tx.rollback();

			if (p1.getY().intValue() == 300) {
				out.println(" PASSED in MakeTransactionalPriorToTransactionRolledback()");
			} else {
				out.println(" FAILED in MakeTransactionalPriorToTransactionRolledback()");
				out.println("\tp1 expected: 300, actual: " + p1.getY().intValue());
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalPriorToTransactionRolledback()");
			if (tx.isActive())
				tx.rollback();
		}

	}


	public void testMakeTransactionalPriorToTransactionRolledbackAll1() {

		try {
			out.println(" ** in testMakeTransactionalPriorToTransactionRolledbackAll1()");
			tx = pm.currentTransaction();
			p2.setY(new Integer(50));
			p3.setY(new Integer(50));
			tx.begin();
			p2.setY(new Integer(100));
			p3.setY(new Integer(100));
			pm.makeTransactionalAll(col1);
			p2.setY(new Integer(200));
			p3.setY(new Integer(200));
			tx.rollback();

			p2.setY(new Integer(300));
			p3.setY(new Integer(300));
			tx.begin();
			p2.setY(new Integer(400));
			p3.setY(new Integer(400));
			tx.rollback();

			if (p2.getY().intValue() == 300 &&
					p3.getY().intValue() == 300) {
				out.println(" PASSED in testMakeTransactionalPriorToTransactionRolledbackAll1()");
			} else {
				out.println(" FAILED in testMakeTransactionalPriorToTransactionRolledbackAll1()");
				out.println("\tp2 expected: 300, actual: " + p2.getY().intValue());
				out.println("\tp3 expected: 300, actual: " + p3.getY().intValue());
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalPriorToTransactionRolledbackAll1()");
			if (tx.isActive())
				tx.rollback();
		}

	}

	public void testMakeTransactionalPriorToTransactionRolledbackAll2() {

		try {
			out.println(" ** in testMakeTransactionalPriorToTransactionRolledbackAll2()");
			tx = pm.currentTransaction();
			p2.setY(new Integer(50));
			p3.setY(new Integer(50));
			tx.begin();
			p2.setY(new Integer(100));
			p3.setY(new Integer(100));

			Object[] objArray = col2.toArray();
			pm.makeTransactionalAll(objArray);
			p2.setY(new Integer(200));
			p3.setY(new Integer(200));
			tx.rollback();

			p2.setY(new Integer(300));
			p3.setY(new Integer(300));
			tx.begin();
			p2.setY(new Integer(400));
			p3.setY(new Integer(400));
			tx.rollback();

			if (p2.getY().intValue() == 300 &&
					p3.getY().intValue() == 300) {
				out.println(" PASSED in testMakeTransactionalPriorToTransactionRolledbackAll2()");
			} else {
				out.println(" FAILED in testMakeTransactionalPriorToTransactionRolledbackAll2()");
				out.println("\tp2 expected: 300, actual: " + p2.getY().intValue());
				out.println("\tp3 expected: 300, actual: " + p3.getY().intValue());
				incrementErrorCount();
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalPriorToTransactionRolledbackAll2()");
		}

	}

}
