/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> MakeTransactionalWithinTransactionRolledback
 *<BR>
 *<B>JavaTest Keywords:</B> transienttransactional
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-22
 *<BR>
 *<B>Assertion Description: </B>
 If the transaction in which an instance is made transactional (by
 calling PersistenceManager.makeTransactional or makeTransactionalAll) is
 rolled back, then the transient instance takes its values as of the call to
 makeTransactional if the call was made within the current transaction.

 *@test
 *@title  MakeTransactional Prior To a Transaction Rolledback
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransactionalWithinTransactionRolledback
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransactionalWithinTransactionRolledback.java
 *@keywords
 */


public class MakeTransactionalWithinTransactionRolledback extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;
	PCPoint p6 = null;
	PCPoint p7 = null;

	Collection col1 = new java.util.HashSet();
	Collection col2 = new java.util.HashSet();

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransactionalWithinTransactionRolledback t = new MakeTransactionalWithinTransactionRolledback();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();
		pm.currentTransaction().setRestoreValues(true);

		testTransactionalInst();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in MakeTransactionalWithinTransactionRolledback tests ");
		} else {
			return Status.failed("FAILED in MakeTransactionalWithinTransactionRolledback: number of errors detected is " + errCnt);
		}

	}

	public void testTransactionalInst() {
		createObjects();
		testMakeTransactionalWithinTransactionRolledback();
		testMakeTransactionalWithinTransactionRolledbackAll1();
		testMakeTransactionalWithinTransactionRolledbackAll2();
	}

	private void createObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			col1.add(p2);
			col1.add(p3);

			col2.add(p4);
			col2.add(p5);

		} catch (Exception ex) {
			out.println(" - createTcleanObjects() FAILED");
		}
	}

	public void testMakeTransactionalWithinTransactionRolledback() {

		try {

			out.println(" ** in testMakeTransactionalWithinTransactionRolledback()");

			tx = pm.currentTransaction();

			p1.setY(new Integer(100));
			tx.begin();
			p1.setY(new Integer(200));
			pm.makeTransactional(p1);
			p1.setY(new Integer(300));
			tx.rollback();

			if (p1.getY().intValue() == 200) {
				out.println(" PASSED in testMakeTransactionalWithinTransactionRolledback()");
			} else {
				out.println(" FAILED in testMakeTransactionalWithinTransactionRolledback()");
				out.println("\tp1 expected: 200, actual: " + p1.getY().intValue());
				incrementErrorCount();
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalWithinTransactionRolledback()");
		}

	}


	public void testMakeTransactionalWithinTransactionRolledbackAll1() {

		try {
			//out.println (" ** in testMakeTransactionalWithinTransactionRolledbackAll1()");
			tx = pm.currentTransaction();
			p2.setY(new Integer(100));
			p3.setY(new Integer(100));
			tx.begin();
			p2.setY(new Integer(200));
			p3.setY(new Integer(200));
			pm.makeTransactionalAll(col1);
			p2.setY(new Integer(300));
			p3.setY(new Integer(300));
			tx.rollback();

			if (p2.getY().intValue() == 200 &&
					p3.getY().intValue() == 200) {
				out.println(" PASSED in testMakeTransactionalWithinTransactionRolledbackAll1()");
			} else {
				out.println(" FAILED in testMakeTransactionalWithinTransactionRolledbackAll1()");
				out.println("\tp2 expected: 200, actual: " + p2.getY().intValue());
				out.println("\tp3 expected: 200, actual: " + p3.getY().intValue());
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalWithinTransactionRolledbackAll1()");
		}

	}

	public void testMakeTransactionalWithinTransactionRolledbackAll2() {

		try {
			//out.println (" ** in testMakeTransactionalWithinTransactionRolledbackAll2()");

			tx = pm.currentTransaction();
			p4.setY(new Integer(100));
			p5.setY(new Integer(100));
			tx.begin();
			p4.setY(new Integer(200));
			p5.setY(new Integer(200));
			Object[] objArray = col2.toArray();
			pm.makeTransactionalAll(objArray);
			p4.setY(new Integer(300));
			p5.setY(new Integer(300));
			tx.rollback();

			if (p4.getY().intValue() == 200 &&
					p5.getY().intValue() == 200) {
				out.println(" PASSED in testMakeTransactionalWithinTransactionRolledbackAll2()");
			} else {
				out.println(" FAILED in testMakeTransactionalWithinTransactionRolledbackAll2()");
				out.println("\tp4 expected: 200, actual: " + p4.getY().intValue());
				out.println("\tp5 expected: 200, actual: " + p5.getY().intValue());
				incrementErrorCount();
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransactionalWithinTransactionRolledbackAll2()");
		}

	}
}
