/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> MakeTransient
 *<BR>
 *<B>JavaTest Keywords:</B> transient
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-13
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.makeTransient and makeTransientAll make a persistent instance transient, so it is not associated with the PersistenceManager instance. It does not affect the persistent state in the data store.

 *@test
 *@title MakeTransient
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransient
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransient.java
 *@keywords transient
 */

public class MakeTransient extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransient t = new MakeTransient();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakeTransient();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakeTransient tests passed");
		} else {
			return Status.failed("MakeTransient: number of errors detected is " + errCnt);
		}

	}

	public void testMakeTransient() {
		createPersistentObjects();
		testMakeTransient1();
		testMakeTransientAll1();
		testMakeTransientAll2();

	}


	private void createPersistentObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);
			tx.commit();
		} catch (Exception ex) {
			out.println(" - createPersistentObjects() FAILED");
		}
	}


	/* test makeTransient (Object pc) */
	public void testMakeTransient1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.makeTransient(p1);
			tx.commit();

			if (testState(p1, TRANSIENT, "transient")) {
				out.println(" \nPASSED in testMakeTransient1()");
			} else {
				out.println(" \nFAILED in testMakeTransient1()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransient1()");
		}

	}


	/* test makeTransientAll (Collection pcs) */
	public void testMakeTransientAll1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			pm.makeTransientAll(col1);
			tx.commit();

			if (testCollection(col1)) {
				out.println(" \nPASSED in testMakeTransientAll1()");
			} else {
				out.println(" \nFAILED in testMakeTransientAll1()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientAll1()");
		}

	}

	/* test makeTransientAll (Object[] o) */
	public void testMakeTransientAll2() {
		try {
			int NUM_OBJS = 2;
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			Object[] obj1 = col1.toArray();

			pm.makeTransientAll(obj1);
			tx.commit();

			if (testArray(obj1, NUM_OBJS)) {
				out.println(" \nPASSED in testMakeTransientAll2()");
			} else {
				out.println(" \nFAILED in testMakeTransientAll2()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientAll2()");
		}

	}


	public boolean testCollection(Collection mycol) {

		int flag = 0;
		Iterator iter = mycol.iterator();
		while (iter.hasNext()) {
			PCPoint p = (PCPoint) iter.next();
			if (testState(p, TRANSIENT, "transient"))
				;  // expected result
			else
				flag++;
		}

		if (flag == 0)
			return true;
		else
			return false;
	}

	public boolean testArray(Object[] arrayObj, int num) {

		int flag = 0;
		int i = 0;
		for (i = 0; i < num; ++i) {
			PCPoint p = (PCPoint) arrayObj[i];
			if (testState(p, TRANSIENT, "transient"))
				;  // expected result
			else
				flag++;
		}

		if (flag == 0)
			return true;
		else
			return false;
	}

}

