/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B>MakeTransientAll Fail
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-3
 *<BR>
 *<B>Assertion Description: </B>
 If a collection or array of instances is passed to PersistenceManager.makeTransientAll, and one or more of the instances fail to complete the required operation, then all instances will be attempted, and a JDOUserException will be thrown which contains a nested exception array, each exception of which contains one of the failing instances. The succeeding instances will transition to the specified life cycle state, and the failing instances will remain in their current state.

 *@test
 *@title MakeTransientAll Fail
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransientAllFails
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransientAllFails.java
 *@keywords persistencemanager
 */

public class MakeTransientAllFails extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransientAllFails t = new MakeTransientAllFails();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakeTransientAllFails();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakeTransientAllFails tests passed");
		} else {
			return Status.failed("MakeTransientAllFails: number of errors detected is " + errCnt);
		}

	}

	public void testMakeTransientAllFails() {

		testMakeTransientAllFails1();
		testMakeTransientAllFails2();

	}


	/* test makeTansientAll (Collection pcs) */
	public void testMakeTransientAllFails1() {
		try {

			tx = pm.currentTransaction();
			tx.begin();

			PCPoint np1 = new PCPoint(3, 3);
			PCPoint np2 = new PCPoint(4, 4);
			Point np3 = new Point(5, 5);

			Collection col1 = new java.util.HashSet();
			col1.add(np1);
			col1.add(np2);
			col1.add(np3);

			pm.makeTransientAll(col1);
			out.println(" FAILED in testMakeTransientAllFails1() - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeTransientAllFails1()");
			tx.rollback();
		}
	}

	/* test makeTransientAll (Object[] o) */
	public void testMakeTransientAllFails2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			PCPoint np1 = new PCPoint(3, 3);
			PCPoint np2 = new PCPoint(4, 4);
			Point np3 = new Point(5, 5);

			Collection col1 = new java.util.HashSet();
			col1.add(np1);
			col1.add(np2);
			col1.add(np3);

			Object[] obj1 = col1.toArray();

			pm.makeTransientAll(obj1);
			out.println(" FAILED in testMakeTransientAllFails2() - should not go that further ");
			incrementErrorCount();
			tx.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeTransientAllFails2()");
			tx.rollback();
		}

	}

}
