/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> MakeTransientCausesLossOfIdentity
 *<BR>
 *<B>JavaTest Keywords:</B> transient identity
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-14
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.makeTransient and makeTransientAll transition an instance to transient, causing it to lose its JDO identity.

 *@test
 *@title MakeTransientCausesLossOfIdentity
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransientCausesLossOfIdentity
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransientCausesLossOfIdentity.java
 *@keywords transient identity
 */

public class MakeTransientCausesLossOfIdentity extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransientCausesLossOfIdentity t = new MakeTransientCausesLossOfIdentity();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakeTransientCausesLossOfIdentity();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakeTransientCausesLossOfIdentity tests passed");
		} else {
			return Status.failed("MakeTransientCausesLossOfIdentity: number of errors detected is " + errCnt);
		}

	}

	public void testMakeTransientCausesLossOfIdentity() {
		createPersistentObjects();
		testMakeTransientCausesLossOfIdentity1();
		testMakeTransientCausesLossOfIdentityAll1();
		testMakeTransientCausesLossOfIdentityAll2();

	}


	private void createPersistentObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);
			tx.commit();
		} catch (Exception ex) {
			out.println(" - createPersistentObjects() FAILED");
		}
	}


	/* test makeTransient (Object pc) */
	public void testMakeTransientCausesLossOfIdentity1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.makeTransient(p1);
			tx.commit();

			Object oid = pm.getObjectId(p1);
			if (oid == null) {
				out.println(" \nPASSED in testMakeTransientCausesLossOfIdentity()");
			} else {
				out.println(" \nFAILED in testMakeTransientCausesLossOfIdentity()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			//ex.printStackTrace(out);
			out.println(" FAILED in testMakeTransientCausesLossOfIdentity()");
			incrementErrorCount();
		}

	}


	/* test makeTransientAll (Collection pcs) */
	public void testMakeTransientCausesLossOfIdentityAll1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			pm.makeTransientAll(col1);
			tx.commit();

			Object oid1 = pm.getObjectId(p2);
			Object oid2 = pm.getObjectId(p3);
			if (oid1 == null &&
					oid2 == null) {
				out.println(" \nPASSED in testMakeTransientCausesLossOfIdentityAll1()");
			} else {
				out.println(" \nFAILED in testMakeTransientCausesLossOfIdentityAll1()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientCausesLossOfIdentityAll1()");
		}

	}

	/* test makeTransientAll (Object[] o) */
	public void testMakeTransientCausesLossOfIdentityAll2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			Object[] obj1 = col1.toArray();

			pm.makeTransientAll(obj1);
			tx.commit();

			Object oid1 = pm.getObjectId(p4);
			Object oid2 = pm.getObjectId(p5);

			if (oid1 == null &&
					oid2 == null) {
				out.println(" \nPASSED in testMakeTransientCausesLossOfIdentityAll2()");
			} else {
				out.println(" \nFAILED in testMakeTransientCausesLossOfIdentityAll2()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientCausesLossOfIdentityAll2()");
		}

	}

}
