/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B>MakeTransientFailsWithDirtyInstance
 *<BR>
 *<B>JavaTest Keywords:</B> exception
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-16
 *<BR>
 *<B>Assertion Description: </B>
 If the instance passed to PersistenceManager.makeTransient or makeTransientAll is dirty, a JDOUserException is thrown.

 *@test
 *@title MakeTransientFailsWithDirtyInstance
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransientFailsWithDirtyInstance
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransientFailsWithDirtyInstance.java
 *@keywords exception
 */

public class MakeTransientFailsWithDirtyInstance extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransientFailsWithDirtyInstance t = new MakeTransientFailsWithDirtyInstance();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakeTransientFailsWithDirtyInstance();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakeTransientFailsWithDirtyInstance tests passed");
		} else {
			return Status.failed("MakeTransientFailsWithDirtyInstance: number of errors detected is " + errCnt);
		}

	}

	public void testMakeTransientFailsWithDirtyInstance() {
		createObjects();
		testMakeTransient();
		testMakeTransientAll1();
		testMakeTransientAll2();

	}


	private void createObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);
			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "createObject()");
		}
	}


	/* test makeTansient (Object pc) */
	public void testMakeTransient() {
		try {

			tx = pm.currentTransaction();
			tx.begin();

			// make instance dirty
			p1.setX(100);

			pm.makeTransient(p1);
			out.println(" FAILED in testMakeTransientFailsWithDirtyInstance1() - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeTransient()");
			tx.rollback();
		}
	}

	/* test makeTansientAll (Collection pcs) */
	public void testMakeTransientAll1() {
		try {

			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			p2.setX(200);
			p3.setX(201);

			pm.makeTransientAll(col1);
			out.println(" FAILED in testMakeTransientAll1() - should not go that further ");
			incrementErrorCount();
			tx.rollback();

		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeTransientAll1()");
			tx.rollback();
		}
	}


	/* test makeTransientAll (Object[] o) */
	public void testMakeTransientAll2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			p4.setX(300);
			p5.setX(301);

			Object[] obj1 = col1.toArray();

			pm.makeTransientAll(obj1);
			out.println(" FAILED in testMakeTransientAll2() - should not go that further ");
			incrementErrorCount();
			tx.rollback();
		} catch (Exception ex) {
			assertJDOUserException(ex, "testMakeTransientAll2()");
			tx.rollback();
		}

	}

}
