/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> MakeTransient Fields Preserved Unchanged
 *<BR>
 *<B>JavaTest Keywords:</B> transient
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-15, A12.5.7-18
 *<BR>
 *<B>Assertion Description: </B>
 If the instance passed to PersistenceManager.makeTransient or makeTransientAll has field values (persistent-nontransactional or persistent-clean), the fields in the cache are preserved unchanged. The instance(s) are not modified in any way.
 *@test
 *@title MakeTransient Fields Preserved Unchanged
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransientFieldsPreservedUnchanged
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransientFieldsPreservedUnchanged.java
 *@keywords transient
 */

public class MakeTransientFieldsPreservedUnchanged extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;

	PCPoint np1 = null;
	PCPoint np2 = null;
	PCPoint np3 = null;
	PCPoint np4 = null;
	PCPoint np5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransientFieldsPreservedUnchanged t = new MakeTransientFieldsPreservedUnchanged();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakeTransientFieldsPreservedUnchanged();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakeTransientFieldsPreservedUnchanged tests passed");
		} else {
			return Status.failed("MakeTransientFieldsPreservedUnchanged: number of errors detected is " + errCnt);
		}

	}

	public void testMakeTransientFieldsPreservedUnchanged() {
		createObjects();
		testMakeTransientFieldsPreservedUnchanged1();
		testMakeTransientFieldsPreservedUnchangedAll1();
		testMakeTransientFieldsPreservedUnchangedAll2();

	}

	private void createObjects() {
		createPCleanObjects();
		createPNonTransactionalObjects();
	}

	private void createPCleanObjects() {
		try {

			// create P-clean instances
			tx = pm.currentTransaction();
			tx.setOptimistic(false);
			tx.begin();

			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);
			tx.commit();

			tx.begin();
			p1.getX();
			p2.getX();
			p3.getX();
			p4.getX();
			p5.getX();
			tx.commit();

		} catch (Exception ex) {
			out.println(" - createPCleanObjects() FAILED");
		}
	}


	private void createPNonTransactionalObjects() {
		try {

			tx = pm.currentTransaction();
			tx.setOptimistic(true);
			tx.begin();

			np1 = new PCPoint(1, 3);
			np2 = new PCPoint(2, 4);
			np3 = new PCPoint(3, 5);
			np4 = new PCPoint(4, 6);
			np5 = new PCPoint(5, 7);

			pm.makePersistent(np1);
			pm.makePersistent(np2);
			pm.makePersistent(np3);
			pm.makePersistent(np4);
			pm.makePersistent(np5);
			tx.commit();

			tx.begin();
			np1.getX();
			np2.getX();
			np3.getX();
			np4.getX();
			np5.getX();
			tx.commit();

		} catch (Exception ex) {
			out.println(" - createPNonTransactionalObjects() FAILED");
		}
	}


	/* test makeTransient (Object pc)
	 * instance passed has field values and is P-nontransactional *
	*/
	public void testMakeTransientFieldsPreservedUnchanged1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.makeTransient(p1);
			tx.commit();

			if (testState(p1, TRANSIENT, "transient")) {
				out.println(" \nPASSED in testMakeTransientFieldsPreservedUnchanged()");
			} else {
				out.println(" \nFAILED in testMakeTransientFieldsPreservedUnchanged()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientFieldsPreservedUnchanged()");
		}

	}


	/* test makeTransientAll (Collection pcs) */
	public void testMakeTransientFieldsPreservedUnchangedAll1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			pm.makeTransientAll(col1);
			tx.commit();

			if (testCollection(col1)) {
				out.println(" \nPASSED in testMakeTransientFieldsPreservedUnchangedAll1()");
			} else {
				out.println(" \nFAILED in testMakeTransientFieldsPreservedUnchangedAll1()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientFieldsPreservedUnchanged1()");
		}

	}

	/* test makeTransientAll (Object[] o) */
	public void testMakeTransientFieldsPreservedUnchangedAll2() {
		try {
			int NUM_OBJS = 2;
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			Object[] obj1 = col1.toArray();

			pm.makeTransientAll(obj1);
			tx.commit();

			if (testArray(obj1, NUM_OBJS)) {
				out.println(" \nPASSED in testMakeTransientFieldsPreservedUnchangedAll2()");
			} else {
				out.println(" \nFAILED in testMakeTransientFieldsPreservedUnchangedAll2()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientFieldsPreservedUnchanged2()");
		}

	}


	public boolean testCollection(Collection mycol) {

		int flag = 0;
		Iterator iter = mycol.iterator();
		while (iter.hasNext()) {
			PCPoint p = (PCPoint) iter.next();
			if (testState(p, TRANSIENT, "transient"))
				;  // expected result
			else
				flag++;
		}

		if (flag == 0)
			return true;
		else
			return false;
	}

	public boolean testArray(Object[] arrayObj, int num) {

		int flag = 0;
		int i = 0;
		for (i = 0; i < num; ++i) {
			PCPoint p = (PCPoint) arrayObj[i];
			if (testState(p, TRANSIENT, "transient"))
				;  // expected result
			else
				flag++;
		}

		if (flag == 0)
			return true;
		else
			return false;
	}

}

