/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> MakeTransientHasNoEffectOnTransientInstances
 *<BR>
 *<B>JavaTest Keywords:</B> transient
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-19
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManager.makeTransient and
 makeTransientAll have no effect if the parameter instance is transient.

 *@test
 *@title MakeTransientHasNoEffectOnTransientInstances
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransientHasNoEffectOnTransientInstances
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransientHasNoEffectOnTransientInstances.java
 *@keywords transient
 */

public class MakeTransientHasNoEffectOnTransientInstances extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransientHasNoEffectOnTransientInstances t = new MakeTransientHasNoEffectOnTransientInstances();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakeTransientHasNoEffectOnTransientInstances();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakeTransientHasNoEffectOnTransientInstances tests passed");
		} else {
			return Status.failed("MakeTransientHasNoEffectOnTransientInstances: number of errors detected is " + errCnt);
		}

	}

	public void testMakeTransientHasNoEffectOnTransientInstances() {
		createTransientObjects();
		testMakeTransientHasNoEffectOnTransientInstances1();
		testMakeTransientHasNoEffectOnTransientInstancesAll1();
		testMakeTransientHasNoEffectOnTransientInstancesAll2();

	}


	private void createTransientObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);
		} catch (Exception ex) {
			out.println(" - createPersistentObjects() FAILED");
		}
	}


	/* test makeTransient (Object pc) */
	public void testMakeTransientHasNoEffectOnTransientInstances1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.makeTransient(p1);
			tx.commit();
			out.println(" \nPASSED in testMakeTransientHasNoEffectOnTransientInstances()");

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientHasNoEffectOnTransientInstances1()");
		}

	}


	/* test makeTransientAll (Collection pcs) */
	public void testMakeTransientHasNoEffectOnTransientInstancesAll1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);

			pm.makeTransientAll(col1);
			tx.commit();
			out.println(" \nPASSED in testMakeTransientHasNoEffectOnTransientInstancesAll1()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientHasNoEffectOnTransientInstancesAll1()");
		}

	}

	/* test makeTransientAll (Object[] o) */
	public void testMakeTransientHasNoEffectOnTransientInstancesAll2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			Object[] obj1 = col1.toArray();

			pm.makeTransientAll(obj1);
			tx.commit();
			out.println(" \nPASSED in testMakeTransientHasNoEffectOnTransientInstancesAll2()");
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientHasNoEffectOnTransientInstancesAll2()");
		}

	}

}
