/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> MakeTransientNotSubjectToRollback
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion IDs:</B> A12.5.7-17
 *<BR>
 *<B>Assertion Description: </B>
 The effect of PersistenceManager.makeTransient or makeTransientAll is immediate and not subject to rollback.

 *@test
 *@title MakeTransientNotSubjectToRollback
 *@executeClass com.sun.jdotck.api.persistencemanager.MakeTransientNotSubjectToRollback
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MakeTransientNotSubjectToRollback.java
 *@keywords transient
 */

public class MakeTransientNotSubjectToRollback extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		MakeTransientNotSubjectToRollback t = new MakeTransientNotSubjectToRollback();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testMakeTransientNotSubjectToRollback();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("MakeTransientNotSubjectToRollback tests passed");
		} else {
			return Status.failed("MakeTransientNotSubjectToRollback: number of errors detected is " + errCnt);
		}

	}

	public void testMakeTransientNotSubjectToRollback() {
		createPersistentObjects();
		testMakeTransientNotSubjectToRollback1();
		testMakeTransientNotSubjectToRollbackAll1();
		testMakeTransientNotSubjectToRollbackAll2();

	}


	private void createPersistentObjects() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			pm.makePersistent(p1);
			pm.makePersistent(p2);
			pm.makePersistent(p3);
			pm.makePersistent(p4);
			pm.makePersistent(p5);
			tx.commit();
		} catch (Exception ex) {
			out.println(" - createPersistentObjects() FAILED");
		}
	}


	/* test makeTransient (Object pc) */
	public void testMakeTransientNotSubjectToRollback1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			pm.makeTransient(p1);
			int curr1 = currentState(p1);
			tx.rollback();

			int curr2 = currentState(p1);
			if (curr1 == curr2) {
				out.println(" \nPASSED in testMakeTransientNotSubjectToRollback()");
			} else {
				out.println(" \nFAILED in testMakeTransientNotSubjectToRollback()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientNotSubjectToRollback()");
		}

	}


	/* test makeTransientAll (Collection pcs) */
	public void testMakeTransientNotSubjectToRollbackAll1() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p2);
			col1.add(p3);
			pm.makeTransientAll(col1);
			int p2_curr = currentState(p2);
			int p3_curr = currentState(p3);
			tx.rollback();

			int p2_ecurr = currentState(p2);
			int p3_ecurr = currentState(p3);

			if (p2_curr == p2_ecurr &&
					p3_curr == p3_ecurr) {
				out.println(" \nPASSED in testMakeTransientNotSubjectToRollbackAll1()");
			} else {
				out.println(" \nFAILED in testMakeTransientNotSubjectToRollbackAll1()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientNotSubjectToRollback1()");
		}

	}

	/* test makeTransientAll (Object[] o) */
	public void testMakeTransientNotSubjectToRollbackAll2() {
		try {
			tx = pm.currentTransaction();
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(p4);
			col1.add(p5);

			Object[] obj1 = col1.toArray();

			pm.makeTransientAll(obj1);
			int p4_curr = currentState(p4);
			int p5_curr = currentState(p5);
			tx.rollback();

			int p4_ecurr = currentState(p4);
			int p5_ecurr = currentState(p5);

			if (p4_curr == p4_ecurr &&
					p5_curr == p5_ecurr) {
				out.println(" \nPASSED in testMakeTransientNotSubjectToRollbackAll2()");
			} else {
				out.println(" \nFAILED in testMakeTransientNotSubjectToRollbackAll2()");
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testMakeTransientNotSubjectToRollback2()");
		}

	}

}
