/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import javax.jdo.*;

import com.sun.jdotck.models.mylib.PCPoint;

/**
 *<B>JavaTest Title:</B> ObjectId Unique Among Instances
 *<BR>
 *<B>JavaTest Keywords:</B> identity
 *<BR>
 *<B>Assertion ID:</B> A12.5.6-12.
 *<BR>
 *<B>Assertion Description: </B>
 Within a transaction, the <code>ObjectId</code> returned will compare equal
 to the <code>ObjectId</code> returned by only one among all JDO instances
 associated with the <code>PersistenceManager</code> regardless of the type of
 <code>ObjectId</code>.

 *@test
 *@title ObjectId Unique Among Instances
 *@executeClass com.sun.jdotck.api.persistencemanager.ObjectIdUniqueAmongInstances
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source ObjectIdUniqueAmongInstances.java
 *@keywords identity
 */

public class ObjectIdUniqueAmongInstances extends JDO_Test {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		ObjectIdUniqueAmongInstances t = new ObjectIdUniqueAmongInstances();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		final int count = 1000;

		HashSet instances = new HashSet(count);
		HashSet oids = new HashSet(count);

		Transaction tx = getPM().currentTransaction();
		tx.begin();

		/** Construct "count" persistent instances, and save them in a hashset.
		 */
		for (int i = 0; i < count; ++i) {
			PCPoint p = new PCPoint(i, count - i);
			pm.makePersistent(p);
			instances.add(p);
		}

		/** For all new persistent instances, get the object ids and save them
		 * in another hashset.  There should be the same number if the ids
		 * are all unique.
		 */
		Iterator it = instances.iterator();
		while (it.hasNext()) {
			oids.add(pm.getObjectId(it.next()));
		}

		tx.commit();

		if (oids.size() == instances.size())
			return Status.passed("Assertion A12.5.6-12 passed");
		else {
			out.println("Oids has size: " + oids.size() + "; expected: " + instances.size());
			out.flush();
			return Status.failed("Assertion A12.5.6-12 failed");
		}
	}
}
