/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.Iterator;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import javax.jdo.*;

import com.sun.jdotck.models.mylib.PCPoint;
import com.sun.jdotck.models.mylib.PCRect;

/**
 *<B>JavaTest Title:</B> Only one instance of persistent object in cache per PersistenceManager
 *<BR>
 *<B>JavaTest Keywords:</B> cache
 *<BR>
 *<B>Assertion ID:</B> A5.4-2.
 *<BR>
 *<B>Assertion Description: </B>
 JDO implementations must manage the cache of JDO Instances such that there is
 only one JDO Instance, associated with each JDO <code>PersistenceManager</code>,
 representing the persistent state of each corresponding data store object.
 (One needs a hashtable that uses <code>==</code> in order to test this).

 *@test
 *@title Only one instance of persistent object in cache per PersistenceManager
 *@executeClass com.sun.jdotck.api.persistencemanager.OneInstanceOfObjectPerPersistenceManager
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source OneInstanceOfObjectPerPersistenceManager.java
 *@keywords cache
 */

public class OneInstanceOfObjectPerPersistenceManager extends JDO_Test {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		OneInstanceOfObjectPerPersistenceManager t = new OneInstanceOfObjectPerPersistenceManager();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
//      Once method has been implemented, remove next line and
//      remove ", but not implemented yet" from String value argument to Status.passed
		out.println("com.sun.jdotck.api.persistencemanager.OneInstanceOfObjectPerPersistenceManager.run not implemented yet");

		initializeArgs(args, err, out);

		Transaction tx = getPM().currentTransaction();
		tx.setRetainValues(false);
		tx.setRestoreValues(false);

		tx.begin();
		PCPoint p1 = new PCPoint(10, 20);
		PCPoint p2 = new PCPoint(20, 40);
		PCRect rect = new PCRect(0, p1, p2);
		pm.makePersistent(rect);
		tx.commit();

		tx.begin();
		Object p1Id = pm.getObjectId(p1);
		PCPoint p1a = (PCPoint) pm.getObjectById(p1Id, true);
		PCPoint p1b = rect.getUpperLeft();
		PCPoint p1c = findPoint(10, 20);
		tx.commit();
		pm.close();

		if (p1 != p1a)
			return Status.failed("Assertion A5.4-2 failed; getObjectById results differ");
		if (p1 != p1b)
			return Status.failed("Assertion A5.4-2 failed; navigation results differ");
		if (p1 != p1c)
			return Status.failed("Assertion A5.4-2 failed; query results differ");

		return Status.passed("Assertion A5.4-2 passed");
	}

	PCPoint findPoint(int x, int y) {
		Query q = getPM().newQuery(PCPoint.class);
		q.declareParameters("int px, int py");
		q.setFilter("x == px & y == py");
		Collection results = (Collection) q.execute(new Integer(x), new Integer(y));
		Iterator it = results.iterator();
		PCPoint ret = (PCPoint) it.next();
		return ret;
	}

}
