/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> PassingNullToEvictAllThrowsException
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion IDs:</B> A12.5-9
 *<BR>
 *<B>Assertion Description: </B>
 Passing a null valued argument to evictAll will throw a NullPointerException.

 *@test
 *@title PassingNullToEvictAllThrowsException
 *@executeClass com.sun.jdotck.api.persistencemanager.PassingNullToEvictAllThrowsException
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source PassingNullToEvictAllThrowsException.java
 *@keywords cache
 */

public class PassingNullToEvictAllThrowsException extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		PassingNullToEvictAllThrowsException t = new PassingNullToEvictAllThrowsException();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testPassingNullToEvictAllThrowsException();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PassingNullToEvictAllThrowsException tests passed");
		} else {
			return Status.failed("PassingNullToEvictAllThrowsException: number of errors detected is " + errCnt);
		}

	}

	public void testPassingNullToEvictAllThrowsException() {
		testEvictAll1();
		testEvictAll2();

	}


	/* test evictAll (Collection pcs) */
	public void testEvictAll1() {
		out.println("**  in testEvictAll1() ");
		try {
			tx = pm.currentTransaction();
			tx.begin();
			Collection col1 = new java.util.HashSet();
			col1 = null;
			pm.evictAll(col1);
			out.println(" \nFAILED in testEvictAll2() - expect exception");
			incrementErrorCount();
		} catch (Exception ex) {
			assertNullPointerException(ex, "testEvictAll1()");
			tx.rollback();
		}

	}

	/* test evictAll (Object[] objArray) */
	public void testEvictAll2() {
		out.println("**  in testEvictAll1() ");
		try {
			tx = pm.currentTransaction();
			tx.begin();
			Collection col1 = new java.util.HashSet();
			col1 = null;
			pm.evictAll(col1.toArray());
			out.println(" \nFAILED in testEvictAll2() - expect exception");
			incrementErrorCount();
		} catch (Exception ex) {
			assertNullPointerException(ex, "testEvictAll2()");
			tx.rollback();
		}

	}


}
