/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Passing Null to RefreshAll Throws Exception
 *<BR>
 *<B>JavaTest Keywords:</B> cache
 *<BR>
 *<B>Assertion IDs:</B> A12.5-10.
 *<BR>
 *<B>Assertion Description: </B>
 Passing a null valued argument to refreshAll will throw a
 NullPointerException.

 *@test
 *@title Passing Null to RefreshAll Throws Exception
 *@executeClass com.sun.jdotck.api.persistencemanager.PassingNullToRefreshAllThrowsException
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source PassingNullToRefreshAllThrowsException.java
 *@keywords cache
 */

public class PassingNullToRefreshAllThrowsException extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		PassingNullToRefreshAllThrowsException t = new PassingNullToRefreshAllThrowsException();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testPassingNullToRefreshAllThrowsException();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PassingNullToRefreshAllThrowsException tests passed");
		} else {
			return Status.failed("PassingNullToRefreshAllThrowsException: number of errors detected is " + errCnt);
		}

	}

	public void testPassingNullToRefreshAllThrowsException() {
		testRefreshAll1();
		testRefreshAll2();

	}


	/* test refreshAll (Collection pcs) */
	public void testRefreshAll1() {
		out.println("**  in testRefreshAll1() ");
		try {
			tx = pm.currentTransaction();
			tx.begin();
			Collection col1 = new java.util.HashSet();
			col1 = null;
			pm.refreshAll(col1);
			out.println(" \nFAILED in testRefreshAll2() - expect exception");
			incrementErrorCount();
		} catch (Exception ex) {
			assertNullPointerException(ex, "testRefreshAll1()");
			tx.rollback();
		}

	}

	/* test refreshAll (Object[] objArray) */
	public void testRefreshAll2() {
		out.println("**  in testRefreshAll1() ");
		try {
			tx = pm.currentTransaction();
			tx.begin();
			Collection col1 = new java.util.HashSet();
			col1 = null;
			pm.refreshAll(col1.toArray());
			out.println(" \nFAILED in testRefreshAll2() - expect exception");
			incrementErrorCount();
		} catch (Exception ex) {
			assertNullPointerException(ex, "testRefreshAll2()");
			tx.rollback();
		}

	}


}
