/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Passing Null to Refresh has no Effect
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion IDs:</B> A12.5-8
 *<BR>
 *<B>Assertion Description: </B>
 Passing a null value to PersistenceManager.refresh will have no effect. A NullPointerException should NOT be thrown.

 *@test
 *@title Passing Null to Refresh has no Effect
 *@executeClass com.sun.jdotck.api.persistencemanager.PassingNullToRefreshHasNoEffect
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source PassingNullToRefreshHasNoEffect.java
 *@keywords
 */

public class PassingNullToRefreshHasNoEffect extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		PassingNullToRefreshHasNoEffect t = new PassingNullToRefreshHasNoEffect();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testPassingNullToRefreshHasNoEffect();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PassingNullToRefreshHasNoEffect tests passed");
		} else {
			return Status.failed("PassingNullToRefreshHasNoEffect: number of errors detected is " + errCnt);
		}

	}


	/* test evict (object pc) */
	public void testPassingNullToRefreshHasNoEffect() {
		try {
			tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = null;
			pm.refresh(p1);
			out.println(" \nPASSED in testPassingNullToRefreshHasNoEffect()");
			tx.rollback();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testPassingNullToRefreshHasNoEffect()");
			tx.rollback();
		}

	}
}
