/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.io.PrintWriter;
import java.util.*;

import javax.jdo.*;

public abstract class PersistenceManagerTest extends JDO_Test {

	public String MYLIB_PKG_NAME = "com.sun.jdotck.models.mylib";

	protected PersistenceManagerTest() {
	}

	public abstract Status runTest(String[] args, PrintWriter err, PrintWriter out);

	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		Status s = null;

		try {
			s = runTest(args, err, out);
		} catch (Exception ex) {
			out.println(" ** reset DB here **");
			ex.printStackTrace(out);
		} finally {
			try {
				cleanupDB();
			} catch (Exception ex) {
				ex.printStackTrace(out);
			} finally {
				out.flush();
				err.flush();
			}
		}
		return s;
	}

	public void cleanupDB() {
		cleanupMylib();
	}


	public void cleanupMylib() {
		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			Transaction tx = pm.currentTransaction();
			tx.begin();
			Collection c = getAllObjects("PCPoint", pm, MYLIB_PKG_NAME);
			pm.deletePersistentAll(c);
			tx.commit();

			tx.begin();
			c = getAllObjects("PCRect", pm, MYLIB_PKG_NAME);
			pm.deletePersistentAll(c);
			tx.commit();
			pm.close();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "initMylib()");
		}

	}

	public Object createPCPointInstance() {
		try {
			PCPoint p1 = new PCPoint(8, 8);
			pm = getPM();

			Transaction tx = pm.currentTransaction();
			tx.begin();
			pm.makePersistent(p1);
			Object oid = pm.getObjectId(p1);
			tx.commit();
			return oid;
		} catch (Exception ex) {
			assertUnexpectedException(ex, "createPCPointInstance()");
			return null;
		}
	}

	public void deletePCPointInstance(Object oid) {
		try {
			pm = getPM();
			Transaction tx = pm.currentTransaction();
			tx.begin();
			Object p1 = pm.getObjectById(oid, true);
			pm.deletePersistent(p1);
			tx.commit();
		} catch (Exception ex) {
			assertUnexpectedException(ex, "deletePCPointInstance()");
		}
	}


	public Collection getAllObjects(String tableName, PersistenceManager pm,
									String PKG_NAME) {
		Collection col = new Vector();
		String className = PKG_NAME + "." + tableName.trim();
		try {
			Class c = Class.forName(className);
			Query query = pm.newQuery();
			query.setClass(c);
			query.setCandidates(pm.getExtent(c, false));
			Object result = query.execute();
			col = (Collection) result;
		} catch (Exception e) {
			out.println("Exception in getAllObjects()");
			e.printStackTrace(out);
		}
		return col;
	}


	public boolean testState(PCPoint obj, int expectState, String str) {
		int actualState = currentState(obj);
		if (actualState != expectState) {
			out.println(" Object not in " + str + " state for X = " +
					+obj.getX());
			out.println(" current state: " + actualState + " expected state: " + expectState);
			return false;
		}
		return true;

	}

	public boolean testState(PCPoint2 obj, int expectState, String str) {
		int actualState = currentState(obj);
		if (actualState != expectState) {
			out.println(" Object not in " + str + " state for X = " +
					+obj.getX());
			out.println(" current state: " + actualState + " expected state: " + expectState);
			return false;
		}
		return true;

	}


	public void assertJDOUserException(Exception ex, String name) {
		if (ex instanceof JDOUserException) {
			; // expected result
			//ex.printStackTrace(out);
			//out.println ("PASSED");
		} else {
			out.println("FAILED in " + name);
			ex.printStackTrace(out);
			incrementErrorCount();
		}
	}

	public void assertJDOUnsupportedOptionException(Exception ex, String name) {
		if (ex instanceof JDOUnsupportedOptionException) {
			; // expected result
			//ex.printStackTrace(out);
			//out.println ("PASSED");
		} else {
			out.println("FAILED in " + name);
			ex.printStackTrace(out);
			incrementErrorCount();
			out.println("length: " + ((JDOException) ex).getNestedExceptions().length);
		}
	}

	public void assertJDODataStoreException(Exception ex, String name) {
		if (ex instanceof JDODataStoreException) {
			; // expected result
			//ex.printStackTrace(out);
			//out.println ("PASSED");
		} else {
			out.println("FAILED in " + name);
			ex.printStackTrace(out);
			incrementErrorCount();
		}
	}


	public void assertJDOFatalUserException(Exception ex, String name) {
		if (ex instanceof JDOFatalUserException) {
			; // expected results
			//ex.printStackTrace(out);
			//out.println ("PASSED");
		} else {
			out.println("FAILED in " + name);
			ex.printStackTrace(out);
			incrementErrorCount();
		}
	}

	public void assertUnexpectedException(Exception ex, String name) {
		out.println(" Unexpected exception caught in " + name);
		ex.printStackTrace(out);
		incrementErrorCount();
	}

	public void assertNullPointerException(Exception ex, String name) {
		if (ex instanceof NullPointerException) {
			//ex.printStackTrace(out);
			out.println("PASSED");
		} else {
			out.println("FAILED in " + name);
			ex.printStackTrace(out);
			incrementErrorCount();
		}
	}

	public void assertGetX(PCPoint p, int value, String name) {
		if (p.getX() != value) {
			incrementErrorCount();
		}
	}

}


