/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;
import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Refresh All With Array Side Effects
 *<BR>
 *<B>JavaTest Keywords:</B> cache
 *<BR>
 *<B>Assertion ID:</B> A12.5.1-5D
 *<BR>
 *<B>Assertion Description: </B>
 The refreshAll(Object[] pcs) updates the values in the parameter
 instance[s] from the data in the data store. ((The intended use is for optimistic transactions where the state of
 the JDO Instance is not guaranteed to reflect the state in the data store. This method can be used to minimize
 the occurrence of commit failures due to mismatch between the state of cached instances and the state of
 data in the data store.)) This can be tested by using 2 PersistenceManagers, independently change an object,
 then refresh.
 *@test
 *@title Refresh All With Array Side Effects
 *@executeClass com.sun.jdotck.api.persistencemanager.RefreshAllWithArraySideEffects
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source RefreshAllWithArraySideEffects.java
 *@keywords cache
 */


class RefreshArrThreadT1 implements Runnable {

	PrintWriter out;
	volatile int state = 0;
	static final int DELAY = 100;
	static boolean commitDone = false;
	static boolean doneFlag = false;
	Transaction tx1 = null;
	static PCPoint n1 = new PCPoint(5, 1);
	static PCPoint n2 = new PCPoint(5, 2);
	static PCPoint n3 = new PCPoint(5, 3);
	static PCPoint n4 = new PCPoint(5, 4);
	static PCPoint n5 = new PCPoint(5, 5);
	static PCPoint n6 = new PCPoint(5, 6);

	RefreshArrThreadT1(PrintWriter out) {
		tx1 = RefreshAllWithArraySideEffects.pm1.currentTransaction();
		tx1.setOptimistic(true);
		this.out = out;
	}

	public static void initVariables() {
		commitDone = false;
		doneFlag = false;
	}

	synchronized public void run() {
		try {
			state = 1;
			tx1.begin();
			n1.setX(500);
			n2.setX(501);

			Collection col1 = new java.util.HashSet();
			col1.add(n1);
			col1.add(n2);

			RefreshAllWithArraySideEffects.pm1.makePersistentAll(col1);
			RefreshAllWithArraySideEffects.pm1.refreshAll(col1.toArray());
			out.println(" setting in T1");

			while (!RefreshArrThreadT2.doneFlag) {
				Thread.sleep(DELAY);
			}

			tx1.commit();
			commitDone = true;
			out.println(" commit in T1 done");

		} catch (Exception ex) {
			out.println(" Exception in T1 run ");
			//ex.printStackTrace(out);
			state = 13;

		} finally {
			out.flush();
		}

	}

}

class RefreshArrThreadT2 implements Runnable {

	PrintWriter out;
	volatile int state = 1;
	static final int DELAY = 100;
	static boolean commitDone = false;
	static boolean doneFlag = false;
	Transaction tx2 = null;
	static PCPoint p1 = new PCPoint(5, 1);
	static PCPoint p2 = new PCPoint(5, 2);

	RefreshArrThreadT2(PrintWriter out) {
		tx2 = RefreshAllWithArraySideEffects.pm2.currentTransaction();
		tx2.setOptimistic(true);
		this.out = out;
	}

	public static void initVariables() {
		commitDone = false;
		doneFlag = false;
	}

	synchronized public void run() {
		try {
			tx2.begin();
			p1.setX(200);
			p2.setX(201);

			Collection col1 = new java.util.HashSet();
			col1.add(p1);
			col1.add(p2);
			out.println(" setting in T2 ");
			RefreshAllWithArraySideEffects.pm2.makePersistentAll(col1);
			RefreshAllWithArraySideEffects.pm2.refreshAll(col1.toArray());
			doneFlag = true;

			while (!RefreshArrThreadT1.commitDone) {
				Thread.sleep(DELAY);

			}
			tx2.commit();
			commitDone = true;
			out.println(" commit in T2 done ");

		} catch (Exception ex) {
			out.println(" Exception in T2 ");
			ex.printStackTrace(out);
			state = 10;

		} finally {
			out.flush();
		}

	}
}


public class RefreshAllWithArraySideEffects extends PersistenceManagerTest {

	static PersistenceManagerFactory pmf = null;
	static PersistenceManager pm1 = null;
	static PersistenceManager pm2 = null;

	static Transaction tx1 = null;
	static Transaction tx2 = null;


	public static void main(String[] args) {

		RefreshAllWithArraySideEffects t = new RefreshAllWithArraySideEffects();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();

		pm1 = pmf.getPersistenceManager();
		pm2 = pmf.getPersistenceManager();


		testRefreshAllWithArraySideEffects();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in RefreshAllWithArraySideEffects tests ");
		} else {
			return Status.failed("FAILED in RefreshAllWithArraySideEffects: number of errors detected is " + errCnt);
		}

	}

	public void testRefreshAllWithArraySideEffects() {
		try {

			RefreshArrThreadT1 thread1 = new RefreshArrThreadT1(out);
			Thread T1 = new Thread(thread1);
			RefreshArrThreadT2 thread2 = new RefreshArrThreadT2(out);
			Thread T2 = new Thread(thread2);

			T1.start();
			T2.start();

			T1.join();
			T2.join();

			if (thread1.state != 1 ||
					thread2.state != 1)
				incrementErrorCount();


		} catch (Exception ex) {
			assertUnexpectedException(ex, "testRefreshAllWithArraySideEffects()");
		}

	}

}
