/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import javax.jdo.*;

import com.sun.jdotck.models.mylib.PCPoint;

/**
 *<B>JavaTest Title:</B> Refresh All With No Parameters
 *<BR>
 *<B>JavaTest Keywords:</B> cache
 *<BR>
 *<B>Assertion ID:</B> A12.5.1-6.
 *<BR>
 *<B>Assertion Description: </B>
 The <code>PersistenceManager.refreshAll</code> method with no parameters
 causes all transactional instances to be refreshed. Note that this method
 will cause loss of changes made to affected instances by the application
 due to refreshing the contents from the data store.
 The JDO <code>PersistenceManager</code>:
 <UL>
 <LI>loads persistent values from the data store;</LI>
 <LI>loads persistent fields into the instance;</LI>
 <LI>calls the <code>jdoPostLoad</code> method on each persistent instance,
 if the class of the instance implements <code>InstanceCallbacks</code>; and</LI>
 <LI>changes the state of persistent-dirty instances to persistent-clean.</LI>
 </UL>

 *@test
 *@title Refresh All With No Parameters
 *@executeClass com.sun.jdotck.api.persistencemanager.RefreshAllWithNoParameters
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source RefreshAllWithNoParameters.java
 *@keywords cache
 */

public class RefreshAllWithNoParameters extends JDO_Test {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		RefreshAllWithNoParameters t = new RefreshAllWithNoParameters();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Transaction tx = getPM().currentTransaction();
		tx.begin();
		PCPoint p = new PCPoint(100, 200);
		pm.makePersistent(p);
		tx.commit();

		tx.begin();
		p.setX(500);
		p.setY(new Integer(800));
		pm.refreshAll();
		int currentX = p.getX();
		Integer currentY = p.getY();
		if ((currentX != 100) | !currentY.equals(new Integer(200))) {
			incrementErrorCount();
			out.println("RefreshAll expected 100, 200; got " + currentX + ", " + currentY);
		}
		tx.commit();

		if (getErrorCount() != 0)
			return Status.failed("Assertion A12.5.1-6 failed; values not refreshed");
		else
			return Status.passed("Assertion A12.5.1-6 passed");
	}
}
