/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Same Transaction Instance For All Calls To Current Transaction
 *<BR>
 *<B>JavaTest Keywords:</B>
 *<BR>
 *<B>Assertion ID:</B> A12.5.2-2.
 *<BR>
 *<B>Assertion Description: </B>
 The identical <code>Transaction</code> instance will be returned by all
 <code>currentTransaction</code> calls to the same <code>PersistenceManager</code>
 until <code>close</code>.

 *@test
 *@title Same Transaction Instance For All Calls To Current Transaction
 *@executeClass com.sun.jdotck.api.persistencemanager.SameTransactionInstanceForAllCallsToCurrentTransaction
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source SameTransactionInstanceForAllCallsToCurrentTransaction.java
 *@keywords
 */

public class SameTransactionInstanceForAllCallsToCurrentTransaction extends JDO_Test {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		SameTransactionInstanceForAllCallsToCurrentTransaction t = new SameTransactionInstanceForAllCallsToCurrentTransaction();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Transaction tx1 = getPM().currentTransaction();
		tx1.begin();
		Transaction tx2 = getPM().currentTransaction();
		tx1.commit();
		Transaction tx3 = getPM().currentTransaction();

		if (tx1 != tx2) {
			out.println("tx1 before begin different from tx2 after begin");
			incrementErrorCount();
		}
		if (tx2 != tx3) {
			out.println("tx2 after begin different from tx3 after commit");
			incrementErrorCount();
		}

		if (getErrorCount() == 0) {
			return Status.passed("Assertion A12.5.2-2 passed");
		} else {
			return Status.failed("Assertion A12.5.2-2 passed");
		}

	}
}
