/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Set IgnoreCache To False
 *<BR>
 *<B>JavaTest Keywords:cache
 *<BR>
 *<B>Assertion ID:</B> A12.5.3-3.
 *<BR>
 *<B>Assertion Description: </B>
 The PersistenceManager.setIgnoreCache method called with a value of false instructs the
 query engine that the user expects queries to return results that reflect changed values in the cache.


 *@test
 *@title Set IgnoreCache To False
 *@executeClass com.sun.jdotck.api.persistencemanager.SetIgnoreCacheToFalse
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetIgnoreCacheToFalse.java
 *@keywords cache
 */

public class SetIgnoreCacheToFalse extends PersistenceManagerTest {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;


	public static void main(String[] args) {

		SetIgnoreCacheToFalse t = new SetIgnoreCacheToFalse();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testSetIgnoreCacheToFalse();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in SetIgnoreCacheToFalse tests ");
		} else {
			return Status.failed("FAILED in SetIgnoreCacheToFalse: number of errors detected is " + errCnt);
		}

	}


	public void testSetIgnoreCacheToFalse() {

		try {

			PCPoint p1 = new PCPoint();

			Transaction tx = pm.currentTransaction();
			tx.begin();
			pm.setIgnoreCache(false);

			boolean returnValue = pm.getIgnoreCache();

			if (returnValue)
				incrementErrorCount();
			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "SetIgnoreCacheToFalse()");
		}

	}


}
