/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Set IgnoreCache To True
 *<BR>
 *<B>JavaTest Keywords:cache
 *<BR>
 *<B>Assertion ID:</B> A12.5.3-2.
 *<BR>
 *<B>Assertion Description: </B>
 The PersistenceManager.setIgnoreCache method called with a value of true is a hint to the
 query engine that the user expects queries to be optimized to return approximate results by ignoring changed values in the
 cache. This is not testable, except to see whether the get/set works.


 *@test
 *@title Set IgnoreCache To True
 *@executeClass com.sun.jdotck.api.persistencemanager.SetIgnoreCacheToTrue
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetIgnoreCacheToTrue.java
 *@keywords cache
 */

public class SetIgnoreCacheToTrue extends PersistenceManagerTest {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;


	public static void main(String[] args) {

		SetIgnoreCacheToTrue t = new SetIgnoreCacheToTrue();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testSetIgnoreCacheToTrue();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in SetIgnoreCacheToTrue tests ");
		} else {
			return Status.failed("FAILED in SetIgnoreCacheToTrue: number of errors detected is " + errCnt);
		}

	}


	public void testSetIgnoreCacheToTrue() {

		try {

			PCPoint p1 = new PCPoint();

			Transaction tx = pm.currentTransaction();
			tx.begin();
			pm.setIgnoreCache(true);

			boolean returnValue = pm.getIgnoreCache();

			if (!returnValue)
				incrementErrorCount();
			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, " SetIgnoreCacheToTrue()");
		}

	}


}
