/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Set Multithreaded True
 *<BR>
 *<B>JavaTest Keywords:</B> multithreaded
 *<BR>
 *<B>Assertion ID:</B> A12.7-1.
 *<BR>
 *<B>Assertion Description: </B>
 If PersistenceManager.setMultithreaded is called with a value of true, then the JDO
 implementation must perform synchronizations to support multiple application threads. A value of true will be returned
 when getMultithreaded is called. In testing, multi-threading should be turned on and then multi-threading tests should
 be run.
 *@test
 *@title Set Multithreaded True
 *@executeClass com.sun.jdotck.api.persistencemanager.SetMultithreadedTrue
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetMultithreadedTrue.java
 *@keywords multithreaded
 */

public class SetMultithreadedTrue extends PersistenceManagerTest {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;


	public static void main(String[] args) {

		SetMultithreadedTrue t = new SetMultithreadedTrue();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testSetMultithreadedTrue();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in SetMultithreadedTrue tests ");
		} else {
			return Status.failed("FAILED in SetMultithreadedTrue: number of errors detected is " + errCnt);
		}

	}


	public void testSetMultithreadedTrue() {

		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();
			pm.setMultithreaded(true);

			boolean returnValue = pm.getMultithreaded();

			if (returnValue) {
				; // expected result
			} else {
				incrementErrorCount();
			}
			tx.commit();

		} catch (Exception ex) {
			assertUnexpectedException(ex, "SetMultithreadedTrue()");
		}

	}


}
