/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Thread Safe
 *<BR>
 *<B>JavaTest Keywords:</B> multithreaded
 *<BR>
 *<B>Assertion ID:</B> A12.4-1.
 *<BR>
 *<B>Assertion Description: </B>
 It is a requirement for all JDO implementations to be thread-safe. That is, the behavior of the implementation must be predictable in the presence of multiple application threads.  This assertion will generate multiple test cases to be evaluated.

 *@test
 *@title Thread Safe
 *@executeClass com.sun.jdotck.api.persistencemanager.ThreadSafe
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source ThreadSafe.java
 *@keywords multithreaded
 */


public class ThreadSafe extends PersistenceManagerTest {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	PCPoint p1 = null;
	private int totalThreadCount = 10;
	private int completedThreadCount = 0;
	private int succeeds = 0;
	private int catchNumber = 0;

	private String expectedErrorText = " expected error here";

	public static void main(String[] args) {

		ThreadSafe t = new ThreadSafe();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);

		testThreadSafe();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in ThreadSafe tests ");
		} else {
			return Status.failed("FAILED in ThreadSafe: number of errors detected is " + errCnt);
		}

	}


	public void testThreadSafe() {

		try {

			p1 = new PCPoint(3, 3);

			pmf = getPMF();

			for (int i = 0; i < totalThreadCount; i++) {
				Thread t = new Thread(new PMThread(pmf, p1, this));
				t.setName("ThreadSafeID-" + i);
				t.start();
			}

			while (totalThreadCount != completedThreadCount) {
				try {
					Thread.currentThread().sleep(100);
				} catch (InterruptedException ex) {
					out.println("Interrupted exception ! ");
				}
			}

			checkResults();

		} catch (Exception tex) {
			assertUnexpectedException(tex, "testThreadSafe()");
		}

	}

	protected synchronized void complete() {
		completedThreadCount++;
	}

	protected synchronized void winning(PCPoint pc) {
		out.println("*** ThreadID : " + Thread.currentThread().getName() + " succeeds");
		succeeds++;
	}


	class PMThread implements Runnable {
		private final PersistenceManager pm;
		private final Object pc;
		private final ThreadSafe owner;

		PMThread(PersistenceManagerFactory pmf, PCPoint pc,
				 ThreadSafe owner) {

			this.pm = pmf.getPersistenceManager();
			this.pc = pc;
			this.owner = owner;
		}

		public void run() {
			Transaction tx = null;
			try {

				out.println(" Running thread: " + Thread.currentThread().getName());
				tx = pm.currentTransaction();
				tx.begin();
				pm.makePersistent(pc);
				tx.commit();
				winning((PCPoint) pc);
				complete();

				while (totalThreadCount != completedThreadCount) {
					try {
						Thread.currentThread().sleep(100);
					} catch (InterruptedException ex) {
						out.println(" interrupted");
					}
				}
			} catch (Exception ex) {
				catchNumber++;
				assertJDOUserException(ex, "PMThread.run()");
				complete();
			} finally {
				try {
					if (tx.isActive()) {
						tx.rollback();
					}
					pm.close();
				} catch (Exception fex) {
					assertUnexpectedException(fex, "PMThread()");
				}

			}
		}		// run

	}  // class PMThread


	public void checkResults() {
		int num = totalThreadCount - 1;
		if (succeeds == 1 &&
				catchNumber == num) {
			; // expected result
		} else {
			incrementErrorCount();
		}
	}

}
