/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanager;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Transient Transactional Instance Retains Values At Commit
 *<BR>
 *<B>JavaTest Keywords:</B> transienttransactional
 *<BR>
 *<B>Assertion ID:</B> A12.5.7-21
 *<BR>
 *<B>Assertion Description: </B>
 If the transaction in which an instance is made transactional (by calling PersistenceManager.makeTransactional or makeTransactionalAll) commits, then the transient instance retains its values.

 *@test
 *@title Transient Transactional Instance Retains Values At Commit
 *@executeClass com.sun.jdotck.api.persistencemanager.TransientTransactionalInstanceRetainsValuesAtCommit
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source TransientTransactionalInstanceRetainsValuesAtCommit.java
 *@keywords transienttransactional
 */

public class TransientTransactionalInstanceRetainsValuesAtCommit extends PersistenceManagerTest {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;
	PCPoint p4 = null;
	PCPoint p5 = null;

	Collection col1 = new java.util.HashSet();
	Collection col2 = new java.util.HashSet();


	public static void main(String[] args) {

		TransientTransactionalInstanceRetainsValuesAtCommit t = new TransientTransactionalInstanceRetainsValuesAtCommit();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {


		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testTransientTransactionalInstanceRetainsValuesAtCommit();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed("PASSED in TransientTransactionalInstanceRetainsValuesAtCommit tests ");
		} else {
			return Status.failed("FAILED in TransientTransactionalInstanceRetainsValuesAtCommit: number of errors detected is " + errCnt);
		}

	}

	public void testTransientTransactionalInstanceRetainsValuesAtCommit() {
		createObjects();
		testTransientTransactional1();
		testTransientTransactional2();
		testTransientTransactional3();
	}


	private void createObjects() {
		try {
			p1 = new PCPoint(1, 3);
			p2 = new PCPoint(2, 4);
			p3 = new PCPoint(3, 5);
			p4 = new PCPoint(4, 6);
			p5 = new PCPoint(5, 7);

			col1.add(p2);
			col1.add(p3);

			col2.add(p4);
			col2.add(p5);

		} catch (Exception ex) {
			out.println(" - createTcleanObjects() FAILED");
		}
	}


	public void testTransientTransactional1() {

		out.println(" ** in testTransientTransactional1() ");
		try {

			PCPoint p1 = new PCPoint(8, 8);

			tx = pm.currentTransaction();

			p1.setX(100);
			tx.begin();
			p1.setX(200);
			pm.makeTransactional(p1);
			p1.setX(300);
			tx.commit();

			assertGetX(p1, 300, "testTransientTransactional1");

		} catch (Exception ex) {
			assertUnexpectedException(ex,
					"testTransientTransactional1");
		}

	}

	public void testTransientTransactional2() {

		out.println(" ** in testTransientTransactional2() ");
		try {

			tx = pm.currentTransaction();

			p2.setX(100);
			p3.setX(100);
			tx.begin();
			p2.setX(201);
			p3.setX(201);
			pm.makeTransactionalAll(col1);
			p2.setX(301);
			p3.setX(301);
			tx.commit();

			assertGetX(p2, 301, "testTransientTransactional2");
			assertGetX(p3, 301, "testTransientTransactional2");

		} catch (Exception ex) {
			assertUnexpectedException(ex,
					"testTransientTransactional2");
		}

	}

	public void testTransientTransactional3() {

		out.println(" ** in testTransientTransactional3() ");
		try {

			tx = pm.currentTransaction();

			p4.setX(100);
			p5.setX(100);
			tx.begin();
			p4.setX(201);
			p5.setX(201);
			pm.makeTransactionalAll(col2.toArray());
			p4.setX(301);
			p5.setX(301);
			tx.commit();

			assertGetX(p4, 301, "testTransientTransactional2");
			assertGetX(p5, 301, "testTransientTransactional2");

		} catch (Exception ex) {
			assertUnexpectedException(ex,
					"testTransientTransactional3");
		}

	}


}
