/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.api.persistencemanager;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B>  ValuesNotRetainedAfterEvictionAndTransactionCompletion
 *<BR>
 *<B>JavaTest Keywords:</B> cache
 *<BR>
 *<B>Assertion IDs:</B> A12.5.1-1.
 *<BR>
 *<B>Assertion Description: </B>
 Evicted instances (via PersistenceManager.evict or evictAll) will not have their values retained after transaction completion, regardless of the setting of the retainValuesflag. They should be in the hollow state after transaction completion.

 *@test
 *@title Values Not Retained After Eviction And Transaction Completion
 *@executeClass com.sun.jdotck.api.persistencemanager.ValuesNotRetainedAfterEvictionAndTransactionCompletion
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source  ValuesNotRetainedAfterEvictionAndTransactionCompletion.java
 *@keywords cache
 */

public class ValuesNotRetainedAfterEvictionAndTransactionCompletion extends PersistenceManagerTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private Transaction tx;

	PCPoint pnt1 = null;
	PCPoint pnt2 = null;
	PCPoint p1 = null;
	PCPoint p2 = null;
	PCPoint p3 = null;


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		ValuesNotRetainedAfterEvictionAndTransactionCompletion t = new ValuesNotRetainedAfterEvictionAndTransactionCompletion();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	public Status runTest(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testValuesNotRetainedAfterEvictionAndTransactionCompletion();

		pm.close();

		int errCnt = getErrorCount();
		if (errCnt == 0) {
			return Status.passed(" ValuesNotRetainedAfterEvictionAndTransactionCompletion tests passed");
		} else {
			return Status.failed(" ValuesNotRetainedAfterEvictionAndTransactionCompletion: number of errors detected is " + errCnt);
		}

	}

	public void testValuesNotRetainedAfterEvictionAndTransactionCompletion() {
		createObjects();
		testEvict();
		testEvictAllCollection();
		testEvictAllArray();

	}


	private void createObjects() {
		try {
			pnt1 = new PCPoint(1, 2);
			pnt2 = new PCPoint(2, 3);
			p1 = new PCPoint(3, 1);
			p2 = new PCPoint(4, 2);
			p3 = new PCPoint(5, 3);
		} catch (Exception ex) {
			assertUnexpectedException(ex, "createObjects()");
		}
	}

	public void testEvict() {
		out.println("**  in testEvict() ");
		try {
			tx = pm.currentTransaction();
			tx.setRetainValues(true);
			tx.begin();

			pm.makePersistent(p3);

			Object oid1 = pm.getObjectId(p3);

			pm.evict(p3);
			tx.commit();

			if (testValue(p3, oid1)) {
				; // expected result
			} else {
				incrementErrorCount();
			}

			if (testState(p3, HOLLOW, "hollow")) {
				; // expected result
			} else {
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testEvict()");
			if (tx.isActive())
				tx.rollback();
		}

	}


	public void testEvictAllCollection() {
		out.println("**  in testEvictAllCollection() ");
		try {
			tx = pm.currentTransaction();
			tx.setRetainValues(true);
			tx.begin();

			pm.makePersistent(pnt1);
			pm.makePersistent(p1);

			Object oid1 = pm.getObjectId(p1);
			Object oid2 = pm.getObjectId(pnt1);

			Collection col1 = new java.util.HashSet();
			col1.add(pnt1);
			col1.add(p1);

			pm.evictAll(col1);
			tx.commit();

			if (testValue(p1, oid1) &&
					testValue(pnt1, oid2)) {
				; // expected result
			} else {
				incrementErrorCount();
			}


			if (testState(p1, HOLLOW, "hollow") &&
					testState(pnt1, HOLLOW, "hollow")) {
				; // expected result
			} else {
				incrementErrorCount();
			}
		} catch (Exception ex) {
			assertUnexpectedException(ex, "testEvictAll()");
			if (tx.isActive())
				tx.rollback();
		}

	}


	public void testEvictAllArray() {
		out.println("**  in testEvictAllArray() ");
		try {
			tx = pm.currentTransaction();
			tx.setRetainValues(true);
			tx.begin();

			Collection col1 = new java.util.HashSet();
			col1.add(pnt2);
			col1.add(p2);

			pm.makePersistent(pnt2);
			pm.makePersistent(p2);

			Object oid1 = pm.getObjectId(p2);
			Object oid2 = pm.getObjectId(pnt2);

			pm.evictAll(col1.toArray());

			tx.commit();

			if (testValue(p2, oid1) &&
					testValue(pnt2, oid2)) {
				; // expected result
			} else {
				incrementErrorCount();
			}

			if (testState(p2, HOLLOW, "hollow") &&
					testState(pnt2, HOLLOW, "hollow")) {
				; // expected result
			} else {
				incrementErrorCount();
			}

		} catch (Exception ex) {
			assertUnexpectedException(ex, "testEvictAllArray()");
			if (tx.isActive())
				tx.rollback();
		}

	}

	public boolean testValue(PCPoint p, Object expectedOid) {

		Object oid = pm.getObjectId(p);
		if (oid == expectedOid) {
			out.println("Value retained !! ");
			return false;
		} else {
			return true;   // value not retained
		}

	}


}
