/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanagerfactory;

import com.sun.javatest.Status;
import com.sun.javatest.Test;
import com.sun.jdotck.JDO_Test;

import java.io.*;
import java.lang.*;
import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B>GetPersistenceManager of PersistenceManagerFactory
 *<BR>
 *<B>JavaTest Keywords:</B> persistencemanagerfactory
 *<BR>
 *<B>Assertion IDs:</B> A11.3-1.
 *<BR>
 *<B>Assertion Description: </B>
 * PersistenceManagerFactory.getPersistenceManager() returns a
 PersistenceManager instance with the configured properties and the
 default values for option settings.
 *@test
 *@title GetPersistenceManager
 *@executeClass com.sun.jdotck.api.persistencemanagerfactory.GetPersistenceManager
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetPersistenceManager.java
 *@keywords
 */

/*
 * Revision History
 * ================
 * Author         :   Linga Neerathilingam
 * Date   :  10/22/01
 *
 */

public class GetPersistenceManager extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private String pmfClass;
	private String url;
	private String username;
	private String password;

	private static String PMFCLASS = "javax.jdo.PersistenceManagerFactoryClass";
	private static String URL = "javax.jdo.option.ConnectionURL";
	private static String USERNAME = "javax.jdo.option.ConnectionUserName";
	private static String PASSWORD = "javax.jdo.option.ConnectionPassword";

	/* standalone interface */
	public static void main(String[] args) throws Exception {
		GetPersistenceManager t = new GetPersistenceManager();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}


	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Properties props = loadProperties(PMFProperties);
		pmfClass = props.getProperty(PMFCLASS);
		url = props.getProperty(URL);
		username = props.getProperty(USERNAME);
		password = props.getProperty(PASSWORD);

		try {
			Class cl = Class.forName(pmfClass);
			pmf = (PersistenceManagerFactory) cl.newInstance();
			pmf.setConnectionURL(url);
			pmf.setConnectionUserName(username);
			pmf.setConnectionPassword(password);
			pm = pmf.getPersistenceManager();
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("Failed in getting the pm.");
		} finally {
			if (pm != null) pm.close();
		}
		out.println("Persistence Manager obtained: " + pm);
		out.flush();

		return Status.passed("\n GetPersistenceManagerForUser() passed.");
	}
}
