/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanagerfactory;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B>Get properties of PersistenceManagerFactory
 *<BR>
 *<B>JavaTest Keywords:</B> persistencemanagerfactory
 *<BR>
 *<B>Assertion IDs:</B> A11.4-1.
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManagerFactory.getProperties() returns a Properties instance
 containing two standard JDO implementation properties
 VendorName: The name of the JDO vendor.
 VersionNumber: The version number string.
 *@test
 *@title get properties
 *@executeClass com.sun.jdotck.api.persistencemanagerfactory.GetProperties
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetProperties.java
 *@keywords
 */

public class GetProperties extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private int foundStandardProperties = 0;

	/* standalone interface */
	public static void main(String[] args) throws Exception {
		GetProperties t = new GetProperties();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}


	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		pmf = getPMF();

		try {
			Properties p = pmf.getProperties();
			for (Enumeration e = p.propertyNames(); e.hasMoreElements();) {
				String s = (String) e.nextElement();
				out.println("\t" + s + ": " + p.getProperty(s));

				if (s.equals("VendorName") || s.equals("VersionNumber")) {
					foundStandardProperties++;
				}
			}
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("Failed in getProperties()");
		}
		out.flush();

		if (foundStandardProperties == 2) {
			return Status.passed("\n getProperties passed.");
		} else {
			return Status.failed("\n Standard properties not found.");
		}
	}
}
