/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanagerfactory;

import com.sun.javatest.Status;
import com.sun.javatest.Test;
import com.sun.jdotck.JDO_Test;

import java.io.*;
import java.lang.*;
import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B>Set ConnectionURL of PersistenceManagerFactory
 *<BR>
 *<B>JavaTest Keywords:</B> persistencemanagerfactory
 *<BR>
 *<B>Assertion IDs:</B> A11.1-16,A11.1-17.
 *<BR>
 *<B>Assertion Description: </B>
 * PersistenceManagerFactory.setConnectionURL(String URL) sets the value of the ConnectionURL property (the URL for the data source).
 * PersistenceManagerFactory.getConnectionURL() returns the value of the ConnectionURL property.
 *@test
 *@title set connection url
 *@executeClass com.sun.jdotck.api.persistencemanagerfactory.SetConnectionURL
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetConnectionURL.java
 *@keywords
 */

public class SetConnectionURL extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private String pmfClass;
	private String url;
	private String username;
	private String password;

	private static String PMFCLASS = "javax.jdo.PersistenceManagerFactoryClass";
	private static String URL = "javax.jdo.option.ConnectionURL";
	private static String USERNAME = "javax.jdo.option.ConnectionUserName";
	private static String PASSWORD = "javax.jdo.option.ConnectionPassword";

	/* standalone interface */
	public static void main(String[] args) throws Exception {
		SetConnectionURL t = new SetConnectionURL();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

/* set ConnectionURL value and get ConnectionURL value to verify */

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Properties props = loadProperties(PMFProperties);
		pmfClass = props.getProperty(PMFCLASS);
		url = props.getProperty(URL);
		username = props.getProperty(USERNAME);
		password = props.getProperty(PASSWORD);

		try {
			Class cl = Class.forName(pmfClass);
			pmf = (PersistenceManagerFactory) cl.newInstance();
			pmf.setConnectionURL(url);
			if (!url.equals(pmf.getConnectionURL())) {
				return Status.failed("ConnectionURL " + url +
						" not equal to value returned by PMF " +
						pmf.getConnectionURL());
			}
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("Failed in setting ConnectionURL.");
		}
		out.println("ConnectionURL: " + pmf.getConnectionURL());
		out.flush();

		return Status.passed("\n SetConnectionURL() passed.");
	}
}
