/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanagerfactory;

import com.sun.javatest.Status;
import com.sun.javatest.Test;
import com.sun.jdotck.JDO_Test;

import java.io.*;
import java.lang.*;
import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B>Set ConnectionUserName of PersistenceManagerFactory
 *<BR>
 *<B>JavaTest Keywords:</B> persistencemanagerfactory
 *<BR>
 *<B>Assertion IDs:</B> A11.1-13.
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManagerFactory.setConnectionUserName(String name) sets the
 value of the ConnectionUserName property (the name of the user establishing the connection).
 *@test
 *@title set connection username
 *@executeClass com.sun.jdotck.api.persistencemanagerfactory.SetConnectionUserName
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetConnectionUserName.java
 *@keywords
 */

public class SetConnectionUserName extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private String pmfClass;
	private String url;
	private String username;
	private String password;

	private static String PMFCLASS = "javax.jdo.PersistenceManagerFactoryClass";
	private static String URL = "javax.jdo.option.ConnectionURL";
	private static String USERNAME = "javax.jdo.option.ConnectionUserName";
	private static String PASSWORD = "javax.jdo.option.ConnectionPassword";

	/* standalone interface */
	public static void main(String[] args) throws Exception {
		SetConnectionUserName t = new SetConnectionUserName();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

/* set ConnectionUserName value and get ConnectionUserName value to verify */

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Properties props = loadProperties(PMFProperties);
		pmfClass = props.getProperty(PMFCLASS);
		url = props.getProperty(URL);
		username = props.getProperty(USERNAME);
		password = props.getProperty(PASSWORD);

		try {
			Class cl = Class.forName(pmfClass);
			pmf = (PersistenceManagerFactory) cl.newInstance();
			pmf.setConnectionUserName(username);
			if (!username.equals(pmf.getConnectionUserName())) {
				return Status.failed("ConnectionUserName " + username +
						" not equal to value returned by PMF " +
						pmf.getConnectionUserName());
			}
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("Failed in setting ConnectionUserName.");
		}
		out.println("ConnectionUserName: " + pmf.getConnectionUserName());
		out.flush();

		return Status.passed("\n SetConnectionUserName() passed.");
	}
}
