/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanagerfactory;

import java.io.*;
import java.lang.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B>Set NonTransactionalRead of persistencemanagerfactory
 *<BR>
 *<B>JavaTest Keywords:</B> persistencemanagerfactory
 *<BR>
 *<B>Assertion IDs:</B> A11.1-7.
 *<BR>
 *<B>Assertion Description: </B>
 *PersistenceManagerFactory.setNontransactionalRead(boolean flag) sets the value of the NontransactionalRead property (the PersistenceManager mode that allows instances to be read outside a transaction).
 *@test
 *@title set NonTransactionalRead
 *@executeClass com.sun.jdotck.api.persistencemanagerfactory.SetNonTransactionalRead
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetNonTransactionalRead.java
 *@keywords
 */

/*
 * Revision History
 * ================
 * Author         :   Linga Neerathilingam
 * Date   :  10/15/01
 * Version  :     1.0
 *
 */

public class SetNonTransactionalRead extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private String pmfClass;
	private String url;
	private String username;
	private String password;

	private static String PMFCLASS = "javax.jdo.PersistenceManagerFactoryClass";
	private static String URL = "javax.jdo.option.ConnectionURL";
	private static String USERNAME = "javax.jdo.option.ConnectionUserName";
	private static String PASSWORD = "javax.jdo.option.ConnectionPassword";

	/* standalone interface */
	public static void main(String[] args) throws Exception {
		SetNonTransactionalRead t = new SetNonTransactionalRead();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

/* set NonTransactionalRead to true or false and use getNonTransactionalRead value to verify */

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Properties props = loadProperties(PMFProperties);
		pmfClass = props.getProperty(PMFCLASS);
		url = props.getProperty(URL);
		username = props.getProperty(USERNAME);
		password = props.getProperty(PASSWORD);

		try {
			Class cl = Class.forName(pmfClass);
			pmf = (PersistenceManagerFactory) cl.newInstance();
			pmf.setNontransactionalRead(false);
			if (pmf.getNontransactionalRead() != false) {
				return Status.failed("NonTransactionalRead set to false, value returned by PMF is " +
						pmf.getNontransactionalRead());
			}
			pmf.setNontransactionalRead(true);
			if (pmf.getNontransactionalRead() != true) {
				return Status.failed("NonTransactionalRead set to true, value returned by PMF is " +
						pmf.getNontransactionalRead());
			}
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("Failed in setting NonTransactionalRead.");
		}
		out.flush();

		return Status.passed("\n SetNonTransactionalRead() passed.");
	}
}
