/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.api.persistencemanagerfactory;

import java.io.*;
import java.lang.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B>Set optimistic of persistencemanagerfactory
 *<BR>
 *<B>JavaTest Keywords:</B> persistencemanagerfactory
 *<BR>
 *<B>Assertion IDs:</B> A11.1-1, A11.1-2.
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManagerFactory.getOptimistic() returns Value of the Optimistic property,persistenceManagerFactory.setOptimistic(boolean flag) sets
 the value of the Optimistic property (the transaction mode that specifies concurrency control
 *@test
 *@title set optimistic
 *@executeClass com.sun.jdotck.api.persistencemanagerfactory.SetOptimistic
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetOptimistic.java
 *@keywords
 */

public class SetOptimistic extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private String pmfClass;
	private String url;
	private String username;
	private String password;

	private static String PMFCLASS = "javax.jdo.PersistenceManagerFactoryClass";
	private static String URL = "javax.jdo.option.ConnectionURL";
	private static String USERNAME = "javax.jdo.option.ConnectionUserName";
	private static String PASSWORD = "javax.jdo.option.ConnectionPassword";

	/* standalone interface */
	public static void main(String[] args) throws Exception {
		SetOptimistic t = new SetOptimistic();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

/* set Optimistic to true or false and use getOptimistic value to verify */

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Properties props = loadProperties(PMFProperties);
		pmfClass = props.getProperty(PMFCLASS);
		url = props.getProperty(URL);
		username = props.getProperty(USERNAME);
		password = props.getProperty(PASSWORD);

		try {
			Class cl = Class.forName(pmfClass);
			pmf = (PersistenceManagerFactory) cl.newInstance();
			pmf.setOptimistic(false);
			if (pmf.getOptimistic() != false) {
				return Status.failed("Optimistic set to false, value returned by PMF is " +
						pmf.getOptimistic());
			}
			pmf.setOptimistic(true);
			if (pmf.getOptimistic() != true) {
				return Status.failed("Optimistic set to true, value returned by PMF is " +
						pmf.getOptimistic());
			}
		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("Failed in setting Optimistic.");
		}
		out.flush();

		return Status.passed("\n SetOptimistic() passed.");
	}
}
