/*
 * EnhancerTest.java
 *
 * Created on February 17, 2002, 1:59 PM
 */

package com.sun.jdotck.enhancement;

import java.util.Properties;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import com.sun.jdotck.JDO_Test;

import com.sun.jdori.enhancer.meta.EnhancerMetaData;
import com.sun.jdori.enhancer.meta.EnhancerMetaDataUserException;
import com.sun.jdori.enhancer.meta.EnhancerMetaDataFatalError;
import com.sun.jdori.enhancer.meta.model.EnhancerMetaDataJDOModelImpl;
import com.sun.jdori.enhancer.meta.prop.EnhancerMetaDataPropertyImpl;

/**
 *
 * @author  Craig Russell
 * @version 1.0
 */
public abstract class EnhancerTest extends JDO_Test {

	boolean verbose = false;

	/** Creates new EnhancerTest */
	public EnhancerTest() {
	}

	protected Properties getProperties(String resourceName) {
		Properties props = null;
		try {
			InputStream in = this.getClass().getClassLoader().getResourceAsStream(resourceName);
			props = new Properties();
			props.load(in);
		} catch (Exception ex) {
			out.flush();
			System.out.flush();
			err.println("EnhancerTest: Error loading properties " + resourceName);
			ex.printStackTrace(err);
		}
		return props;
	}

	protected abstract void testOnePackage(String packageName, List fullyQualifiedClassNameList);

	protected abstract String convertClassName(String packageName, String className);

	void testAllPackages() {
		/** First, get classes to test from properties file.
		 */
		Properties classesToTest = getProperties("enhancement-test.properties"); //NOI18N

		Enumeration enum = classesToTest.propertyNames();
		int numberOfPackages = 0;

		/** Each key is a package name; the value is a list of class names to test.
		 */
		while (enum.hasMoreElements()) {
			++numberOfPackages;
			String packageName = (String) enum.nextElement();
			out.println("EnhancerTest Package: " + packageName);
			String classNames = (String) classesToTest.get(packageName);
			out.println("EnhancerTest Classes: " + classNames);
			StringTokenizer st = new StringTokenizer(classNames, " ,");
			ArrayList classNameList = new ArrayList();
			/** Each entry is a list of class names separated by comma or space
			 */
			while (st.hasMoreTokens()) {
				String className = st.nextToken();
				String listEntry = convertClassName(packageName, className);
				classNameList.add(listEntry);
				out.println("EnhancerTest Class: " + className);
			}
			incrementTestCount(classNameList.size());
			testOnePackage(packageName, classNameList);
		}
		out.println("EnhancerTest numberOfPackages: " + numberOfPackages);
		return;
	}

	EnhancerMetaData getEnhancerMetaData(String packageName) {

		String packageFileName = packageName.replace('.', '/');
		String jdoPropertiesFileName = packageFileName + "/jdoTest.properties"; //NOI18N
		out.println("EnhancerTest.getEnhancerMetaData properties resource name: " + jdoPropertiesFileName);
		Properties metaDataProperties = getProperties(jdoPropertiesFileName);
		out.println("EnhancerTest.getEnhancerMetaData got properties: " + metaDataProperties);
		EnhancerMetaData jdoMetaData = null;
		if (jdoPropertiesFileName != null) {
			try {
				jdoMetaData
						= new EnhancerMetaDataPropertyImpl(out, verbose,
								metaDataProperties);
			} catch (EnhancerMetaDataFatalError ex) {
				out.flush();
				System.out.flush();
				err.println("EnhancerTest: Cannot read meta-data properties " + jdoPropertiesFileName);
				err.println("EnhancerTest: EnhancerMetaDataFatalError caught:" + ex);
				ex.printStackTrace(err);
				return null;
			} catch (Exception ex) {
				out.flush();
				System.out.flush();
				out.println("EnhancerTest: Cannot open meta-data properties " + jdoPropertiesFileName);
				out.println("EnhancerTest: Exception caught:" + ex);
				ex.printStackTrace(out);
				return null;
			}
			if (verbose) {
				out.println(
						"EnhancerTest: using meta-data from properties: "
						+ jdoPropertiesFileName);
			}
			return jdoMetaData;

		} else {
			err.println("EnhancerTest: No properties: " + jdoPropertiesFileName);
			return null;
		}
	}

}
