package com.sun.jdotck.enhancement;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.List;

import com.sun.jdori.enhancer.test.AnnotationTest;
import com.sun.jdori.enhancer.meta.EnhancerMetaData;
import com.sun.jdori.enhancer.meta.EnhancerMetaDataUserException;
import com.sun.jdori.enhancer.meta.EnhancerMetaDataFatalError;

/**
 *<B>JavaTest Title:</B> FieldAccessModified
 *<BR>
 *<B>JavaTest Keywords:</B> enhancement
 *<BR>
 *<B>Assertion ID:</B> .
 *<BR>
 *<B>Assertion Description: </B>
 The enhancer modifies field accesses to guarantee that the values of fields
 are retrieved from the data store prior to application usage.
 <OL TYPE="A">
 <LI>For any field access that reads the value of a field, the getfield byte code
 is replaced with a call to a generated local method, <code>jdoGetXXX</code></LI>
 <LI>For any field access that stores the new value of a field, the putfield
 byte code is replaced with a call to a generated local method,
 <code>jdoSetXXX</code></LI>
 </OL>

 *@test
 *@title FieldAccessModified
 *@executeClass com.sun.jdotck.enhancement.FieldAccessModified
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source FieldAccessModified.java
 *@keywords enhancement
 */

public class FieldAccessModified extends EnhancerTest {

	String workDir = null;

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		FieldAccessModified t = new FieldAccessModified();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		out.println("com.sun.jdotck.enhancement.FieldAccessModified.run");

		initializeArgs(args, err, out);
		PMFPropertiesObject = loadProperties(PMFProperties); // will exit here if no properties

		testAllPackages();

		cleanup();
		if (getErrorCount() == 0) {
			return Status.passed("Assertion A20.3-0 passed " + getTestCount() + " tests");
		} else {
			return Status.failed("Assertion A20.3-0 failed, " + getErrorCount() +
					" failed tests out of " + getTestCount());
		}
	}

	protected void testOnePackage(String packageName, List fullyQualifiedClassNames) {
		out.println("ImplementsPersistenceCapable.testOnePackage: " + packageName + fullyQualifiedClassNames);
		out.flush();
		EnhancerMetaData metaData = getEnhancerMetaData(packageName);
		out.println("ImplementPersistenceCapable.testOnePackage got metaData: " + metaData);

		try {
			final AnnotationTest test = new AnnotationTest(metaData);
			final int res = test.test(out, verbose, fullyQualifiedClassNames);
			incrementErrorCount(res);
		} catch (EnhancerMetaDataUserException ex) {
			err.println("Problem with reading JDO meta-data;");
			err.println("Exception caught:" + ex);
			ex.printStackTrace(err);
		} catch (EnhancerMetaDataFatalError ex) {
			err.println("Problem with reading JDO meta-data;");
			err.println("Exception caught:" + ex);
			ex.printStackTrace(err);
		} catch (RuntimeException ex) {
			err.println("Internal error;");
			err.println("Exception caught:" + ex);
			ex.printStackTrace(err);
		}
	}

	protected String convertClassName(String packageName, String className) {
		String classFileName = packageName.replace('.', '/') + "/" + className + ".class";
		out.println("FieldAccessModified: classFileName: " + classFileName);
		return classFileName;
	}

}
