/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.extents;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.Iterator;
import java.util.Collection;

import javax.jdo.*;

import com.sun.jdotck.models.company.Employee;

/**
 *<B>JavaTest Title:</B> Close of Extent Iterator is Iterator Specific
 *<BR>
 *<B>JavaTest Keywords:</B> extent
 *<BR>
 *<B>Assertion ID:</B> A15.3-11.
 *<BR>
 *<B>Assertion Description: </B>
 After a call to <code>Extent.close(Iterator i)</code>,
 the <code>Extent</code> itself can still be used to acquire other
 iterators and can be used as the <code>Extent</code> for queries.

 *@test
 *@title Close of Extent Iterator is Iterator Specific
 *@executeClass com.sun.jdotck.extents.CloseOfExtentIteratorIsIteratorSpecific
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source CloseOfExtentIteratorIsIteratorSpecific.java
 *@keywords extent
 */

public class CloseOfExtentIteratorIsIteratorSpecific extends ExtentTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		CloseOfExtentIteratorIsIteratorSpecific t = new CloseOfExtentIteratorIsIteratorSpecific();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Extent ex = getExtent();
		Iterator it1 = ex.iterator();
		ex.close(it1);
		int count = countIterator(ex.iterator());
		if (count != 2)
			return Status.failed("Assertion A15.3-11 failed iterating Employees after close of first iterator; counted " + count + " instances; should be 2");
		Query q = getPM().newQuery(ex);
		Collection c = (Collection) q.execute();
		int count2 = countIterator(c.iterator());
		if (count2 != 2)
			return Status.failed("Assertion A15.3-11 failed in query after closing iterator; counted " + count + " instances; should be 2");
		return Status.passed("Assertion A15.3-11 passed");

	}

}
