/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.extents;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.Iterator;
import java.util.Date;

import com.sun.jdotck.models.company.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Instances Deleted Prior to Iteration not Returned
 *<BR>
 *<B>JavaTest Keywords:</B> extent
 *<BR>
 *<B>Assertion ID:</B> A15.3-3.
 *<BR>
 *<B>Assertion Description: </B>
 If instances were deleted in the transaction prior to the execution of
 <code>Extent.iterator()</code>, the returned <code>Iterator</code>
 will not contain the instances.

 *@test
 *@title Instances Deleted Prior to Iteration not Returned
 *@executeClass com.sun.jdotck.extents.InstancesDeletedPriorToIterationNotReturned
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source InstancesDeletedPriorToIterationNotReturned.java
 *@keywords extent
 */

public class InstancesDeletedPriorToIterationNotReturned extends ExtentTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		InstancesDeletedPriorToIterationNotReturned t = new InstancesDeletedPriorToIterationNotReturned();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {
			beginTransaction();
			getPM().setIgnoreCache(false);
			Extent ex = getPM().getExtent(Employee.class, true);
			Iterator it1 = ex.iterator();
			deleteEmployee((Employee) it1.next());
			Iterator it2 = ex.iterator();
			int count = countIterator(it2);
			rollbackTransaction();

			Iterator it3 = ex.iterator();
			int count3 = countIterator(it3);

			if (count == 1 & count3 == 2)
				return Status.passed("Assertion A15.3-3 passed");
			else
				return Status.failed("Assertion A15.3-3 failed; counted " +
						count + " after delete; should be 1; counted " +
						count3 + "after rollback; should be 2");
		} finally {
			cleanup();
		}
	}

}
