/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.extents;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.Iterator;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Iterator Mutating Methods
 *<BR>
 *<B>JavaTest Keywords:</B> extent exception
 *<BR>
 *<B>Assertion ID:</B> A15.3-4.
 *<BR>
 *<B>Assertion Description: </B>
 If any mutating method, including the <code>remove</code> method,
 is called on the <code>Iterator</code> returned by <code>Extent.iterator()</code>,
 an <code>UnsupportedOperationException</code> is thrown.

 *@test
 *@title Iterator Mutating Methods
 *@executeClass com.sun.jdotck.extents.IteratorMutatingMethods
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source IteratorMutatingMethods.java
 *@keywords extent exception
 */

public class IteratorMutatingMethods extends ExtentTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		IteratorMutatingMethods t = new IteratorMutatingMethods();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		Extent ex = getExtent();
		Iterator it = ex.iterator();
		it.next();
		try {
			it.remove();
		} catch (UnsupportedOperationException expected) {
			return Status.passed("Assertion A15.3-4 passed");
		} finally {
			cleanup();
		}
		return Status.failed("Assertion A15.3-4 failed");
	}
}
