/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.extents;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Date;

import com.sun.jdotck.models.company.*;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Iterator Next After Extent Close
 *<BR>
 *<B>JavaTest Keywords:</B> extent exception
 *<BR>
 *<B>Assertion ID:</B> A15.3-10.
 *<BR>
 *<B>Assertion Description: </B>
 After a call to <code>Extent.close(Iterator i)</code>,
 the parameter <code>Iterator</code>
 will throw a <code>NoSuchElementException</code> to <code>next()</code>.

 *@test
 *@title Iterator Next After Extent Close
 *@executeClass com.sun.jdotck.extents.IteratorNextAfterExtentClose
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source IteratorNextAfterExtentClose.java
 *@keywords extent exception
 */

public class IteratorNextAfterExtentClose extends ExtentTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		IteratorNextAfterExtentClose t = new IteratorNextAfterExtentClose();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {
			beginTransaction();
			Extent ex = getPM().getExtent(Employee.class, true);
			Iterator it1 = ex.iterator();
			deleteEmployee((Employee) it1.next());
			Iterator it2 = ex.iterator();
			addEmployee();
			Iterator it3 = ex.iterator();
			ex.close(it1);
			ex.close(it2);
			ex.close(it3);

			try {
				tryNext(it1);
				tryNext(it2);
				tryNext(it3);
				rollbackTransaction();

				Iterator it4 = ex.iterator();
				int count4 = countIterator(it4);
				if (count4 == 2)
					return Status.passed("Assertion A15.3-10 passed");
				else {
					out.println("Iterator4 after rollback: " + count4 + "; should be 2");
					return Status.failed("Assertion A15.3-10 failed");
				}

			} catch (Exception unexpected) {
				rollbackTransaction();
				cleanup();
				return Status.failed("Assertion A15.3-10 failed");
			}

		} catch (Exception unexpected) {
			return Status.failed("Assertion A15.3-10 failed");
		} finally {
			cleanup();
		}
	}

	void tryNext(Iterator it) throws Exception {
		try {
			it.next();
		} catch (NoSuchElementException expected) {
			return;
		}
		throw new Exception("Expected exception not thrown");
	}

}
