/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.models.company;

import java.io.Serializable;

/**
 *This class represents a postal address, using U.S. address conventions.
 */

public class Address implements Serializable {
	private long addrid;
	private String street;
	private String city;
	private String state;
	private String zipcode;

	/** This is the JDO-required no-args constructor
	 */
	private Address() {
	}

	/**
	 *This constructor initializes the <code>Address</code> components.
	 *@param addrid The address ID.
	 *@param street The street address.
	 *@param city The city.
	 *@param state The state.
	 *@param zipcode The zip code.
	 */
	public Address(long addrid, String street, String city, String state, String zipcode) {
		this.addrid = addrid;
		this.street = street;
		this.city = city;
		this.state = state;
		this.zipcode = zipcode;
	}

	/**
	 *Get the addr ID associated with this object.
	 */
	public long getAddrID() {
		return addrid;
	}

	/**
	 *Get the street component of the address.
	 *@return The street component of the address.
	 */
	public String getStreet() {
		return street;
	}

	/**
	 *Set the street component of the address.
	 *@param street The street component.
	 */
	public void setStreet(String street) {
		this.street = street;
	}

	/**
	 *Get the city.
	 *@return The city component of the address.
	 */
	public String getCity() {
		return city;
	}

	/**
	 *Set the city component of the address.
	 *@param city The city.
	 */
	public void setCity(String city) {
		this.city = city;
	}

	/**
	 *Get the state component of the address.
	 *@return The state.
	 */
	public String getState() {
		return state;
	}

	/**
	 *Set the state component of the address.
	 *@param state The state.
	 */
	public void setState(String state) {
		this.state = state;
	}

	/**
	 *Get the zipcode component of the address.
	 *@return The zipcode.
	 */
	public String getZipcode() {
		return zipcode;
	}

	/**
	 *Set the zip code component of the address.
	 *@param zipcode The zipcode.
	 */
	public void setZipcode(String zipcode) {
		this.zipcode = zipcode;
	}

	/**
	 *This class is used to represent the application identifier
	 for the <code>Address</code> class.
	 */
	public static class Oid implements Serializable {
		/**
		 *This is the identifier field for <code>Address</code> and must correspond
		 *in type and name to the field in <code>Address</code>.
		 */
		public long addrid;

		/**
		 *The required public, null constructor.
		 */
		public Oid() {
			addrid = 0;
		}

		/**
		 *A constructor to initialize the identifier field.
		 */
		public Oid(long addrid) {
			this.addrid = addrid;
		}

		public boolean equals(java.lang.Object obj) {
			if (obj == null || !this.getClass().equals(obj.getClass()))
				return (false);
			Oid o = (Oid) obj;
			if (this.addrid != o.addrid) return (false);
			return (true);
		}

		public int hashCode() {
			return ((int) addrid);
		}

	}

}
