/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.models.company;

import java.util.Date;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 *This class represents a full-time employee.
 */
public class FullTimeEmployee extends Employee {
	private double salary;

	/** This is the JDO-required no-args constructor
	 */
	private FullTimeEmployee() {
	}

	/**
	 *Initialize a full-time employee.
	 *@param personid The person identifier.
	 *@param first The person's first name.
	 *@param last The person's last name.
	 *@param addr The person's address.
	 *@param born The person's birthdate.
	 *@param hired The date that the person was hired.
	 *@param sal The salary of the full-time employee.
	 */
	public FullTimeEmployee(
			long personid,
			String first,
			String last,
			Address addr,
			Date born,
			Date hired,
			double sal) {
		super(personid, first, last, addr, born, hired);
		salary = sal;
	}

	/**
	 *Get the salary of the full time employee.
	 *@return The salary of the full time employee.
	 */
	public double getSalary() {
		return salary;
	}

	/**
	 *Set the salary for the full-time employee.
	 *@param salary The salary to set for the full-time employee.
	 */
	public void setSalary(double salary) {
		this.salary = salary;
	}

	public String toString() {
		StringBuffer rc = new StringBuffer("FullTimeEmployee: ");
		rc.append(super.toString());
		rc.append(" $" + salary);
		return rc.toString();
	}

	protected void writeObject(ObjectOutputStream out)
			throws IOException {
		super.writeObject(out);
		out.writeDouble(salary);
	}

	protected void readObject(ObjectInputStream in)
			throws IOException, ClassNotFoundException {
		super.readObject(in);
		salary = in.readDouble();
	}

}
