/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.models.company;

import java.io.Serializable;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 *This class represents an insurance carrier selection for a particular
 *<code>Employee</code>.
 */
public class Insurance implements Serializable {
	private long insid;
	private String carrier;
	private Employee employee;

	/** This is the JDO-required no-args constructor
	 */
	private Insurance() {
	}

	/**
	 *Initialize an <code>Insurance</code> instance.
	 *@param insid The insurance instance identifier.
	 *@param carrier The insurance carrier.
	 *@param employee The employee associated with this insurance.
	 */
	public Insurance(long insid, String carrier, Employee employee) {
		this.insid = insid;
		this.carrier = carrier;
		this.employee = employee;
	}

	/**
	 *Get the insurance ID.
	 */
	public long getId() {
		return insid;
	}

	/**
	 *Set the insurance ID.
	 *@param insid The insurance ID value.
	 */
	public void setId(long insid) {
		this.insid = insid;
	}

	/**
	 *Get the insurance carrier.
	 *@return The insurance carrier.
	 */
	public String getCarrier() {
		return carrier;
	}

	/**
	 *Set the insurance carrier.
	 *@param carrier The insurance carrier.
	 */
	public void setCarrier(String carrier) {
		this.carrier = carrier;
	}

	/**
	 *Get the associated employee.
	 *@return The employee for this insurance.
	 */
	public Employee getEmployee() {
		return employee;
	}

	/**
	 *Set the associated employee.
	 *@param employee The associated employee.
	 */
	public void setEmployee(Employee employee) {
		this.employee = employee;
	}

	private void writeObject(ObjectOutputStream out)
			throws IOException {
		out.writeLong(insid);
		out.writeObject(carrier);
		out.writeObject(employee);
	}

	private void readObject(ObjectInputStream in)
			throws IOException, ClassNotFoundException {
		insid = in.readLong();
		carrier = (String) in.readObject();
		employee = (Employee) in.readObject();
	}

	/**
	 *This class is used to represent the application
	 *identifier for the <code>Insurance</code> class.
	 */
	public static class Oid implements Serializable {
		/**
		 *This field represents the application identifier for the <code>Insurance</code>
		 *class. It must match the field in the <code>Insurance</code> class in both name and type.
		 */
		public long insid;

		/**
		 *The required null constructor.
		 */
		public Oid() {
		}

		/**
		 *Initialize with an insurance identifier.
		 */
		public Oid(long insid) {
			this.insid = insid;
		}

		public boolean equals(java.lang.Object obj) {
			if (obj == null || !this.getClass().equals(obj.getClass()))
				return (false);
			Oid o = (Oid) obj;
			if (this.insid != o.insid) return (false);
			return (true);
		}

		public int hashCode() {
			return ((int) insid);
		}

	}

}

