/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.models.company;

import java.util.Date;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 *This class represents a part-time employee.
 */
public class PartTimeEmployee extends Employee {
	private double wage;

	/** This is the JDO-required no-args constructor
	 */
	private PartTimeEmployee() {
	}

	/**
	 *Initialize a part-time employee.
	 *@param personid The identifier for the person.
	 *@param first The person's first name.
	 *@param last The person's last name.
	 *@param addr The person's address.
	 *@param born The person's birthdate.
	 *@param hired The date the person was hired.
	 *@param wage The person's wage.
	 */
	public PartTimeEmployee(
			long personid,
			String first,
			String last,
			Address addr,
			Date born,
			Date hired,
			double wage) {
		super(personid, first, last, addr, born, hired);
		this.wage = wage;
	}

	/**
	 *Get the wage of the part-time employee.
	 *@return The wage of the part-time employee.
	 */
	public double getWage() {
		return wage;
	}

	/**
	 *Set the wage of the part-time employee.
	 *@param wage The wage of the part-time employee.
	 */
	public void setWage(double wage) {
		this.wage = wage;
	}

	public String toString() {
		StringBuffer rc = new StringBuffer("PartTimeEmployee: ");
		rc.append(super.toString());
		rc.append(" $" + wage);
		return rc.toString();
	}

	protected void writeObject(ObjectOutputStream out)
			throws IOException {
		super.writeObject(out);
		out.writeDouble(wage);
	}

	protected void readObject(ObjectInputStream in)
			throws IOException, ClassNotFoundException {
		super.readObject(in);
		wage = in.readDouble();
	}
}
