/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.models.company;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 *This class represents a person.
 */
public class Person implements Serializable {
	private long personid;
	private String firstname;
	private String lastname;
	private Address address;
	private Date birthdate;

	protected static SimpleDateFormat formatter =
			new SimpleDateFormat("d/MMM/yyyy");


	/** This is the JDO-required no-args constructor
	 */
	protected Person() {
	}

	/**
	 *Initialize a <code>Person</code> instance.
	 *@param personid The person identifier.
	 *@param firstname The person's first name.
	 *@param lastname The person's last name.
	 *@param address The person's address.
	 *@param birthdate The person's birthdate.
	 */
	public Person(long personid, String firstname, String lastname, Address address, Date birthdate) {
		this.personid = personid;
		this.firstname = firstname;
		this.lastname = lastname;
		this.address = address;
		this.birthdate = birthdate;
	}

	/**
	 *Get the person's last name.
	 *@return The last name.
	 */
	public String getLastname() {
		return lastname;
	}

	/**
	 *Set the person's last name.
	 *@param lastname The last name.
	 */
	public void setLastname(String lastname) {
		this.lastname = lastname;
	}

	/**
	 *Get the person's first name.
	 *@return The first name.
	 */
	public String getFirstname() {
		return firstname;
	}

	/**
	 *Set the person's first name.
	 *@param firstname The first name.
	 */
	public void setFirstname(String firstname) {
		this.firstname = firstname;
	}

	/**
	 *Get the address.
	 *@return The address.
	 */
	public Address getAddress() {
		return address;
	}

	/**
	 *Set the address.
	 *@param address The address.
	 */
	public void setAddress(Address address) {
		this.address = address;
	}

	/**
	 *Get the person's birthdate.
	 *@return The person's birthdate.
	 */
	public Date getBirthdate() {
		return birthdate;
	}

	/**
	 *Set the person's birthdate.
	 *@param birthdate The person's birthdate.
	 */
	public void setBirthdate(Date birthdate) {
		this.birthdate = birthdate;
	}

	protected void writeObject(ObjectOutputStream out)
			throws IOException {
		out.writeLong(personid);
		out.writeObject(lastname);
		out.writeObject(firstname);
		out.writeObject(address);
		out.writeObject(birthdate);
	}

	protected void readObject(ObjectInputStream in)
			throws IOException, ClassNotFoundException {
		personid = in.readLong();
		lastname = (String) in.readObject();
		firstname = (String) in.readObject();
		address = (Address) in.readObject();
		birthdate = (Date) in.readObject();
	}

	/**
	 *Return a String representation of a <code>Person</code> object.
	 */
	public String toString() {
		StringBuffer rc = new StringBuffer("Person: ");
		rc.append(personid);
		rc.append(", ");
		rc.append(lastname);
		rc.append(", " + firstname);
		rc.append(", born " + formatter.format(birthdate));
		return rc.toString();
	}

	/**
	 *This class is used to represent the application identifier
	 *for the <code>Person</code> class.
	 */
	public static class Oid implements Serializable {
		/**
		 *This field represents the identifier for the <code>Person</code> class.
		 *It must match a field in the <code>Person</code> class in both name and type.
		 */
		public long personid;

		/**
		 *The required null constructor.
		 */
		public Oid() {
			personid = 0;
		}

		/**
		 *Initialize the identifier.
		 *@param personid The person identifier.
		 */
		public Oid(long personid) {
			this.personid = personid;
		}

		public boolean equals(java.lang.Object obj) {
			if (obj == null ||
					!this.getClass().equals(obj.getClass()))
				return (false);
			Oid o = (Oid) obj;
			if (this.personid != o.personid) return (false);
			return (true);
		}

		public int hashCode() {
			return ((int) personid);
		}

	}

}
