/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.models.fieldtypes;

import java.util.Date;

public class SimpleClass implements SimpleInterface, Comparable {

	private static long counter = new Date().getTime();

	private static long newId() {
		synchronized (SimpleClass.class) {
			return counter++;
		}
	}

	private long id = newId();
	private int intField = 0;
	private String stringField = new String("Test");

	public SimpleClass() {
	};
	public SimpleClass(int intField, String stringField) {
		this.intField = intField;
		this.stringField = stringField;
	}

	public void setIntValue(int intField) {
		this.intField = intField;
	}

	public int getIntValue() {
		return intField;
	}

	public void setStringValue(String stringField) {
		this.stringField = stringField;
	}

	public String getStringValue() {
		return stringField;
	}

	public boolean equals(Object obj) {
		if (!(obj instanceof SimpleClass))
			return false;

		if (intField == ((SimpleClass) obj).getIntValue()
				&& stringField.equals(((SimpleClass) obj).getStringValue()))
			return true;
		else
			return false;
	}

	public int hashCode() {
		return intField ^ stringField.hashCode();
	}

	public int compareTo(Object p) {
		SimpleClass sc = (SimpleClass) p;
		if (intField < sc.intField) return -1;
		if (intField > sc.intField) return 1;
		return stringField.compareTo(sc.stringField);
	}

	public static class Oid {
		public long id;

		public Oid() {
		}

		public Oid(String s) {
			id = Long.parseLong(s);
		}

		public String toString() {
			return this.getClass().getName() + ": " + id;
		}

		public int hashCode() {
			return (int) id;
		}

		public boolean equals(Object other) {
			if (other != null && (other instanceof Oid)) {
				Oid k = (Oid) other;
				return k.id == this.id;
			}
			return false;
		}

	}
}
