/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.models.fieldtypes;

import java.io.PrintWriter;
import javax.jdo.*;
import java.util.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

/**
 *<B>JavaTest Title:</B> Support of field type array.
 *<BR>
 *<B>JavaTest Keywords:</B> model
 *<BR>
 *<B>Assertion ID:</B>
 *<BR>
 *<B>Assertion Description: </B>

 *@test
 *@title Support of field type array
 *@executeClass com.sun.jdotck.models.fieldtypes.TestArrayCollections
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source TestArrayCollections.java
 *@keywords model
 */


public class TestArrayCollections extends JDO_Test {
	private PersistenceManager pm;
	private Transaction tx;
	private static String prefix = "TestArrayCollections: ";
	private Object[] defaultValue; // do not initialize, should be 0 for int

	public TestArrayCollections() {
	}

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		TestArrayCollections t = new TestArrayCollections();
		Status s = t.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);

		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			tx = pm.currentTransaction();
			// turn on datastore transactions
			tx.setOptimistic(false);
			runTest();
			if (pm != null) {
				if (tx != null) {
					if (tx.isActive()) {
						tx.rollback();
					}
				}
				pm.close();
			}
		} catch (Exception e) {
			out.println("Exception caught in com.sun.jdotck.models.fieldtypes.TestArrayCollections");
			out.println(e);
			e.printStackTrace(out);
			return Status.failed("Assertion support of arrays failed, exception thrown");
		}
		pm = null;
		tx = null;
		if (getErrorCount() > 0)
			return Status.failed("Assertion support of arrays failed");
		else
			return Status.passed("Assertion support of arrays passed");
	}

	public void runTest() {
		int i, n;
		FirstSetOfTestValuesForCollection firstSetOfValues = new FirstSetOfTestValuesForCollection();
		SecondSetOfTestValuesForCollection secondSetOfValues = new SecondSetOfTestValuesForCollection();

		tx.begin();
		ArrayCollections pi = new ArrayCollections();
		pi.identifier = 1;
		pm.makePersistent(pi);
		Object oid = pm.getObjectId(pi);
		n = pi.getLength();
// Provide initial set of values
		for (i = 0; i < n; ++i) {
			String valueType = TestUtil.getFieldSpecs(pi.fieldSpecs[i]);
			//create an array of initial values for each value type
			Object[] firstValueArray = null;
			Vector firstValueVector = (Vector) firstSetOfValues.get(valueType);
			firstValueArray = (Object[]) java.lang.reflect.Array.newInstance(firstValueVector.get(0).getClass(),
					firstValueVector.size());
			for (int j = 0; j < firstValueVector.size(); j++) {
				firstValueArray[j] = firstValueVector.get(j);
			}

			//set the initial set of values
			pi.set(i, firstValueArray);
		}
		tx.commit();
// cache will be flushed
		pi = null;
		System.gc();

		tx.begin();

		checkValues(oid, firstSetOfValues); // check if persistent fields have values set
		pi = (ArrayCollections) pm.getObjectById(oid, true);

// Provide new set of values
		for (i = 0; i < n; ++i) {
			String valueType = TestUtil.getFieldSpecs(pi.fieldSpecs[i]);
			//create an array of second set of values for each value type
			Object[] secondValueArray = null;
			Vector secondValueVector = (Vector) secondSetOfValues.get(valueType);
			secondValueArray = (Object[]) java.lang.reflect.Array.newInstance(secondValueVector.get(0).getClass(),
					secondValueVector.size());
			for (int j = 0; j < secondValueVector.size(); j++) {
				secondValueArray[j] = secondValueVector.get(j);
			}
			pi.set(i, secondValueArray);
		}
		tx.commit();
// cache will be flushed
		pi = null;
		System.gc();

		tx.begin();
// check new values
		checkValues(oid, secondSetOfValues);
		pi = (ArrayCollections) pm.getObjectById(oid, true);
		pm.deletePersistent(pi);
		tx.commit();
	}

	private void checkValues(Object oid, Hashtable startValue) {
		int i;
		ArrayCollections pi = (ArrayCollections) pm.getObjectById(oid, true);
		int n = pi.getLength();
		for (i = 0; i < n; ++i) {
			String valueType = TestUtil.getFieldSpecs(pi.fieldSpecs[i]);
			//build the compareWith array
			Object[] compareWith = null;
			Vector compareWithVector = (Vector) startValue.get(valueType);
			compareWith = (Object[]) java.lang.reflect.Array.newInstance(compareWithVector.get(0).getClass(),
					compareWithVector.size());
			for (int j = 0; j < compareWithVector.size(); j++) {
				compareWith[j] = compareWithVector.get(j);
			}

			Object[] val = pi.get(i);

			if (!Arrays.equals(val, compareWith)) {
				incrementErrorCount();
				out.print(prefix);
				out.print("Incorrect value for ");
				out.println(ArrayCollections.fieldSpecs[i]);
			}
		}
	}
}
