/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.models.fieldtypes;

import java.io.PrintWriter;
import javax.jdo.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

/**
 *<B>JavaTest Title:</B> Support of field type boolean.
 *<BR>
 *<B>JavaTest Keywords:</B> model
 *<BR>
 *<B>Assertion ID:</B> A6.4.3-1.
 *<BR>
 *<B>Assertion Description: </B>
 JDO implementations must support fields of the primitive type <code>boolean</code>.
 *@test
 *@title Support of field type boolean
 *@executeClass com.sun.jdotck.models.fieldtypes.TestFieldsOfPrimitiveboolean
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source TestFieldsOfPrimitiveboolean.java
 *@keywords model
 */


public class TestFieldsOfPrimitiveboolean extends JDO_Test {
	private PersistenceManager pm;
	private Transaction tx;
	private static String prefix = "TestFieldsOfPrimitiveboolean: ";
	private boolean defaultValue; // do not initialize, should be set to default

	public TestFieldsOfPrimitiveboolean() {
	}

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		TestFieldsOfPrimitiveboolean t = new TestFieldsOfPrimitiveboolean();
		Status s = t.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);

		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			tx = pm.currentTransaction();
			runTest();
			if (pm != null) {
				if (tx != null) {
					if (tx.isActive()) {
						tx.rollback();
					}
				}
				pm.close();
			}
		} catch (Exception e) {
			out.println("Exception caught in com.sun.jdotck.models.fieldtypes.TestFieldsOfPrimitiveboolean");
			out.println(e);
			e.printStackTrace(out);
			return Status.failed("Assertion A6.4.3-1 failed, exception thrown");
		}
		pm = null;
		tx = null;
		if (getErrorCount() > 0)
			return Status.failed("Assertion A6.4.3-1 failed");
		else
			return Status.passed("Assertion A6.4.3-1 passed");
	}

	public void runTest() {
		int i, n;
		boolean value;
		tx.begin();
		FieldsOfPrimitiveboolean pi = new FieldsOfPrimitiveboolean();
		pi.identifier = 1;
		pm.makePersistent(pi);
		Object oid = pm.getObjectId(pi);
		n = pi.getLength();
// Provide initial set of values
		for (i = 0, value = true; i < n; ++i) {
			pi.set(i, value);
		}
		tx.commit();
// cache will be flushed
		pi = null;
		System.gc();

		tx.begin();

		pi = (FieldsOfPrimitiveboolean) pm.getObjectById(oid, true);
		checkValues(oid, true); // check if persistent fields have values set

// Provide new set of values
		for (i = 0, value = false; i < n; ++i) {
			pi.set(i, value);
		}
		tx.commit();
// cache will be flushed
		pi = null;
		System.gc();

		tx.begin();
// check new values
		checkValues(oid, false);
		pi = (FieldsOfPrimitiveboolean) pm.getObjectById(oid, true);
		pm.deletePersistent(pi);
		tx.commit();
	}

	private void checkValues(Object oid, boolean startValue) {
		int i;
		FieldsOfPrimitiveboolean pi = (FieldsOfPrimitiveboolean) pm.getObjectById(oid, true);
		int n = pi.getLength();
		for (i = 0; i < n; ++i) {
			if (!FieldsOfPrimitiveboolean.isPersistent[i]) continue;
			boolean val = pi.get(i);
			if (val != startValue) {
				incrementErrorCount();
				out.print(prefix);
				out.print("Incorrect value for ");
				out.print(FieldsOfPrimitiveboolean.fieldSpecs[i]);
				out.print(", expected value ");
				out.print(startValue);
				out.print(", value is ");
				out.println(val);
			}
		}
	}
}
