/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.models.fieldtypes;

import java.io.PrintWriter;
import javax.jdo.*;
import java.util.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

/**
 *<B>JavaTest Title:</B> Support of field type List
 *<BR>
 *<B>JavaTest Keywords:</B> model
 *<BR>
 *<B>Assertion ID:</B> A6.4.3-36.
 *<BR>
 *<B>Assertion Description: </B>
 If the List optional feature is supported, then JDO implementations must
 support fields of the interface type <code>List</code>,
 supporting them as Second Class Objects or First Class Objects.
 *@test
 *@title Support of field type List
 *@executeClass com.sun.jdotck.models.fieldtypes.TestListCollections
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source TestListCollections.java
 *@keywords model
 */


public class TestListCollections extends JDO_Test {
	private PersistenceManager pm;
	private Transaction tx;
	private static String prefix = "TestListCollections: ";
	private List defaultValue; // do not initialize, should be 0 for int

	public TestListCollections() {
	}

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		TestListCollections t = new TestListCollections();
		Status s = t.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		if (!isListSupported())
			return Status.passed("Assertion A6.4.3-36 passed, implementation does not support List");
		if (!isLinkedListSupported() && !isArrayListSupported())
			return Status.failed("Assertion A6.4.3-36 failed, List is supported, but neither ArrayList or LinkedList is supported");

		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			tx = pm.currentTransaction();
			// turn on datastore transactions
			tx.setOptimistic(false);
			runTest();
			if (pm != null) {
				if (tx != null) {
					if (tx.isActive()) {
						tx.rollback();
					}
				}
				pm.close();
			}
		} catch (Exception e) {
			out.println("Exception caught in com.sun.jdotck.models.fieldtypes.TestListCollections");
			out.println(e);
			e.printStackTrace(out);
			return Status.failed("Assertion A6.4.3-36 failed, exception thrown");
		}
		pm = null;
		tx = null;
		if (getErrorCount() > 0)
			return Status.failed("Assertion A6.4.3-36 failed");
		else
			return Status.passed("Assertion A6.4.3-36 passed");
	}


	public void runTest() {
		int i, n;
		FirstSetOfTestValuesForCollection firstValue = new FirstSetOfTestValuesForCollection();
		SecondSetOfTestValuesForCollection secondValue = new SecondSetOfTestValuesForCollection();

		tx.begin();
		ListCollections pi = new ListCollections();
		pi.identifier = 1;
		pm.makePersistent(pi);
		Object oid = pm.getObjectId(pi);
		n = pi.getLength();
// Provide initial set of values
		for (i = 0; i < n; ++i) {
			String valueType = TestUtil.getFieldSpecs(pi.fieldSpecs[i]);
			pi.set(i, new LinkedList((Collection) firstValue.get(valueType)));
		}
		tx.commit();
// cache will be flushed
		pi = null;
		System.gc();

		tx.begin();

		checkValues(oid, firstValue); // check if persistent fields have values set
		pi = (ListCollections) pm.getObjectById(oid, true);

// Provide new set of values
		for (i = 0; i < n; ++i) {
			String valueType = TestUtil.getFieldSpecs(pi.fieldSpecs[i]);
			pi.set(i, new LinkedList((Collection) secondValue.get(valueType)));
		}
		tx.commit();
// cache will be flushed
		pi = null;
		System.gc();

		tx.begin();
// check new values
		checkValues(oid, secondValue);
		pi = (ListCollections) pm.getObjectById(oid, true);
		pm.deletePersistent(pi);
		tx.commit();
	}

	private void checkValues(Object oid, Hashtable startValue) {
		int i;

		ListCollections pi = (ListCollections) pm.getObjectById(oid, true);
		int n = pi.getLength();
		for (i = 0; i < n; ++i) {
			String valueType = TestUtil.getFieldSpecs(pi.fieldSpecs[i]);
			List compareWith = new LinkedList((Collection) startValue.get(valueType));

			List val = pi.get(i);

			if (!val.equals(compareWith)) {
				incrementErrorCount();
				out.print(prefix);
				out.print("Incorrect value for ");
				out.println(ListCollections.fieldSpecs[i]);
			}
		}
	}

}
