/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.models.fieldtypes;

import java.io.PrintWriter;
import javax.jdo.*;
import java.util.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;

/**
 *<B>JavaTest Title:</B> Support of field type Map, varying value type
 *<BR>
 *<B>JavaTest Keywords:</B> model
 *<BR>
 *<B>Assertion ID:</B> A6.4.3-35.
 *<BR>
 *<B>Assertion Description: </B>
 If the Map optional feature is supported, then JDO implementations
 must support fields of the interface type <code>Map</code>,
 supporting them as Second Class Objects or First Class Objects.
 *@test
 *@title Support of field type Map, varying value type
 *@executeClass com.sun.jdotck.models.fieldtypes.TestMapStringKeyCollections
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source TestMapStringKeyCollections.java
 *@keywords model
 */


public class TestMapStringKeyCollections extends JDO_Test {
	private PersistenceManager pm;
	private Transaction tx;
	private static String prefix = "TestMapStringKeyCollections: ";
	private Map defaultValue; // do not initialize, should be 0 for int

	public TestMapStringKeyCollections() {
	}

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		TestMapStringKeyCollections t = new TestMapStringKeyCollections();
		Status s = t.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		if (!isMapSupported())
			return Status.passed("Assertion A6.4.3-35 passed, implementation does not support Map");
		if (!isHashMapSupported() && !isTreeMapSupported())
			return Status.failed("Assertion A6.4.3-35 failed, Map is supported, but neither HashMap or TreeMap is supported");

		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			tx = pm.currentTransaction();
			// turn on datastore transactions
			tx.setOptimistic(false);
			runTest();
			if (pm != null) {
				if (tx != null) {
					if (tx.isActive()) {
						tx.rollback();
					}
				}
				pm.close();
			}
		} catch (Exception e) {
			out.println("Exception caught in com.sun.jdotck.models.fieldtypes.TestMapStringKeyCollections");
			out.println(e);
			e.printStackTrace(out);
			return Status.failed("Assertion A6.4.3-35 failed, exception thrown");
		}
		pm = null;
		tx = null;
		if (getErrorCount() > 0)
			return Status.failed("Assertion A6.4.3-35 failed");
		else
			return Status.passed("Assertion A6.4.3-35 passed");
	}


	public void runTest() {
		int i, j, n;
		FirstSetOfTestValuesForCollection firstValue = new FirstSetOfTestValuesForCollection();
		SecondSetOfTestValuesForCollection secondValue = new SecondSetOfTestValuesForCollection();

		tx.begin();
		MapStringKeyCollections pi = new MapStringKeyCollections();
		pi.identifier = 1;
		pm.makePersistent(pi);
		Object oid = pm.getObjectId(pi);
		n = pi.getLength();
// Provide initial set of values
		for (i = 0; i < n; ++i) {
			Vector fieldSpecs = TestUtil.getFieldSpecsForMap(pi.fieldSpecs[i]);
			String fieldType = (String) fieldSpecs.get(0);
			String valueType = (String) fieldSpecs.get(1);
			Map map = new HashMap();
			Vector keys = (Vector) firstValue.get(fieldType);
			Vector values = (Vector) secondValue.get(valueType);

			for (j = 0; j < keys.size(); j++) {
				map.put(keys.get(j), values.get(j));
			}

			pi.set(i, map);
		}
		tx.commit();
// cache will be flushed
		pi = null;
		System.gc();

		tx.begin();

		Hashtable firstSet = new Hashtable();
		firstSet.put("keys", firstValue);
		firstSet.put("values", secondValue);

		checkValues(oid, firstSet); // check if persistent fields have values set
		pi = (MapStringKeyCollections) pm.getObjectById(oid, true);

// Provide new set of values     -- reverse the keys and values
		for (i = 0; i < n; ++i) {
			Vector fieldSpecs = TestUtil.getFieldSpecsForMap(pi.fieldSpecs[i]);
			String fieldType = (String) fieldSpecs.get(0);
			String valueType = (String) fieldSpecs.get(1);
			Map map = new HashMap();
			Vector keys = (Vector) secondValue.get(fieldType);
			Vector values = (Vector) firstValue.get(valueType);

			for (j = 0; j < keys.size(); j++) {
				map.put(keys.get(j), values.get(j));
			}

			pi.set(i, map);
		}

		tx.commit();
// cache will be flushed
		pi = null;
		System.gc();

		tx.begin();
// check new values
		Hashtable secondSet = new Hashtable();
		secondSet.put("keys", secondValue);
		secondSet.put("values", firstValue);

		checkValues(oid, secondSet);
		pi = (MapStringKeyCollections) pm.getObjectById(oid, true);
		pm.deletePersistent(pi);
		tx.commit();
	}

	private void checkValues(Object oid, Hashtable startValue) {
		int i, j;

		MapStringKeyCollections pi = (MapStringKeyCollections) pm.getObjectById(oid, true);
		int n = pi.getLength();

		Hashtable keySet = (Hashtable) startValue.get("keys");
		Hashtable valueSet = (Hashtable) startValue.get("values");

		for (i = 0; i < n; ++i) {
			Vector fieldSpecs = TestUtil.getFieldSpecsForMap(pi.fieldSpecs[i]);
			String fieldType = (String) fieldSpecs.get(0);
			String valueType = (String) fieldSpecs.get(1);
			Map compareWith = new HashMap();

			Vector keys = (Vector) keySet.get(fieldType);
			Vector values = (Vector) valueSet.get(valueType);
			int sz = keys.size();
			for (j = 0; j < sz; j++) {
				compareWith.put(keys.get(j), values.get(j));
			}

			Map val = pi.get(i);

			if (!val.equals(compareWith)) {
				incrementErrorCount();
				out.print(prefix);
				out.print("Incorrect value for ");
				out.println(MapStringKeyCollections.fieldSpecs[i]);
			}
		}
	}
}
