/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.models.inheritance;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import javax.jdo.*;
import java.io.*;

import com.sun.jdotck.*;

/**
 *<B>JavaTest Title:</B> Field With the Same Name as a Field in a Superclass
 *<BR>
 *<B>JavaTest Keywords:</B> inheritance
 *<BR>
 *<B>Assertion ID:</B> A6.5-7.
 *<BR>
 *<B>Assertion Description: </B>
 A class might define a new field with the same name as the field declared
 in the superclass, and might define it to be different (persistent or not)
 from the inherited field. But Java treats the declared field as a different
 field from the inherited field, so there is no conflict.

 *@test
 *@title Field With the Same Name as a Field in a Superclass
 *@executeClass com.sun.jdotck.models.inheritance.FieldWithSameNameInSuperclass
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source FieldWithSameNameInSuperclass.java
 *@keywords inheritance
 */

public class FieldWithSameNameInSuperclass extends TestParts {

	public FieldWithSameNameInSuperclass() {
	}

	static public void main(String[] args) {
		FieldWithSameNameInSuperclass testInstance = new FieldWithSameNameInSuperclass();
		Status s = testInstance.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	public Status run(String[] args, PrintWriter errPrt, PrintWriter outPrt) {

		initializeArgs(args, errPrt, outPrt);

		PersistenceManagerFactory fact = getPMF();
		PersistenceManager pm = fact.getPersistenceManager();
		Transaction t = pm.currentTransaction();
		t.setRestoreValues(true);

		t.begin();
		FieldSameName4.removeAllInstances(pm); // always start fresh with no instances
		t.commit();
		t.begin();
		FieldSameName4 refa = new FieldSameName4(intA_V[1], doubleB_V[1], intB_V[1], charC_V[1], booleanD_V[1], floatE_V[1], shortF_V[1], shortG_V[1], intH_V[1]);
		pm.makePersistent(refa);
		Object objPtrA = pm.getObjectId(refa);

		refa.setSecondObj(new FieldSameName4(intA_V[2], doubleB_V[2], intB_V[2], charC_V[2], booleanD_V[2], floatE_V[2], shortF_V[2], shortG_V[2], intH_V[2]));
		secondObj_V[1] = refa.getSecondObj();
		refa.setThirdObj(new FieldSameName4(intA_V[3], doubleB_V[3], intB_V[3], charC_V[3], booleanD_V[3], floatE_V[3], shortF_V[3], shortG_V[3], intH_V[3]));
		thirdObj_V[1] = refa.getThirdObj();
		pm.makePersistent(thirdObj_V[1]);
		Object objPtrB = pm.getObjectId(thirdObj_V[1]);
		refa.setFourthObj(new FieldSameName4(intA_V[4], doubleB_V[4], intB_V[4], charC_V[4], booleanD_V[4], floatE_V[4], shortF_V[4], shortG_V[4], intH_V[4]));
		fourthObj_V[1] = refa.getFourthObj();
		try {
			t.commit();
		} catch (JDOException e) {
			Object o = e.getFailedObject();
			String cname = o == null ? "null" : o.getClass().getName();
			outPrt.print("Exception thrown, failed object class is ");
			outPrt.println(cname);
			outPrt.flush();
			pm.close();
			throw e;
		}
		t.begin();
		FieldSameName4 a = null;
		FieldSameName4 b = null;

		try {  // retrieve object created in previous transaction & store in value array for later comparison
			thirdObj_V[1] = (FieldSameName4) pm.getObjectById(objPtrB, true);
		} catch (JDOUserException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDOUserException " + e + " could not reference thirdObj.");
			incrementErrorCount();
			t.rollback();
			pm.close();
			return Status.failed("Test FieldWithSameNameInSuperclass could not locate persistent object created in previous transaction");
		}

		try {  // retrieve object created in previous transaction
			a = (FieldSameName4) pm.getObjectById(objPtrA, true);
			checkPersistentAreCorrect(persistentAfterCommit, 1, a.getDoubleB(), a.getIntB(), a.getShortF(), a.getThirdObj(), a.getIntH());

			// verify referenced persistent object contains correct values
			b = a.getThirdObj();
			if (b != null) {  // if previous error caused b to be null, then these tests cannot be performed.
				checkPersistentAreCorrect(persistentAfterCommit, 3, b.getDoubleB(), b.getIntB(), b.getShortF(), b.getThirdObj(), b.getIntH());
			}
		} catch (JDOUserException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDOUserException " + e + " could not reference previously created object.");
			incrementErrorCount();
			t.rollback();
			pm.close();
			return Status.failed("Test FieldWithSameNameInSuperclass could not locate persistent object created in previous transaction");
		}

		// set in new values
		a.setIntA(intA_V[5]);
		a.setCharC(charC_V[5]);
		a.setBooleanD(booleanD_V[5]);
		a.setShortG(shortG_V[5]);
		FieldSameName4 fourth = new FieldSameName4(intA_V[6], doubleB_V[6], intB_V[6], charC_V[6], booleanD_V[6], floatE_V[6], shortF_V[6], shortG_V[6], intH_V[6]);
		a.setFourthObj(fourth);
		a.setFloatE(floatE_V[5]);
		a.setSecondObj(null);
		a.setDoubleB(doubleB_V[5]);
		a.setIntB(intB_V[5]);
		a.setShortF(shortF_V[5]);
		a.setThirdObj(null);
		a.setIntH(intH_V[5]);

		b.setIntA(intA_V[7]);
		b.setCharC(charC_V[7]);
		b.setBooleanD(booleanD_V[7]);
		b.setShortG(shortG_V[7]);
		b.setFourthObj(null);
		b.setFloatE(floatE_V[7]);
		b.setSecondObj(null);
		b.setDoubleB(doubleB_V[7]);
		b.setIntB(intB_V[7]);
		b.setShortF(shortF_V[7]);
		b.setThirdObj(null);
		b.setIntH(intH_V[7]);

		// create new objects and make persistent
		FieldSameName4 c = new FieldSameName4(intA_V[8], doubleB_V[8], intB_V[8], charC_V[8], booleanD_V[8], floatE_V[8], shortF_V[8], shortG_V[8], intH_V[8]);
		FieldSameName4 d = new FieldSameName4(intA_V[9], doubleB_V[9], intB_V[9], charC_V[9], booleanD_V[9], floatE_V[9], shortF_V[9], shortG_V[9], intH_V[9]);
		c.setThirdObj(d);
		c.setFourthObj(d);
		thirdObj_V[8] = d;
		fourthObj_V[8] = d;
		pm.makePersistent(c);

		// change values of newly persistent object
		c.setIntA(intA_V[10]);
		c.setCharC(charC_V[10]);
		c.setBooleanD(booleanD_V[10]);
		c.setShortG(shortG_V[10]);
		c.setFourthObj(null);
		c.setFloatE(floatE_V[10]);
		c.setSecondObj(null);
		c.setDoubleB(doubleB_V[10]);
		c.setIntB(intB_V[10]);
		c.setShortF(shortF_V[10]);
		c.setThirdObj(null);
		c.setIntH(intH_V[10]);

		t.rollback();

		// verify objects revert back to transient after rollback
		checkPersistentAreCorrect(transientAfterRollback, 8, c.getDoubleB(), c.getIntB(), c.getShortF(), c.getThirdObj(), c.getIntH());
		checkTransactionalAreCorrect(transientAfterRollback, 8, c.getFloatE(), c.getSecondObj());
		checkNonpersistentAreCorrect(transientAfterRollback, 10, c.getIntA(), c.getCharC(), c.getBooleanD(), c.getShortG(), c.getFourthObj());

		t.begin();

		// verify rollback lost all persistent changes.
		try {  // retrieve object created in previous transaction & store in value array for later comparison
			thirdObj_V[1] = (FieldSameName4) pm.getObjectById(objPtrB, true);
		} catch (JDOUserException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDOUserException " + e + " could not reference thirdObj.");
			incrementErrorCount();
			t.rollback();
			pm.close();
			return Status.failed("Test FieldWithSameNameInSuperclass could not locate persistent object created in previous transaction");
		}

		try {  // retrieve object created in previous transaction
			a = (FieldSameName4) pm.getObjectById(objPtrA, true);
			checkPersistentAreCorrect(persistentAfterRollback, 1, a.getDoubleB(), a.getIntB(), a.getShortF(), a.getThirdObj(), a.getIntH());
			b = a.getThirdObj();
			if (b != null) {  // if previous error caused b to be null, then these tests cannot be performed.
				checkPersistentAreCorrect(persistentAfterRollback, 3, b.getDoubleB(), b.getIntB(), b.getShortF(), b.getThirdObj(), b.getIntH());
			}
		} catch (JDOUserException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDOUserException " + e + " could not reference previously created object.");
			incrementErrorCount();
			t.rollback();
			pm.close();
			return Status.failed("Test FieldWithSameNameInSuperclass could not locate persistent object created in previous transaction");
		}

		t.rollback();
		pm.close();

		if (getErrorCount() > 0)
			return Status.passed("Assertion A6.5-7 (FieldWithSameNameInSuperclass) failed");
		else
			return Status.passed("Assertion A6.5-7 (FieldWithSameNameInSuperclass) passed");
	}
}
