/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.models.inheritance;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import javax.jdo.*;
import java.io.*;

import com.sun.jdotck.*;

/**
 *<B>JavaTest Title:</B> Persistent Fields Are Persistent in SubClasses
 *<BR>
 *<B>JavaTest Keywords:</B> inheritance
 *<BR>
 *<B>Assertion ID:</B> A6.5-4.
 *<BR>
 *<B>Assertion Description: </B>
 Fields identified as persistent in persistence-capable classes
 will be persistent in subclasses.

 *@test
 *@title Persistent Fields Are Persistent in SubClasses
 *@executeClass com.sun.jdotck.models.inheritance.PersistentFieldsArePersistentInSubClasses
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source PersistentFieldsArePersistentInSubClasses.java
 *@keywords inheritance
 */


public class PersistentFieldsArePersistentInSubClasses extends TestParts {

	public PersistentFieldsArePersistentInSubClasses() {
	}

	static public void main(String[] args) {
		PersistentFieldsArePersistentInSubClasses testInstance = new PersistentFieldsArePersistentInSubClasses();
		Status s = testInstance.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	public Status run(String[] args, PrintWriter errPrt, PrintWriter outPrt) {

		initializeArgs(args, errPrt, outPrt);

		PersistenceManagerFactory fact = getPMF();
		PersistenceManager pm = fact.getPersistenceManager();
		Transaction t = pm.currentTransaction();
		t.setRestoreValues(true);

		t.begin();
		AllPersist4.removeAllInstances(pm); // always start fresh with no instances
		t.commit();
		t.begin();
		AllPersist4 refa = new AllPersist4(intA_V[1], doubleB_V[1], intB_V[1], charC_V[1], booleanD_V[1], floatE_V[1], shortF_V[1], shortG_V[1], intH_V[1]);
		pm.makePersistent(refa);
		Object objPtrA = pm.getObjectId(refa);

		refa.secondObj = new AllPersist4(intA_V[2], doubleB_V[2], intB_V[2], charC_V[2], booleanD_V[2], floatE_V[2], shortF_V[2], shortG_V[2], intH_V[2]);
		secondObj_V[1] = refa.secondObj;
		refa.thirdObj = new AllPersist4(intA_V[3], doubleB_V[3], intB_V[3], charC_V[3], booleanD_V[3], floatE_V[3], shortF_V[3], shortG_V[3], intH_V[3]);
		thirdObj_V[1] = refa.thirdObj;
		pm.makePersistent(refa.thirdObj);
		Object objPtrB = pm.getObjectId(refa.thirdObj);
		refa.fourthObj = new AllPersist4(intA_V[4], doubleB_V[4], intB_V[4], charC_V[4], booleanD_V[4], floatE_V[4], shortF_V[4], shortG_V[4], intH_V[4]);
		fourthObj_V[1] = refa.fourthObj;
		t.commit();

		t.begin();
		AllPersist4 a = null;
		AllPersist4 b = null;

		try {  // retrieve object created in previous transaction & store in value array for later comparison
			thirdObj_V[1] = (AllPersist4) pm.getObjectById(objPtrB, true);
		} catch (JDOUserException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDOUserException " + e + " could not reference thirdObj.");
			incrementErrorCount();
			t.rollback();
			pm.close();
			return Status.failed("PersistentFieldsArePersistentInSubClasses could not locate persistent object created in previous transaction");
		}

		try {  // retrieve object created in previous transaction
			a = (AllPersist4) pm.getObjectById(objPtrA, true);
			checkPersistentAreCorrect(persistentAfterCommit, 1, a.doubleB, a.intB, a.shortF, a.thirdObj, a.intH);

			// verify referenced persistent object contains correct values
			b = a.thirdObj;
			if (b != null) {  // if previous error caused b to be null, then these tests cannot be performed.
				checkPersistentAreCorrect(persistentAfterCommit, 3, b.doubleB, b.intB, b.shortF, b.thirdObj, b.intH);
			}
		} catch (JDOUserException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDOUserException " + e + " could not reference previously created object.");
			incrementErrorCount();
			t.rollback();
			pm.close();
			return Status.failed("PersistentFieldsArePersistentInSubClasses could not locate persistent object created in previous transaction");
		}

		// set in new values
		a.intA = intA_V[5];
		a.charC = charC_V[5];
		a.booleanD = booleanD_V[5];
		a.shortG = shortG_V[5];
		a.fourthObj = new AllPersist4(intA_V[6], doubleB_V[6], intB_V[6], charC_V[6], booleanD_V[6], floatE_V[6], shortF_V[6], shortG_V[6], intH_V[6]);
		fourthObj_V[5] = a.fourthObj;
		a.floatE = floatE_V[5];
		a.secondObj = null;
		a.doubleB = doubleB_V[5];
		a.intB = intB_V[5];
		a.shortF = shortF_V[5];
		a.thirdObj = null;
		a.intH = intH_V[5];

		b.intA = intA_V[7];
		b.charC = charC_V[7];
		b.booleanD = booleanD_V[7];
		b.shortG = shortG_V[7];
		b.fourthObj = null;
		b.floatE = floatE_V[7];
		b.secondObj = null;
		b.doubleB = doubleB_V[7];
		b.intB = intB_V[7];
		b.shortF = shortF_V[7];
		b.thirdObj = null;
		b.intH = intH_V[7];

		// create new objects and make persistent
		AllPersist4 c = new AllPersist4(intA_V[8], doubleB_V[8], intB_V[8], charC_V[8], booleanD_V[8], floatE_V[8], shortF_V[8], shortG_V[8], intH_V[8]);
		AllPersist4 d = new AllPersist4(intA_V[9], doubleB_V[9], intB_V[9], charC_V[9], booleanD_V[9], floatE_V[9], shortF_V[9], shortG_V[9], intH_V[9]);
		c.thirdObj = d;
		c.fourthObj = d;
		thirdObj_V[8] = d;
		fourthObj_V[8] = d;
		pm.makePersistent(c);

		// change values of newly persistent object
		c.intA = intA_V[10];
		c.charC = charC_V[10];
		c.booleanD = booleanD_V[10];
		c.shortG = shortG_V[10];
		c.fourthObj = null;
		c.floatE = floatE_V[10];
		c.secondObj = null;
		c.doubleB = doubleB_V[10];
		c.intB = intB_V[10];
		c.shortF = shortF_V[10];
		c.thirdObj = null;
		c.intH = intH_V[10];

		t.rollback();

		// verify objects revert back to transient after rollback
		checkPersistentAreCorrect(transientAfterRollback, 8, c.doubleB, c.intB, c.shortF, c.thirdObj, c.intH);

		t.begin();

		// verify rollback lost all persistent changes.
		try {  // retrieve object created in previous transaction & store in value array for later comparison
			thirdObj_V[1] = (AllPersist4) pm.getObjectById(objPtrB, true);
		} catch (JDOUserException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDOUserException " + e + " could not reference thirdObj.");
			incrementErrorCount();
			t.rollback();
			pm.close();
			return Status.failed("PersistentFieldsArePersistentInSubClasses could not locate persistent object created in previous transaction");
		}

		try {  // retrieve object created in previous transaction
			a = (AllPersist4) pm.getObjectById(objPtrA, true);
			checkPersistentAreCorrect(persistentAfterRollback, 1, a.doubleB, a.intB, a.shortF, a.thirdObj, a.intH);
			b = a.thirdObj;
			if (b != null) {  // if previous error caused b to be null, then these tests cannot be performed.
				checkPersistentAreCorrect(persistentAfterRollback, 3, b.doubleB, b.intB, b.shortF, b.thirdObj, b.intH);
			}
		} catch (JDOUserException e) {
			// could not locate persistent object created in previous transaction
			errPrt.println("JDOUserException " + e + " could not reference previously created object.");
			incrementErrorCount();
			t.rollback();
			pm.close();
			return Status.failed("PersistentFieldsArePersistentInSubClasses could not locate persistent object created in previous transaction");
		}

		t.rollback();
		pm.close();

		if (getErrorCount() > 0)
			return Status.passed("Assertion A6.5-4 (PersistentFieldsArePersistentInSubClasses) failed");
		else
			return Status.passed("Assertion A6.5-4 (PersistentFieldsArePersistentInSubClasses) passed");
	}
}
