/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.models.inheritance;

import com.sun.jdotck.*;

import java.io.*;

/**
 * The test data is in arrays, making it easy to change values.  The Inheritance test classes refer to the
 * data via array index and the checkXXXX() routines carry an index value--indicating the expected value.
 * If those attributes are references and should be other than null, as is the case with thirdObj, then the
 * test code inserts the reference into the data array.
 *
 * Each test class follows the same pattern:
 * 1.  Persistent and transient object are created in the first transaction.
 * 2.  A second transaction is started, the first persistent object retrieved and checking is done on
 *     its attribute values.
 * 3.  A rollback is performed and value comparisions are made on the transient instances that were created.
 * 4.  A third transation is started and the first persistent object again retrieved and checking is done on
 *     its attribute values.
 */
public abstract class TestParts extends JDO_Test {

	//                             0   1      2      3      4     5     6      7     8         9     10    11
	static int intA_V[] = {-1, 350, 43, 336, -447, 1, 337, 10, 43, -447, 20, 30};
	static double doubleB_V[] = {2.0, 9.8, 94.5, -7.2, -13., 4.0, -8.2, -13, 94.5, -13, 23, 33};
	static int intB_V[] = {-5, 3334, -76, 28, 12497, 5, 38, 14, -76, 12497, 24, 34};
	static char charC_V[] = {'3', 'D', 'J', 'L', 'N', 'w', '1', 'x', 'J', 'P', 'y', 'z'};
	static boolean booleanD_V[] = {false, true, true, true, true, false, true, true, true, true, true, false};
	static float floatE_V[] = {-4.4f, -32.7f, -21.5f, -3266f, -3.6f, 3.0f, -4266f, 12f, -21.5f, -3.6f, 22f, 32f};
	static short shortF_V[] = {1238, 258, 394, 8, 21, 6, 9, 15, 394, 21, 25, 35};
	static short shortG_V[] = {-360, -1000, -4321, -6, -44, 2, -7, 11, 4321, -44, 21, 31};
	static int intH_V[] = {-6, -25, -347600, -603, -7613, 7, -703, 16, -347600, -7613, 26, 36};
	static Object secondObj_V[] = {null, null, null, null, null, null, null, null, null, null, null, null};
	static Object thirdObj_V[] = {null, null, null, null, null, null, null, null, null, null, null, null};
	static Object fourthObj_V[] = {null, null, null, null, null, null, null, null, null, null, null, null};

	static String persistentAfterCommit = "Read object back after committed to Database.  ";
	static String persistentAfterRollback = "Read object back after rollback.  ";
	static String transientAfterRollback = "Object reverted to transient after rollback.  ";

	public TestParts() {
	}

	void checkPersistentAreCorrect(String title, int index, double doubleB, int intB, short shortF, Object thirdObj, int intH) {
		if (doubleB != doubleB_V[index]) {
			out.println(title + "Persistent attribute doubleB is " + doubleB + ", it should be " + doubleB_V[index]);
			incrementErrorCount();
		}
		if (intB != intB_V[index]) {
			out.println(title + "Persistent attribute intB is " + intB + ", it should be " + intB_V[index]);
			incrementErrorCount();
		}
		if (shortF != shortF_V[index]) {
			out.println(title + "Persistent attribute shortF is " + shortF + ", it should be " + shortF_V[index]);
			incrementErrorCount();
		}
		if (thirdObj != thirdObj_V[index]) {
			out.println(title + "Persistent attribute thirdObj is " + thirdObj + ", it should be " + thirdObj_V[index]);
			incrementErrorCount();
		}
		if (intH != intH_V[index]) {
			out.println(title + "Persistent attribute intH is " + intH + ", it should be " + intH_V[index]);
			incrementErrorCount();
		}
	}

	void checkTransactionalAreCorrect(String title, int index, float floatE, Object secondObj) {

		if (floatE != floatE_V[index]) {
			out.println(title + "Transactional attribute floatE is " + floatE + ", it should be " + floatE_V[index]);
			incrementErrorCount();
		}
		if (secondObj != secondObj_V[index]) {
			out.println(title + "Transactional attribute secondObj is " + secondObj + ", it should be " + secondObj_V[index]);
			incrementErrorCount();
		}
	}

	void checkNonpersistentAreCorrect(String title, int index, int intA, char charC, boolean booleanD, short shortG, Object fourthObj) {
		if (intA != intA_V[index]) {
			out.println(title + "In non-persistent class, attribute intA is " + intA + ", it should be " + intA_V[index]);
			incrementErrorCount();
		}
		if (charC != charC_V[index]) {
			out.println(title + "In non-persistent class, attribute charC is " + charC + ", it should be " + charC_V[index]);
			incrementErrorCount();
		}
		if (booleanD != booleanD_V[index]) {
			out.println(title + "In non-persistent class, attribute booleanD is " + booleanD + ", it should be " + booleanD_V[index]);
			incrementErrorCount();
		}
		if (shortG != shortG_V[index]) {
			out.println(title + "In non-persistent class, attribute shortG is " + shortG + ", it should be " + shortG_V[index]);
			incrementErrorCount();
		}
		if (fourthObj != fourthObj_V[index]) {
			out.println(title + "In non-persistent class, attribute fourthObj is " + fourthObj + ", it should be " + fourthObj_V[index]);
			incrementErrorCount();
		}
	}
}
