/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.models.inheritance;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import javax.jdo.*;
import java.io.*;

import com.sun.jdotck.*;

/**
 *<B>JavaTest Title:</B> Transactional Fields are Transactional in Subclasses
 *<BR>
 *<B>JavaTest Keywords:</B> inheritance
 *<BR>
 *<B>Assertion ID:</B> A6.5-5.
 *<BR>
 *<B>Assertion Description: </B>
 Fields marked as transactional in persistence-capable classes
 will be transactional in subclasses.

 *@test
 *@title Transactional Fields are Transactional in Subclasses
 *@executeClass com.sun.jdotck.models.inheritance.TransactionalFieldsAreTransactionalInSubclasses
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source TransactionalFieldsAreTransactionalInSubclasses.java
 *@keywords inheritance
 */

public class TransactionalFieldsAreTransactionalInSubclasses extends TestParts {

	public TransactionalFieldsAreTransactionalInSubclasses() {
	}

	static public void main(String[] args) {
		TransactionalFieldsAreTransactionalInSubclasses testInstance = new TransactionalFieldsAreTransactionalInSubclasses();
		Status s = testInstance.run(args, new PrintWriter(System.err, true), new PrintWriter(System.out, true));
		s.exit();
	}

	public Status run(String[] args, PrintWriter errPrt, PrintWriter outPrt) {

		initializeArgs(args, errPrt, outPrt);

		PersistenceManagerFactory fact = getPMF();
		PersistenceManager pm = fact.getPersistenceManager();
		Transaction t = pm.currentTransaction();
		t.setRestoreValues(true);

		t.begin();
		AllPersist4.removeAllInstances(pm); // always start fresh with no instances
		t.commit();
		t.begin();
		// create new objects and make persistent
		AllPersist4 c = new AllPersist4(intA_V[8], doubleB_V[8], intB_V[8], charC_V[8], booleanD_V[8], floatE_V[8], shortF_V[8], shortG_V[8], intH_V[8]);
		AllPersist4 d = new AllPersist4(intA_V[9], doubleB_V[9], intB_V[9], charC_V[9], booleanD_V[9], floatE_V[9], shortF_V[9], shortG_V[9], intH_V[9]);
		c.thirdObj = d;
		c.fourthObj = d;
		thirdObj_V[8] = d;
		fourthObj_V[8] = d;
		pm.makePersistent(c);

		// change values of newly persistent object
		c.intA = intA_V[10];
		c.charC = charC_V[10];
		c.booleanD = booleanD_V[10];
		c.shortG = shortG_V[10];
		c.fourthObj = null;
		c.floatE = floatE_V[10];
		c.secondObj = d;
		c.doubleB = doubleB_V[10];
		c.intB = intB_V[10];
		c.shortF = shortF_V[10];
		c.thirdObj = null;
		c.intH = intH_V[10];

		t.rollback();

		// verify objects revert back to transient after rollback
		checkTransactionalAreCorrect(transientAfterRollback, 8, c.floatE, c.secondObj);

		pm.close();

		if (getErrorCount() > 0)
			return Status.passed("Assertion A6.5-5 (TransactionalFieldsAreTransactionalInSubClasses) failed");
		else
			return Status.passed("Assertion A6.5-5 (TransactionalFieldsAreTransactionalInSubClasses) passed");
	}
}
