/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.models.mylib.PCPoint;

import java.io.PrintWriter;
import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Each parameter named in the parameter declaration must be bound to a value when executing the query.
 *<BR>
 *<B>JavaTest Keywords:</B> query parameter check
 *<BR>
 *<B>Assertion ID:</B> A14.3-3.
 *<BR>
 *<B>Assertion Description: </B> Each parameter named in the parameter declaration must be bound to a value when executing the query.
 *@test
 *@title Each parameter named in the parameter declaration must be bound to a value when executing the query.
 *@executeClass com.sun.jdotck.query.BoundParameterCheck
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source BoundParameterCheck
 *@keywords query
 */

public class BoundParameterCheck extends QueryTest {
	protected PersistenceManagerFactory pmf;
	protected PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {
		BoundParameterCheck queryTest = new BoundParameterCheck();
		Status status = queryTest.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			checkQueryParameters();
			pm.close();
			return Status.passed("\nTest BoundParameterCheck - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest BoundParameterCheck - Failed");
		}
	}

	void checkQueryParameters() {
		Transaction tx = pm.currentTransaction();
		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null) {
				out.println("Query is null...");
			}
			query.setClass(PCPoint.class);
			query.setCandidates(pm.getExtent(PCPoint.class, false));
			query.declareParameters("int a");
			query.setFilter("x == 0");
			Object results = query.execute();
			out.println("checkQueryParameters failed");
			tx.rollback();
			throw new RuntimeException("Test BoundParameterCheck: Failed");
		} catch (JDOException e) {
			e.printStackTrace(out);
			tx.rollback();
		}
	}
}

