/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Close
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.7-1.
 *<BR>
 *<B>Assertion Description: </B> <code>Query.close(Object)</code> closes the result of one <code>execute(...)</code> method, and releases resources associated with it. After this method completes, the query result can no longer be used, for example to iterate the returned elements. Any elements returned previously by iteration of the results remain in their current state. Any iterators acquired from the queryResult will return <code>false</code> to <code>hasNext()</code> and will throw <code>NoSuchElementException</code> to <code>next(</code>).
 *@test
 *@title Close
 *@executeClass com.sun.jdotck.query.Close
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source Close
 *@keywords query
 */

public class Close extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		Close testQuery = new Close();

		Status status = testQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testClose();

			pm.close();

			return Status.passed("\nTest Close - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest Close - Failed");
		}
	}

	void testClose() throws Exception {

		out.println("\nExecuting test Close()...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p1 = new PCPoint(0, 0);
		Object p2 = new PCPoint(1, 1);
		Object p3 = new PCPoint(2, 2);
		Object p4 = new PCPoint(3, 3);
		Object p5 = new PCPoint(4, 4);

		expected.add(p1);
		expected.add(p2);
		expected.add(p3);
		expected.add(p4);
		expected.add(p5);

		insertExpected(expected);

		// printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test Close: Results are as expected and accessible before query is closed");
		} catch (Exception e) {
			out.println("Test Close: Failed");
			e.printStackTrace(out);
		}

		Iterator resIterator = ((Collection) results).iterator();

		query.closeAll();

		if (resIterator.hasNext()) {
			out.println("iterator.hasNext() should return null.");
			out.println("Test Close() failed.");
			//throw new Exception("Test Close() failed.");
		} else {
			out.println("iterator.hasNext() return null.");
		}

		try {
			resIterator.next();
		} catch (Exception e) {
			e.printStackTrace(out);
		}

		out.println("NoSuchElement exception expected.");

		tx.commit();
	}

}

