/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Close All
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.7-2.
 *<BR>
 *<B>Assertion Description: </B> <code>Query.closeAll()</code> closes all results of <code>execute(...)</code> methods on this <code>Query</code> instance, as in A14.6.7-1.  The <code>Query</code> instance is still valid and can still be used.
 *@test
 *@title Close All
 *@title get properties
 *@executeClass com.sun.jdotck.query.CloseAll
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source CloseAll
 *@keywords query
 */

public class CloseAll extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		CloseAll testQuery = new CloseAll();

		Status status = testQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testCloseAll();

			pm.close();

			return Status.passed("\nTest CloseAll - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest CloseAll - Failed");
		}
	}

	void testCloseAll() throws Exception {

		out.println("\nExecuting test CloseAll()...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		Collection results = (Collection) query.execute();

		Iterator resIterator = results.iterator();

		query.closeAll();

		if (resIterator.hasNext() == false) {
			out.println("iterator.hasNext() return null.");
		} else {
			out.println("iterator.hasNext() should return null.");
			out.println("Test CloseAll() failed.");
			//throw new Exception("Test CloseAll() failed.");
		}

		try {
			resIterator.next();
		} catch (Exception e) {
			e.printStackTrace(out);
		}

		out.println("NoSuchElement exception expected.");

		tx.commit();
	}

}

