/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Compile Query
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6-14.
 *<BR>
 *<B>Assertion Description: </B> <code>Query.compile()</code> requires the <code>Query</code> instance to validate any elements bound to the query instance and report any inconsistencies by throwing a <code>JDOUserException</code>. It is only a hint to the <code>Query</code> instance to prepare and optimize an execution plan for the query, implementations are not required to support query compilation.
 *@test
 *@title Compile Query
 *@executeClass com.sun.jdotck.query.CompileQuery
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source CompileQuery
 *@keywords query
 */

public class CompileQuery extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		CompileQuery newQuery = new CompileQuery();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testCompileQuery();

			pm.close();

			return Status.passed("\nTest CompileQuery - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest CompileQuery - Failed");
		}
	}

	Status testCompileQuery() {

		String params = "Integer param";

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters(params);
		query.setFilter("x == param");

		query.compile();

		Object results = query.execute(new Integer(4));

		// check query result
		Collection expected = new ArrayList();

		Object p5 = new PCPoint(4, 4);

		expected.add(p5);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("Test CompileQuery - Failed");
		}

		tx.commit();
		return Status.passed("\nTest CompileQuery - Passed");
	}

}

