/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Declare Variables
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6-9.
 *<BR>
 *<B>Assertion Description: </B> <code>Query.declareVariables(String variables)</code> binds the unbound variable statements to the query instance. This method defines the types and names of variables that will be used in the filter but not provided as values by the <code>execute</code> method.
 *@test
 *@title Declare Variables
 *@executeClass com.sun.jdotck.query.DeclareVariables
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source DeclareVariables
 *@keywords query
 */

public class DeclareVariables extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		DeclareVariables newQuery = new DeclareVariables();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testDeclareVariables();

			pm.close();

			return Status.passed("\nTest DeclareVariables - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest DeclareVariables - Failed");
		}
	}

	Status testDeclareVariables() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters("Integer param1, Integer param2");
		query.setFilter("x == param1 && y == param2");

		Object results = query.execute(new java.lang.Integer(2), new java.lang.Integer(2));

		// check query result
		Collection expected = new ArrayList();
		Object p3 = new PCPoint(2, 2);
		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("Test DeclareVariables - Failed");
		}

		tx.commit();
		return Status.passed("\nTest DeclareVariables - Passed");
	}
}

