/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Equality and Comparisons Between Primitives and Wrapper
 Instances
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.2-3.
 *<BR>
 *<B>Assertion Description: </B> Equality and ordering comparisons between primitives and instances of wrapper classes are valid in a <code>Query</code> filter.
 *@test
 *@title Equality and Comparisons Between Primitives and Wrapper Instances
 *@executeClass com.sun.jdotck.query.EqualityAndComparisonsBetweenPrimitivesAndWrapperInstances
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source EqualityAndComparisonsBetweenPrimitivesAndWrapperInstances
 *@keywords query
 */

public class EqualityAndComparisonsBetweenPrimitivesAndWrapperInstances extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		EqualityAndComparisonsBetweenPrimitivesAndWrapperInstances newQuery = new EqualityAndComparisonsBetweenPrimitivesAndWrapperInstances();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testEqualityAndComparisonsBetweenPrimitivesAndWrapperInstances();

			pm.close();

			return Status.passed("\nTest EqualityAndComparisonsBetweenPrimitivesAndWrapperInstances - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest EqualityAndComparisonsBetweenPrimitivesAndWrapperInstances - Failed");
		}
	}

	Status testEqualityAndComparisonsBetweenPrimitivesAndWrapperInstances() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("y == 1");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p2 = new PCPoint(1, 1);

		expected.add(p2);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("Test EqualityAndComparisonsBetweenPrimitivesAndWrapperInstances - Failed");
		}

		tx.commit();
		return Status.passed("\nTest EqualityAndComparisonsBetweenPrimitivesAndWrapperInstances - Passed");
	}

}

