/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Execute Query
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.1-3.
 *<BR>
 *<B>Assertion Description: </B> The <code>Query.execute</code> methods execute the query using the parameters and return the result, which is a collection of instances that satisfy the boolean filter. Each parameter of the <code>execute</code> method(s) is an object which is either the value of the corresponding parameter or the wrapped value of a primitive parameter.
 *@test
 *@title Execute Query
 *@executeClass com.sun.jdotck.query.ExecuteQuery
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source ExecuteQuery
 *@keywords query
 */

public class ExecuteQuery extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		ExecuteQuery newQuery = new ExecuteQuery();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testExecuteQuery01();
			testExecuteQuery02();
			testExecuteQuery03();

			pm.close();

			return Status.passed("\nTest ExecuteQuery - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest ExecuteQuery - Failed");
		}
	}

	Status testExecuteQuery01() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == 4");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p5 = new PCPoint(4, 4);

		expected.add(p5);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ExecuteQuery01 - Failed");
			return Status.failed("Test ExecuteQuery01 - Failed");
		}

		tx.commit();
		out.println("Test ExecuteQuery01 - Passed\n");
		return Status.passed("\nTest ExecuteQuery01 - Passed");
	}

	Status testExecuteQuery02() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters("Integer param");
		query.setFilter("x == param");

		Object results = query.execute(new java.lang.Integer(2));

		// check query result
		Collection expected = new ArrayList();
		Object p3 = new PCPoint(2, 2);
		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ExecuteQuery02 - Failed\n");
			return Status.failed("Test ExecuteQuery02 - Failed");
		}

		tx.commit();
		out.println("Test ExecuteQuery02 - Passed\n");
		return Status.passed("\nTest ExecuteQuery02 - Passed");
	}

	Status testExecuteQuery03() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters("Integer param1, Integer param2");
		query.setFilter("x == param1 && y == param2");

		Object results = query.execute(new java.lang.Integer(2), new java.lang.Integer(2));

		// check query result
		Collection expected = new ArrayList();
		Object p3 = new PCPoint(2, 2);
		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ExecuteQuery03 - Failed\n");
			return Status.failed("Test ExecuteQuery03 - Failed");
		}

		tx.commit();
		out.println("Test ExecuteQuery03 - Passed\n");
		return Status.passed("\nTest ExecuteQuery03 - Passed");
	}
}

