/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Execute Query when Persistence Manager is Closed
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.1-1.
 *<BR>
 *<B>Assertion Description: </B> If the <code>PersistenceManager</code> which constructed the <code>Query</code> is closed, then the <code>Query.execute</code> methods throw <code>JDOUserException</code>.
 *@test
 *@title Execute Query when Persistence Manager is Closed
 *@executeClass com.sun.jdotck.query.ExecuteQueryWhenPersistenceManagerIsClosed
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source ExecuteQueryWhenPersistenceManagerIsClosed
 *@keywords query
 */

public class ExecuteQueryWhenPersistenceManagerIsClosed extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		ExecuteQueryWhenPersistenceManagerIsClosed newQuery = new ExecuteQueryWhenPersistenceManagerIsClosed();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			testExecuteQueryWhenPersistenceManagerIsClosed();

			//pm.close();

			return Status.passed("\nTest ExecuteQueryWhenPersistenceManagerIsClosed - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest ExecuteQueryWhenPersistenceManagerIsClosed - Failed");
		}
	}

	void testExecuteQueryWhenPersistenceManagerIsClosed() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == 4");

		query.compile();

		Object results = query.execute();

		tx.commit();

		pm.close();

		if (query == null) {
			out.println("Query is null...");
		}

		try {
			Object results1 = query.execute();
		} catch (Exception ex) {
			ex.printStackTrace(out);
		}
	}

}

