/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Execute Query with Array
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.1-6.
 *<BR>
 *<B>Assertion Description: </B> The <code>executeWithArray</code> method is similar to the <code>execute</code> method, but takes its parameters from an array instance. The array contains <code>Object</code>s, in which the positional <code>Object</code> is the value to use in the query for that parameter. Unlike <code>execute</code>, there is no limit on the number of parameters.
 *@test
 *@title Execute Query with Array
 *@executeClass com.sun.jdotck.query.ExecuteQueryWithArray
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source ExecuteQueryWithArray
 *@keywords query
 */

public class ExecuteQueryWithArray extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		ExecuteQueryWithArray newQuery = new ExecuteQueryWithArray();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testExecuteQueryWithArray01();
			testExecuteQueryWithArray02();

			pm.close();

			return Status.passed("\nTest ExecuteQueryWithArray - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest ExecuteQueryWithArray - Failed");
		}
	}

	Status testExecuteQueryWithArray01() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters("Integer param");
		query.setFilter("x == param");

		Object[] actualParams = {new Integer(2)};
		Object results = query.executeWithArray(actualParams);

		// check query result
		Collection expected = new ArrayList();
		Object p3 = new PCPoint(2, 2);
		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ExecuteQueryWithArray01 - Failed\n");
			return Status.failed("Test ExecuteQueryWithArray01 - Failed");
		}

		tx.commit();
		out.println("Test ExecuteQueryWithArray01 - Passed\n");
		return Status.passed("\nTest ExecuteQueryWithArray01 - Passed");
	}

	Status testExecuteQueryWithArray02() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters("Integer param1, Integer param2");
		query.setFilter("x == param1 && y == param2");

		Object[] actualParams = {new Integer(2), new Integer(2)};
		Object results = query.executeWithArray(actualParams);

		// check query result
		Collection expected = new ArrayList();
		Object p3 = new PCPoint(2, 2);
		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ExecuteQueryWithArray02 - Failed\n");
			return Status.failed("Test ExecuteQueryWithArray02 - Failed");
		}

		tx.commit();
		out.println("Test ExecuteQueryWithArray02 - Passed\n");
		return Status.passed("\nTest ExecuteQueryWithArray02 - Passed");
	}
}

