/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.query;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.Iterator;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Executing Multiple Queries Simultaneously is ThreadSafe
 *<BR>
 *<B>JavaTest Keywords:</B> query concurrency
 *<BR>
 *<B>Assertion ID:</B> A14.3-2.
 *<BR>
 *<B>Assertion Description: </B>
 Multiple queries might be executed simultaneously by different threads
 (but the implementation might choose to execute them serially).
 In either case, the execution must be thread safe.

 *@test
 *@title Executing Multiple Queries Simultaneously is ThreadSafe
 *@executeClass com.sun.jdotck.query.ExecutingMultipleQueriesSimultaneouslyIsThreadSafe
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source ExecutingMultipleQueriesSimultaneouslyIsThreadSafe.java
 *@keywords query concurrency
 */

public class ExecutingMultipleQueriesSimultaneouslyIsThreadSafe
		extends MultipleActiveQueryInstanceInSamePersistenceManager
		implements Runnable {

	static int nrOfThreads = 20;
	static Thread[] threads = new Thread[nrOfThreads];
	static Status[] stati = new Status[nrOfThreads];
	static Class clazz = com.sun.jdotck.models.mylib.PCPoint.class;

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) throws Exception {
		ExecutingMultipleQueriesSimultaneouslyIsThreadSafe t = new ExecutingMultipleQueriesSimultaneouslyIsThreadSafe();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	int index;

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			out.println(Thread.currentThread().getName() + ": Starting " + nrOfThreads + " concurrent threads.");
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			initDatabase(pm, clazz);
			pm.close();
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest ExecutingMultipleQueriesSimultaneouslyIsThreadSafe - Parent thread Failed");
		}

		for (int i = 0; i < nrOfThreads; i++) {
			ExecutingMultipleQueriesSimultaneouslyIsThreadSafe queryTest =
					new ExecutingMultipleQueriesSimultaneouslyIsThreadSafe();
			queryTest.initializeArgs(args, err, out);
			queryTest.index = i;
			threads[i] = new Thread(queryTest);
			threads[i].start();
		}

		out.println(Thread.currentThread().getName() + ": Waiting for threads to join...");

		for (int i = 0; i < nrOfThreads; i++)
			try {
				threads[i].join();
			} catch (InterruptedException e) {
				return Status.failed(e.toString());
			}

		out.println("\n" + Thread.currentThread().getName() + ": All threads joined.");

		for (int i = 0; i < nrOfThreads; i++)
			if (stati[i].getType() != Status.PASSED)
				return stati[i];

		return stati[0];
	}

	public void run() {
		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			setInsertedObjects();
			testMultipleActiveQueryInstanceInSamePersistenceManager();
			pm.close();
			stati[index] = Status.passed("Assertion A14.3-2 passed for thread " + Thread.currentThread().getName());
		} catch (Exception e) {
			e.printStackTrace(out);
			stati[index] = Status.failed("Assertion A14.3-2 failed for thread " + Thread.currentThread().getName());
		}
	}

	public void setInsertedObjects() throws Exception {
		Transaction tx = pm.currentTransaction();
		tx.begin();
		Query query = pm.newQuery();
		query.setClass(clazz);
		query.setCandidates(pm.getExtent(clazz, false));
		Object results = query.execute();
		for (Iterator i = ((Collection) results).iterator(); i.hasNext();)
			inserted.add(i.next());
		tx.commit();
	}

}
