/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get IgnoreCache
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6-13.
 *<BR>
 *<B>Assertion Description: </B> <code>Query.getIgnoreCache</code> returns the current setting of the IgnoreCache option.
 *@test
 *@title Get IgnoreCache
 *@executeClass com.sun.jdotck.query.GetIgnoreCache
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source GetIgnoreCache
 *@keywords query
 */

public class GetIgnoreCache extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		GetIgnoreCache newQuery = new GetIgnoreCache();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			testGetIgnoreCache01();
			testGetIgnoreCache02();

			pm.close();

			return Status.passed("\nTest GetIgnoreCache - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest GetIgnoreCache - Failed");
		}
	}

	void testGetIgnoreCache01() throws Exception {

		Transaction tx = pm.currentTransaction();
		tx.setOptimistic(false);
		tx.begin();

		Query query = pm.newQuery();

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		if (query.getIgnoreCache() == false) {
			out.println("Pessimistic: IgnoreCache - Default value = " + query.getIgnoreCache());
			out.println("Pessimistic: IgnoreCache - Default value test: Passed");
		} else {
			out.println("Pessimistic: IgnoreCache - Default value = " + query.getIgnoreCache());
			out.println("Pessimistic: IgnoreCache - Default value test: Failed");
			throw new Exception("Pessimistic: IgnoreCache - Default value test - failed");
		}

		query.compile();

		tx.commit();
	}

	void testGetIgnoreCache02() throws Exception {

		Transaction tx = pm.currentTransaction();
		tx.setOptimistic(true);
		tx.begin();

		Query query = pm.newQuery();

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		if (query.getIgnoreCache() == false) {
			out.println("Optimistic: IgnoreCache - Default value = " + query.getIgnoreCache());
			out.println("Optimistic: IgnoreCache - Default value test: Passed");
		} else {
			out.println("Optimistic: IgnoreCache - Default value = " + query.getIgnoreCache());
			out.println("Optimistic: IgnoreCache - Default value test: Failed");
			throw new Exception("Optimistic: IgnoreCache - Default value test: failed");
		}

		query.compile();

		tx.commit();
	}

}

