/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get PersistenceManager
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6-2.
 *<BR>
 *<B>Assertion Description: </B> <code>Query.getPersistenceManager()</code> returns the associated <code>PersistenceManager</code> instance.
 *@test
 *@title Get PersistenceManager
 *@executeClass com.sun.jdotck.query.GetPersistenceManager
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source GetPersistenceManager
 *@keywords query
 */

public class GetPersistenceManager extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;

	public static void main(String[] args) throws Exception {

		GetPersistenceManager newQuery = new GetPersistenceManager();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			testGetPersistenceManager01();
			testGetPersistenceManager02();

			pm.close();

			return Status.passed("\nTest GetPersistenceManager - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest GetPersistenceManager - Failed");
		}
	}

	void testGetPersistenceManager01() throws Exception {

		Transaction tx = pm.currentTransaction();
		tx.setOptimistic(false);
		tx.begin();

		Query query = pm.newQuery();

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		PersistenceManager pm1 = query.getPersistenceManager();
		if (pm.equals(pm1)) {
			out.println("Pessimistic: GetPersistenceManager Test: Passed");
		} else {
			out.println("Pessimistic: GetPersistenceManager Test: Failed");
			throw new Exception("Pessimistic: GetPersistenceManager Test: Failed");
		}

		query.compile();

		tx.commit();
	}

	void testGetPersistenceManager02() throws Exception {

		Transaction tx = pm.currentTransaction();
		tx.setOptimistic(true);
		tx.begin();

		Query query = pm.newQuery();

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		PersistenceManager pm1 = query.getPersistenceManager();
		if (pm.equals(pm1)) {
			out.println("Optimistic: GetPersistenceManager Test: Passed");
		} else {
			out.println("Optimistic: GetPersistenceManager Test: Failed");
			throw new Exception("Optimistic: GetPersistenceManager Test: Failed");
		}

		query.compile();

		tx.commit();
	}

}

