/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;
import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.PipedOutputStream;
import java.io.ObjectInputStream;
import java.io.PipedInputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Get PersistenceManager From Restored Serialized Query
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6-3.
 *<BR>
 *<B>Assertion Description: </B> If a <code>Query</code> instance has been restored from a serialized form, <code>Query.getPersistenceManager()</code> returns <code>null</code>.
 *@test
 *@title Get PersistenceManager From Restored Serialized Query
 *@executeClass com.sun.jdotck.query.GetPersistenceManagerFromRestoredSerializedQuery
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source GetPersistenceManagerFromRestoredSerializedQuery
 *@keywords query
 */

public class GetPersistenceManagerFromRestoredSerializedQuery extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		GetPersistenceManagerFromRestoredSerializedQuery queryTest = new GetPersistenceManagerFromRestoredSerializedQuery();

		Status status = queryTest.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			// initDatabase(pm, PCPoint.class);
			testGetPersistenceManagerFromRestoredSerializedQuery();

			pm.close();

			return Status.passed("\nTest GetPersistenceManagerFromRestoredSerializedQuery - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest GetPersistenceManagerFromRestoredSerializedQuery - Failed");
		}
	}

	void serializeQuery() throws Exception {

		Query query = pm.newQuery();

		if (query == null) {
			throw new Exception("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == 3");

		query.compile();

		ObjectOutputStream oos = null;

		try {
			out.println("Attempting to serialize Query object.");
			oos = new ObjectOutputStream(new FileOutputStream("query.ser"));
			oos.writeObject(query);
			out.println("Query object serialized.");
		} catch (Exception ex) {
			ex.printStackTrace(out);
			throw new Exception("Unable to serialize Query object.");
		} finally {
			if (oos != null) {
				try {
					oos.close();
				} catch (Exception ex) {
				}
			}
		}
	}

	void testGetPersistenceManagerFromRestoredSerializedQuery() throws Exception {

		out.println("\nExecuting test GetPersistenceManagerFromRestoredSerializedQuery() ...");

		Query restoredQuery = null;
		ObjectInputStream ois = null;
		PersistenceManager pm1 = null;

		serializeQuery();

		try {
			out.println("Attempting to de-serialize Query object.");
			ois = new ObjectInputStream(new FileInputStream("query.ser"));
			restoredQuery = (Query) ois.readObject();
			out.println("Query object restored.");
		} catch (Exception ex) {
			ex.printStackTrace(out);
			throw new Exception("Unable to deserialize Query object.");
		} finally {
			if (ois != null) {
				try {
					ois.close();
				} catch (Exception ex) {
				}
			}
		}


		try {
			pm1 = restoredQuery.getPersistenceManager();

			if (pm1 == null) {
				out.println("Test GetPersistenceManagerFromRestoredSerializedQuery(): Passed");
			} else {
				out.println("Test GetPersistenceManagerFromRestoredSerializedQuery(): Failed");
				throw new Exception("PersistenceManager from restored serialized query is not null");
			}
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test GetPersistenceManagerFromRestoredSerializedQuery(): Failed");
		}
	}
}
