/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.ArrayList;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Methods and Object Construction not Supported
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.2-8.
 *<BR>
 *<B>Assertion Description: </B> Methods, including object construction, are not supported in a <code>Query</code> filter.
 *@test
 *@title Methods and Object Construction not Supported
 *@executeClass com.sun.jdotck.query.MethodsAndObjectConstructionNotSupported
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MethodsAndObjectConstructionNotSupported
 *@keywords query
 */

public class MethodsAndObjectConstructionNotSupported extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;

	public static void main(String[] args) throws Exception {

		MethodsAndObjectConstructionNotSupported queryTest = new MethodsAndObjectConstructionNotSupported();

		Status status = queryTest.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			initDatabase(pm, PCPoint.class);

			testUnsupportedOperators01("this.getX() == 1");
			testUnsupportedOperators01("y.intValue() == 1");
			testUnsupportedOperators01("y == new Integer(1)");

			pm.close();
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest MethodsAndObjectConstructionNotSupported() - Failed");
		}

		if (getErrorCount() == 0) {
			return Status.passed("\nTest MethodsAndObjectConstructionNotSupported() - Passed");
		} else {
			return Status.failed("\nTest MethodsAndObjectConstructionNotSupported() - Failed");
		}
	}

	void testUnsupportedOperators01(String filter) throws Exception {

		Query query = pm.newQuery();
		String expectedMsg = "setFilter: Invalid method call ....";

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		try {
			query.setFilter(filter);
			query.compile();

			out.println("Test UnsupportedOperators01(" + filter + ") - Failed\n");
			error("Missing " + JDOUserException.class.getName() + "(" +
					expectedMsg + ")");
		} catch (JDOException ex) {
			out.println("Exception message: " + ex.getMessage());
			if (!(ex instanceof JDOUserException)) {
				error("Wrong exception of " + ex.getClass().getName() + "\n" +
						"  Expected exception of " + JDOUserException.class.getName());
			}
			out.println("Test UnsupportedOperators01(\"" + filter + "\") - Passed\n");
		}
	}

	private void error(String message) {
		err.println(message);
		incrementErrorCount();
	}

}

