/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Multiple Active Query Instances in Same PersistenceMana
 ger
 *<BR>
 *<B>JavaTest Keywords:</B> query concurrency
 *<BR>
 *<B>Assertion ID:</B> A14.3-1.
 *<BR>
 *<B>Assertion Description: </B> Multiple JDO <code>Query</code> instances might be active simultaneously in the same JDO <code>PersistenceManager</code> instance.
 *@test
 *@title Multiple Active Query Instances in Same PersistenceManager
 *@executeClass com.sun.jdotck.query.MultipleActiveQueryInstanceInSamePersistenceManager
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MultipleActiveQueryInstanceInSamePersistenceManager
 *@keywords query
 */

public class MultipleActiveQueryInstanceInSamePersistenceManager extends QueryTest {
	protected PersistenceManagerFactory pmf;
	protected PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		MultipleActiveQueryInstanceInSamePersistenceManager queryTest = new MultipleActiveQueryInstanceInSamePersistenceManager();

		Status status = queryTest.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testMultipleActiveQueryInstanceInSamePersistenceManager();

			pm.close();

			return Status.passed("Assertion A14.3-1 passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("Assertion A14.3-1 failed");
		}
	}

	void testMultipleActiveQueryInstanceInSamePersistenceManager() throws Exception {

		out.println("\nExecuting test MultipleActiveQueryInstanceInSamePersistenceManager()...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p1 = new PCPoint(0, 0);
		Object p2 = new PCPoint(1, 1);
		Object p3 = new PCPoint(2, 2);
		Object p4 = new PCPoint(3, 3);
		Object p5 = new PCPoint(4, 4);

		expected.add(p1);
		expected.add(p2);
		expected.add(p3);
		expected.add(p4);
		expected.add(p5);

		insertExpected(expected);

		printOutput(results);

		checkQueryResultWithoutOrder(results);
		out.println("Test MultipleActiveQueryInstanceInSamePersistenceManager: Passed");

		tx.commit();
	}

}

