/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Status;
import com.sun.jdotck.models.mylib.PCPoint;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.Iterator;
import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Multiple Active Query Instances in Same PersistenceMana
 ger executed simultaneously
 *<BR>
 *<B>JavaTest Keywords:</B> query concurrency
 *<BR>
 *<B>Assertion ID:</B> A14.3-2.
 *<BR>
 *<B>Assertion Description: </B> Multiple JDO <code>Query</code> instances might be executed simultaneously in the same JDO <code>PersistenceManager</code> instance.
 *@test
 *@title Multiple Active Query Instances in Same PersistenceManager executed simultaneously
 *@executeClass com.sun.jdotck.query.MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously
 *@keywords query
 */

public class MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously
		extends MultipleActiveQueryInstanceInSamePersistenceManager
		implements Runnable {

	static int nrOfThreads = 20;
	static Thread[] threads = new Thread[nrOfThreads];
	static Status[] stati = new Status[nrOfThreads];

	public static void main(String[] args) throws Exception {

		MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously queryTest =
				new MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously();

		Status status = queryTest.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		status.exit();
	}

	int index;

	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			out.println("number of concurrent threads: " + nrOfThreads);
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			initDatabase(pm, PCPoint.class);
			pm.close();
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously - Parent thread Failed");
		}

		for (int i = 0; i < nrOfThreads; i++) {
			MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously queryTest =
					new MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously();
			queryTest.initializeArgs(args, err, out);
			queryTest.index = i;
			threads[i] = new Thread(queryTest);
			threads[i].start();
		}

		out.println("waiting for threads to join...");

		for (int i = 0; i < nrOfThreads; i++)
			try {
				threads[i].join();
			} catch (InterruptedException e) {
				return Status.failed(e.toString());
			}

		out.println("all threads joined...");

		for (int i = 0; i < nrOfThreads; i++)
			if (stati[i].getType() != Status.PASSED)
				return stati[i];

		return stati[0];
	}

	public void run() {
		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			setInsertedObjects();
			testMultipleActiveQueryInstanceInSamePersistenceManager();
			pm.close();
			stati[index] = Status.passed("\nTest MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously - Passed for thread " + Thread.currentThread().getName());
		} catch (Exception e) {
			e.printStackTrace(out);
			stati[index] = Status.failed("\nTest MultipleActiveQueryInstanceInSamePersistenceManagerExecutedSimultaneously - Failed for thread " + Thread.currentThread().getName());
		}
	}

	public void setInsertedObjects() throws Exception {
		Transaction tx = pm.currentTransaction();
		tx.begin();
		Query query = pm.newQuery();
		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		Object results = query.execute();
		for (Iterator i = ((Collection) results).iterator(); i.hasNext();)
			inserted.add(i.next());
		tx.commit();
	}

}
