/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.query;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.models.company.Person;
import com.sun.jdotck.models.company.Employee;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Namespace of Field Names Separate From Types, Variables, Parameters
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.4-1.
 *<BR>
 *<B>Assertion Description: </B>
 Field names are in a separate namespace from type names,
 variable names, and parameters.

 *@test
 *@title Namespace of Field Names Separate From Types, Variables, Parameters
 *@executeClass com.sun.jdotck.query.NamespaceOfFieldNamesSeparateFromTypesVariablesParameters
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source NamespaceOfFieldNamesSeparateFromTypesVariablesParameters.java
 *@keywords query
 */

public class NamespaceOfFieldNamesSeparateFromTypesVariablesParameters extends QueryTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		NamespaceOfFieldNamesSeparateFromTypesVariablesParameters t = new NamespaceOfFieldNamesSeparateFromTypesVariablesParameters();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			PersistenceManagerFactory pmf = getPMF();
			PersistenceManager pm = pmf.getPersistenceManager();
			checkQueryNameSpaces(pm);
			pm.close();
			return Status.passed("Assertion A14.4-1 passed");
		} catch (Exception e) {
			e.printStackTrace(out);
		}
		return Status.failed("Assertion A14.4-1 failed");
	}

	void checkQueryNameSpaces(PersistenceManager pm) {
		Transaction tx = pm.currentTransaction();
		Class clazz = com.sun.jdotck.models.company.Project.class;
		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null)
				throw new RuntimeException("checkQueryNameSpaces(): query instance is null");
			query.setClass(clazz);
			query.setCandidates(pm.getExtent(clazz, false));
			query.declareParameters("com.sun.jdotck.models.company.Employee Employee");
			query.declareVariables("com.sun.jdotck.models.company.Person person;");
			query.setFilter("reviewers.contains(person) && person==Employee");
			Object results = query.execute(new Employee(0, "", "", null, null, null));
		} catch (JDOException e) {
			throw new RuntimeException("Namespaces of types are not separated from namespaces of parameters");
		} finally {
			tx.rollback();
		}

		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null)
				throw new RuntimeException("checkQueryNameSpaces(): query instance is null");
			query.setClass(clazz);
			query.setCandidates(pm.getExtent(clazz, false));
			query.declareParameters("com.sun.jdotck.models.company.Employee employee");
			query.declareVariables("com.sun.jdotck.models.company.Person Person;");
			query.setFilter("reviewers.contains(Person) && Person==employee");
			Object results = query.execute(new Employee(0, "", "", null, null, null));
		} catch (JDOException e) {
			throw new RuntimeException("Namespaces of types are not separated from namespaces of variables");
		} finally {
			tx.rollback();
		}

		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null)
				throw new RuntimeException("checkQueryNameSpaces(): query instance is null");
			query.setClass(clazz);
			query.setCandidates(pm.getExtent(clazz, false));
			query.declareParameters("com.sun.jdotck.models.company.Employee reviewers");
			query.setFilter("reviewers.firstName == name");
			Object results = query.execute(new Employee(0, "", "", null, null, null));
			throw new RuntimeException("Parameters and fields are not in the same namespace");
		} catch (JDOException e) {
			out.println("Caught expected " + e);
		} finally {
			tx.rollback();
		}

		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null)
				throw new RuntimeException("checkQueryNameSpaces(): query instance is null");
			query.setClass(clazz);
			query.setCandidates(pm.getExtent(clazz, false));
			query.declareParameters("com.sun.jdotck.models.company.Employee person");
			query.declareVariables("com.sun.jdotck.models.company.Person person;");
			query.setFilter("reviewers.contains(person) && person==person");
			Object results = query.execute(new Employee(0, "", "", null, null, null));
			throw new RuntimeException("Variables and parameters are not in the same namespace");
		} catch (JDOException e) {
			out.println("Caught expected " + e);
		} finally {
			tx.rollback();
		}
	}
}
