/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> New Query From Existing Query Bound to PersistenceManag
 er From Same Vendor
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.5-3.
 *<BR>
 *<B>Assertion Description: </B> <code>PersistenceManager.newQuery(Object query)</code> constructs a <code>Query</code> instance from another query, where the parameter is currently bound to a <code>PersistenceManager</code> from the same JDO vendor.  Any of the elements Class, Filter, Import declarations, Variable declarations, Parameter declarations, and Ordering from the parameter <code>Query</code> are copied to the new <code>Query</code> instance, but a candidate <code>Collection</code> or <code>Extent</code> element is discarded.
 *@test
 *@title New Query From Existing Query Bound to PersistenceManager From Same Ven dor

 *@executeClass com.sun.jdotck.query.NewQueryFromExistingQueryBoundToPMFromSameVendor
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source NewQueryFromExistingQueryBoundToPMFromSameVendor
 *@keywords query
 */

public class NewQueryFromExistingQueryBoundToPMFromSameVendor extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		NewQueryFromExistingQueryBoundToPMFromSameVendor newQuery = new NewQueryFromExistingQueryBoundToPMFromSameVendor();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testNewQueryFromExistingQueryBoundToPMFromSameVendor01();
			testNewQueryFromExistingQueryBoundToPMFromSameVendor02();


			return Status.passed("\nTest NewQueryFromExistingQueryBoundToPMFromSameVendor - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest NewQueryFromExistingQueryBoundToPMFromSameVendor - Failed");
		} finally {
			pm.close();
		}
	}

	void testNewQueryFromExistingQueryBoundToPMFromSameVendor01() throws Exception {

		out.println("\nExecuting test NewQueryFromExistingQueryBoundToPMFromSameVendor01()...");

		Transaction tx = pm.currentTransaction();
		tx.begin();

		try {
			Query query = pm.newQuery();
			query.setClass(PCPoint.class);

			Query query1 = pm.newQuery(query);
			query1.compile();

			Object results = query1.execute();

			Collection expected = new ArrayList();

			Object p1 = new PCPoint(0, 0);
			Object p2 = new PCPoint(1, 1);
			Object p3 = new PCPoint(2, 2);
			Object p4 = new PCPoint(3, 3);
			Object p5 = new PCPoint(4, 4);

			expected.add(p1);
			expected.add(p2);
			expected.add(p3);
			expected.add(p4);
			expected.add(p5);

			insertExpected(expected);

			printOutput(results);

			checkQueryResultWithoutOrder(results);
			out.println("Test NewQueryFromExistingQueryBoundToPMFromSameVendor01() - Passed\n");
		} catch (Exception e) {
			out.println("Test NewQueryFromExistingQueryBoundToPMFromSameVendor01() - Failed\n");
			e.printStackTrace(out);
		} finally {
			tx.commit();
		}
	}

	void testNewQueryFromExistingQueryBoundToPMFromSameVendor02() throws Exception {
		out.println("\nExecuting test NewQueryFromExistingQueryBoundToPMFromSameVendor02()...");

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		Query query1 = pm.newQuery(query);
		query1.setCandidates(pm.getExtent(PCPoint.class, true));
		query1.compile();

		Object results = query1.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p1 = new PCPoint(0, 0);
		Object p2 = new PCPoint(1, 1);
		Object p3 = new PCPoint(2, 2);
		Object p4 = new PCPoint(3, 3);
		Object p5 = new PCPoint(4, 4);

		expected.add(p1);
		expected.add(p2);
		expected.add(p3);
		expected.add(p4);
		expected.add(p5);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test NewQueryFromExistingQueryBoundToPMFromSameVendor02() - Passed\n");
		} catch (Exception e) {
			out.println("Test NewQueryFromExistingQueryBoundToPMFromSameVendor02() - Failed\n");
			e.printStackTrace(out);
		} finally {
			tx.commit();
		}
	}

}

