/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;
import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.PipedOutputStream;
import java.io.ObjectInputStream;
import java.io.PipedInputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> New Query From Existing Serialized Query
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.5-2.
 *<BR>
 *<B>Assertion Description: </B> <code>PersistenceManager.newQuery(Object query)</code> constructs a <code>Query</code> instance from another query, where the parameter is a serialized/re-stored <code>Query</code> instance from the same JDO vendor but a different execution environment.  Any of the elements Class, Filter, Import declarations, Variable declarations, Parameter declarations, and Ordering from the parameter <code>Query</code> are copied to the new <code>Query</code> instance, but a candidate <code>Collection</code> or <code>Extent</code> element is discarded.
 *@test
 *@title New Query From Existing Serialized Query
 *@executeClass com.sun.jdotck.query.NewQueryFromRestoredSerializedQuery
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source NewQueryFromRestoredSerializedQuery
 *@keywords query
 */

public class NewQueryFromRestoredSerializedQuery extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		NewQueryFromRestoredSerializedQuery queryTest = new NewQueryFromRestoredSerializedQuery();

		Status status = queryTest.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testNewQueryFromRestoredSerializedQuery01();

			pm.close();

			return Status.passed("\nTest NewQueryFromRestoredSerializedQuery01 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest NewQueryFromRestoredSerializedQuery01 - Failed");
		}
	}

	void testNewQueryFromRestoredSerializedQuery01() throws Exception {

		out.println("\nExecuting test NewQueryFromRestoredSerializedQuery01() ...");

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == 3");

		query.compile();

		ObjectOutputStream oos = null;

		try {
			out.println("Attempting to serialize Query object.");
			oos = new ObjectOutputStream(new FileOutputStream("query.ser"));
			oos.writeObject(query);
			out.println("Query object serialized.");
			out.println("Test QueryIsSerializable02(): Passed");
		} catch (Exception ex) {
			out.println("Unable to serialize Query object.");
			out.println("Test QueryIsSerializable02(): Failed");
			ex.printStackTrace(out);
			//throw new Exception("Test QueryIsSerializable02(): Failed");
		} finally {
			if (oos != null) {
				try {
					oos.flush();
				} catch (Exception ex) {
				}
				try {
					oos.close();
				} catch (Exception ex) {
				}
			}
//            if (tx.isActive())
//                tx.rollback();
		}


		/**        // create and connect pipe
		 java.io.PipedOutputStream pipeOut = new java.io.PipedOutputStream();
		 java.io.PipedInputStream pipeIn = new java.io.PipedInputStream(pipeOut);
		 // serialize query object
		 java.io.ObjectOutputStream out = new java.io.ObjectOutputStream(pipeOut)
		 ;
		 out.writeObject(query);
		 out.flush();
		 **/
		// deserialize query object
		java.io.ObjectInputStream in = new java.io.ObjectInputStream(new FileInputStream("query.ser"));
		Query query1 = (Query) in.readObject();

		// init and execute query
		query = pm.newQuery(query1);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p4 = new PCPoint(3, 3);

		expected.add(p4);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test NewQueryFromRestoredSerializedQuery01(): Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test NewQueryFromRestoredSerializedQuery01() - Failed");
		}

		if (tx.isActive())
			tx.commit();
	}
}

