/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> New Query with Candidate Class
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.5-5.
 *<BR>
 *<B>Assertion Description: </B> <code>PersistenceManager.newQuery(Class cls)</code> constructs a query instance with the candidate class specified.
 *@test
 *@title New Query with Candidate Class
 *@executeClass com.sun.jdotck.query.NewQueryWithCandidateClass
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source NewQueryWithCandidateClass
 *@keywords query
 */

public class NewQueryWithCandidateClass extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		NewQueryWithCandidateClass newQuery = new NewQueryWithCandidateClass();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testNewQuery();

			pm.close();

			return Status.passed("\nTest NewQueryWithCandidateClass - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest NewQueryWithCandidateClass - Failed");
		}
	}

	Status testNewQuery() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery(PCPoint.class);

		if (query == null) {
			out.println("Query is null...");
		}

		query.setCandidates(pm.getExtent(PCPoint.class, false));

		Object results = query.execute();

		// check query result

		insertExpected(inserted);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("Test NewQueryWithCandidateClass - Failed");
		}

		tx.commit();
		return Status.passed("\nTest NewQueryWithCandidateClass - Passed");
	}

}

