/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.query;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.models.mylib.PCPoint;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> New Query with Candidate Class and Extent
 *<BR>
 *<B>JavaTest Keywords:</B> query extent
 *<BR>
 *<B>Assertion ID:</B> A14.5-6.
 *<BR>
 *<B>Assertion Description: </B>
 <code>PersistenceManager.newQuery(Class cls, Extent cln)</code>
 constructs a query instance with the candidate class and candidate
 <code>Extent</code> specified.

 *@test
 *@title New Query with Candidate Class and Extent
 *@executeClass com.sun.jdotck.query.NewQueryWithCandidateClassAndExtent
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source NewQueryWithCandidateClassAndExtent.java
 *@keywords query extent
 */

public class NewQueryWithCandidateClassAndExtent extends QueryTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		NewQueryWithCandidateClassAndExtent t = new NewQueryWithCandidateClassAndExtent();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			PersistenceManagerFactory pmf = getPMF();
			PersistenceManager pm = pmf.getPersistenceManager();
			initDatabase(pm, PCPoint.class);
			testNewQuery(pm);
			pm.close();
			return Status.passed("Assertion A14.5-6 passed");
		} catch (Exception e) {
			e.printStackTrace(out);
		}
		return Status.failed("Assertion A14.5-6 failed");
	}

	void testNewQuery(PersistenceManager pm) throws Exception {
		Transaction tx = pm.currentTransaction();
		Class clazz = PCPoint.class;
		Extent extent = pm.getExtent(clazz, true);
		tx.begin();
		Query query = pm.newQuery();
		if (query == null)
			throw new RuntimeException("testNewQuery(): query instance is null");
		query.setClass(clazz);
		query.setCandidates(extent);
		Object results = query.execute();
		insertExpected(inserted);
		printOutput(results);
		checkQueryResultWithoutOrder(results);
		tx.commit();
	}
}
