/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> New Query
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion IDs:</B> A14.5-1.
 *<BR>
 *<B>Assertion Description: </B>
 PersistenceManagerFactory.newQuery() constructs and empty Query instance.

 *@test
 *@title get properties
 *@executeClass com.sun.jdotck.query.NewQueryWithExtentAndFilter
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source NewQueryWithExtentAndFilter
 *@keywords query
 */

public class NewQueryWithExtentAndFilter extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		NewQueryWithExtentAndFilter newQuery = new NewQueryWithExtentAndFilter();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testNewQuery();

			pm.close();

			return Status.passed("\nTest NewQueryWithExtentAndFilter - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest NewQueryWithExtentAndFilter - Failed");
		}
	}

	Status testNewQuery() {

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery(pm.getExtent(PCPoint.class, false), "x == 1");

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);

		Object results = query.execute();

		// check query result

		Collection expected = new ArrayList();

		Object pcp1 = new PCPoint(1, 1);
		expected.add(pcp1);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("Test NewQueryWithExtentAndFilter - Failed");
		}

		tx.commit();
		return Status.passed("\nTest NewQueryWithExtentAndFilter - Passed");
	}

}

