/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> New Query With Specified Language and Query
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.5-4.
 *<BR>
 *<B>Assertion Description: </B> <code>PersistenceManager.newQuery(String language, Object query)</code> constructs a query instance using the specified language and the specified query .  The query instance returned will be of a class defined by the query language.  The language parameter for the JDO Query language as herein documented is &quot;javax.jdo.query.JDOQL&quot;.
 *@test
 *@title New Query With Specified Language and Query
 *@executeClass com.sun.jdotck.query.NewQueryWithSpecifiedLanguageAndQuery
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source NewQueryWithSpecifiedLanguageAndQuery
 *@keywords query
 */

public class NewQueryWithSpecifiedLanguageAndQuery extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		NewQueryWithSpecifiedLanguageAndQuery newQuery = new NewQueryWithSpecifiedLanguageAndQuery();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testNewQueryWithSpecifiedLanguageAndQuery();
			//testNewQueryWithSpecifiedLanguageAndQuery();

			pm.close();

			return Status.passed("\nTest NewQueryWithSpecifiedLanguageAndQuery - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest NewQueryWithSpecifiedLanguageAndQuery - Failed");
		}
	}

	/***************
	 void testNewQueryWithSpecifiedLanguageAndQuery() throws Exception {

	 out.println("\nExecuting test NewQueryWithSpecifiedLanguageAndQuery()...");

	 String expectedMsg = "Missing candidate collection or extent specification";

	 Transaction tx = pm.currentTransaction();
	 tx.begin();

	 try {
	 Query query = pm.newQuery();
	 query.setClass(PCPoint.class);
	 query.setCandidates(pm.getExtent(PCPoint.class, false));


	 Query query1 = pm.newQuery(query);
	 query1.compile();

	 Object results = query1.execute();

	 out.println("Test NewQueryWithSpecifiedLanguageAndQuery() - Failed\n");
	 Status.failed ("\nTest NewQueryWithSpecifiedLanguageAndQuery() - Failed");
	 checkMissingException(JDOUserException.class, expectedMsg);

	 } catch (JDOException ex) {
	 out.println("Exception message: " + ex.getMessage());
	 if(tx.isActive()) tx.rollback();
	 checkJDOException(ex, JDOUserException.class, expectedMsg);
	 out.println("Test NewQueryWithSpecifiedLanguageAndQuery() - Passed\n");
	 }

	 if(tx.isActive()) tx.rollback();

	 }
	 *************/

	void testNewQueryWithSpecifiedLanguageAndQuery() throws Exception {
		out.println("\nExecuting test NewQueryWithSpecifiedLanguageAndQuery()...");

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		Query query1 = pm.newQuery("javax.jdo.query.JDOQL", query);
		if (query1 == null) {
			out.println("Query1 is null...");
		}
		query1.setCandidates(pm.getExtent(PCPoint.class, false));
		query1.compile();

		Object results = query1.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p1 = new PCPoint(0, 0);
		Object p2 = new PCPoint(1, 1);
		Object p3 = new PCPoint(2, 2);
		Object p4 = new PCPoint(3, 3);
		Object p5 = new PCPoint(4, 4);

		expected.add(p1);
		expected.add(p2);
		expected.add(p3);
		expected.add(p4);
		expected.add(p5);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test NewQueryWithSpecifiedLanguageAndQuery() - Passed\n");
		} catch (Exception e) {
			out.println("Test NewQueryWithSpecifiedLanguageAndQuery() - Failed\n");
			e.printStackTrace(out);
		}

		tx.commit();
	}

}

