/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Ordering Specification
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.6-1.
 *<BR>
 *<B>Assertion Description: </B> The <code>Query</code> ordering specification includes a list of one or more ordering declarations separated by commas. Each ordering declaration includes an expression whose type is one of:
 <UL>
 <LI>primitive types except <code>boolean</code></LI>
 <LI>wrapper types except <code>Boolean</code></LI>
 <LI><code>BigDecimal</code></LI>
 <LI><code>BigInteger</code></LI>
 <LI><code>String</code></LI>
 <LI><code>Date</code></LI>
 </UL>
 followed by one of the following words: <code>&quot;ascending</code>&quot; or <code>&quot;descending</code>&quot; which indicates the ordering of the values for that expression.  Ordering might be specified including navigation. The name of the field to be used in ordering via navigation through single-valued fields is specified by the Java language syntax of <code>field_name.field_name....field_name</code>.
 *@test
 *@title Ordering Specification
 *@executeClass com.sun.jdotck.query.OrderingSpecification
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source OrderingSpecification
 *@keywords query
 */

public class OrderingSpecification extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		OrderingSpecification queryTest = new OrderingSpecification();

		Status status = queryTest.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testOrderingSpecification01();
			testOrderingSpecification02();

			pm.close();

			return Status.passed("\nTest OrderingSpecification - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest OrderingSpecification - Failed");
		}
	}

	void testOrderingSpecification01() {

		out.println("\nExecuting test OrderingSpecification01() ...");

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setOrdering("x ascending");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p1 = new PCPoint(0, 0);
		Object p2 = new PCPoint(1, 1);
		Object p3 = new PCPoint(2, 2);
		Object p4 = new PCPoint(3, 3);
		Object p5 = new PCPoint(4, 4);

		expected.add(p1);
		expected.add(p2);
		expected.add(p3);
		expected.add(p4);
		expected.add(p5);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithOrder(results);
			out.println("Test OrderingSpecification01(): Passed");
		} catch (Exception e) {
			out.println("Test OrderingSpecification01(): Failed");
			e.printStackTrace(out);
		}

		tx.commit();
	}

	void testOrderingSpecification02() {

		out.println("\nExecuting test OrderingSpecification02() ...");

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setOrdering("x descending");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p1 = new PCPoint(0, 0);
		Object p2 = new PCPoint(1, 1);
		Object p3 = new PCPoint(2, 2);
		Object p4 = new PCPoint(3, 3);
		Object p5 = new PCPoint(4, 4);

		expected.add(p5);
		expected.add(p4);
		expected.add(p3);
		expected.add(p2);
		expected.add(p1);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithOrder(results);
			out.println("Test OrderingSpecification02(): Passed");
		} catch (Exception e) {
			out.println("Test OrderingSpecification02(): Failed");
			e.printStackTrace(out);
		}

		tx.commit();
	}

}

