/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Parameter Declared with Same Name as Field of Candidate
 Class
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.4-2.
 *<BR>
 *<B>Assertion Description: </B> A field of the candidate class of a <code>Query</code> can be hidden if a parameter is declared with the same name.
 *@test
 *@title Parameter Declared with Same Name as Field of Candidate Class
 *@executeClass com.sun.jdotck.query.ParameterDeclaredWithSameNameAsFieldOfCandidateClass
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source ParameterDeclaredWithSameNameAsFieldOfCandidateClass
 *@keywords query
 */

public class ParameterDeclaredWithSameNameAsFieldOfCandidateClass extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		ParameterDeclaredWithSameNameAsFieldOfCandidateClass queryTest = new ParameterDeclaredWithSameNameAsFieldOfCandidateClass();

		Status status = queryTest.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testParameterDeclaredWithSameNameAsFieldOfCandidateClass01();
			testParameterDeclaredWithSameNameAsFieldOfCandidateClass02();

			pm.close();

			return Status.passed("\nTest ParameterDeclaredWithSameNameAsFieldOfCandidateClass - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest ParameterDeclaredWithSameNameAsFieldOfCandidateClass - Failed");
		}
	}

	Status testParameterDeclaredWithSameNameAsFieldOfCandidateClass01() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters("Integer x");
		query.setFilter("x == x");

		Object results = query.execute(new java.lang.Integer(2));

		// check query result
		Collection expected = new ArrayList();

		Object p1 = new PCPoint(0, 0);
		Object p2 = new PCPoint(1, 1);
		Object p3 = new PCPoint(2, 2);
		Object p4 = new PCPoint(3, 3);
		Object p5 = new PCPoint(4, 4);

		expected.add(p1);
		expected.add(p2);
		expected.add(p3);
		expected.add(p4);
		expected.add(p5);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("Test ParameterDeclaredWithSameNameAsFieldOfCandidateClass - Failed");
		}

		tx.commit();
		return Status.passed("\nTest ParameterDeclaredWithSameNameAsFieldOfCandidateClass - Passed");
	}

	Status testParameterDeclaredWithSameNameAsFieldOfCandidateClass02() {
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters("Integer y");
		query.setFilter("y == y");

		Object results = query.execute(new java.lang.Integer(2));

		// check query result
		Collection expected = new ArrayList();

		Object p1 = new PCPoint(0, 0);
		Object p2 = new PCPoint(1, 1);
		Object p3 = new PCPoint(2, 2);
		Object p4 = new PCPoint(3, 3);
		Object p5 = new PCPoint(4, 4);

		expected.add(p1);
		expected.add(p2);
		expected.add(p3);
		expected.add(p4);
		expected.add(p5);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("Test ParameterDeclaredWithSameNameAsFieldOfCandidateClass - Failed");
		}

		tx.commit();
		return Status.passed("\nTest ParameterDeclaredWithSameNameAsFieldOfCandidateClass - Passed");
	}
}

