/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Parentheses Mark Operator Precedence
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.2-37.
 *<BR>
 *<B>Assertion Description: </B> Parentheses can be used to explicitly mark operator precedence.

 *@test
 *@title Parentheses Mark Operator Precedence
 *@executeClass com.sun.jdotck.query.ParenthesesMarkOperatorPrecedence
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source ParenthesesMarkOperatorPrecedence
 *@keywords query
 */

public class ParenthesesMarkOperatorPrecedence extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		ParenthesesMarkOperatorPrecedence queryTest = new ParenthesesMarkOperatorPrecedence();

		Status status = queryTest.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testParenthesesMarkOperatorPrecedence01();
			testParenthesesMarkOperatorPrecedence02();
			testParenthesesMarkOperatorPrecedence03();
			testParenthesesMarkOperatorPrecedence04();
			testParenthesesMarkOperatorPrecedence05();
			testParenthesesMarkOperatorPrecedence06();
			testParenthesesMarkOperatorPrecedence07();
			testParenthesesMarkOperatorPrecedence08();
			testParenthesesMarkOperatorPrecedence09();

			pm.close();

			return Status.passed("\nTest ParenthesesMarkOperatorPrecedence - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest ParenthesesMarkOperatorPrecedence - Failed");
		}
	}

	void testParenthesesMarkOperatorPrecedence01() {

		out.println("\nExecuting Test ParenthesesMarkOperatorPrecedence01...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == (1 + 1)");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p3 = new PCPoint(2, 2);

		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test ParenthesesMarkOperatorPrecedence01 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ParenthesesMarkOperatorPrecedence01 - Failed");
		}

		tx.commit();
	}

	void testParenthesesMarkOperatorPrecedence02() {

		out.println("\nExecuting Test ParenthesesMarkOperatorPrecedence01...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == (1 + 1) * 2");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p5 = new PCPoint(4, 4);

		expected.add(p5);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test ParenthesesMarkOperatorPrecedence02 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ParenthesesMarkOperatorPrecedence02 - Failed");
		}

		tx.commit();
	}

	void testParenthesesMarkOperatorPrecedence03() {

		out.println("\nExecuting Test ParenthesesMarkOperatorPrecedence03...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == (9 - 1) * (3 + 5) / 32");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p3 = new PCPoint(2, 2);

		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test ParenthesesMarkOperatorPrecedence03 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ParenthesesMarkOperatorPrecedence03 - Failed");
		}

		tx.commit();
	}

	void testParenthesesMarkOperatorPrecedence04() {

		out.println("\nExecuting Test ParenthesesMarkOperatorPrecedence04...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == 2 && y == 2");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p3 = new PCPoint(2, 2);

		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test ParenthesesMarkOperatorPrecedence04 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ParenthesesMarkOperatorPrecedence04 - Failed");
		}

		tx.commit();
	}

	void testParenthesesMarkOperatorPrecedence05() {

		out.println("\nExecuting Test ParenthesesMarkOperatorPrecedence05...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == (1 + 1) || y == (1002 - 1000)");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p3 = new PCPoint(2, 2);

		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test ParenthesesMarkOperatorPrecedence05 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ParenthesesMarkOperatorPrecedence05 - Failed");
		}

		tx.commit();
	}

	void testParenthesesMarkOperatorPrecedence06() {

		out.println("\nExecuting Test ParenthesesMarkOperatorPrecedence06...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == (1 * 2) && true");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p3 = new PCPoint(2, 2);

		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test ParenthesesMarkOperatorPrecedence06 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ParenthesesMarkOperatorPrecedence06 - Failed");
		}

		tx.commit();
	}

	void testParenthesesMarkOperatorPrecedence07() {

		out.println("\nExecuting Test ParenthesesMarkOperatorPrecedence07...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == (10000 / 5000) || false");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p3 = new PCPoint(2, 2);

		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test ParenthesesMarkOperatorPrecedence07 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ParenthesesMarkOperatorPrecedence07 - Failed");
		}

		tx.commit();
	}

	void testParenthesesMarkOperatorPrecedence08() {

		out.println("\nExecuting Test ParenthesesMarkOperatorPrecedence08...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("(x == 2) == true");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p3 = new PCPoint(2, 2);

		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test ParenthesesMarkOperatorPrecedence08 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ParenthesesMarkOperatorPrecedence08 - Failed");
		}

		tx.commit();
	}

	void testParenthesesMarkOperatorPrecedence09() {

		out.println("\nExecuting Test ParenthesesMarkOperatorPrecedence09...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("(x == ((21 - 1/1)/10 + 1)) | (false && true)");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p4 = new PCPoint(3, 3);

		expected.add(p4);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test ParenthesesMarkOperatorPrecedence09 - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test ParenthesesMarkOperatorPrecedence09 - Failed");
		}

		tx.commit();
	}

}

