/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.query;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Query Import Statement
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.3-5.
 *<BR>
 *<B>Assertion Description: </B>
 A <code>Query</code> import statement is specified as a <code>String</code>
 with semicolon-separated statements. The syntax is the same as in the Java
 language import statement.

 *@test
 *@title Query Import Statement
 *@executeClass com.sun.jdotck.query.QueryImportStatement
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source QueryImportStatement.java
 *@keywords query
 */

public class QueryImportStatement extends QueryTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		QueryImportStatement t = new QueryImportStatement();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			PersistenceManagerFactory pmf = getPMF();
			PersistenceManager pm = pmf.getPersistenceManager();
			checkQueryImports(pm);
			pm.close();
			return Status.passed("Assertion A14.3-5 passed");
		} catch (Exception e) {
			e.printStackTrace(out);
		}
		return Status.failed("Assertion A14.3-5 failed");
	}

	void checkQueryImports(PersistenceManager pm) {
		Transaction tx = pm.currentTransaction();
		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null)
				throw new RuntimeException("checkQueryImports(): query instance is null");
			Class clazz = com.sun.jdotck.models.company.Project.class;
			query.setClass(clazz);
			query.setCandidates(pm.getExtent(clazz, false));
			query.declareImports("import com.sun.jdotck.models.company.Person; import com.sun.jdotck.models.company.Employee");
			query.declareVariables("Person a; Employee b");
			query.setFilter("reviewers.contains(a) && a.firstname==\"brazil\" || reviewers.contains(b) && b.firstname==\"brazil\"");
			Object results = query.execute();
		} catch (JDOException e) {
			e.printStackTrace(out);
			throw new RuntimeException("checkQueryImports() failed");
		} finally {
			tx.rollback();
		}
	}
}
