/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;
import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.FileInputStream;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Query is Serializable
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.3-7 A14.6-1.
 *<BR>
 *<B>Assertion Description: </B> The class implementing the <code>Query</code> interface must be serializable.  The serialized fields include the candidate class, the filter, parameter declarations, variable declarations, imports, and ordering specification.
 *@test
 *@title Query is Serializable
 *@executeClass com.sun.jdotck.query.QueryIsSerializable
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source QueryIsSerializable
 *@keywords query
 */

public class QueryIsSerializable extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		QueryIsSerializable newQuery = new QueryIsSerializable();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testQueryIsSerializable01();
			testQueryIsSerializable02();
			testQueryIsSerializable03();

			pm.close();

			return Status.passed("\nTest QueryIsSerializable - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest QueryIsSerializable - Failed");
		}
	}

	void testQueryIsSerializable01() {

		out.println("\nExecuting test QueryIsSerializable01() ...");

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		if (query instanceof Serializable) {
			out.println("Query extends serializable interface.");
			out.println("Test QueryIsSerializable01(): Passed");
		} else {
			out.println("Query does not extends serializable interface.");
			out.println("Test QueryIsSerializable01(): Failed");
		}
		tx.rollback();
	}

	void testQueryIsSerializable02() throws Exception {

		out.println("\nExecuting test QueryIsSerializable02() ...");

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.setFilter("x == 3");

		query.compile();

		ObjectOutputStream oos = null;

		try {
			out.println("Attempting to serialize Query object.");
			oos = new ObjectOutputStream(new FileOutputStream("query.ser"));
			oos.writeObject(query);
			out.println("Query object serialized.");
			out.println("Test QueryIsSerializable02(): Passed");
		} catch (Exception ex) {
			out.println("Unable to serialize Query object.");
			out.println("Test QueryIsSerializable02(): Failed");
			ex.printStackTrace(out);
			//throw new Exception("Test QueryIsSerializable02(): Failed");
		} finally {
			if (oos != null) {
				try {
					oos.flush();
				} catch (Exception ex) {
				}
				try {
					oos.close();
				} catch (Exception ex) {
				}
			}
			if (tx.isActive())
				tx.rollback();
		}

		if (tx.isActive())
			tx.commit();
	}

	void testQueryIsSerializable03() throws Exception {

		out.println("\nExecuting test QueryIsSerializable03() ...");

		Object restoredQuery = null;

		ObjectInputStream ois = null;

		try {
			out.println("Attempting to de-serialize Query object.");
			ois = new ObjectInputStream(new FileInputStream("query.ser"));
			restoredQuery = ois.readObject();
			out.println("Query object restored.");
		} catch (Exception ex) {
			out.println("Unable to deserialize Query object.");
			out.println("Test QueryIsSerializable02(): Failed");
			ex.printStackTrace(out);
			//throw new Exception("Test QueryIsSerializable02(): Failed");
		} finally {
			if (ois != null) {
				try {
					ois.close();
				} catch (Exception ex) {
				}
			}
		}

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery(restoredQuery);

		if (query == null) {
			out.println("Query is null...");
		}

		query.setCandidates(pm.getExtent(PCPoint.class, false));

		query.compile();

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p4 = new PCPoint(3, 3);

		expected.add(p4);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test QueryIsSerializable03(): Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test QueryIsSerializable03() - Failed");
		}

		if (tx.isActive())
			tx.commit();
	}
}

