/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.query;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Query Parameter Declaration
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.3-3.
 *<BR>
 *<B>Assertion Description: </B>
 A <code>Query</code> parameter declaration is a <code>String</code>
 containing one or more query parameter declarations separated with commas.
 It follows the syntax for formal parameters in the Java language.
 Each parameter named in the parameter declaration must be bound to a value
 when the query is executed.

 *@test
 *@title Query Parameter Declaration
 *@executeClass com.sun.jdotck.query.QueryParameterDeclaration
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source QueryParameterDeclaration.java
 *@keywords query
 */

public class QueryParameterDeclaration extends QueryTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		QueryParameterDeclaration t = new QueryParameterDeclaration();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			PersistenceManagerFactory pmf = getPMF();
			PersistenceManager pm = pmf.getPersistenceManager();
			checkQueryParameters(pm);
			pm.close();
			return Status.passed("\nAssertion A14.3-3 passed");
		} catch (Exception e) {
			e.printStackTrace(out);
		}
		return Status.failed("\nAssertion A14.3-3 failed");
	}

	void checkQueryParameters(PersistenceManager pm) {
		Transaction tx = pm.currentTransaction();
		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null)
				throw new RuntimeException("checkQueryParameters(): query instance is null");
			Class clazz = com.sun.jdotck.models.mylib.PCPoint.class;
			query.setClass(clazz);
			query.setCandidates(pm.getExtent(clazz, false));
			query.declareParameters("int a");
			query.setFilter("x == 0");
			Object results = query.execute();
			out.println("checkQueryParameters failed");
			throw new RuntimeException("checkQueryParameters() failed");
		} catch (JDOException e) {
			out.println("Caught expected " + e);
		} finally {
			tx.rollback();
		}
	}
}
