/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Query Result Passed to Another Query
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.1-4.
 *<BR>
 *<B>Assertion Description: </B> The collection returned by <code>Query.execute</code> can be passed to another <code>Query</code>.
 *@test
 *@title Query Result Passed to Another Query
 *@executeClass com.sun.jdotck.query.QueryResultPassedToAnotherQuery
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source QueryResultPassedToAnotherQuery
 *@keywords query
 */

public class QueryResultPassedToAnotherQuery extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;
	Collection resultCln;

	public static void main(String[] args) throws Exception {

		QueryResultPassedToAnotherQuery newQuery = new QueryResultPassedToAnotherQuery();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			setResultCollection();
			testQueryResultPassedToAnotherQuery01();
			testQueryResultPassedToAnotherQuery02();
			testQueryResultPassedToAnotherQuery03();

			pm.close();

			return Status.passed("\nTest QueryResultPassedToAnotherQuery - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest QueryResultPassedToAnotherQuery - Failed");
		}
	}

	void setResultCollection() {

		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		resultCln = (Collection) query.execute();

		tx.commit();

	}

	void testQueryResultPassedToAnotherQuery01() {

		out.println("\nExecuting test QueryResultPassedToAnotherQuery01()...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(resultCln);
		query.setFilter("x == 1");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p2 = new PCPoint(1, 1);

		expected.add(p2);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test QueryResultPassedToAnotherQuery01: Passed");
		} catch (Exception e) {
			out.println("Test QueryResultPassedToAnotherQuery01: Failed");
			e.printStackTrace(out);
		}

		tx.commit();
	}

	void testQueryResultPassedToAnotherQuery02() {

		out.println("\nExecuting test QueryResultPassedToAnotherQuery02()...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery(PCPoint.class, resultCln);

		if (query == null) {
			out.println("Query is null...");
		}

		query.setFilter("x == 1");

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p2 = new PCPoint(1, 1);

		expected.add(p2);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test QueryResultPassedToAnotherQuery02: Passed");
		} catch (Exception e) {
			out.println("Test QueryResultPassedToAnotherQuery02: Failed");
			e.printStackTrace(out);
		}

		tx.commit();
	}

	void testQueryResultPassedToAnotherQuery03() {

		out.println("\nExecuting test QueryResultPassedToAnotherQuery03()...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery(PCPoint.class, resultCln, "x == 1");

		if (query == null) {
			out.println("Query is null...");
		}

		Object results = query.execute();

		// check query result
		Collection expected = new ArrayList();

		Object p2 = new PCPoint(1, 1);

		expected.add(p2);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test QueryResultPassedToAnotherQuery03: Passed");
		} catch (Exception e) {
			out.println("Test QueryResultPassedToAnotherQuery03: Failed");
			e.printStackTrace(out);
		}

		tx.commit();
	}

}

