/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

public abstract class QueryTest extends JDO_Test {
	public Collection inserted = new ArrayList();
	public Collection reference = new ArrayList();


	public Status initDatabase(PersistenceManager pm, Class cls, int numInsert) {
		cleanupDatabase(pm, cls);
		return insertObjects(pm, cls, numInsert);
	}

	public Status initDatabase(PersistenceManager pm, Class cls) {
		cleanupDatabase(pm, cls);
		return insertObjects(pm, cls, 5);
	}

	public Status insertObjects(PersistenceManager pm, Class cls, int numInsert) {

		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();

			for (int i = 0; i < numInsert; i++) {
				Object pc = new PCPoint(i, i);
				pm.makePersistent(pc);
				inserted.add(pc);
			}

			tx.commit();

		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("Failed to insert objects.");
		}
		out.println("Total objects inserted : " + numInsert);
		return Status.passed("Objects Inserted: Done.");
	}


	public Status insertObjects(PersistenceManager pm, Class cls) {

		return insertObjects(pm, cls, 5);
	}

	public void insertExpected(Collection cls) {

		reference.clear();
		for (Iterator iter = cls.iterator(); iter.hasNext();) {
			Object pc = iter.next();
			for (Iterator iteri = inserted.iterator(); iteri.hasNext();) {
				Object pci = iteri.next();
				if (((PCPoint) pc).getX() == ((PCPoint) pci).getX()) {
					reference.add(pci);
					break;
				}
			}
		}
	}

	public Status cleanupDatabase(PersistenceManager pm, Class cls) {

		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();

			Query query = pm.newQuery();
			query.setClass(cls);
			query.setCandidates(pm.getExtent(cls, false));

			Collection result = (Collection) query.execute();

			if (result.size() != 0) {
				for (Iterator iter = result.iterator(); iter.hasNext();) {
					PCPoint pc = (PCPoint) iter.next();
					pm.deletePersistent(pc);
				}
			}

			out.println("Deleted the existing " + cls.getName() + " object...");
			tx.commit();

		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("Failed to cleanup database.");
		}
		out.println("Database cleaned...");
		return Status.passed("Database cleanup: Done.");
	}

	protected void checkQueryResultWithOrder(Object result, Collection expected)
			throws Exception {

		if (result == null) {
			throw new Exception("Query returns null");
		}
		if (!(result instanceof Collection)) {
			throw new Exception("Query result is not a collection: " + result.getClass().getName());
		}

		if (!result.equals(expected)) {
			String lf = System.getProperty("line.separator");
			throw new Exception("Wrong query result: " + lf +
					"query returns: " + result + lf +
					"expected result: " + expected);
		} else {
			out.println("Test PASSED...");
		}
	}

	protected void checkQueryResultWithOrder(Object result)
			throws Exception {

		if (result == null) {
			throw new Exception("Query returns null");
		}
		if (!(result instanceof Collection)) {
			throw new Exception("Query result is not a collection: " + result.getClass().getName());
		}

		if (!result.equals(reference)) {
			String lf = System.getProperty("line.separator");
			throw new Exception("Wrong query result: " + lf +
					"query returns: " + result + lf +
					"expected result: " + reference);
		} else {
			out.println("Test PASSED...");
		}
	}

	protected void checkQueryResultWithoutOrder(Object result, Collection expected)
			throws Exception {

		if (result == null) {
			throw new Exception("Query returns null");
		}
		if (!(result instanceof Collection)) {
			throw new Exception("Query result is not a collection: " + result.getClass().getName());
		}

		if ((!((Collection) result).containsAll(expected)) || (((Collection) result).size() != expected.size())) {
			String lf = System.getProperty("line.separator");
			throw new Exception("Wrong query result: " + lf +
					"Result size: " + ((Collection) result).size() +
					lf + "Expected size: " + expected.size() + lf +
					"query returns: " + result + lf +
					"expected result: " + expected);
		} else {
			out.println("Test PASSED...");
		}
	}

	protected void checkQueryResultWithoutOrder(Object result)
			throws Exception {

		if (result == null) {
			throw new Exception("Query returns null");
		}
		if (!(result instanceof Collection)) {
			throw new Exception("Query result is not a collection: " + result.getClass().getName());
		}

		if ((!((Collection) result).containsAll(reference)) || (((Collection) result).size() != reference.size())) {
			String lf = System.getProperty("line.separator");
			throw new Exception("Wrong query result: " + lf +
					"Result size: " + ((Collection) result).size() +
					lf + "Expected size: " + reference.size() + lf +
					"query returns: " + result + lf +
					"expected result: " + reference);
		} else {
			out.println("Test PASSED...");
		}
	}


	protected void printOutput(Object results, Collection expected) {

		Iterator iter = null;
		PCPoint pcp = null;
		if (results == null) {
			out.println("Query returns null");
		}
		if (!(results instanceof Collection)) {
			out.println("Query result is not a collection: " + results.getClass().getName());
		}
		out.println("Retrived Objects are:");
		iter = ((Collection) results).iterator();
		while (iter.hasNext()) {
			pcp = (PCPoint) iter.next();
			out.println("X = " + pcp.getX() + "\tY = " + pcp.getY());
		}

		out.println("Expected Objects are:");
		iter = ((Collection) expected).iterator();
		while (iter.hasNext()) {
			pcp = (PCPoint) iter.next();
			out.println("X = " + pcp.getX() + "\tY = " + pcp.getY());
		}
	}


	protected void printOutput(Object results) {

		Iterator iter = null;
		PCPoint pcp = null;
		if (results == null) {
			out.println("Query returns null");
		}
		if (!(results instanceof Collection)) {
			out.println("Query result is not a collection: " + results.getClass().getName());
		}
		out.println("Retrived Objects are:");
		iter = ((Collection) results).iterator();
		while (iter.hasNext()) {
			pcp = (PCPoint) iter.next();
			out.println("X = " + pcp.getX() + "\tY = " + pcp.getY());
		}

		out.println("Expected Objects are:");
		iter = ((Collection) reference).iterator();
		while (iter.hasNext()) {
			pcp = (PCPoint) iter.next();
			out.println("X = " + pcp.getX() + "\tY = " + pcp.getY());
		}
	}

	/**
	 * Checks whether the JDOExecption has the expected message
	 */

	protected void checkJDOException(JDOException ex, Class expectedException, String expectedMsg)
			throws Exception {
		String lf = System.getProperty("line.separator");
		String msg = ex.getMessage();
		if (!expectedException.isInstance(ex)) {
			throw new Exception("Wrong exception of " + ex.getClass() + lf +
					" with message " + msg + lf +
					"  Expected exception of " + expectedException + lf +
					"  with message " + expectedMsg);
		}
	}

	/**
	 * Thows an exception that indicates that the test case expected to catch a
	 * JDOQueryException which did not occur.
	 */

	protected void checkMissingException(Class expectedException, String expectedMsg)
			throws Exception {
		throw new Exception("Missing " + expectedException.getName() + "(" +
				expectedMsg + ")");
	}

}
