/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.query;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Restored Serialized Query Instance Loses Association With PM
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.3-8.
 *<BR>
 *<B>Assertion Description: </B>
 If a serialized instance is restored, it loses its association with its former
 <code>PersistenceManager</code>.

 *@test
 *@title Restored Serialized Query Instance Loses Association With PM
 *@executeClass com.sun.jdotck.query.RestoredSerializedQueryInstanceLosesAssociationWithPM
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source RestoredSerializedQueryInstanceLosesAssociationWithPM.java
 *@keywords query
 */

public class RestoredSerializedQueryInstanceLosesAssociationWithPM extends QueryTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		RestoredSerializedQueryInstanceLosesAssociationWithPM t = new RestoredSerializedQueryInstanceLosesAssociationWithPM();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			PersistenceManagerFactory pmf = getPMF();
			PersistenceManager pm = pmf.getPersistenceManager();
			checkRestoredQueryInstance(pm);
			pm.close();
			return Status.passed("Assertion A14.3-8 passed");
		} catch (Exception e) {
			e.printStackTrace(out);
		}
		return Status.failed("Assertion A14.3-8 failed");
	}

	void checkRestoredQueryInstance(PersistenceManager pm) {
		Transaction tx = pm.currentTransaction();
		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null)
				throw new RuntimeException("checkRestoredQueryInstance(): query instance is null");
			Class clazz = com.sun.jdotck.models.company.Project.class;
			query.setClass(clazz);
			query.setCandidates(pm.getExtent(clazz, false));
			query.declareVariables("com.sun.jdotck.models.company.Person a; com.sun.jdotck.models.company.Person b");
			query.setFilter("reviewers.contains(a) && a.firstname==\"brazil\" || reviewers.contains(b) && b.firstname==\"brazil\"");
			try {
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				ObjectOutputStream oos = new ObjectOutputStream(baos);
				oos.writeObject(query);
				ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(baos.toByteArray()));
				query = (Query) ois.readObject();
			} catch (IOException e) {
				e.printStackTrace(out);
				throw new RuntimeException(e.toString());
			} catch (ClassNotFoundException e) {
				e.printStackTrace(out);
				throw new RuntimeException(e.toString());
			}
			Object results = query.execute();
			throw new RuntimeException("A deserialized query instance should not execute successfully without associating that instance to a persistence manager");
		} catch (JDOException e) {
			out.println("Caught expected " + e);
		} finally {
			tx.rollback();
		}
	}
}
