/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.query;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;

import com.sun.javatest.Status;
import com.sun.jdotck.models.mylib.PCPoint;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Set Filter
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6-7.
 *<BR>
 *<B>Assertion Description: </B>
 <code>Query.setFilter(String filter)</code> binds the query filter
 to the query instance.

 *@test
 *@title Set Filter
 *@executeClass com.sun.jdotck.query.SetFilter
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source SetFilter.java
 *@keywords query
 */

public class SetFilter extends QueryTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		SetFilter t = new SetFilter();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			PersistenceManagerFactory pmf = getPMF();
			PersistenceManager pm = pmf.getPersistenceManager();
			initDatabase(pm, PCPoint.class);
			testNewQuery(pm);
			pm.close();
			return Status.passed("Assertion A14.6-7 passed");
		} catch (Exception e) {
			e.printStackTrace(out);
		}
		return Status.failed("Assertion A14.6-7 failed");
	}

	void testNewQuery(PersistenceManager pm) throws Exception {
		Transaction tx = pm.currentTransaction();
		Class clazz = PCPoint.class;
		Extent extent = pm.getExtent(clazz, true);
		tx.begin();
		Query query = pm.newQuery();
		if (query == null)
			throw new RuntimeException("testNewQuery(): query instance is null");
		query.setClass(clazz);
		query.setCandidates(extent);
		query.setFilter("x == 2");
		Object results = query.execute();
		Collection expected = new ArrayList();
		expected.add(new PCPoint(2, 2));
		insertExpected(expected);
		printOutput(results);
		checkQueryResultWithoutOrder(results);
		tx.commit();
	}
}
