/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Set IgnoreCache
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6-12.
 *<BR>
 *<B>Assertion Description: </B> <code>Query.setIgnoreCache(boolean flag)</code> sets the IgnoreCache option for queries.
 *@test
 *@title Set IgnoreCache
 *@executeClass com.sun.jdotck.query.SetIgnoreCache
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source SetIgnoreCache
 *@keywords query
 */

public class SetIgnoreCache extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		SetIgnoreCache newQuery = new SetIgnoreCache();

		Status status = newQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			testSetIgnoreCache01();
			testSetIgnoreCache02();

			pm.close();

			return Status.passed("\nTest SetIgnoreCache - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest SetIgnoreCache - Failed");
		}
	}

	void testSetIgnoreCache01() throws Exception {

		Transaction tx = pm.currentTransaction();
		tx.setOptimistic(false);
		tx.begin();

		Query query = pm.newQuery();

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		out.println("Pessimistic: IgnoreCache - Setting value = true");
		query.setIgnoreCache(true);
		if (query.getIgnoreCache() == true) {
			out.println("Pessimistic: IgnoreCache - value = " + query.getIgnoreCache());
			out.println("Pessimistic: IgnoreCache - Setting value to true: Passed");
		} else {
			out.println("Pessimistic: IgnoreCache - value = " + query.getIgnoreCache());
			out.println("Pessimistic: IgnoreCache - Setting value to true: Failed");
			throw new Exception("Pessimistic: IgnoreCache - Setting value to true - failed");
		}

		out.println("Pessimistic: IgnoreCache - Setting value = false");
		query.setIgnoreCache(false);
		if (query.getIgnoreCache() == false) {
			out.println("Pessimistic: IgnoreCache - value = " + query.getIgnoreCache());
			out.println("Pessimistic: IgnoreCache - Setting value to false: Passed");
		} else {
			out.println("Pessimistic: IgnoreCache - value = " + query.getIgnoreCache());
			out.println("Pessimistic: IgnoreCache - Setting value to false: Failed");
			throw new Exception("Pessimistic: IgnoreCache - Setting value to false - failed");
		}

		query.compile();

		tx.commit();
	}

	void testSetIgnoreCache02() throws Exception {

		Transaction tx = pm.currentTransaction();
		tx.setOptimistic(true);
		tx.begin();

		Query query = pm.newQuery();

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));

		out.println("Optimistic: IgnoreCache - Setting value = true");
		query.setIgnoreCache(true);
		if (query.getIgnoreCache() == true) {
			out.println("Optimistic: IgnoreCache - value = " + query.getIgnoreCache());
			out.println("Optimistic: IgnoreCache - Setting value to true: Passed");
		} else {
			out.println("Optimistic: IgnoreCache - value = " + query.getIgnoreCache());
			out.println("Optimistic: IgnoreCache - Setting value to true: Failed");
			throw new Exception("Optimistic: IgnoreCache - Setting value to true - failed");
		}

		out.println("Optimistic: IgnoreCache - Setting value = false");
		query.setIgnoreCache(false);
		if (query.getIgnoreCache() == false) {
			out.println("Optimistic: IgnoreCache - value = " + query.getIgnoreCache());
			out.println("Optimistic: IgnoreCache - Setting value to false: Passed");
		} else {
			out.println("Optimistic: IgnoreCache - value = " + query.getIgnoreCache());
			out.println("Optimistic: IgnoreCache - Setting value to false: Failed");
			throw new Exception("Optimistic: IgnoreCache - Setting value to false - failed");
		}

		query.compile();

		tx.commit();
	}
}

