/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Use of this to Access Hidden Field
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.4-4.
 *<BR>
 *<B>Assertion Description: </B> A hidden field may be accessed using the <code>'this'</code> qualifier: <code>this.fieldName</code>.
 *@test
 *@title Use of this to Access Hidden Field
 *@executeClass com.sun.jdotck.query.UseOfThisToAcessHiddenField
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source UseOfThisToAcessHiddenField
 *@keywords query
 */

public class UseOfThisToAcessHiddenField extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {

		UseOfThisToAcessHiddenField queryTest = new UseOfThisToAcessHiddenField();

		Status status = queryTest.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);
			testUseOfThisToAcessHiddenField01();
			testUseOfThisToAcessHiddenField02();

			pm.close();

			return Status.passed("\nTest UseOfThisToAcessHiddenField - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest UseOfThisToAcessHiddenField - Failed");
		}
	}

	void testUseOfThisToAcessHiddenField01() {

		out.println("\nExecuting test UseOfThisToAcessHiddenField01() ...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters("Integer x");
		query.setFilter("this.x == x");

		Object results = query.execute(new java.lang.Integer(2));

		// check query result
		Collection expected = new ArrayList();
		Object p3 = new PCPoint(2, 2);
		expected.add(p3);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test UseOfThisToAcessHiddenField01(): Passed");
		} catch (Exception e) {
			out.println("Test UseOfThisToAcessHiddenField01(): Failed");
			e.printStackTrace(out);
		}

		tx.commit();
	}

	void testUseOfThisToAcessHiddenField02() {

		out.println("\nExecuting test UseOfThisToAcessHiddenField02() ...");
		Transaction tx = pm.currentTransaction();
		tx.begin();

		Query query = pm.newQuery();

		if (query == null) {
			out.println("Query is null...");
		}

		query.setClass(PCPoint.class);
		query.setCandidates(pm.getExtent(PCPoint.class, false));
		query.declareParameters("Integer y");
		query.setFilter("this.y == y");

		Object results = query.execute(new java.lang.Integer(3));

		// check query result
		Collection expected = new ArrayList();
		Object p4 = new PCPoint(3, 3);
		expected.add(p4);

		insertExpected(expected);

		printOutput(results);

		try {
			checkQueryResultWithoutOrder(results);
			out.println("Test UseOfThisToAcessHiddenField02(): Passed");

		} catch (Exception e) {
			e.printStackTrace(out);
			out.println("Test UseOfThisToAcessHiddenField02(): Failed");
		}

		tx.commit();
	}
}

