/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.models.company.Project;

import java.io.PrintWriter;
import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Variable declarations follow the Java syntax.
 *<BR>
 *<B>JavaTest Keywords:</B> query parameter check
 *<BR>
 *<B>Assertion ID:</B> A14.3-3.
 *<BR>
 *<B>Assertion Description: </B> A query variable declaration is a string containing oue or more variable
 *declarations. It follows the syntax for local variables in the Java language.
 *@test
 *@title Variable declarations follow the Java syntax.
 *@executeClass com.sun.jdotck.query.VariableDeclarationCheck
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source VariableDeclarationCheck
 *@keywords query
 */

public class VariableDeclarationCheck extends QueryTest {
	protected PersistenceManagerFactory pmf;
	protected PersistenceManager pm;
	int numInsert = 5;

	public static void main(String[] args) throws Exception {
		VariableDeclarationCheck queryTest = new VariableDeclarationCheck();
		Status status = queryTest.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			checkQueryParameters();
			pm.close();
			return Status.passed("\nTest VariableDeclarationCheck - Passed");
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest VariableDeclarationCheck - Failed");
		}
	}

	void checkQueryParameters() {
		Transaction tx = pm.currentTransaction();
		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null) {
				out.println("Query is null...");
			}
			query.setClass(Project.class);
			query.setCandidates(pm.getExtent(Project.class, false));
			query.declareVariables("com.sun.jdotck.models.company.Person a; com.sun.jdotck.models.company.Person b");
			query.setFilter("reviewers.contains(a) && a.firstname==\"brazil\" || reviewers.contains(b) && b.firstname==\"brazil\"");
			Object results = query.execute();
			tx.rollback();
		} catch (JDOException e) {
			e.printStackTrace(out);
			tx.rollback();
			throw new RuntimeException("Test VariableDeclarationCheck: Failed");
		}
	}
}

