/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */


package com.sun.jdotck.query;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> Variable Declared with Same Name as Field of Candidate Class
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.4-3.
 *<BR>
 *<B>Assertion Description: </B>
 A field of the candidate class of a <code>Query</code> can be hidden if a
 variable is declared with the same name.

 *@test
 *@title Variable Declared with Same Name as Field of Candidate Class
 *@executeClass com.sun.jdotck.query.VariableDeclaredWithSameNameAsFieldOfCandidateClass
 *@executeArgs -PMFProperties $PMFProperties
 * -ApplicationIdentity $ApplicationIdentity
 * -Array $Array
 * -ArrayList $ArrayList
 * -ChangeApplicationIdentity $ChangeApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -List $List
 * -Map $Map
 * -NonDurableIdentity $NonDurableIdentity
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -NullCollection $NullCollection
 * -Optimistic $Optimistic
 * -RetainValues $RetainValues
 * -TransientTransactional $TransientTransactional
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 *@source VariableDeclaredWithSameNameAsFieldOfCandidateClass.java
 *@keywords query
 */

public class VariableDeclaredWithSameNameAsFieldOfCandidateClass extends QueryTest {

	/**
	 *The <code>main</code> is called when the class
	 *is directly executed from the command line.
	 *@param args The arguments passed to the program.
	 */
	public static void main(String[] args) {
		VariableDeclaredWithSameNameAsFieldOfCandidateClass t = new VariableDeclaredWithSameNameAsFieldOfCandidateClass();
		Status s = t.run(args, new PrintWriter(System.err), new PrintWriter(System.out));
		s.exit();
	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {
		initializeArgs(args, err, out);
		try {
			PersistenceManagerFactory pmf = getPMF();
			PersistenceManager pm = pmf.getPersistenceManager();
			checkQueryVariables(pm);
			pm.close();
			return Status.passed("Assertion A14.4-3 passed");
		} catch (Exception e) {
			e.printStackTrace(out);
		}
		return Status.failed("Assertion A14.4-3 failed");
	}

	void checkQueryVariables(PersistenceManager pm) {
		Transaction tx = pm.currentTransaction();
		Class clazz = com.sun.jdotck.models.company.Project.class;
		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null)
				throw new RuntimeException("checkQueryVariables(): query instance is null");
			query.setClass(clazz);
			query.setCandidates(pm.getExtent(clazz, false));
			query.declareVariables("com.sun.jdotck.models.company.Person reviewers;");
			query.setFilter("reviewers.contains(reviewers)");
			Object results = query.execute();
			throw new RuntimeException("Variable declaration \"Person reviewers\" did not hide field Person.reviewers");
		} catch (JDOException e) {
			out.println("Caught expected " + e);
		} finally {
			tx.rollback();
		}

		try {
			tx.begin();
			Query query = pm.newQuery();
			if (query == null)
				throw new RuntimeException("checkQueryVariables(): query instance is null");
			query.setClass(clazz);
			query.setCandidates(pm.getExtent(clazz, false));
			query.declareVariables("com.sun.jdotck.models.company.Person reviewers;");
			query.setFilter("this.reviewers.contains(reviewers) && reviewers.firstname==\"brazil\"");
			Object results = query.execute();
		} finally {
			tx.rollback();
		}
	}
}
