/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;

/**
 *<B>JavaTest Title:</B> White Space is a Character and is Ignored
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.2-6.
 *<BR>
 *<B>Assertion Description: </B> White space (non-printing characters space, tab, carriage return, and line feed) is a separator and is otherwise ignored in a <code>Query</code> filter.
 *@test
 *@title White Space is a Character and is Ignored
 *@executeClass com.sun.jdotck.query.WhiteSpaceIsACharacterAndIgnored
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source WhiteSpaceIsACharacterAndIgnored
 *@keywords query
 */

public class WhiteSpaceIsACharacterAndIgnored extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;

	public static void main(String[] args) throws Exception {

		WhiteSpaceIsACharacterAndIgnored queryTest = new WhiteSpaceIsACharacterAndIgnored();

		Status status = queryTest.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);

		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			initDatabase(pm, PCPoint.class);

			initExpectedResult("x == 0");

			// Escape Sequence
			testWhiteSpaceIsACharacterAndIgnored01("x\t == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\n == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\f == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\r == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\n == \t 0 \r");
			// Unicode
			testWhiteSpaceIsACharacterAndIgnored01("x\u0009 == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\u000c == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\u0020 == 0");
			// Octal Escape
			testWhiteSpaceIsACharacterAndIgnored01("x\11 == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\12 == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\14 == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\15 == 0");
			testWhiteSpaceIsACharacterAndIgnored01("x\40 == 0");

			pm.close();
		} catch (Exception e) {
			e.printStackTrace(out);
			return Status.failed("\nTest WhiteSpaceIsACharacterAndIgnored - Failed");
		}

		if (getErrorCount() == 0) {
			return Status.passed("\nTest WhiteSpaceIsACharacterAndIgnored - Passed");
		} else {
			return Status.failed("\nTest WhiteSpaceIsACharacterAndIgnored - Failed");
		}
	}

	void initExpectedResult(String filter) {

		Transaction tx = pm.currentTransaction();
		Query query = pm.newQuery();

		try {
			tx.begin();

			query.setClass(PCPoint.class);
			query.setCandidates(pm.getExtent(PCPoint.class, false));
			query.setFilter(filter);

			reference = (Collection) query.execute();
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			error(ex.getLocalizedMessage());
		}
	}

	void testWhiteSpaceIsACharacterAndIgnored01(String filter) {

		Transaction tx = pm.currentTransaction();
		Query query = pm.newQuery();
		Collection results = null;

		try {
			tx.begin();

			query.setClass(PCPoint.class);
			query.setCandidates(pm.getExtent(PCPoint.class, false));
			query.setFilter(filter);

			results = (Collection) query.execute();
			printOutput(results);

			checkQueryResultWithoutOrder(results);
			out.println("Test WhiteSpaceIsACharacterAndIgnored01(\"" + filter + "\"): Passed");
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			error(ex.getLocalizedMessage());
			out.println("Test WhiteSpaceIsACharacterAndIgnored01(\"" + filter + "\"): Failed");
		}
	}

	private void error(String message) {
		err.println("Exception caught: " + message);
		incrementErrorCount();
	}

}
