package com.sun.jdotck.query.operators;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.query.QueryTest;
import com.sun.jdotck.InitCompany;
import com.sun.jdotck.models.mylib.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.io.PrintWriter;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Binary Addition Query Operator
 *<BR>
 *<B>JavaTest Keywords:</B> query
 *<BR>
 *<B>Assertion ID:</B> A14.6.2-26.
 *<BR>
 *<B>Assertion Description: </B> The binary addition operator (<code>+</code>) is supported for all types as they are defined in the Java language. This includes the following types:
 <UL>
 <LI><code>byte, short, int, long, char, Byte, Short Integer, Long, Character</code></LI>
 <LI><code>float, double, Float, Double</code></LI>
 <LI><code>BigDecimal, BigInteger</code></LI>
 </UL>
 The operation on object-valued fields of wrapper types (<code>Boolean, Byte, Short, Integer, Long, Float</code>, and <code>Double</code>), and numeric types (<code>BigDecimal</code> and <code>BigInteger</code>) use the wrapped values as operands.
 *@test
 *@title Binary Addition Query Operator
 *@executeClass com.sun.jdotck.query.operators.BinaryAddition
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection

 *@source BinaryAddition
 *@keywords query
 */

public class BinaryAddition extends QueryTest {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;

	public static void main(String[] args) throws Exception {

		BinaryAddition testQuery = new BinaryAddition();

		Status status = testQuery.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		status.exit();
	}

	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		InitCompany company = new InitCompany();
		Status status = null;

		initializeArgs(args, err, out);

		try {
			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();
			company.loadPrimitiveTypes(pm);
			status = testBinaryAddition();
			pm.close();
		} catch (Exception e) {
			e.printStackTrace(out);
			status = Status.failed("\nTest BinaryAddition - Failed");
		}

		return status;
	}

	Status testBinaryAddition() {

		out.println("\nExecuting test BinaryAddition() ...");

		Transaction tx = pm.currentTransaction();
		Collection expected = null;
		Query query = null;

		try {
			tx.begin();
			query = pm.newQuery();

			if (query == null) {
				out.println("Query is null...");
			}

			query.setClass(PrimitiveTypes.class);
			query.setCandidates(pm.getExtent(PrimitiveTypes.class, false));
			query.setFilter("id == 9");
			expected = (Collection) query.execute();

			testQuery(query, "id = 1 + 8", expected);
			testQuery(query, "byteNotNull == 1 + 8", expected);
			testQuery(query, "shortNotNull == 1 + 8", expected);
			testQuery(query, "intNotNull == 1 + 8", expected);
			testQuery(query, "longNotNull == 1 + 8", expected);
			testQuery(query, "charNotNull == 1 + 8", expected);
			testQuery(query, "floatNotNull == 1 + 8", expected);
			testQuery(query, "doubleNotNull == 1 + 8", expected);
			testQuery(query, "byteNull == 1 + 8", expected);
			testQuery(query, "shortNull == 1 + 8", expected);
			testQuery(query, "intNull == 1 + 8", expected);
			testQuery(query, "longNull == 1 + 8", expected);
			testQuery(query, "charNull == 1 + 8", expected);
			testQuery(query, "floatNull == 1 + 8", expected);
			testQuery(query, "doubleNull == 1 + 8", expected);
			testQuery(query, "bigDecimal == 1 + 8", expected);
			testQuery(query, "bigInteger == 1 + 8", expected);

			tx.commit();
		} catch (Exception e) {
			tx.rollback();
			e.printStackTrace(out);
			return Status.failed("\nTest BinaryAddition - Failed");
		}

		if (getErrorCount() > 0) {
			return Status.failed("\nTest BinaryAddition - Failed");
		} else {
			return Status.passed("\nTest BinaryAddition - Passed");
		}
	}

	private void testQuery(Query query, String filter, Collection expected) {
		Collection results = null;

		query.setFilter(filter);
		results = (Collection) query.execute();

		// check query result
		if (expected.size() != results.size()) {
			incrementErrorCount();
		} else {
			out.println("Size = " + results.size());
			for (Iterator i1 = results.iterator(), i2 = expected.iterator();
				 i1.hasNext();) {
				Object pc1 = i1.next();
				Object pc2 = i2.next();
				out.println("Object is " + pc1);
				if (pc1 == null || pc2 == null || !pc1.equals(pc2)) {
					incrementErrorCount();
				}
			}
		}
	}

}

