/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.query.operators;

import java.io.PrintWriter;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.fieldtypes.AllTypes;

import java.util.Iterator;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import java.util.Calendar;
import java.util.Locale;
import java.text.DateFormat;
import java.math.BigInteger;
import java.math.BigDecimal;
import javax.jdo.*;

public abstract class ComparisonTests extends JDO_Test {
	protected PersistenceManager pm;
	protected Query query;
	protected Transaction tx;
	protected Collection query_result;

	protected static String BooleanParameter = "Boolean value";
	protected static String booleanParameter = "boolean value";
	protected static String ByteParameter = "Byte value";
	protected static String byteParameter = "byte value";
	protected static String CharacterParameter = "Character value";
	protected static String charParameter = "char value";
	protected static String DoubleParameter = "Double value";
	protected static String doubleParameter = "double value";
	protected static String FloatParameter = "Float value";
	protected static String floatParameter = "float value";
	protected static String IntegerParameter = "Integer value";
	protected static String intParameter = "int value";
	protected static String LongParameter = "Long value";
	protected static String longParameter = "long value";
	protected static String ShortParameter = "Short value";
	protected static String shortParameter = "short value";
	protected static String StringParameter = "String value";
	protected static String LocaleParameter = "java.util.Locale value";
	protected static String BigDecimalParameter = "java.math.BigDecimal value";
	protected static String BigIntegerParameter = "java.math.BigInteger value";
	protected static String DateParameter = "java.util.Date value";
	protected static String AllTypesParameter = "com.sun.jdotck.models.fieldtypes.AllTypes value";

	protected void verifyDataLoaded() {
		tx.begin();
		Extent e = pm.getExtent(AllTypes.class, false);
		Iterator i = e.iterator();
		int cnt = 0;
		while (i.hasNext()) {
			Object o = i.next();
			cnt++;
		}
		e.close(i);
		tx.rollback();
		if (cnt == AllTypes.NUM_VALUES)
			return;
		i = e.iterator();
		while (i.hasNext()) {
			Object o = i.next();
			pm.deletePersistent(o);
		}
		e.close(i);
		AllTypes.load(pm);
	}

	protected void printErrorAndContext(String message, String filter, String parameter) {
		out.println();
		out.print(message);
		out.print(" with filter \"");
		out.print(filter);
		out.print("\"");
		if (parameter != null) {
			out.print(", parameter \"");
			out.print(parameter);
			out.print("\"");
		}
		out.println();
		incrementErrorCount();
	}

	protected void runQuery(String filter, String parameter, Object parameterValue) {
		Extent e = pm.getExtent(AllTypes.class, false);
		query = pm.newQuery(e, filter);
		query_result = null;
		try {
			if (parameter != null) {
				query.declareParameters(parameter);
				query_result = (Collection) query.execute(parameterValue);
			} else {
				query_result = (Collection) query.execute();
			}
		} catch (Exception exception) {
			printErrorAndContext("Exception on Query.execute", filter, parameter);
			out.println(e);
			exception.printStackTrace(out);
			query_result = null;
			if (tx.isActive()) tx.rollback();
			return;
		}
		if (query_result == null) {
			printErrorAndContext("Query.execute returned a null", filter, parameter);
			if (tx.isActive()) tx.rollback();
		}
	}
}

