/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> After Completion Method Called When Rolledback
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.3-7.
 *<BR>
 *<B>Assertion Description: </B>
 The afterCompletion method of the Synchronization instance registered with a Transaction
 will be called after the behavior specified for the method rollback with a value of Status.STATUS_ROLLEDBACK.


 *@test
 *@title  After Completion Method Called When Rolledback
 *@executeClass com.sun.jdotck.transactions.AfterCompletionMethodCalledWhenRolledback
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source AfterCompletionMethodCalledWhenRolledback.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/11/01	 1.0
 */
public class AfterCompletionMethodCalledWhenRolledback extends JDO_Test implements javax.transaction.Synchronization {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction tx;
	private boolean flag;


	public void beforeCompletion() {
		//out.println ("before Complition isActive returns :"  + tx.isActive());
	}

	public void afterCompletion(int status) {

		if (status == javax.transaction.Status.STATUS_ROLLEDBACK) {
			flag = true;
		} else {
			out.println(" incorrect status , status is :" + status);
			incrementErrorCount();
		}
	}


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		AfterCompletionMethodCalledWhenRolledback t = new AfterCompletionMethodCalledWhenRolledback();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.setSynchronization()
	 *
	*/



	public void testAfterCompletionMethodCalledWhenRolledback() {
		try {

			tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);

			tx.setSynchronization(this);
			flag = false;
			tx.rollback();
			//tx.commit();
			if (flag == true) {
				/* Test passes */
			} else {
				out.println(" Commit didn't invoke afterCompletion method");
				incrementErrorCount();
			}


		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexception caught in AfterCompletionMethodCalledWhenRolledback.java");
			incrementErrorCount();

		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();


		this.testAfterCompletionMethodCalledWhenRolledback();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in AfterCompletionMethodCalledWhenRolledback");
		else
			return Status.failed("FAILED in AfterCompletionMethodCalledWhenRolledback: number of errors detected is " + errCnt);


	}

}
