/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Commit
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.4-1.
 *<BR>
 *<B>Assertion Description: </B>
 The <code>commit</code> method performs the following operations:
 <UL>
 <LI>calls the <code>beforeCompletion</code> method of the
 <code>Synchronization</code> instance registered with the
 <code>Transaction</code>;</LI>
 <LI>flushes dirty persistent instances;</LI>
 <LI>notifies the underlying data store to commit the transaction
 (this cannot be tested)</LI>
 <LI>transitions persistent instances according to the life cycle specification;</LI>
 <LI>calls the <code>afterCompletion</code> method of the <code>Synchronization</code>
 instance registered with the <code>Transaction</code> with the results of the
 data store commit operation.</LI>
 </UL>
 *@test
 *@title  Commit
 *@executeClass com.sun.jdotck.transactions.Commit
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source Commit.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/15/01	 1.0
 */
public class Commit extends JDO_Test implements javax.transaction.Synchronization {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction tx;
	private boolean flag1;
	private boolean flag2;
	private boolean flag3;
	private boolean flag4;


	public void beforeCompletion() {
		flag1 = true;
		out.println("beforeCompletion called ");
	}

	public void afterCompletion(int status) {

		if (status == javax.transaction.Status.STATUS_COMMITTED) {
			flag2 = true;
			out.println("afterCompletion called\n ");
		} else {
			out.println(" incorrect status\n ");
			incrementErrorCount();
		}
	}


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		Commit t = new Commit();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.setSynchronization()
	 *
	*/



	public void testCommit() {
		try {

			tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			if (JDOHelper.isDirty(p1))
				flag3 = true;
			else {
				out.println(" incorrect Object status ");
				incrementErrorCount();
			}

			tx.setSynchronization(this);
			flag1 = false;
			flag2 = false;
			flag4 = false;
			tx.commit();
			flag3 = JDOHelper.isDirty(p1);
			if (flag3 == false)
				out.println("Object changed from dirty to non-dirty\n");
			if (flag1 == true && flag2 == true && flag3 == false) {
				/* Test passes */
			} else {
				out.println(" Commit didn't complete the process\n");
				incrementErrorCount();
			}


		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexception caught in Commit.java");
			incrementErrorCount();

		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();


		this.testCommit();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in Commit");
		else
			return Status.failed("FAILED in Commit: number of errors detected is " + errCnt);


	}

}
