/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Get Persistence Manager
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.1-1.
 *<BR>
 *<B>Assertion Description: </B>
 The transactions.getPersistenceManager method returns associated persistence manager
 if the object parameter ia not null and implements persistenceCapable.
 evaluating to true when == is used.
 *@test
 *@title Get Persistence Manager
 *@executeClass com.sun.jdotck.transactions.GetPersistenceManager
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source GetPersistenceManager.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	9/26/01		 1.0
 */
public class GetPersistenceManager extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction transaction;

	//private JDOHelper                   jhlpr;
	/* standalone interface */
	public static void main(String[] args) throws Exception {

		GetPersistenceManager t = new GetPersistenceManager();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.getPersistenceManager(Object pc)
	 *
	*/
	public Status testGetPersistenceManager() {
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			pm1 = tx.getPersistenceManager();
			tx.commit();
			if (pm1 == null)
				return Status.failed("FAILED in testGetPersistenceManager()");

			else
				return Status.passed("PASSED in testGetPersistenceManager()");

		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in testGetPersistenceManager()");
		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);
		try {

			PersistenceManagerFactory pmf = getPMF();
			pm = pmf.getPersistenceManager();

			testGetPersistenceManager();

			Status s1 = this.testGetPersistenceManager();
			pm.close();

			err.flush();
			out.flush();

			if (s1.isPassed())
				return Status.passed("PASSED in GetPersistenceManager");
			else
				return Status.failed("FAILED in GetPersistenceManager");

		} catch (Exception ex) {
			ex.printStackTrace(out);
			return Status.failed("FAILED in GetPersistenceManager()");
		}

	}

}
