/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Is Active Until After Completion Method Called
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.1-3.
 *<BR>
 *<B>Assertion Description: </B>
 Transaction.isActive returns true after the transaction has been started, until the afterCompletion
 synchronization method is called
 *@test
 *@title Is Active Until After Completion Method Called
 *@executeClass com.sun.jdotck.transactions.IsActiveUntilAfterCompletionMethodCalled
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source IsActiveUntilAfterCompletionMethodCalled.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/09/01	 1.0
 */
public class IsActiveUntilAfterCompletionMethodCalled extends JDO_Test implements javax.transaction.Synchronization {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction tx;
	private boolean flag;
	private boolean flag2;


	public void beforeCompletion() {
		out.println("before Complition isActive returns :" + tx.isActive());
	}

	public void afterCompletion(int status) {
		out.println("after Complition isActive returns :" + tx.isActive());
	}


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		IsActiveUntilAfterCompletionMethodCalled t = new IsActiveUntilAfterCompletionMethodCalled();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.isActive()
	 *
	*/



	public void testIsActiveUntilAfterCompletionMethodCalled() {
		try {

			tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			tx.setSynchronization(this);

			flag = tx.isActive();
			tx.commit();
			out.println("after Commit isActive returns :" + tx.isActive());
			if (flag == true) {
				;
			} else {
				incrementErrorCount();
			}

			flag2 = tx.isActive();
			if (flag2 == false) {
				;
			} else {
				incrementErrorCount();
			}

		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexception caught in IsActiveUntilAfterCompletionMethodCalled");
			incrementErrorCount();

		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();


		this.testIsActiveUntilAfterCompletionMethodCalled();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in IsActiveUntilAfterCompletionMethodCalled");
		else
			return Status.failed("FAILED in IsActiveUntilAfterCompletionMethodCalled: number of errors detected is " + errCnt);


	}

}
