/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Rollback
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.4-2.
 *<BR>
 *<B>Assertion Description: </B>
 The <code>commit</code> method performs the following operations:
 <UL>
 <LI>transitions persistent instances
 according to the life cycle specification;</LI>
 <LI>rolls back changes made in this transaction from the data store;</LI>
 <LI>calls the <code>afterCompletion</code> method of the
 <code>Synchronization</code> instance registered with the
 <code>Transaction</code>.</LI>
 </UL>

 *@test
 *@title  Rollback
 *@executeClass com.sun.jdotck.transactions.Rollback
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source Rollback.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/17/01	 1.0
 */
public class Rollback extends JDO_Test implements javax.transaction.Synchronization {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction tx;
	private boolean flag1;
	private boolean flag2;
	private boolean flag3;
	private boolean flag4;


	public void beforeCompletion() {
		flag1 = true;
		out.println("beforeCompletion called ");
	}

	public void afterCompletion(int status) {

		if (status == javax.transaction.Status.STATUS_ROLLEDBACK) {
			flag2 = true;
			out.println("afterCompletion called ");
		} else {
			out.println(" incorrect status\n ");
			incrementErrorCount();
		}
	}


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		Rollback t = new Rollback();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.setSynchronization()
	 *
	*/



	public void testRollback() {
		try {

			tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			tx.commit();

			tx.begin();
			p1.setX(55);

			if (JDOHelper.isDirty(p1))
				flag3 = true;
			else {
				out.println(" incorrect Object status ");
				incrementErrorCount();
			}

			tx.setSynchronization(this);
			flag1 = false;
			flag2 = false;
			flag4 = false;
			tx.rollback();

			flag3 = JDOHelper.isDirty(p1);
			if (flag3 == false)
				out.println("Object changed from dirty to non-dirty");


			int xx = p1.getX();
			out.println("xx =" + xx);
			if (xx == 55)
				flag4 = true;
			else
				flag4 = false;


			if (flag1 == false && flag2 == true && flag3 == false && flag4 == false) {
				/* Test passes */
			} else {
				out.println("Rollback didn't complete the process\n");
				incrementErrorCount();
			}


		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexception caught in Rollback.java");
			incrementErrorCount();

		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();


		this.testRollback();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in Rollback");
		else
			return Status.failed("FAILED in Rollback: number of errors detected is " + errCnt);


	}

}
