/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Set Nontransactional Read
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.2-9.
 *<BR>
 *<B>Assertion Description: </B>
 If an implementation supports nontransactional read, then a call to
 <code>Transaction.setNontransactionalRead</code> with a parameter value of
 <code>true</code> will set the flag to <code>true</code> and allows persistent
 instances to be read outside of a transaction.
 Queries and navigation will be allowed without an active transaction.
 *@test
 *@title Set Nontransactional Read
 *@executeClass com.sun.jdotck.transactions.SetNontransactionalRead
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetNontransactionalRead.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/18/01	 1.0
 */
public class SetNontransactionalRead extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction transaction;
	private boolean flag;
	private boolean ntrf;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		SetNontransactionalRead t = new SetNontransactionalRead();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.setNonteansactionalRead()
	 *
	*/
	public void testSetNontransactionalRead() {
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			p1.setX(55);
			if (ntrf == false)
				out.println(" This test can't be performed due to nature of implementation ");
			else
				tx.setNontransactionalRead(true);
			if (tx.getNontransactionalRead()) {
				;
			} else {
				out.println(" getNontransactionalRead is not set properly ");
				incrementErrorCount();
			}

			tx.commit();
			if (tx.isActive()) {
				out.println("Transaction still is active ");
				incrementErrorCount();
			} else {
				int x = p1.getX();
				out.println("x= " + x);
			}


		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexception caught in SetNontransactionalRead");
			incrementErrorCount();

		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		ntrf = pmf.supportedOptions().contains("javax.jdo.option.NontransactionalRead");

		testSetNontransactionalRead();
		this.testSetNontransactionalRead();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in SetNontransactionalRead");
		else
			return Status.failed("FAILED in SetNontransactionalRead: number of errors detected is " + errCnt);


	}

}
