/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Set Nontransactional Read Called During TX Completion
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.2-1.
 *<BR>
 *<B>Assertion Description: </B>
 If the setNontransactionalRead method of the Transaction interface is
 called during commit or rollback processing (within the beforeCompletion and afterCompletion
 synchronization methods), a JDOUserException is thrown.

 *@test
 *@title Set Nontransactional Read Called During TX Completion
 *@executeClass com.sun.jdotck.transactions.SetNontransactionalReadCalledDuringTxCompletion
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetNontransactionalReadCalledDuringTxCompletion.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/18/01	 1.0
 */
public class SetNontransactionalReadCalledDuringTxCompletion extends JDO_Test implements javax.transaction.Synchronization {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction tx;
	private boolean flag;
	private boolean ntrf;

	public void beforeCompletion() {
		try {
			out.println(" beforeCompletion ");
			tx.setNontransactionalRead(true);

		} catch (Exception ex1) {
			if (ex1 instanceof javax.jdo.JDOUserException) {
				ex1.printStackTrace(out);
				out.println(" Test pass ");
			} else {
				ex1.printStackTrace(out);
				out.println(" Unexception caught in beforeCompletion");
				incrementErrorCount();
			}


		}
	}

	public void afterCompletion(int status) {
		try {
			out.println(" afterCompletion ");
			tx.setNontransactionalRead(true);
		} catch (Exception ex2) {
			if (ex2 instanceof javax.jdo.JDOUserException) {
				ex2.printStackTrace(out);
				out.println(" Test pass ");
			} else {
				ex2.printStackTrace(out);
				out.println(" Unexception caught in afterCompletion");
				incrementErrorCount();
			}


		}
	}


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		SetNontransactionalReadCalledDuringTxCompletion t = new SetNontransactionalReadCalledDuringTxCompletion();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.setNonteansactionalRead()
	 *
	*/
	public void testSetNontransactionalReadCalledDuringTxCompletion() {
		try {

			tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			p1.setX(55);
			if (ntrf == false)
				out.println(" This test can't be performed due to nature of implementation ");
			else
				tx.setSynchronization(this);
			tx.commit();

		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexception caught in SetNontransactionalReadCalledDuringTxCompletion");
			incrementErrorCount();

		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		ntrf = pmf.supportedOptions().contains("javax.jdo.option.NontransactionalRead");

		testSetNontransactionalReadCalledDuringTxCompletion();
		this.testSetNontransactionalReadCalledDuringTxCompletion();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in SetNontransactionalReadCalledDuringTxCompletion");
		else
			return Status.failed("FAILED in SetNontransactionalReadCalledDuringTxCompletion: number of errors detected is " + errCnt);


	}

}
