/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Set Retain Values
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.2-18.
 *<BR>
 *<B>Assertion Description: </B>
 The retainValues setting passed to setRetainValues replaces
 the retainValues setting currently active.
 *@test
 *@title Set Retain Values
 *@executeClass com.sun.jdotck.transactions.SetRetainValues
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetRetainValues.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/09/01	 1.0
 */
public class SetRetainValues extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction transaction;
	private boolean flag;
	private boolean flag2;
	private boolean flag3;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		SetRetainValues t = new SetRetainValues();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.isActive()
	 *
	*/
	public void testSetRetainValues() {
		try {
			flag = true;
			Transaction tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			tx.commit();
			tx.setRetainValues(flag);

			flag2 = tx.getRetainValues();
			tx.setRetainValues(false);
			flag3 = tx.getRetainValues();

			if (flag2 == true && flag3 == false) {
				;
			} else {
				out.println(" flag2 =" + flag2);
				out.println(" flag3 =" + flag3);
				incrementErrorCount();
			}


		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexception caught in SetRetainValues");
			incrementErrorCount();

		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		testSetRetainValues();
		this.testSetRetainValues();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)

			return Status.passed("PASSED in SetRetainValues" + errCnt);
		else
			return Status.failed("FAILED in SetRetainValues: number of errors detected is " + errCnt);


	}

}
