/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Set Optimistic Called During TX Completion
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.2-4.
 *<BR>
 *<B>Assertion Description: </B>
 If the setOptimistic method of the Transaction interface is called during commit or rollback processing (within the
 beforeCompletion and afterCompletion synchronization methods), a JDOUserException is thrown.


 *@test
 *@title Set Optimistic Called During TX Completion
 *@executeClass com.sun.jdotck.transactions.SetRetainValuesCalledDuringTxCompletion
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetRetainValuesCalledDuringTxCompletion.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/20/01	 1.0
 */
public class SetRetainValuesCalledDuringTxCompletion extends JDO_Test implements javax.transaction.Synchronization {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction tx;
	private boolean flag;
	private boolean ntrf;

	public void beforeCompletion() {
		try {
			out.println(" beforeCompletion ");
			tx.setRetainValues(true);

		} catch (Exception ex1) {
			if (ex1 instanceof javax.jdo.JDOUserException) {
				ex1.printStackTrace(out);
				out.println(" Test pass ");
			} else {
				ex1.printStackTrace(out);
				out.println(" Unexception caught in beforeCompletion");
				incrementErrorCount();
			}


		}
	}

	public void afterCompletion(int status) {
		try {
			out.println(" afterCompletion ");
			tx.setRetainValues(true);
		} catch (Exception ex2) {
			if (ex2 instanceof javax.jdo.JDOUserException) {
				ex2.printStackTrace(out);
				out.println(" Test pass ");
			} else {
				ex2.printStackTrace(out);
				out.println(" Unexception caught in afterCompletion");
				incrementErrorCount();
			}


		}
	}


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		SetRetainValuesCalledDuringTxCompletion t = new SetRetainValuesCalledDuringTxCompletion();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.setOptimistic()
	 *
	*/
	public void testSetRetainValuesCalledDuringTxCompletion() {
		try {

			tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);
			tx.setSynchronization(this);
			tx.commit();

		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexception caught in SetRetainValuesCalledDuringTxCompletion");
			incrementErrorCount();

		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		//ntrf = pmf.supportedOptions().contains("javax.jdo.option.NontransactionalRead");

		testSetRetainValuesCalledDuringTxCompletion();
		this.testSetRetainValuesCalledDuringTxCompletion();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in SetRetainValuesCalledDuringTxCompletion");
		else
			return Status.failed("FAILED in SetRetainValuesCalledDuringTxCompletion: number of errors detected is " + errCnt);


	}

}
