/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Set Retain Values True When Not Supported
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.2-8.
 *<BR>
 *<B>Assertion Description: </B>
 If the optional feature RetainValues is not supported, then a call to Transaction.setRetainValues with a
 value of true will cause a JDOUnsupportedOptionException to be thrown.

 *@test
 *@title Set Retain Values True When Not Supported
 *@executeClass com.sun.jdotck.transactions.SetRetainValuesTrueWhenNotSupported
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetRetainValuesTrueWhenNotSupported.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/22/01	 1.0
 */
public class SetRetainValuesTrueWhenNotSupported extends JDO_Test {
	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction transaction;
	private boolean flag;
	private boolean srvf;

	/* standalone interface */
	public static void main(String[] args) throws Exception {

		SetRetainValuesTrueWhenNotSupported t = new SetRetainValuesTrueWhenNotSupported();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.setRetainValues()
	 *
	*/
	public void testSetRetainValuesTrueWhenNotSupported() {
		try {

			Transaction tx = pm.currentTransaction();
			tx.begin();

			if (srvf == true)
				out.println(" This test can't be performed due to nature of implementation ");
			else
				tx.setRetainValues(true);

			tx.commit();

		} catch (Exception ex) {
			if (ex instanceof javax.jdo.JDOUnsupportedOptionException) {
				out.println(" excepted EXCEPTION");
				ex.printStackTrace(out);
			} else {
				ex.printStackTrace(out);
				out.println(" Unexception caught in SetRetainValuesTrueWhenNotSupported");
				incrementErrorCount();

			}
		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();

		srvf = pmf.supportedOptions().contains("javax.jdo.option.RetainValues");
		out.println(" srvf =" + srvf);
		testSetRetainValuesTrueWhenNotSupported();
		this.testSetRetainValuesTrueWhenNotSupported();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in SetRetainValuesTrueWhenNotSupported");
		else
			return Status.failed("FAILED in SetRetainValuesTrueWhenNotSupported: number of errors detected is " + errCnt);


	}

}
