/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 *
 */

package com.sun.jdotck.transactions;

import java.io.*;

import com.sun.javatest.Test;
import com.sun.javatest.Status;
import com.sun.jdotck.JDO_Test;
import com.sun.jdotck.models.mylib.*;


import java.util.Hashtable;
import java.util.Iterator;

import java.util.*;

import javax.jdo.*;


/**
 *<B>JavaTest Title:</B> Set Synchronization To Null
 *<BR>
 *<B>JavaTest Keywords:</B> transactions
 *<BR>
 *<B>Assertion ID:</B> A13.4.3-2.
 *<BR>
 *<B>Assertion Description: </B>
 If the parameter to Transaction.setSynchronization is null, then no instance
 will be notified.
 *@test
 *@title Set Synchronization To Null
 *@executeClass com.sun.jdotck.transactions.SetSynchronizationToNull
 *@executeArgs -PMFProperties $PMFProperties
 * -TransientTransactional $TransientTransactional
 * -NontransactionalRead $NontransactionalRead
 * -NontransactionalWrite $NontransactionalWrite
 * -RetainValues $RetainValues
 * -Optimistic $Optimistic
 * -ApplicationIdentity $ApplicationIdentity
 * -DatastoreIdentity $DatastoreIdentity
 * -NonDurableIdentity $NonDurableIdentity
 * -ArrayList $ArrayList
 * -HashMap $HashMap
 * -Hashtable $Hashtable
 * -LinkedList $LinkedList
 * -TreeMap $TreeMap
 * -TreeSet $TreeSet
 * -Vector $Vector
 * -Map $Map
 * -List $List
 * -Array $Array
 * -NullCollection $NullCollection
 *@source SetSynchronizationToNull.java
 *@keywords transactions
 */


/*
 * Revision History
 * ================
 * Author         :   	Date   : 	Version
 * Azita Kamangar  	10/11/01	 1.0
 */
public class SetSynchronizationToNull extends JDO_Test implements javax.transaction.Synchronization {

	private PersistenceManagerFactory pmf;
	private PersistenceManager pm;
	private PersistenceManager pm1;
	private Transaction tx;
	private boolean flag;
	private boolean flag2;


	public void beforeCompletion() {
		//out.println ("before Complition isActive returns :"  + tx.isActive());
	}

	public void afterCompletion(int status) {
		//out.println ("after Complition isActive returns :"  + tx.isActive());

	}


	/* standalone interface */
	public static void main(String[] args) throws Exception {

		SetSynchronizationToNull t = new SetSynchronizationToNull();

		Status s = t.run(args, new PrintWriter(System.err),
				new PrintWriter(System.out));
		s.exit();
	}

	/* test transactions.setSynchronization()
	 *
	*/



	public void testSetSynchronizationToNull() {
		try {

			tx = pm.currentTransaction();
			tx.begin();
			PCPoint p1 = new PCPoint(1, 3);
			pm.makePersistent(p1);

			tx.setSynchronization(this);
			tx.setSynchronization(null);


			if (tx.getSynchronization() == null) {
				;
			} else {
				incrementErrorCount();
			}

			tx.commit();

		} catch (Exception ex) {
			ex.printStackTrace(out);
			out.println(" Unexception caught in SetSynchronizationToNull");
			incrementErrorCount();

		}

	}

	/**
	 *The <code>run</code> method is invoked by JavaTest to
	 *execute the test defined by this class.
	 *@param args The arguments provided by the test script,
	 *providing configuration information for a test.
	 *@param err A stream to which to report errors.
	 *@param out A stream to report messages.
	 *@return A <code>Status</code> object representing the outcome of the test.
	 */
	public Status run(String[] args, PrintWriter err, PrintWriter out) {

		initializeArgs(args, err, out);


		PersistenceManagerFactory pmf = getPMF();
		pm = pmf.getPersistenceManager();


		this.testSetSynchronizationToNull();
		pm.close();

		err.flush();
		out.flush();

		int errCnt = getErrorCount();
		if (errCnt == 0)
			return Status.passed("PASSED in SetSynchronizationToNull");
		else
			return Status.failed("FAILED in SetSynchronizationToNull: number of errors detected is " + errCnt);


	}

}
